@@ -8,10 +8,17 @@ import re
 import sys
 import warnings
 
+from ._version import get_versions
+__version__ = get_versions()['version']
+del get_versions
+
 from . import config, errors, _runtests as runtests, types
 
 # Re-export typeof
-from .special import typeof, prange, pndindex, gdb, gdb_breakpoint, gdb_init
+from .special import (
+    typeof, prange, pndindex, gdb, gdb_breakpoint, gdb_init,
+    literally, literal_unroll
+)
 
 # Re-export error classes
 from .errors import *
@@ -19,10 +26,8 @@ from .errors import *
 # Re-export all type names
 from .types import *
 
-from .smartarray import SmartArray
-
 # Re-export decorators
-from .decorators import autojit, cfunc, generated_jit, jit, njit, stencil
+from .decorators import (cfunc, generated_jit, jit, njit, stencil, jit_module)
 
 # Re-export vectorize decorators and the thread layer querying function
 from .npyufunc import vectorize, guvectorize, threading_layer
@@ -37,12 +42,16 @@ from .jitclass import jitclass
 import numba.withcontexts
 from numba.withcontexts import objmode_context as objmode
 
+# Enable bytes/unicode array support (Python 3.x only)
+from .utils import IS_PY3
+if IS_PY3:
+    import numba.charseq
+
 # Keep this for backward compatibility.
 test = runtests.main
 
 
 __all__ = """
-    autojit
     cfunc
     from_dtype
     guvectorize
@@ -50,6 +59,7 @@ __all__ = """
     jitclass
     njit
     stencil
+    jit_module
     typeof
     prange
     gdb
@@ -58,10 +68,11 @@ __all__ = """
     stencil
     vectorize
     objmode
+    literal_unroll
     """.split() + types.__all__ + errors.__all__
 
 
-_min_llvmlite_version = (0, 27, 0)
+_min_llvmlite_version = (0, 31, 0)
 _min_llvm_version = (7, 0, 0)
 
 def _ensure_llvm():
@@ -172,6 +183,14 @@ Is set to True if Intel SVML is in use.
 """
 config.USING_SVML = _try_enable_svml()
 
-from ._version import get_versions
-__version__ = get_versions()['version']
-del get_versions
+
+# ---------------------- WARNING WARNING WARNING ----------------------------
+# The following imports occur below here (SVML init) because somewhere in their
+# import sequence they have a `@njit` wrapped function. This triggers too early
+# a bind to the underlying LLVM libraries which then irretrievably sets the LLVM
+# SVML state to "no SVML". See https://github.com/numba/numba/issues/4689 for
+# context.
+# ---------------------- WARNING WARNING WARNING ----------------------------
+
+# Initialize typed containers
+import numba.typed
