@@ -5,12 +5,14 @@
  */
 
 #include "_pymodule.h"
+#include <stddef.h>
 #include <stdio.h>
 #include <math.h>
 #include "_math_c99.h"
 #ifdef _MSC_VER
     #define int64_t signed __int64
     #define uint64_t unsigned __int64
+    #define uint32_t unsigned __int32
 #else
     #include <stdint.h>
 #endif
@@ -575,13 +577,13 @@ import_cython_function(const char *module_name, const char *function_name)
     Py_DECREF(module);
     if (capi == NULL)
         return NULL;
-    cobj = PyMapping_GetItemString(capi, function_name);
+    cobj = PyMapping_GetItemString(capi, (char *)function_name);
     Py_DECREF(capi);
     if (cobj == NULL) {
-	PyErr_Clear();
-	PyErr_Format(PyExc_ValueError,
-		     "No function '%s' found in __pyx_capi__ of '%s'",
-		     function_name, module_name);
+        PyErr_Clear();
+        PyErr_Format(PyExc_ValueError,
+                     "No function '%s' found in __pyx_capi__ of '%s'",
+                     function_name, module_name);
         return NULL;
     }
     /* 2.7+ => Cython exports a PyCapsule */
@@ -602,16 +604,16 @@ _numba_import_cython_function(PyObject *self, PyObject *args)
     PyObject *res;
 
     if (!PyArg_ParseTuple(args, "ss", &module_name, &function_name)) {
-	return NULL;
+        return NULL;
     }
     p = import_cython_function(module_name, function_name);
     if (p == NULL) {
-	return NULL;
+        return NULL;
     }
     res = PyLong_FromVoidPtr(p);
     if (res == NULL) {
       PyErr_SetString(PyExc_RuntimeError,
-		      "Could not convert function address to int");
+                      "Could not convert function address to int");
       return NULL;
     }
     return res;
@@ -885,7 +887,7 @@ numba_do_raise(PyObject *exc_packed)
         /* Unpack a (class/inst/tuple, arguments, location) tuple. */
         if (!PyArg_ParseTuple(exc_packed, "OOO", &exc, &value, &loc)) {
             Py_DECREF(exc_packed);
-            goto raise_error;
+            goto raise_error_w_loc;
         }
 
         if (exc == Py_None) {
@@ -919,13 +921,13 @@ numba_do_raise(PyObject *exc_packed)
         Py_XINCREF(loc);
         if (PyExceptionClass_Check(exc)) {
             /* It is a class, type used here just as a tmp var */
-            type = PyObject_CallObject(exc, NULL);
+            type = PyObject_CallObject(exc, value);
             if (type == NULL)
-                goto raise_error;
+                goto raise_error_w_loc;
             if (!PyExceptionInstance_Check(type)) {
                 PyErr_SetString(PyExc_TypeError,
                                 "exceptions must derive from BaseException");
-                goto raise_error;
+                goto raise_error_w_loc;
             }
             /* all ok, set type to the exc */
             Py_DECREF(type);
@@ -937,20 +939,9 @@ numba_do_raise(PyObject *exc_packed)
             Py_DECREF(exc_packed);
             PyErr_SetString(PyExc_TypeError,
                             "exceptions must derive from BaseException");
-            goto raise_error;
+            goto raise_error_w_loc;
         }
 
-        /* for injecting a frame into the traceback with location info in it */
-        if (loc != Py_None && PyTuple_Check(loc)) {
-            pos = 0;
-            function_name = PyTuple_GET_ITEM(loc, pos);
-            function_name_str = PyString_AsString(function_name);
-            pos = 1;
-            filename = PyTuple_GET_ITEM(loc, pos);
-            filename_str = PyString_AsString(filename);
-            pos = 2;
-            lineno = PyTuple_GET_ITEM(loc, pos);
-        }
         /* as this branch is exited:
          * - type should be an exception class
          * - value should be the args for the exception class instantiation
@@ -986,7 +977,7 @@ numba_do_raise(PyObject *exc_packed)
         /* exc should be an exception class or an instance of an exception */
         if (PyExceptionClass_Check(exc)) {
             type = exc;
-            value = PyObject_CallObject(exc, NULL);
+            value = PyObject_CallObject(exc, value);
             if (value == NULL)
                 goto raise_error;
             if (!PyExceptionInstance_Check(value)) {
@@ -1012,9 +1003,19 @@ numba_do_raise(PyObject *exc_packed)
 
     PyErr_SetObject(type, value);
 
-    /* instance is instantiated, if loc is present add a frame for it */
-    if(loc && loc != Py_None)
+raise_error_w_loc:
+    /* instance is instantiated/internal exception is raised, if loc is present
+     * add a frame for it into the traceback */
+    if(loc && loc != Py_None && PyTuple_Check(loc))
     {
+        pos = 0;
+        function_name = PyTuple_GET_ITEM(loc, pos);
+        function_name_str = PyString_AsString(function_name);
+        pos = 1;
+        filename = PyTuple_GET_ITEM(loc, pos);
+        filename_str = PyString_AsString(filename);
+        pos = 2;
+        lineno = PyTuple_GET_ITEM(loc, pos);
         traceback_add(function_name_str, filename_str, \
                       (int)PyLong_AsLong(lineno));
     }
@@ -1064,12 +1065,57 @@ numba_unpickle(const char *data, int n)
  * Unicode helpers
  */
 
+/* Developer note:
+ *
+ * The hash value of unicode objects is obtained via:
+ * ((PyASCIIObject *)(obj))->hash;
+ * The use comes from this definition:
+ * https://github.com/python/cpython/blob/6d43f6f081023b680d9db4542d19b9e382149f0a/Objects/unicodeobject.c#L119-L120
+ * and it's used extensively throughout the `cpython/Object/unicodeobject.c`
+ * source, not least in `unicode_hash` itself:
+ * https://github.com/python/cpython/blob/6d43f6f081023b680d9db4542d19b9e382149f0a/Objects/unicodeobject.c#L11662-L11679
+ *
+ * The Unicode string struct layouts are described here:
+ * https://github.com/python/cpython/blob/6d43f6f081023b680d9db4542d19b9e382149f0a/Include/cpython/unicodeobject.h#L82-L161
+ * essentially, all the unicode string layouts start with a `PyASCIIObject` at
+ * offset 0 (as of commit 6d43f6f081023b680d9db4542d19b9e382149f0a, somewhere
+ * in the 3.8 development cycle).
+ *
+ * For safety against future CPython internal changes, the code checks that the
+ * _base members of the unicode structs are what is expected in 3.7, and that
+ * their offset is 0. It then walks the struct to the hash location to make sure
+ * the offset is indeed the same as PyASCIIObject->hash.
+ * Note: The large condition in the if should evaluate to a compile time
+ * constant.
+ */
+
+#define MEMBER_SIZE(structure, member) sizeof(((structure *)0)->member)
+
 NUMBA_EXPORT_FUNC(void *)
-numba_extract_unicode(PyObject *obj, Py_ssize_t *length, int *kind) {
+numba_extract_unicode(PyObject *obj, Py_ssize_t *length, int *kind,
+                      unsigned int *ascii, Py_ssize_t *hash) {
 #if (PY_MAJOR_VERSION >= 3) && (PY_MINOR_VERSION >= 3)
     if (!PyUnicode_READY(obj)) {
         *length = PyUnicode_GET_LENGTH(obj);
         *kind = PyUnicode_KIND(obj);
+        /* could also use PyUnicode_IS_ASCII but it is not publicly advertised in https://docs.python.org/3/c-api/unicode.html */
+        *ascii = (unsigned int)(PyUnicode_MAX_CHAR_VALUE(obj) == (0x7f));
+        /* this is here as a crude check for safe casting of all unicode string
+         * structs to a PyASCIIObject */
+        if (MEMBER_SIZE(PyCompactUnicodeObject, _base) == sizeof(PyASCIIObject)             &&
+            MEMBER_SIZE(PyUnicodeObject, _base) == sizeof(PyCompactUnicodeObject)           &&
+            offsetof(PyCompactUnicodeObject, _base) == 0                                    &&
+            offsetof(PyUnicodeObject, _base) == 0                                           &&
+            offsetof(PyCompactUnicodeObject, _base.hash) == offsetof(PyASCIIObject, hash)   &&
+            offsetof(PyUnicodeObject, _base._base.hash) == offsetof(PyASCIIObject, hash)
+           ) {
+            /* Grab the hash from the type object cache, do not compute it. */
+            *hash = ((PyASCIIObject *)(obj))->hash;
+        }
+        else {
+            /* cast is not safe, fail */
+            return NULL;
+        }
         return PyUnicode_DATA(obj);
     } else {
         return NULL;
@@ -1080,6 +1126,52 @@ numba_extract_unicode(PyObject *obj, Py_ssize_t *length, int *kind) {
 #endif
 }
 
+/* this is late included as it #defines e.g. SHIFT that should not impact
+ * the above */
+#include "_unicodetype_db.h"
+
+/* This function is a modified copy of the private function gettyperecord from
+ * CPython's Objects/unicodectype.c
+ *
+ * See:https://github.com/python/cpython/blob/1d4b6ba19466aba0eb91c4ba01ba509acf18c723/Objects/unicodectype.c#L45-L59
+ */
+NUMBA_EXPORT_FUNC(void)
+numba_gettyperecord(Py_UCS4 code, int *upper, int *lower, int *title,
+                    unsigned char *decimal, unsigned char *digit,
+                    unsigned short *flags)
+{
+    int index;
+    const numba_PyUnicode_TypeRecord *rec;
+
+    if (code >= 0x110000)
+        index = 0;
+    else
+    {
+        index = index1[(code>>SHIFT)];
+        index = index2[(index<<SHIFT)+(code&((1<<SHIFT)-1))];
+    }
+
+    rec = &numba_PyUnicode_TypeRecords[index];
+    *upper = rec->upper;
+    *lower = rec->lower;
+    *title = rec->title;
+    *decimal = rec->decimal;
+    *digit = rec->digit;
+    *flags = rec->flags;
+}
+
+/* This function provides a consistent access point for the
+ * _PyUnicode_ExtendedCase array defined in CPython's Objects/unicodectype.c
+ * and now also as numba_PyUnicode_ExtendedCase in Numba's _unicodetype_db.h
+ */
+NUMBA_EXPORT_FUNC(Py_UCS4)
+numba_get_PyUnicode_ExtendedCase(int code)
+{
+    return numba_PyUnicode_ExtendedCase[code];
+}
+
+/* from _unicodetype_db.h */
+#undef SHIFT
 
 /*
  * defined break point for gdb
