@@ -1,12 +1,10 @@
 /*
 Expose all functions as pointers in a dedicated C extension.
 */
-
-#define NUMBA_EXPORT_FUNC(_rettype) static _rettype
-#define NUMBA_EXPORT_DATA(_vartype) static _vartype
-
+#include "cext/cext.h"
 /* Import _pymodule.h first, for a recent _POSIX_C_SOURCE */
 #include "_pymodule.h"
+
 #include <math.h>
 #ifdef _MSC_VER
     #define false 0
@@ -15,6 +13,12 @@ Expose all functions as pointers in a dedicated C extension.
 #else
     #include <stdbool.h>
 #endif
+
+/*
+Include C-extension here
+*/
+#include "cext/cext.h"
+
 /* Numba C helpers */
 #include "_helperlib.c"
 
@@ -87,7 +91,6 @@ build_c_helpers_dict(void)
     declmethod(unpack_slice);
     declmethod(do_raise);
     declmethod(unpickle);
-    declmethod(extract_unicode);
     declmethod(attempt_nocopy_reshape);
     declmethod(get_pyobject_private_data);
     declmethod(set_pyobject_private_data);
@@ -119,10 +122,44 @@ build_c_helpers_dict(void)
 
     /* Unicode string support */
     declmethod(extract_unicode);
+    declmethod(gettyperecord);
+    declmethod(get_PyUnicode_ExtendedCase);
 
     /* for gdb breakpoint */
     declmethod(gdb_breakpoint);
 
+    /* for dictionary support */
+    declmethod(test_dict);
+    declmethod(dict_new_minsize);
+    declmethod(dict_set_method_table);
+    declmethod(dict_free);
+    declmethod(dict_length);
+    declmethod(dict_lookup);
+    declmethod(dict_insert);
+    declmethod(dict_insert_ez);
+    declmethod(dict_delitem);
+    declmethod(dict_popitem);
+    declmethod(dict_iter_sizeof);
+    declmethod(dict_iter);
+    declmethod(dict_iter_next);
+    declmethod(dict_dump);
+
+    /* for list support */
+    declmethod(test_list);
+    declmethod(list_new);
+    declmethod(list_set_method_table);
+    declmethod(list_free);
+    declmethod(list_length);
+    declmethod(list_allocated);
+    declmethod(list_setitem);
+    declmethod(list_getitem);
+    declmethod(list_append);
+    declmethod(list_pop);
+    declmethod(list_delete_slice);
+    declmethod(list_iter_sizeof);
+    declmethod(list_iter);
+    declmethod(list_iter_next);
+
 #define MATH_UNARY(F, R, A) declmethod(F);
 #define MATH_BINARY(F, R, A, B) declmethod(F);
     #include "mathnames.h"
@@ -167,6 +204,18 @@ build_npymath_exports_dict(void)
     return dct;
 }
 
+
+/*
+ * Helper to deal with flushing stdout
+ */
+PyAPI_FUNC(void) _numba_flush_stdout(void) ;
+
+void
+_numba_flush_stdout(void) {
+  fflush(stdout);
+}
+
+
 static PyMethodDef ext_methods[] = {
     { "rnd_get_state", (PyCFunction) _numba_rnd_get_state, METH_O, NULL },
     { "rnd_get_py_state_ptr", (PyCFunction) _numba_rnd_get_py_state_ptr, METH_NOARGS, NULL },
