@@ -687,13 +687,19 @@ int typecode_ndarray(PyObject *dispatcher, PyArrayObject *ary) {
 
     /* The order in which we check for the right contiguous-ness is important.
        The order must match the order by numba.numpy_support.map_layout.
+       Further, only *contiguous-ness* is checked, not alignment, byte order or
+       write permissions.
     */
-    if (PyArray_ISCARRAY(ary)){
+    if (PyArray_IS_C_CONTIGUOUS(ary)){
         layout = 1;
-    } else if (PyArray_ISFARRAY(ary)) {
+    } else if (PyArray_IS_F_CONTIGUOUS(ary)) {
         layout = 2;
     }
 
+    /* the typecode cache by convention is for "behaved" arrays (aligned and
+     * writeable), all others must be forced to the fall back */
+    if (!PyArray_ISBEHAVED(ary)) goto FALLBACK;
+
     if (ndim <= 0 || ndim > N_NDIM) goto FALLBACK;
 
     dtype = dtype_num_to_typecode(PyArray_TYPE(ary));
