@@ -162,7 +162,10 @@ class _SourceFileBackedLocatorMixin(object):
     """
 
     def get_source_stamp(self):
-        st = os.stat(self._py_file)
+        if getattr(sys, 'frozen', False):
+            st = os.stat(sys.executable)
+        else:
+            st = os.stat(self._py_file)
         # We use both timestamp and size as some filesystems only have second
         # granularity.
         return st.st_mtime, st.st_size
@@ -224,8 +227,8 @@ class _InTreeCacheLocator(_SourceFileBackedLocatorMixin, _CacheLocator):
 
 class _UserWideCacheLocator(_SourceFileBackedLocatorMixin, _CacheLocator):
     """
-    A locator for functions backed by a regular Python module, cached
-    into a user-wide cache directory.
+    A locator for functions backed by a regular Python module or a
+    frozen executable, cached into a user-wide cache directory.
     """
 
     def __init__(self, py_func, py_file):
@@ -234,17 +237,33 @@ class _UserWideCacheLocator(_SourceFileBackedLocatorMixin, _CacheLocator):
         appdirs = AppDirs(appname="numba", appauthor=False)
         cache_dir = appdirs.user_cache_dir
         cache_subpath = os.path.dirname(py_file)
-        if os.name != "nt":
+        if not (os.name == "nt" or getattr(sys, 'frozen', False)):
             # On non-Windows, further disambiguate by appending the entire
             # absolute source path to the cache dir, e.g.
             # "$HOME/.cache/numba/usr/lib/.../mypkg/mysubpkg"
             # On Windows, this is undesirable because of path length limitations
+            # For frozen applications, there is no existing "full path"
+            # directory, and depends on a relocatable executable.
             cache_subpath = os.path.abspath(cache_subpath).lstrip(os.path.sep)
         self._cache_path = os.path.join(cache_dir, cache_subpath)
 
     def get_cache_path(self):
         return self._cache_path
 
+    @classmethod
+    def from_function(cls, py_func, py_file):
+        if not (os.path.exists(py_file) or getattr(sys, 'frozen', False)):
+            # Perhaps a placeholder (e.g. "<ipython-XXX>")
+            # stop function exit if frozen, since it uses a temp placeholder
+            return
+        self = cls(py_func, py_file)
+        try:
+            self.ensure_cache_path()
+        except OSError:
+            # Cannot ensure the cache directory exists or is writable
+            return
+        return self
+
 
 class _IPythonCacheLocator(_CacheLocator):
     """
@@ -271,7 +290,7 @@ class _IPythonCacheLocator(_CacheLocator):
         except ImportError:
             # older IPython version
             from IPython.utils.path import get_ipython_cache_dir
-        return os.path.join(get_ipython_cache_dir(), 'numba')
+        return os.path.join(get_ipython_cache_dir(), 'numba_cache')
 
     def get_source_stamp(self):
         return hashlib.sha256(self._bytes_source).hexdigest()
@@ -329,9 +348,10 @@ class _CacheImpl(object):
             raise RuntimeError("cannot cache function %r: no locator available "
                                "for file %r" % (qualname, source_path))
         self._locator = locator
-        # Keep the last dotted component, since the package name is already
-        # encoded in the directory.
-        modname = py_func.__module__.split('.')[-1]
+        # Use filename base name as module name to avoid conflict between
+        # foo/__init__.py and foo/foo.py
+        filename = inspect.getfile(py_func)
+        modname = os.path.splitext(os.path.basename(filename))[0]
         fullname = "%s.%s" % (modname, qualname)
         abiflags = getattr(sys, 'abiflags', '')
         self._filename_base = self.get_filename_base(fullname, abiflags)
@@ -394,7 +414,7 @@ class CompileResultCacheImpl(_CacheImpl):
             cannot_cache = "as it uses outer variables in a closure"
         elif cres.lifted:
             cannot_cache = "as it uses lifted loops"
-        elif cres.has_dynamic_globals:
+        elif cres.library.has_dynamic_globals:
             cannot_cache = ("as it uses dynamic globals "
                             "(such as ctypes pointers and large global arrays)")
         if cannot_cache:
@@ -411,7 +431,7 @@ class CodeLibraryCacheImpl(_CacheImpl):
     Implements the logic to cache CodeLibrary objects.
     """
 
-    _filename_prefix = None  # must be overriden
+    _filename_prefix = None  # must be overridden
 
     def reduce(self, codelib):
         """
@@ -586,7 +606,7 @@ class Cache(_Cache):
     by a subclass of ``_CacheImpl`` specified as *_impl_class* in the subclass.
     """
 
-    # The following class variables must be overriden by subclass.
+    # The following class variables must be overridden by subclass.
     _impl_class = None
 
     def __init__(self, py_func):
