@@ -37,7 +37,6 @@ def compile_cuda(pyfunc, return_type, args, debug, inline):
     flags.set('no_compile')
     flags.set('no_cpython_wrapper')
     if debug:
-        flags.set('boundcheck')
         flags.set('debuginfo')
     if inline:
         flags.set('forceinline')
@@ -104,6 +103,8 @@ class DeviceFunctionTemplate(object):
 
         Each signature is compiled once by caching the compiled function inside
         this object.
+
+        Returns the `CompileResult`.
         """
         if args not in self._compileinfos:
             cres = compile_cuda(self.py_func, None, args, debug=self.debug,
@@ -122,7 +123,46 @@ class DeviceFunctionTemplate(object):
         else:
             cres = self._compileinfos[args]
 
-        return cres.signature
+        return cres
+
+    def inspect_llvm(self, args):
+        """Returns the LLVM-IR text compiled for *args*.
+
+        Parameters
+        ----------
+        args: tuple[Type]
+            Argument types.
+
+        Returns
+        -------
+        llvmir : str
+        """
+        cres = self._compileinfos[args]
+        mod = cres.library._final_module
+        return str(mod)
+
+    def inspect_ptx(self, args, nvvm_options={}):
+        """Returns the PTX compiled for *args* for the currently active GPU
+
+        Parameters
+        ----------
+        args: tuple[Type]
+            Argument types.
+        nvvm_options : dict; optional
+            See `CompilationUnit.compile` in `numba/cuda/cudadrv/nvvm.py`.
+
+        Returns
+        -------
+        ptx : bytes
+        """
+        llvmir = self.inspect_llvm(args)
+        # Make PTX
+        cuctx = get_context()
+        device = cuctx.device
+        cc = device.compute_capability
+        arch = nvvm.get_arch_option(*cc)
+        ptx = nvvm.llvm_to_ptx(llvmir, opt=3, arch=arch, **nvvm_options)
+        return ptx
 
 
 def compile_device_template(pyfunc, debug=False, inline=False):
@@ -138,7 +178,7 @@ def compile_device_template(pyfunc, debug=False, inline=False):
 
         def generic(self, args, kws):
             assert not kws
-            return dft.compile(args)
+            return dft.compile(args).signature
 
     typingctx = CUDATargetDesc.typingctx
     typingctx.insert_user_function(dft, device_function_template)
@@ -212,9 +252,11 @@ class ExternFunction(object):
         self.sig = sig
 
 
-
 class ForAll(object):
     def __init__(self, kernel, ntasks, tpb, stream, sharedmem):
+        if ntasks < 0:
+            raise ValueError("Can't create ForAll with negative task count: %s"
+                             % ntasks)
         self.kernel = kernel
         self.ntasks = ntasks
         self.thread_per_block = tpb
@@ -222,6 +264,9 @@ class ForAll(object):
         self.sharedmem = sharedmem
 
     def __call__(self, *args):
+        if self.ntasks == 0:
+            return
+
         if isinstance(self.kernel, AutoJitCUDAKernel):
             kernel = self.kernel.specialize(*args)
         else:
@@ -244,7 +289,7 @@ class ForAll(object):
             ctx = get_context()
             kwargs = dict(
                 func=kernel._func.get(),
-                b2d_func=lambda tpb: 0,
+                b2d_func=0,     # dynamic-shared memory is constant to blksz
                 memsize=self.sharedmem,
                 blocksizelimit=1024,
             )
@@ -621,13 +666,7 @@ class CUDAKernel(CUDAKernelBase):
                 retr=retr)
 
         if isinstance(ty, types.Array):
-            if isinstance(ty, types.SmartArrayType):
-                devary = val.get('gpu')
-                retr.append(lambda: val.mark_changed('gpu'))
-                outer_parent = ctypes.c_void_p(0)
-                kernelargs.append(outer_parent)
-            else:
-                devary = wrap_arg(val).to_device(retr, stream)
+            devary = wrap_arg(val).to_device(retr, stream)
 
             c_intp = ctypes.c_ssize_t
 
