@@ -23,7 +23,7 @@ ADDRSPACE_SHARED = 3
 ADDRSPACE_CONSTANT = 4
 ADDRSPACE_LOCAL = 5
 
-# Opaque handle for comilation unit
+# Opaque handle for compilation unit
 nvvm_program = c_void_p
 
 # Result code
@@ -108,7 +108,7 @@ class NVVM(object):
             if cls.__INSTANCE is None:
                 cls.__INSTANCE = inst = object.__new__(cls)
                 try:
-                    inst.driver = open_cudalib('nvvm', ccc=True)
+                    inst.driver = open_cudalib('nvvm')
                 except OSError as e:
                     cls.__INSTANCE = None
                     errmsg = ("libNVVM cannot be found. Do `conda install "
@@ -164,7 +164,7 @@ class CompilationUnit(object):
         self.driver.check_error(err, 'Failed to add module')
 
     def compile(self, **options):
-        """Perform Compliation
+        """Perform Compilation
 
         The valid compiler options are
 
@@ -299,7 +299,7 @@ def _find_arch(mycc):
             if i == 0:
                 # CC lower than supported
                 raise NvvmSupportError("GPU compute capability %d.%d is "
-                                       "not supported (requires >=2.0)" % mycc)
+                                       "not supported (requires >=%d.%d)" % (mycc + cc))
             else:
                 # return the previous CC
                 return SUPPORTED_CC[i - 1]
@@ -318,17 +318,11 @@ def get_arch_option(major, minor):
     return 'compute_%d%d' % arch
 
 
-MISSING_LIBDEVICE_MSG = '''
-Please define environment variable NUMBAPRO_LIBDEVICE=/path/to/libdevice
-/path/to/libdevice -- is the path to the directory containing the libdevice.*.bc
-files in the installation of CUDA.  (requires CUDA >=8.0)
-'''
-
 MISSING_LIBDEVICE_FILE_MSG = '''Missing libdevice file for {arch}.
-Please ensure you have package cudatoolkit 8.0.
+Please ensure you have package cudatoolkit >= 8.
 Install package by:
 
-    conda install cudatoolkit=8.0
+    conda install cudatoolkit
 '''
 
 
