@@ -2,7 +2,7 @@ from __future__ import print_function, absolute_import, division
 
 from functools import reduce
 import operator
-import six
+import math
 
 from llvmlite.llvmpy.core import Type
 import llvmlite.llvmpy.core as lc
@@ -10,7 +10,9 @@ import llvmlite.binding as ll
 
 from numba.targets.imputils import Registry
 from numba import cgutils
+from numba import six
 from numba import types
+from numba.utils import IS_PY3
 from .cudadrv import nvvm
 from . import nvvmutils, stubs
 
@@ -271,16 +273,21 @@ def ptx_warp_sync(context, builder, sig, args):
     return context.get_dummy_value()
 
 
-@lower(stubs.shfl_sync_intrinsic, types.i4, types.i4, types.i4, types.i4, types.i4)
-@lower(stubs.shfl_sync_intrinsic, types.i4, types.i4, types.i8, types.i4, types.i4)
-@lower(stubs.shfl_sync_intrinsic, types.i4, types.i4, types.f4, types.i4, types.i4)
-@lower(stubs.shfl_sync_intrinsic, types.i4, types.i4, types.f8, types.i4, types.i4)
+@lower(stubs.shfl_sync_intrinsic, types.i4, types.i4, types.i4, types.i4,
+       types.i4)
+@lower(stubs.shfl_sync_intrinsic, types.i4, types.i4, types.i8, types.i4,
+       types.i4)
+@lower(stubs.shfl_sync_intrinsic, types.i4, types.i4, types.f4, types.i4,
+       types.i4)
+@lower(stubs.shfl_sync_intrinsic, types.i4, types.i4, types.f8, types.i4,
+       types.i4)
 def ptx_shfl_sync_i32(context, builder, sig, args):
     """
-    The NVVM intrinsic for shfl only supports i32, but the cuda intrinsic function supports
-    both 32 and 64 bit ints and floats, so for feature parity, i64, f32, and f64 are implemented.
-    Floats by way of bitcasting the float to an int, then shuffling, then bitcasting back.
-    And 64-bit values by packing them into 2 32bit values, shuffling thoose, and then packing back together.
+    The NVVM intrinsic for shfl only supports i32, but the cuda intrinsic
+    function supports both 32 and 64 bit ints and floats, so for feature parity,
+    i64, f32, and f64 are implemented. Floats by way of bitcasting the float to
+    an int, then shuffling, then bitcasting back. And 64-bit values by packing
+    them into 2 32bit values, shuffling thoose, and then packing back together.
     """
     mask, mode, value, index, clamp = args
     value_type = sig.args[2]
@@ -414,6 +421,90 @@ def ptx_selp(context, builder, sig, args):
     return builder.select(test, a, b)
 
 
+@lower(max, types.f4, types.f4)
+def ptx_max_f4(context, builder, sig, args):
+    fn = builder.module.get_or_insert_function(
+        lc.Type.function(
+            lc.Type.float(),
+            (lc.Type.float(), lc.Type.float())),
+        '__nv_fmaxf')
+    return builder.call(fn, args)
+
+
+@lower(max, types.f8, types.f4)
+@lower(max, types.f4, types.f8)
+@lower(max, types.f8, types.f8)
+def ptx_max_f8(context, builder, sig, args):
+    fn = builder.module.get_or_insert_function(
+        lc.Type.function(
+            lc.Type.double(),
+            (lc.Type.double(), lc.Type.double())),
+        '__nv_fmax')
+
+    return builder.call(fn, [
+        context.cast(builder, args[0], sig.args[0], types.double),
+        context.cast(builder, args[1], sig.args[1], types.double),
+    ])
+
+
+@lower(min, types.f4, types.f4)
+def ptx_min_f4(context, builder, sig, args):
+    fn = builder.module.get_or_insert_function(
+        lc.Type.function(
+            lc.Type.float(),
+            (lc.Type.float(), lc.Type.float())),
+        '__nv_fminf')
+    return builder.call(fn, args)
+
+
+@lower(min, types.f8, types.f4)
+@lower(min, types.f4, types.f8)
+@lower(min, types.f8, types.f8)
+def ptx_min_f8(context, builder, sig, args):
+    fn = builder.module.get_or_insert_function(
+        lc.Type.function(
+            lc.Type.double(),
+            (lc.Type.double(), lc.Type.double())),
+        '__nv_fmin')
+
+    return builder.call(fn, [
+        context.cast(builder, args[0], sig.args[0], types.double),
+        context.cast(builder, args[1], sig.args[1], types.double),
+    ])
+
+
+@lower(round, types.f4)
+@lower(round, types.f8)
+def ptx_round(context, builder, sig, args):
+    if not IS_PY3:
+        raise NotImplementedError(
+            "round returns a float on Python 2.x")
+    fn = builder.module.get_or_insert_function(
+        lc.Type.function(
+            lc.Type.int(64),
+            (lc.Type.double(),)),
+        '__nv_llrint')
+    return builder.call(fn, [
+        context.cast(builder, args[0], sig.args[0], types.double),
+    ])
+
+
+def gen_deg_rad(const):
+    def impl(context, builder, sig, args):
+        argty, = sig.args
+        factor = context.get_constant(argty, const)
+        return builder.fmul(factor, args[0])
+    return impl
+
+
+_deg2rad = math.pi / 180.
+_rad2deg = 180. / math.pi
+lower(math.radians, types.f4)(gen_deg_rad(_deg2rad))
+lower(math.radians, types.f8)(gen_deg_rad(_deg2rad))
+lower(math.degrees, types.f4)(gen_deg_rad(_rad2deg))
+lower(math.degrees, types.f8)(gen_deg_rad(_rad2deg))
+
+
 def _normalize_indices(context, builder, indty, inds):
     """
     Convert integer indices into tuple of intp
@@ -445,7 +536,7 @@ def _atomic_dispatcher(dispatch_fn):
                             (aryty.ndim, len(indty)))
 
         lary = context.make_array(aryty)(context, builder, ary)
-        ptr = cgutils.get_item_pointer(builder, aryty, lary, indices)
+        ptr = cgutils.get_item_pointer(context, builder, aryty, lary, indices)
         # dispatcher to implementation base on dtype
         return dispatch_fn(context, builder, dtype, ptr, val)
     return imp
@@ -458,10 +549,12 @@ def _atomic_dispatcher(dispatch_fn):
 def ptx_atomic_add_tuple(context, builder, dtype, ptr, val):
     if dtype == types.float32:
         lmod = builder.module
-        return builder.call(nvvmutils.declare_atomic_add_float32(lmod), (ptr, val))
+        return builder.call(nvvmutils.declare_atomic_add_float32(lmod),
+                            (ptr, val))
     elif dtype == types.float64:
         lmod = builder.module
-        return builder.call(nvvmutils.declare_atomic_add_float64(lmod), (ptr, val))
+        return builder.call(nvvmutils.declare_atomic_add_float64(lmod),
+                            (ptr, val))
     else:
         return builder.atomic_rmw('add', ptr, val, 'monotonic')
 
@@ -473,9 +566,11 @@ def ptx_atomic_add_tuple(context, builder, dtype, ptr, val):
 def ptx_atomic_max(context, builder, dtype, ptr, val):
     lmod = builder.module
     if dtype == types.float64:
-        return builder.call(nvvmutils.declare_atomic_max_float64(lmod), (ptr, val))
+        return builder.call(nvvmutils.declare_atomic_max_float64(lmod),
+                            (ptr, val))
     elif dtype == types.float32:
-        return builder.call(nvvmutils.declare_atomic_max_float32(lmod), (ptr, val))
+        return builder.call(nvvmutils.declare_atomic_max_float32(lmod),
+                            (ptr, val))
     elif dtype in (types.int32, types.int64):
         return builder.atomic_rmw('max', ptr, val, ordering='monotonic')
     elif dtype in (types.uint32, types.uint64):
@@ -491,9 +586,11 @@ def ptx_atomic_max(context, builder, dtype, ptr, val):
 def ptx_atomic_min(context, builder, dtype, ptr, val):
     lmod = builder.module
     if dtype == types.float64:
-        return builder.call(nvvmutils.declare_atomic_min_float64(lmod), (ptr, val))
+        return builder.call(nvvmutils.declare_atomic_min_float64(lmod),
+                            (ptr, val))
     elif dtype == types.float32:
-        return builder.call(nvvmutils.declare_atomic_min_float32(lmod), (ptr, val))
+        return builder.call(nvvmutils.declare_atomic_min_float32(lmod),
+                            (ptr, val))
     elif dtype in (types.int32, types.int64):
         return builder.atomic_rmw('min', ptr, val, ordering='monotonic')
     elif dtype in (types.uint32, types.uint64):
@@ -510,12 +607,14 @@ def ptx_atomic_cas_tuple(context, builder, sig, args):
 
     lary = context.make_array(aryty)(context, builder, ary)
     zero = context.get_constant(types.intp, 0)
-    ptr = cgutils.get_item_pointer(builder, aryty, lary, (zero,))
+    ptr = cgutils.get_item_pointer(context, builder, aryty, lary, (zero,))
     if aryty.dtype == types.int32:
         lmod = builder.module
-        return builder.call(nvvmutils.declare_atomic_cas_int32(lmod), (ptr, old, val))
+        return builder.call(nvvmutils.declare_atomic_cas_int32(lmod),
+                            (ptr, old, val))
     else:
-        raise TypeError('Unimplemented atomic compare_and_swap with %s array' % dtype)
+        raise TypeError('Unimplemented atomic compare_and_swap '
+                        'with %s array' % dtype)
 
 
 # -----------------------------------------------------------------------------
@@ -542,7 +641,10 @@ def _generic_array(context, builder, shape, dtype, symbol_name, addrspace,
         # Create global variable in the requested address-space
         gvmem = lmod.add_global_variable(laryty, symbol_name, addrspace)
         # Specify alignment to avoid misalignment bug
-        gvmem.align = context.get_abi_sizeof(lldtype)
+        align = context.get_abi_sizeof(lldtype)
+        # Alignment is required to be a power of 2 for shared memory. If it is
+        # not a power of 2 (e.g. for a Record array) then round up accordingly.
+        gvmem.align = 1 << (align - 1 ).bit_length()
 
         if elemcount <= 0:
             if can_dynsized:    # dynamic shared memory
@@ -558,7 +660,8 @@ def _generic_array(context, builder, shape, dtype, symbol_name, addrspace,
 
             gvmem.initializer = lc.Constant.undef(laryty)
 
-        if dtype not in types.number_domain:
+        other_supported_type = isinstance(dtype, (types.Record, types.Boolean))
+        if dtype not in types.number_domain and not other_supported_type:
             raise TypeError("unsupported type: %s" % dtype)
 
         # Convert to generic address-space
