@@ -1,12 +1,11 @@
-from numba.testing import SerialSuite
+from numba.testing import unittest
 from numba.testing import load_testsuite
 from numba import cuda
 from os.path import dirname, join
 
 
 def load_tests(loader, tests, pattern):
-
-    suite = SerialSuite()
+    suite = unittest.TestSuite()
     this_dir = dirname(__file__)
     suite.addTests(load_testsuite(loader, join(this_dir, 'nocuda')))
     if cuda.is_available():
