@@ -3,6 +3,7 @@ from __future__ import print_function, absolute_import, division
 from numba import unittest_support as unittest
 import numpy as np
 from numba import cuda
+from numba.cuda.testing import SerialMixin
 
 
 def reinterpret_array_type(byte_arr, start, stop, output):
@@ -11,7 +12,7 @@ def reinterpret_array_type(byte_arr, start, stop, output):
     output[0] = val
 
 
-class TestCudaArrayMethods(unittest.TestCase):
+class TestCudaArrayMethods(SerialMixin, unittest.TestCase):
     def test_reinterpret_array_type(self):
         """
         Reinterpret byte array as int32 in the GPU.
