@@ -4,9 +4,10 @@ import numpy as np
 
 from numba import unittest_support as unittest
 from numba import cuda, float32
+from numba.cuda.testing import SerialMixin
 
 
-class TestFastMathOption(unittest.TestCase):
+class TestFastMathOption(SerialMixin, unittest.TestCase):
     def test_kernel(self):
 
         def foo(arr, val):
