@@ -1,10 +1,10 @@
 from __future__ import print_function, division, absolute_import
 import numpy as np
 from numba import cuda, float32, float64, int32
-from numba.cuda.testing import unittest
+from numba.cuda.testing import unittest, SerialMixin
 
 
-class TestCudaIDiv(unittest.TestCase):
+class TestCudaIDiv(SerialMixin, unittest.TestCase):
     def test_inplace_div(self):
 
         @cuda.jit(argtypes=[float32[:, :], int32, int32])
