@@ -1,10 +1,10 @@
 from __future__ import print_function, absolute_import
 import math
 from numba import cuda
-from numba.cuda.testing import unittest
+from numba.cuda.testing import unittest, SerialMixin
 
 
-class TestCudaMonteCarlo(unittest.TestCase):
+class TestCudaMonteCarlo(SerialMixin, unittest.TestCase):
     def test_montecarlo(self):
         """Just make sure we can compile this
         """
