@@ -1,7 +1,7 @@
 from __future__ import print_function, absolute_import
 import numpy as np
 from numba import cuda, float32, int32
-from numba.cuda.testing import unittest
+from numba.cuda.testing import unittest, SerialMixin
 
 
 def foo(inp, out):
@@ -14,7 +14,7 @@ def copy(inp, out):
     cufoo(inp[i, :], out[i, :])
 
 
-class TestCudaSlicing(unittest.TestCase):
+class TestCudaSlicing(SerialMixin, unittest.TestCase):
     def test_slice_as_arg(self):
         global cufoo
         cufoo = cuda.jit("void(int32[:], int32[:])", device=True)(foo)
@@ -25,6 +25,13 @@ class TestCudaSlicing(unittest.TestCase):
 
         cucopy[1, 10](inp, out)
 
+    def test_assign_empty_slice(self):
+        # Issue #5017. Assigning to an empty slice should not result in a
+        # CudaAPIError.
+        N = 0
+        a = range(N)
+        arr = cuda.device_array(len(a))
+        arr[:] = cuda.to_device(a)
 
 if __name__ == '__main__':
     unittest.main()
