@@ -3,7 +3,7 @@ import numpy as np
 from numba import vectorize
 from numba import cuda, float64
 from numba import unittest_support as unittest
-from numba.cuda.testing import skip_on_cudasim
+from numba.cuda.testing import skip_on_cudasim, SerialMixin
 from numba import config
 
 sig = [float64(float64, float64)]
@@ -15,7 +15,7 @@ if config.ENABLE_CUDASIM:
 
 
 @skip_on_cudasim('ufunc API unsupported in the simulator')
-class TestCUDAVectorizeScalarArg(unittest.TestCase):
+class TestCUDAVectorizeScalarArg(SerialMixin, unittest.TestCase):
 
     def test_vectorize_scalar_arg(self):
         @vectorize(sig, target=target)
@@ -24,14 +24,20 @@ class TestCUDAVectorizeScalarArg(unittest.TestCase):
 
         A = np.arange(10, dtype=np.float64)
         dA = cuda.to_device(A)
-        vector_add(1.0, dA)
+        v = vector_add(1.0, dA)
+
+        np.testing.assert_array_almost_equal(
+            v.copy_to_host(),
+            np.arange(1, 11, dtype=np.float64))
 
     def test_vectorize_all_scalars(self):
         @vectorize(sig, target=target)
         def vector_add(a, b):
             return a + b
-        
-        vector_add(1.0, 1.0)
+
+        v = vector_add(1.0, 1.0)
+
+        np.testing.assert_almost_equal(2.0, v)
 
 
 if __name__ == '__main__':
