@@ -6,6 +6,8 @@ import sys
 import warnings
 
 from numba import utils
+from .errors import UnsupportedError
+from .ir import Loc
 
 
 class DataFlowAnalysis(object):
@@ -88,9 +90,11 @@ class DataFlowAnalysis(object):
         fn(info, inst)
 
     def handle_unknown_opcode(self, info, inst):
-        msg = "Use of unknown opcode {} at line {} of {}"
-        raise NotImplementedError(msg.format(inst.opname, inst.lineno,
-                                             self.bytecode.func_id.filename))
+        raise UnsupportedError(
+            "Use of unknown opcode '{}'".format(inst.opname),
+            loc=Loc(filename=self.bytecode.func_id.filename,
+                    line=inst.lineno)
+        )
 
     def dup_topx(self, info, inst, count):
         orig = [info.pop() for _ in range(count)]
@@ -121,6 +125,9 @@ class DataFlowAnalysis(object):
             info.pop(discard=True)
         return block
 
+    def op_NOP(self, info, inst):
+        pass
+
     def op_DUP_TOPX(self, info, inst):
         count = inst.arg
         assert 1 <= count <= 5, "Invalid DUP_TOPX count"
@@ -358,7 +365,7 @@ class DataFlowAnalysis(object):
             info.append(inst, func=func, vararg=vararg, res=res)
             info.push(res)
 
-    def op_BUILD_TUPLE_UNPACK_WITH_CALL(self, info, inst):
+    def _build_tuple_unpack(self, info, inst):
         # Builds tuple from other tuples on the stack
         tuples = list(reversed([info.pop() for _ in range(inst.arg)]))
         temps = [info.make_temp() for _ in range(len(tuples) - 1)]
@@ -366,6 +373,13 @@ class DataFlowAnalysis(object):
         # The result is in the last temp var
         info.push(temps[-1])
 
+    def op_BUILD_TUPLE_UNPACK_WITH_CALL(self, info, inst):
+        # just unpack the input tuple, call inst will be handled afterwards
+        self._build_tuple_unpack(info, inst)
+
+    def op_BUILD_TUPLE_UNPACK(self, info, inst):
+        self._build_tuple_unpack(info, inst)
+
     def op_BUILD_CONST_KEY_MAP(self, info, inst):
         keys = info.pop()
         vals = list(reversed([info.pop() for _ in range(inst.arg)]))
@@ -668,6 +682,32 @@ class DataFlowAnalysis(object):
         self.add_syntax_block(info, LoopBlock())
         info.append(inst)
 
+    def op_SETUP_WITH(self, info, inst):
+        cm = info.pop()    # the context-manager
+        self.add_syntax_block(info, WithBlock())
+        yielded = info.make_temp()
+        info.push(yielded)
+        info.append(inst, contextmanager=cm)
+
+    def op_WITH_CLEANUP(self, info, inst):
+        """
+        Note: py2 only opcode
+        """
+        # TOS is the return value of __exit__()
+        info.pop()
+        info.append(inst)
+
+    def op_WITH_CLEANUP_START(self, info, inst):
+        # TOS is the return value of __exit__()
+        info.pop()
+        info.append(inst)
+
+    def op_WITH_CLEANUP_FINISH(self, info, inst):
+        info.append(inst)
+
+    def op_END_FINALLY(self, info, inst):
+        info.append(inst)
+
     def op_POP_BLOCK(self, info, inst):
         block = self.pop_syntax_block(info)
         info.append(inst)
@@ -696,7 +736,7 @@ class DataFlowAnalysis(object):
                 defaults = []
                 for i in range(num_posdefaults):
                     defaults.append(info.pop())
-                defaults = tuple(defaults)
+                defaults = tuple(reversed(defaults))
         elif utils.PYVERSION >= (3, 0) and utils.PYVERSION < (3, 6):
             num_posdefaults = inst.arg & 0xff
             num_kwdefaults = (inst.arg >> 8) & 0xff
@@ -704,7 +744,7 @@ class DataFlowAnalysis(object):
             if MAKE_CLOSURE:
                 closure = info.pop()
             if num_annotations > 0:
-                annotations = info.pop() 
+                annotations = info.pop()
             if num_kwdefaults > 0:
                 kwdefaults = []
                 for i in range(num_kwdefaults):
@@ -716,7 +756,7 @@ class DataFlowAnalysis(object):
                 defaults = []
                 for i in range(num_posdefaults):
                     defaults.append(info.pop())
-                defaults = tuple(defaults)
+                defaults = tuple(reversed(defaults))
         else:
             if inst.arg & 0x8:
                 closure = info.pop()
@@ -727,7 +767,7 @@ class DataFlowAnalysis(object):
             if inst.arg & 0x1:
                 defaults = info.pop()
         res = info.make_temp()
-        info.append(inst, name=name, code=code, closure=closure, annotations=annotations, 
+        info.append(inst, name=name, code=code, closure=closure, annotations=annotations,
                     kwdefaults=kwdefaults, defaults=defaults, res=res)
         info.push(res)
 
@@ -739,6 +779,15 @@ class DataFlowAnalysis(object):
         info.append(inst, res=res)
         info.push(res)
 
+    #NOTE: Please see notes in `interpreter.py` surrounding the implementation
+    # of LOAD_METHOD and CALL_METHOD.
+
+    def op_LOAD_METHOD(self, *args, **kws):
+        self.op_LOAD_ATTR(*args, **kws)
+
+    def op_CALL_METHOD(self, *args, **kws):
+        self.op_CALL_FUNCTION(*args, **kws)
+
     def _ignored(self, info, inst):
         pass
 
@@ -750,6 +799,13 @@ class LoopBlock(object):
         self.stack_offset = None
 
 
+class WithBlock(object):
+    __slots__ = ('stack_offset',)
+
+    def __init__(self):
+        self.stack_offset = None
+
+
 class BlockInfo(object):
     def __init__(self, block, offset, incoming_blocks):
         self.block = block
@@ -861,3 +917,11 @@ class BlockInfo(object):
     @terminator.setter
     def terminator(self, inst):
         self._term = inst
+
+    @property
+    def active_try_block(self):
+        """Try except not supported.
+
+        See byteflow.py
+        """
+        return None
