@@ -190,7 +190,15 @@ class BooleanModel(DataModel):
         return self.as_data(builder, value)
 
     def from_data(self, builder, value):
-        return builder.trunc(value, self.get_value_type())
+        ty = self.get_value_type()
+        resalloca = cgutils.alloca_once(builder, ty)
+        cond = builder.icmp_unsigned('==', value, value.type(0))
+        with builder.if_else(cond) as (then, otherwise):
+            with then:
+                builder.store(ty(0), resalloca)
+            with otherwise:
+                builder.store(ty(1), resalloca)
+        return builder.load(resalloca)
 
     def from_argument(self, builder, value):
         return self.from_data(builder, value)
@@ -295,14 +303,15 @@ class EnumModel(ProxyModel):
 @register_default(types.Dummy)
 @register_default(types.ExceptionInstance)
 @register_default(types.ExternalFunction)
-@register_default(types.NumbaFunction)
 @register_default(types.Macro)
 @register_default(types.EnumClass)
 @register_default(types.IntEnumClass)
 @register_default(types.NumberClass)
+@register_default(types.TypeRef)
 @register_default(types.NamedTupleClass)
 @register_default(types.DType)
 @register_default(types.RecursiveCall)
+@register_default(types.MakeFunctionLiteral)
 class OpaqueModel(PrimitiveModel):
     """
     Passed as opaque pointers
@@ -410,6 +419,7 @@ class ExternalFuncPointerModel(PrimitiveModel):
 
 @register_default(types.UniTuple)
 @register_default(types.NamedUniTuple)
+@register_default(types.StarArgUniTuple)
 class UniTupleModel(DataModel):
     def __init__(self, dmm, fe_type):
         super(UniTupleModel, self).__init__(dmm, fe_type)
@@ -711,12 +721,25 @@ class ComplexModel(StructModel):
 
 @register_default(types.Tuple)
 @register_default(types.NamedTuple)
+@register_default(types.StarArgTuple)
 class TupleModel(StructModel):
     def __init__(self, dmm, fe_type):
         members = [('f' + str(i), t) for i, t in enumerate(fe_type)]
         super(TupleModel, self).__init__(dmm, fe_type, members)
 
 
+@register_default(types.UnionType)
+class UnionModel(StructModel):
+    def __init__(self, dmm, fe_type):
+        members = [
+            ('tag', types.uintp),
+            # XXX: it should really be a MemInfoPointer(types.voidptr)
+            ('payload', types.Tuple.from_types(fe_type.types)),
+        ]
+        super(UnionModel, self).__init__(dmm, fe_type, members)
+
+
+
 @register_default(types.Pair)
 class PairModel(StructModel):
     def __init__(self, dmm, fe_type):
@@ -857,16 +880,6 @@ class ArrayFlagsModel(StructModel):
         super(ArrayFlagsModel, self).__init__(dmm, fe_type, members)
 
 
-@register_default(types.SmartArrayType)
-class SmartArrayModel(StructModel):
-    def __init__(self, dmm, fe_type):
-        members = [
-            ('parent', types.pyobject),
-            ('data', fe_type.as_array)
-        ]
-        super(SmartArrayModel, self).__init__(dmm, fe_type, members)
-
-
 @register_default(types.NestedArray)
 class NestedArrayModel(ArrayModel):
     def __init__(self, dmm, fe_type):
@@ -966,6 +979,24 @@ class UnicodeCharSeq(DataModel):
     def get_data_type(self):
         return self._be_type
 
+    def as_data(self, builder, value):
+        return value
+
+    def from_data(self, builder, value):
+        return value
+
+    def as_return(self, builder, value):
+        return value
+
+    def from_return(self, builder, value):
+        return value
+
+    def as_argument(self, builder, value):
+        return value
+
+    def from_argument(self, builder, value):
+        return value
+
 
 @register_default(types.CharSeq)
 class CharSeq(DataModel):
