@@ -1,4 +1,4 @@
-# -*- coding: utf8 -*-
+# -*- coding: utf-8 -*-
 
 from __future__ import print_function, division, absolute_import
 
@@ -9,8 +9,10 @@ import struct
 import sys
 import uuid
 import weakref
+from copy import deepcopy
 
 from numba import _dispatcher, compiler, utils, types, config, errors
+from numba.compiler_lock import global_compiler_lock
 from numba.typeconv.rules import default_type_manager
 from numba import sigutils, serialize, typing
 from numba.typing.templates import fold_arguments
@@ -37,7 +39,6 @@ class OmittedArg(object):
 
 
 class _FunctionCompiler(object):
-
     def __init__(self, py_func, targetdescr, targetoptions, locals,
                  pipeline_class):
         self.py_func = py_func
@@ -46,6 +47,10 @@ class _FunctionCompiler(object):
         self.locals = locals
         self.pysig = utils.pysignature(self.py_func)
         self.pipeline_class = pipeline_class
+        # Remember key=(args, return_type) combinations that will fail
+        # compilation to avoid compilation attempt on them.  The values are
+        # the exceptions.
+        self._failed_cache = {}
 
     def fold_argument_types(self, args, kws):
         """
@@ -59,7 +64,7 @@ class _FunctionCompiler(object):
         def default_handler(index, param, default):
             return types.Omitted(default)
         def stararg_handler(index, param, values):
-            return types.Tuple(values)
+            return types.StarArgTuple(values)
         # For now, we take argument values from the @jit function, even
         # in the case of generated jit.
         args = fold_arguments(self.pysig, args, kws,
@@ -69,6 +74,28 @@ class _FunctionCompiler(object):
         return self.pysig, args
 
     def compile(self, args, return_type):
+        status, retval = self._compile_cached(args, return_type)
+        if status:
+            return retval
+        else:
+            raise retval
+
+    def _compile_cached(self, args, return_type):
+        key = tuple(args), return_type
+        try:
+            return False, self._failed_cache[key]
+        except KeyError:
+            pass
+
+        try:
+            retval = self._compile_core(args, return_type)
+        except errors.TypingError as e:
+            self._failed_cache[key] = e
+            return False, e
+        else:
+            return True, retval
+
+    def _compile_core(self, args, return_type):
         flags = compiler.Flags()
         self.targetdescr.options.parse_as_flags(flags, self.targetoptions)
         flags = self._customize_flags(flags)
@@ -166,7 +193,8 @@ class _DispatcherBase(_dispatcher.Dispatcher):
 
     __numba__ = "py_func"
 
-    def __init__(self, arg_count, py_func, pysig, can_fallback):
+    def __init__(self, arg_count, py_func, pysig, can_fallback,
+                 exact_match_required):
         self._tm = default_type_manager
 
         # A mapping of signatures to compile results
@@ -191,11 +219,12 @@ class _DispatcherBase(_dispatcher.Dispatcher):
                                         arg_count, self._fold_args,
                                         argnames, defargs,
                                         can_fallback,
-                                        has_stararg)
+                                        has_stararg,
+                                        exact_match_required)
 
         self.doc = py_func.__doc__
         self._compiling_counter = _CompilingCounter()
-        utils.finalize(self, self._make_finalizer())
+        weakref.finalize(self, self._make_finalizer())
 
     def _reset_overloads(self):
         self._clear()
@@ -308,7 +337,7 @@ class _DispatcherBase(_dispatcher.Dispatcher):
             """
             if config.SHOW_HELP:
                 help_msg = errors.error_extras[issue_type]
-                e.patch_message(''.join(e.args) + help_msg)
+                e.patch_message('\n'.join((str(e).rstrip(), help_msg)))
             if config.FULL_TRACEBACKS:
                 raise e
             else:
@@ -322,6 +351,30 @@ class _DispatcherBase(_dispatcher.Dispatcher):
                 argtypes.append(self.typeof_pyval(a))
         try:
             return self.compile(tuple(argtypes))
+        except errors.ForceLiteralArg as e:
+            # Received request for compiler re-entry with the list of arguments
+            # indicated by e.requested_args.
+            # First, check if any of these args are already Literal-ized
+            already_lit_pos = [i for i in e.requested_args
+                               if isinstance(args[i], types.Literal)]
+            if already_lit_pos:
+                # Abort compilation if any argument is already a Literal.
+                # Letting this continue will cause infinite compilation loop.
+                m = ("Repeated literal typing request.\n"
+                     "{}.\n"
+                     "This is likely caused by an error in typing. "
+                     "Please see nested and suppressed exceptions.")
+                info = ', '.join('Arg #{} is {}'.format(i, args[i])
+                                 for i in  sorted(already_lit_pos))
+                raise errors.CompilerError(m.format(info))
+            # Convert requested arguments into a Literal.
+            args = [(types.literal
+                     if i in e.requested_args
+                     else lambda x: x)(args[i])
+                    for i, v in enumerate(args)]
+            # Re-enter compilation with the Literal-ized arguments
+            return self._compile_for_args(*args)
+
         except errors.TypingError as e:
             # Intercept typing error that may be due to an argument
             # that failed inferencing as a Numba type
@@ -362,11 +415,26 @@ class _DispatcherBase(_dispatcher.Dispatcher):
             if config.SHOW_HELP:
                 if hasattr(e, 'patch_message'):
                     help_msg = errors.error_extras['reportable']
-                    e.patch_message(''.join(e.args) + help_msg)
+                    e.patch_message('\n'.join((str(e).rstrip(), help_msg)))
             # ignore the FULL_TRACEBACKS config, this needs reporting!
             raise e
 
     def inspect_llvm(self, signature=None):
+        """Get the LLVM intermediate representation generated by compilation.
+
+        Parameters
+        ----------
+        signature : tuple of numba types, optional
+            Specify a signature for which to obtain the LLVM IR. If None, the
+            IR is returned for all available signatures.
+
+        Returns
+        -------
+        llvm : dict[signature, str] or str
+            Either the LLVM IR string for the specified signature, or, if no
+            signature was given, a dictionary mapping signatures to LLVM IR
+            strings.
+        """
         if signature is not None:
             lib = self.overloads[signature].library
             return lib.get_llvm_str()
@@ -374,28 +442,66 @@ class _DispatcherBase(_dispatcher.Dispatcher):
         return dict((sig, self.inspect_llvm(sig)) for sig in self.signatures)
 
     def inspect_asm(self, signature=None):
+        """Get the generated assembly code.
+
+        Parameters
+        ----------
+        signature : tuple of numba types, optional
+            Specify a signature for which to obtain the assembly code. If
+            None, the assembly code is returned for all available signatures.
+
+        Returns
+        -------
+        asm : dict[signature, str] or str
+            Either the assembly code for the specified signature, or, if no
+            signature was given, a dictionary mapping signatures to assembly
+            code.
+        """
         if signature is not None:
             lib = self.overloads[signature].library
             return lib.get_asm_str()
 
         return dict((sig, self.inspect_asm(sig)) for sig in self.signatures)
 
-    def inspect_types(self, file=None, **kwargs):
-        """
-        print or return annotated source with Numba intermediate IR
+    def inspect_types(self, file=None, signature=None,
+                      pretty=False, style='default', **kwargs):
+        """Print/return Numba intermediate representation (IR)-annotated code.
 
-        Pass `pretty=True` to attempt color highlighting, and HTML rendering in
-        Jupyter and IPython by returning an Annotate Object. `file` must be
-        None if used in conjunction with `pretty=True`.
+        Parameters
+        ----------
+        file : file-like object, optional
+            File to which to print. Defaults to sys.stdout if None. Must be
+            None if ``pretty=True``.
+        signature : tuple of numba types, optional
+            Print/return the intermediate representation for only the given
+            signature. If None, the IR is printed for all available signatures.
+        pretty : bool, optional
+            If True, an Annotate object will be returned that can render the
+            IR with color highlighting in Jupyter and IPython. ``file`` must
+            be None if ``pretty`` is True. Additionally, the ``pygments``
+            library must be installed for ``pretty=True``.
+        style : str, optional
+            Choose a style for rendering. Ignored if ``pretty`` is ``False``.
+            This is directly consumed by ``pygments`` formatters. To see a
+            list of available styles, import ``pygments`` and run
+            ``list(pygments.styles.get_all_styles())``.
+
+        Returns
+        -------
+        annotated : Annotate object, optional
+            Only returned if ``pretty=True``, otherwise this function is only
+            used for its printing side effect. If ``pretty=True``, an Annotate
+            object is returned that can render itself in Jupyter and IPython.
         """
-        pretty = kwargs.get('pretty', False)
-        style = kwargs.get('style', 'default')
+        overloads = self.overloads
+        if signature is not None:
+            overloads = {signature: self.overloads[signature]}
 
         if not pretty:
             if file is None:
                 file = sys.stdout
 
-            for ver, res in utils.iteritems(self.overloads):
+            for ver, res in utils.iteritems(overloads):
                 print("%s %s" % (self.py_func.__name__, ver), file=file)
                 print('-' * 80, file=file)
                 print(res.type_annotation, file=file)
@@ -404,7 +510,7 @@ class _DispatcherBase(_dispatcher.Dispatcher):
             if file is not None:
                 raise ValueError("`file` must be None if `pretty=True`")
             from .pretty_annotate import Annotate
-            return Annotate(self, style=style)
+            return Annotate(self, signature=signature, style=style)
 
     def inspect_cfg(self, signature=None, show_wrapper=None):
         """
@@ -434,10 +540,15 @@ class _DispatcherBase(_dispatcher.Dispatcher):
         signature. If no signature is supplied a dictionary of signature to
         annotation information is returned.
         """
-        if signature is not None:
-            cres = self.overloads[signature]
-            return cres.type_annotation.annotate_raw()
-        return dict((sig, self.annotate(sig)) for sig in self.signatures)
+        signatures = self.signatures if signature is None else [signature]
+        out = collections.OrderedDict()
+        for sig in signatures:
+            cres = self.overloads[sig]
+            ta = cres.type_annotation
+            key = (ta.func_id.filename + ':' + str(ta.func_id.firstlineno + 1),
+                   ta.signature)
+            out[key] = ta.annotate_raw()[key]
+        return out
 
     def _explain_ambiguous(self, *args, **kws):
         """
@@ -520,7 +631,7 @@ class Dispatcher(_DispatcherBase):
     __numba__ = 'py_func'
 
     def __init__(self, py_func, locals={}, targetoptions={},
-                 impl_kind='direct', pipeline_class=compiler.Pipeline):
+                 impl_kind='direct', pipeline_class=compiler.Compiler):
         """
         Parameters
         ----------
@@ -532,7 +643,7 @@ class Dispatcher(_DispatcherBase):
             Target-specific config options.
         impl_kind: str
             Select the compiler mode for `@jit` and `@generated_jit`
-        pipeline_class: type numba.compiler.BasePipeline
+        pipeline_class: type numba.compiler.CompilerBase
             The compiler pipeline type.
         """
         self.typingctx = self.targetdescr.typing_context
@@ -541,7 +652,9 @@ class Dispatcher(_DispatcherBase):
         pysig = utils.pysignature(py_func)
         arg_count = len(pysig.parameters)
         can_fallback = not targetoptions.get('nopython', False)
-        _DispatcherBase.__init__(self, arg_count, py_func, pysig, can_fallback)
+
+        _DispatcherBase.__init__(self, arg_count, py_func, pysig, can_fallback,
+                                 exact_match_required=False)
 
         functools.update_wrapper(self, py_func)
 
@@ -628,7 +741,7 @@ class Dispatcher(_DispatcherBase):
         self._memo[u] = self
         self._recent.append(self)
 
-    @compiler.global_compiler_lock
+    @global_compiler_lock
     def compile(self, sig):
         if not self._can_compile:
             raise RuntimeError("compilation disabled")
@@ -639,7 +752,6 @@ class Dispatcher(_DispatcherBase):
             existing = self.overloads.get(tuple(args))
             if existing is not None:
                 return existing.entry_point
-
             # Try to load from disk cache
             cres = self._cache.load_overload(sig, self.targetctx)
             if cres is not None:
@@ -652,7 +764,12 @@ class Dispatcher(_DispatcherBase):
                 return cres.entry_point
 
             self._cache_misses[sig] += 1
-            cres = self._compiler.compile(args, return_type)
+            try:
+                cres = self._compiler.compile(args, return_type)
+            except errors.ForceLiteralArg as e:
+                def folded(args, kws):
+                    return self._compiler.fold_argument_types(args, kws)[1]
+                raise e.bind_fold_arguments(folded)
             self.add_overload(cres)
             self._cache.save_overload(sig, cres)
             return cres.entry_point
@@ -730,7 +847,8 @@ class LiftedCode(_DispatcherBase):
         _DispatcherBase.__init__(self, self.func_ir.arg_count,
                                  self.func_ir.func_id.func,
                                  self.func_ir.func_id.pysig,
-                                 can_fallback=True)
+                                 can_fallback=True,
+                                 exact_match_required=False)
 
     def get_source_location(self):
         """Return the starting line number of the loop.
@@ -742,7 +860,7 @@ class LiftedCode(_DispatcherBase):
         """
         pass
 
-    @compiler.global_compiler_lock
+    @global_compiler_lock
     def compile(self, sig):
         # Use counter to track recursion compilation depth
         with self._compiling_counter:
@@ -758,15 +876,16 @@ class LiftedCode(_DispatcherBase):
 
             self._pre_compile(args, return_type, flags)
 
-            # Clone IR to avoid mutation in rewrite pass
+            # Clone IR to avoid (some of the) mutation in the rewrite pass
             cloned_func_ir = self.func_ir.copy()
             cres = compiler.compile_ir(typingctx=self.typingctx,
-                                        targetctx=self.targetctx,
-                                        func_ir=cloned_func_ir,
-                                        args=args, return_type=return_type,
-                                        flags=flags, locals=self.locals,
-                                        lifted=(),
-                                        lifted_from=self.lifted_from)
+                                       targetctx=self.targetctx,
+                                       func_ir=cloned_func_ir,
+                                       args=args, return_type=return_type,
+                                       flags=flags, locals=self.locals,
+                                       lifted=(),
+                                       lifted_from=self.lifted_from,
+                                       is_lifted_loop=True,)
 
             # Check typing error if object mode is used
             if cres.typing_error is not None and not flags.enable_pyobject:
