@@ -25,22 +25,55 @@ class NumbaWarning(Warning):
     Base category for all Numba compiler warnings.
     """
 
+    def __init__(self, msg, loc=None, highlighting=True, ):
+        self.msg = msg
+        self.loc = loc
+        if highlighting:
+            highlight = termcolor().errmsg
+        else:
+            def highlight(x):
+                return x
+        if loc:
+            super(NumbaWarning, self).__init__(
+                highlight("%s\n%s\n" % (msg, loc.strformat())))
+        else:
+            super(NumbaWarning, self).__init__(highlight("%s" % (msg,)))
 
-class PerformanceWarning(NumbaWarning):
+
+class NumbaPerformanceWarning(NumbaWarning):
     """
     Warning category for when an operation might not be
     as fast as expected.
     """
 
 
-class ParallelSafetyWarning(RuntimeWarning):
+class NumbaDeprecationWarning(NumbaWarning):
+    """
+    Warning category for use of a deprecated feature.
+    """
+
+
+class NumbaPendingDeprecationWarning(NumbaWarning):
+    """
+    Warning category for use of a feature that is pending deprecation.
+    """
+
+
+class NumbaParallelSafetyWarning(NumbaWarning):
     """
     Warning category for when an operation in a prange
     might not have parallel semantics.
     """
 
+
+class NumbaTypeSafetyWarning(NumbaWarning):
+    """
+    Warning category for unsafe casting operations.
+    """
+
 # These are needed in the color formatting of errors setup
 
+
 @six.add_metaclass(abc.ABCMeta)
 class _ColorScheme(object):
 
@@ -109,7 +142,7 @@ try:
     # coloring so CI system output is consistently readable without having
     # to read between shell escape characters.
     if os.environ.get('NUMBA_DISABLE_ERROR_MESSAGE_HIGHLIGHTING', None):
-        raise ImportError # just to trigger the exception handler below
+        raise ImportError  # just to trigger the exception handler below
 
 except ImportError:
 
@@ -142,8 +175,7 @@ except ImportError:
 
 else:
 
-    from colorama import init, reinit, deinit, Fore, Back, Style
-    from contextlib import contextmanager
+    from colorama import init, reinit, deinit, Fore, Style
 
     class ColorShell(object):
         _has_initialized = False
@@ -175,10 +207,10 @@ else:
 
     # No color added, just bold weighting
     themes['no_color'] = {'code': None,
-                         'errmsg': None,
-                         'filename': None,
-                         'indicate': None,
-                         'highlight': None, }
+                          'errmsg': None,
+                          'filename': None,
+                          'indicate': None,
+                          'highlight': None, }
 
     # suitable for terminals with a dark background
     themes['dark_bg'] = {'code': Fore.BLUE,
@@ -203,10 +235,10 @@ else:
 
     # suitable for use in jupyter notebooks
     themes['jupyter_nb'] = {'code': Fore.BLACK,
-                         'errmsg': Fore.BLACK,
-                         'filename': Fore.GREEN,
-                         'indicate': Fore.CYAN,
-                         'highlight': Fore.RED, }
+                            'errmsg': Fore.BLACK,
+                            'filename': Fore.GREEN,
+                            'indicate': Fore.CYAN,
+                            'highlight': Fore.RED, }
 
     default_theme = themes['no_color']
 
@@ -253,6 +285,15 @@ else:
             _termcolor_inst = HighlightColorScheme(scheme)
         return _termcolor_inst
 
+feedback_details = """
+Please report the error message and traceback, along with a minimal reproducer
+at: https://github.com/numba/numba/issues/new
+
+If more help is needed please feel free to speak to the Numba core developers
+directly at: https://gitter.im/numba/numba
+
+Thanks in advance for your help in improving Numba!
+"""
 
 unsupported_error_info = """
 Unsupported functionality was found in the code Numba was trying to compile.
@@ -271,26 +312,33 @@ If the code is valid and the unsupported functionality is important to you
 please file a feature request at: https://github.com/numba/numba/issues/new
 
 To see Python/NumPy features supported by the latest release of Numba visit:
-http://numba.pydata.org/numba-doc/dev/reference/pysupported.html
+http://numba.pydata.org/numba-doc/latest/reference/pysupported.html
 and
-http://numba.pydata.org/numba-doc/dev/reference/numpysupported.html
+http://numba.pydata.org/numba-doc/latest/reference/numpysupported.html
 """
 
 constant_inference_info = """
 Numba could not make a constant out of something that it decided should be
 a constant. This could well be a current limitation in Numba's internals,
-please either raise a bug report along with a minimal reproducer at:
-https://github.com/numba/numba/issues/new
-"""
+however please first check that your code is valid for compilation,
+particularly with respect to string interpolation (not supported!) and
+the requirement of compile time constants as arguments to exceptions:
+http://numba.pydata.org/numba-doc/latest/reference/pysupported.html?highlight=exceptions#constructs
+
+If the code is valid and the unsupported functionality is important to you
+please file a feature request at: https://github.com/numba/numba/issues/new
+
+If you think your code should work with Numba. %s
+""" % feedback_details
 
 typing_error_info = """
 This is not usually a problem with Numba itself but instead often caused by
 the use of unsupported features or an issue in resolving types.
 
 To see Python/NumPy features supported by the latest release of Numba visit:
-http://numba.pydata.org/numba-doc/dev/reference/pysupported.html
+http://numba.pydata.org/numba-doc/latest/reference/pysupported.html
 and
-http://numba.pydata.org/numba-doc/dev/reference/numpysupported.html
+http://numba.pydata.org/numba-doc/latest/reference/numpysupported.html
 
 For more information about typing errors and how to debug them visit:
 http://numba.pydata.org/numba-doc/latest/user/troubleshoot.html#my-code-doesn-t-compile
@@ -303,15 +351,9 @@ https://github.com/numba/numba/issues/new
 reportable_issue_info = """
 -------------------------------------------------------------------------------
 This should not have happened, a problem has occurred in Numba's internals.
-
-Please report the error message and traceback, along with a minimal reproducer
-at: https://github.com/numba/numba/issues/new
-
-If more help is needed please feel free to speak to the Numba core developers
-directly at: https://gitter.im/numba/numba
-
-Thanks in advance for your help in improving Numba!
-"""
+You are currently using Numba version %s.
+%s
+""" % (numba.__version__, feedback_details)
 
 error_extras = dict()
 error_extras['unsupported_error'] = unsupported_error_info
@@ -388,7 +430,15 @@ class WarningsFixer(object):
         """
         Emit all stored warnings.
         """
-        for (filename, lineno, category), messages in sorted(self._warnings.items()):
+        def key(arg):
+            # It is possible through codegen to create entirely identical
+            # warnings, this leads to comparing types when sorting which breaks
+            # on Python 3. Key as str() and if the worse happens then `id`
+            # creates some uniqueness
+            return str(arg) + str(id(arg))
+
+        for (filename, lineno, category), messages in sorted(
+                self._warnings.items(), key=key):
             for msg in sorted(messages):
                 warnings.warn_explicit(msg, category, filename, lineno)
         self._warnings.clear()
@@ -402,7 +452,8 @@ class NumbaError(Exception):
         if highlighting:
             highlight = termcolor().errmsg
         else:
-            def highlight(x): return x
+            def highlight(x):
+                return x
         if loc:
             super(NumbaError, self).__init__(
                 highlight("%s\n%s\n" % (msg, loc.strformat())))
@@ -461,6 +512,7 @@ class NotDefinedError(IRError):
     """
     An undefined variable is encountered during interpretation of IR.
     """
+
     def __init__(self, name, loc=None):
         self.name = name
         msg = "Variable '%s' is not defined." % name
@@ -496,6 +548,7 @@ class LoweringError(NumbaError):
     """
     An error occurred during lowering.
     """
+
     def __init__(self, msg, loc=None):
         super(LoweringError, self).__init__(msg, loc=loc)
 
@@ -507,7 +560,6 @@ class UnsupportedParforsError(NumbaError):
     pass
 
 
-
 class ForbiddenConstruct(LoweringError):
     """
     A forbidden Python construct was encountered (e.g. use of locals()).
@@ -539,6 +591,7 @@ class ByteCodeSupportError(NumbaError):
     """
     Failure to extract the bytecode of the user's function.
     """
+
     def __init__(self, msg, loc=None):
         super(ByteCodeSupportError, self).__init__(msg, loc=loc)
 
@@ -554,10 +607,9 @@ class ConstantInferenceError(NumbaError):
     """
     Failure during constant inference.
     """
+
     def __init__(self, value, loc=None):
-        self.value = value
-        msg = "Cannot make a constant from: %s" % value
-        super(ConstantInferenceError, self).__init__(msg, loc=loc)
+        super(ConstantInferenceError, self).__init__(value, loc=loc)
 
 
 class InternalError(NumbaError):
@@ -578,6 +630,55 @@ class RequireLiteralValue(TypingError):
     pass
 
 
+class ForceLiteralArg(NumbaError):
+    """A Pseudo-exception to signal the dispatcher to type an argument literally
+
+    Attributes
+    ----------
+    requested_args : frozenset[int]
+        requested positions of the arguments.
+    """
+    def __init__(self, arg_indices, fold_arguments=None, loc=None):
+        """
+        Parameters
+        ----------
+        arg_indices : Sequence[int]
+            requested positions of the arguments.
+        fold_arguments: callable
+            A function ``(tuple, dict) -> tuple`` that binds and flattens
+            the ``args`` and ``kwargs``.
+        loc : numba.ir.Loc or None
+        """
+        super(ForceLiteralArg, self).__init__(
+            "Pseudo-exception to force literal arguments in the dispatcher",
+            loc=loc,
+        )
+        self.requested_args = frozenset(arg_indices)
+        self.fold_arguments = fold_arguments
+
+    def bind_fold_arguments(self, fold_arguments):
+        """Bind the fold_arguments function
+        """
+        from numba import utils
+
+        e = ForceLiteralArg(self.requested_args, fold_arguments,
+                            loc=self.loc)
+        return utils.chain_exception(e, self)
+
+    def combine(self, other):
+        """Returns a new instance by or'ing the requested_args.
+        """
+        if not isinstance(other, ForceLiteralArg):
+            m = '*other* must be a {} but got a {} instead'
+            raise TypeError(m.format(ForceLiteralArg, type(other)))
+        return ForceLiteralArg(self.requested_args | other.requested_args)
+
+    def __or__(self, other):
+        """Same as self.combine(other)
+        """
+        return self.combine(other)
+
+
 class LiteralTypingError(TypingError):
     """
     Failure in typing a Literal type
@@ -589,7 +690,6 @@ def _format_msg(fmt, args, kwargs):
     return fmt.format(*args, **kwargs)
 
 
-import os.path
 _numba_path = os.path.dirname(__file__)
 loc_info = {}
 
