@@ -1,7 +1,10 @@
+from __future__ import print_function, absolute_import
+
 import types as pytypes  # avoid confusion with numba.types
 import ctypes
 import numba
-from numba import config, ir, ir_utils, utils, prange, rewrites, types, typing
+from numba import (config, ir, ir_utils, utils, prange, rewrites, types, typing,
+                   errors)
 from numba.parfor import internal_prange
 from numba.ir_utils import (
     mk_unique_var,
@@ -9,7 +12,6 @@ from numba.ir_utils import (
     add_offset_to_labels,
     replace_vars,
     remove_dels,
-    remove_dead,
     rename_labels,
     find_topo_order,
     merge_adjacent_blocks,
@@ -22,7 +24,8 @@ from numba.ir_utils import (
     get_np_ufunc_typ,
     get_ir_of_code,
     simplify_CFG,
-    canonicalize_array_math
+    canonicalize_array_math,
+    dead_code_elimination,
     )
 
 from numba.analysis import (
@@ -40,16 +43,28 @@ Variable enable_inline_arraycall is only used for testing purpose.
 """
 enable_inline_arraycall = True
 
+
+def callee_ir_validator(func_ir):
+    """Checks the IR of a callee is supported for inlining
+    """
+    for blk in func_ir.blocks.values():
+        for stmt in blk.find_insts(ir.Assign):
+            if isinstance(stmt.value, ir.Yield):
+                msg = "The use of yield in a closure is unsupported."
+                raise errors.UnsupportedError(msg, loc=stmt.loc)
+
+
 class InlineClosureCallPass(object):
     """InlineClosureCallPass class looks for direct calls to locally defined
     closures, and inlines the body of the closure function to the call site.
     """
 
-    def __init__(self, func_ir, parallel_options, swapped={}):
+    def __init__(self, func_ir, parallel_options, swapped={}, typed=False):
         self.func_ir = func_ir
         self.parallel_options = parallel_options
         self.swapped = swapped
         self._processed_stencils = []
+        self.typed = typed
 
     def run(self):
         """Run inline closure call pass.
@@ -97,19 +112,19 @@ class InlineClosureCallPass(object):
             for k, s in sorted(sized_loops, key=lambda tup: tup[1], reverse=True):
                 visited.append(k)
                 if guard(_inline_arraycall, self.func_ir, cfg, visited, loops[k],
-                         self.swapped, self.parallel_options.comprehension):
+                         self.swapped, self.parallel_options.comprehension,
+                         self.typed):
                     modified = True
             if modified:
                 _fix_nested_array(self.func_ir)
 
         if modified:
             remove_dels(self.func_ir.blocks)
-            # repeat dead code elimintation until nothing can be further
-            # removed
-            while (remove_dead(self.func_ir.blocks, self.func_ir.arg_names,
-                                                                self.func_ir)):
-                pass
+            # run dead code elimination
+            dead_code_elimination(self.func_ir)
+            # do label renaming
             self.func_ir.blocks = rename_labels(self.func_ir.blocks)
+
         debug_print("END")
 
     def _inline_reduction(self, work_list, block, i, expr, call_name):
@@ -118,20 +133,24 @@ class InlineClosureCallPass(object):
         require(not self.parallel_options.reduction)
         require(call_name == ('reduce', 'builtins') or
                 call_name == ('reduce', '_functools'))
-        if len(expr.args) != 3:
+        if len(expr.args) not in (2, 3):
             raise TypeError("invalid reduce call, "
-                            "three arguments including initial "
-                            "value required")
+                            "two arguments are required (optional initial "
+                            "value can also be specified)")
         check_reduce_func(self.func_ir, expr.args[0])
-        def reduce_func(f, A, v):
-            s = v
+        def reduce_func(f, A, v=None):
             it = iter(A)
+            if v is not None:
+                s = v
+            else:
+                s = next(it)
             for a in it:
                s = f(s, a)
             return s
         inline_closure_call(self.func_ir,
                         self.func_ir.func_id.func.__globals__,
-                        block, i, reduce_func, work_list=work_list)
+                        block, i, reduce_func, work_list=work_list,
+                        callee_validator=callee_ir_validator)
         return True
 
     def _inline_stencil(self, instr, call_name, func_def):
@@ -211,27 +230,37 @@ class InlineClosureCallPass(object):
                 func_def.op == "make_function")
         inline_closure_call(self.func_ir,
                             self.func_ir.func_id.func.__globals__,
-                            block, i, func_def, work_list=work_list)
+                            block, i, func_def, work_list=work_list,
+                            callee_validator=callee_ir_validator)
         return True
 
 def check_reduce_func(func_ir, func_var):
+    """Checks the function at func_var in func_ir to make sure it's amenable
+    for inlining. Returns the function itself"""
     reduce_func = guard(get_definition, func_ir, func_var)
     if reduce_func is None:
         raise ValueError("Reduce function cannot be found for njit \
                             analysis")
-    if not (hasattr(reduce_func, 'code')
+    if isinstance(reduce_func, (ir.FreeVar, ir.Global)):
+        if not isinstance(reduce_func.value,
+                          numba.targets.registry.CPUDispatcher):
+            raise ValueError("Invalid reduction function")
+        # pull out the python function for inlining
+        reduce_func = reduce_func.value.py_func
+    elif not (hasattr(reduce_func, 'code')
             or hasattr(reduce_func, '__code__')):
         raise ValueError("Invalid reduction function")
     f_code = (reduce_func.code if hasattr(reduce_func, 'code')
                                     else reduce_func.__code__)
     if not f_code.co_argcount == 2:
         raise TypeError("Reduction function should take 2 arguments")
-    return
+    return reduce_func
 
 
 def inline_closure_call(func_ir, glbls, block, i, callee, typingctx=None,
                         arg_typs=None, typemap=None, calltypes=None,
-                        work_list=None):
+                        work_list=None, callee_validator=None,
+                        replace_freevars=True):
     """Inline the body of `callee` at its callsite (`i`-th instruction of `block`)
 
     `func_ir` is the func_ir object of the caller function and `glbls` is its
@@ -241,6 +270,12 @@ def inline_closure_call(func_ir, glbls, block, i, callee, typingctx=None,
     make_function node. `typingctx`, `typemap` and `calltypes` are typing
     data structures of the caller, available if we are in a typed pass.
     `arg_typs` includes the types of the arguments at the callsite.
+    `callee_validator` is an optional callable which can be used to validate the
+    IR of the callee to ensure that it contains IR supported for inlining, it
+    takes one argument, the func_ir of the callee
+
+    Returns IR blocks of the callee and the variable renaming dictionary used
+    for them to facilitate further processing of new blocks.
     """
     scope = block.scope
     instr = block.body[i]
@@ -249,10 +284,19 @@ def inline_closure_call(func_ir, glbls, block, i, callee, typingctx=None,
     debug_print("Found closure call: ", instr, " with callee = ", callee)
     # support both function object and make_function Expr
     callee_code = callee.code if hasattr(callee, 'code') else callee.__code__
-    callee_defaults = callee.defaults if hasattr(callee, 'defaults') else callee.__defaults__
     callee_closure = callee.closure if hasattr(callee, 'closure') else callee.__closure__
     # first, get the IR of the callee
-    callee_ir = get_ir_of_code(glbls, callee_code)
+    if isinstance(callee, pytypes.FunctionType):
+        from numba import compiler
+        callee_ir = compiler.run_frontend(callee, inline_closures=True)
+    else:
+        callee_ir = get_ir_of_code(glbls, callee_code)
+
+    # check that the contents of the callee IR is something that can be inlined
+    # if a validator is supplied
+    if callee_validator is not None:
+        callee_validator(callee_ir)
+
     callee_blocks = callee_ir.blocks
 
     # 1. relabel callee_ir by adding an offset
@@ -284,25 +328,13 @@ def inline_closure_call(func_ir, glbls, block, i, callee, typingctx=None,
     _debug_dump(callee_ir)
 
     # 3. replace formal parameters with actual arguments
-    args = list(call_expr.args)
-    if callee_defaults:
-        debug_print("defaults = ", callee_defaults)
-        if isinstance(callee_defaults, tuple): # Python 3.5
-            args = args + list(callee_defaults)
-        elif isinstance(callee_defaults, ir.Var) or isinstance(callee_defaults, str):
-            defaults = func_ir.get_definition(callee_defaults)
-            assert(isinstance(defaults, ir.Const))
-            loc = defaults.loc
-            args = args + [ir.Const(value=v, loc=loc)
-                           for v in defaults.value]
-        else:
-            raise NotImplementedError(
-                "Unsupported defaults to make_function: {}".format(defaults))
+    args = _get_callee_args(call_expr, callee, block.body[i].loc, func_ir)
+
     debug_print("After arguments rename: ")
     _debug_dump(callee_ir)
 
     # 4. replace freevar with actual closure var
-    if callee_closure:
+    if callee_closure and replace_freevars:
         closure = func_ir.get_definition(callee_closure)
         debug_print("callee's closure = ", closure)
         if isinstance(closure, tuple):
@@ -320,12 +352,15 @@ def inline_closure_call(func_ir, glbls, block, i, callee, typingctx=None,
         _debug_dump(callee_ir)
 
     if typingctx:
-        from numba import compiler
+        from numba import typed_passes
+        # call branch pruning to simplify IR and avoid inference errors
+        callee_ir._definitions = ir_utils.build_definitions(callee_ir.blocks)
+        numba.analysis.dead_branch_prune(callee_ir, arg_typs)
         try:
-            f_typemap, f_return_type, f_calltypes = compiler.type_inference_stage(
+            f_typemap, f_return_type, f_calltypes = typed_passes.type_inference_stage(
                     typingctx, callee_ir, arg_typs, None)
-        except BaseException:
-            f_typemap, f_return_type, f_calltypes = compiler.type_inference_stage(
+        except Exception:
+            f_typemap, f_return_type, f_calltypes = typed_passes.type_inference_stage(
                     typingctx, callee_ir, arg_typs, None)
             pass
         canonicalize_array_math(callee_ir, f_typemap,
@@ -351,9 +386,12 @@ def inline_closure_call(func_ir, glbls, block, i, callee, typingctx=None,
     # 6. replace Return with assignment to LHS
     topo_order = find_topo_order(callee_blocks)
     _replace_returns(callee_blocks, instr.target, new_label)
-    #    remove the old definition of instr.target too
-    if (instr.target.name in func_ir._definitions):
-        func_ir._definitions[instr.target.name] = []
+
+    # remove the old definition of instr.target too
+    if (instr.target.name in func_ir._definitions
+            and call_expr in func_ir._definitions[instr.target.name]):
+        # NOTE: target can have multiple definitions due to control flow
+        func_ir._definitions[instr.target.name].remove(call_expr)
 
     # 7. insert all new blocks, and add back definitions
     for label in topo_order:
@@ -369,7 +407,71 @@ def inline_closure_call(func_ir, glbls, block, i, callee, typingctx=None,
     if work_list != None:
         for block in new_blocks:
             work_list.append(block)
-    return callee_blocks
+    return callee_blocks, var_dict
+
+
+def _get_callee_args(call_expr, callee, loc, func_ir):
+    """Get arguments for calling 'callee', including the default arguments.
+    keyword arguments are currently only handled when 'callee' is a function.
+    """
+    if call_expr.op == 'call':
+        args = list(call_expr.args)
+    elif call_expr.op == 'getattr':
+        args = [call_expr.value]
+    else:
+        raise TypeError("Unsupported ir.Expr.{}".format(call_expr.op))
+
+    debug_print = _make_debug_print("inline_closure_call default handling")
+
+    # handle defaults and kw arguments using pysignature if callee is function
+    if isinstance(callee, pytypes.FunctionType):
+        pysig = numba.utils.pysignature(callee)
+        normal_handler = lambda index, param, default: default
+        default_handler = lambda index, param, default: ir.Const(default, loc)
+        # Throw error for stararg
+        # TODO: handle stararg
+        def stararg_handler(index, param, default):
+            raise NotImplementedError(
+                "Stararg not supported in inliner for arg {} {}".format(
+                    index, param))
+        if call_expr.op == 'call':
+            kws = dict(call_expr.kws)
+        else:
+            kws = {}
+        return numba.typing.fold_arguments(
+            pysig, args, kws, normal_handler, default_handler,
+            stararg_handler)
+    else:
+        # TODO: handle arguments for make_function case similar to function
+        # case above
+        callee_defaults = (callee.defaults if hasattr(callee, 'defaults')
+                           else callee.__defaults__)
+        if callee_defaults:
+            debug_print("defaults = ", callee_defaults)
+            if isinstance(callee_defaults, tuple):  # Python 3.5
+                defaults_list = []
+                for x in callee_defaults:
+                    if isinstance(x, ir.Var):
+                        defaults_list.append(x)
+                    else:
+                        # this branch is predominantly for kwargs from
+                        # inlinable functions
+                        defaults_list.append(ir.Const(value=x, loc=loc))
+                args = args + defaults_list
+            elif (isinstance(callee_defaults, ir.Var)
+                    or isinstance(callee_defaults, str)):
+                default_tuple = func_ir.get_definition(callee_defaults)
+                assert(isinstance(default_tuple, ir.Expr))
+                assert(default_tuple.op == "build_tuple")
+                const_vals = [func_ir.get_definition(x) for
+                              x in default_tuple.items]
+                args = args + const_vals
+            else:
+                raise NotImplementedError(
+                    "Unsupported defaults to make_function: {}".format(
+                        defaults))
+        return args
+
 
 def _make_debug_print(prefix):
     def debug_print(*args):
@@ -377,6 +479,7 @@ def _make_debug_print(prefix):
             print(prefix + ": " + "".join(str(x) for x in args))
     return debug_print
 
+
 def _debug_dump(func_ir):
     if config.DEBUG_INLINE_CLOSURE:
         func_ir.dump()
@@ -514,7 +617,8 @@ def _find_iter_range(func_ir, range_iter_var, swapped):
     func_var = range_def.func
     func_def = get_definition(func_ir, func_var)
     debug_print("func_var = ", func_var, " func_def = ", func_def)
-    require(isinstance(func_def, ir.Global) and func_def.value == range)
+    require(isinstance(func_def, ir.Global) and
+            (func_def.value == range or func_def.value == numba.special.prange))
     nargs = len(range_def.args)
     swapping = [('"array comprehension"', 'closure of'), range_def.func.loc]
     if nargs == 1:
@@ -529,7 +633,8 @@ def _find_iter_range(func_ir, range_iter_var, swapped):
     else:
         raise GuardException
 
-def _inline_arraycall(func_ir, cfg, visited, loop, swapped, enable_prange=False):
+def _inline_arraycall(func_ir, cfg, visited, loop, swapped, enable_prange=False,
+                      typed=False):
     """Look for array(list) call in the exit block of a given loop, and turn list operations into
     array operations in the loop if the following conditions are met:
       1. The exit block contains an array call on the list;
@@ -584,8 +689,8 @@ def _inline_arraycall(func_ir, cfg, visited, loop, swapped, enable_prange=False)
                 if isinstance(func_def, ir.Expr) and func_def.op == 'getattr' \
                   and func_def.attr == 'append':
                     list_def = get_definition(func_ir, func_def.value)
-                    debug_print("list_def = ", list_def, list_def == list_var_def)
-                    if list_def == list_var_def:
+                    debug_print("list_def = ", list_def, list_def is list_var_def)
+                    if list_def is list_var_def:
                         # found matching append call
                         list_append_stmts.append((label, block, stmt))
 
@@ -640,7 +745,7 @@ def _inline_arraycall(func_ir, cfg, visited, loop, swapped, enable_prange=False)
     # Skip list construction and skip terminator, add the rest to stmts
     for i in range(len(loop_entry.body) - 1):
         stmt = loop_entry.body[i]
-        if isinstance(stmt, ir.Assign) and (stmt.value == list_def or is_removed(stmt.value, removed)):
+        if isinstance(stmt, ir.Assign) and (stmt.value is list_def or is_removed(stmt.value, removed)):
             removed.append(stmt.target)
         else:
             stmts.append(stmt)
@@ -674,10 +779,16 @@ def _inline_arraycall(func_ir, cfg, visited, loop, swapped, enable_prange=False)
             range_func_def.value = internal_prange
 
     else:
-        len_func_var = ir.Var(scope, mk_unique_var("len_func"), loc)
-        stmts.append(_new_definition(func_ir, len_func_var,
-                     ir.Global('range_iter_len', range_iter_len, loc=loc), loc))
-        size_val = ir.Expr.call(len_func_var, (iter_var,), (), loc=loc)
+        # this doesn't work in objmode as it's effectively untyped
+        if typed:
+            len_func_var = ir.Var(scope, mk_unique_var("len_func"), loc)
+            stmts.append(_new_definition(func_ir, len_func_var,
+                        ir.Global('range_iter_len', range_iter_len, loc=loc),
+                        loc))
+            size_val = ir.Expr.call(len_func_var, (iter_var,), (), loc=loc)
+        else:
+            raise GuardException
+
 
     stmts.append(_new_definition(func_ir, size_var, size_val, loc))
 
@@ -689,7 +800,7 @@ def _inline_arraycall(func_ir, cfg, visited, loop, swapped, enable_prange=False)
     array_var = ir.Var(scope, mk_unique_var("array"), loc)
     empty_func = ir.Var(scope, mk_unique_var("empty_func"), loc)
     if dtype_def and dtype_mod_def:
-        # when dtype is present, we'll call emtpy with dtype
+        # when dtype is present, we'll call empty with dtype
         dtype_mod_var = ir.Var(scope, mk_unique_var("dtype_mod"), loc)
         dtype_var = ir.Var(scope, mk_unique_var("dtype"), loc)
         stmts.append(_new_definition(func_ir, dtype_mod_var, dtype_mod_def, loc))
@@ -699,11 +810,16 @@ def _inline_arraycall(func_ir, cfg, visited, loop, swapped, enable_prange=False)
                          ir.Global('empty', np.empty, loc=loc), loc))
         array_kws = [('dtype', dtype_var)]
     else:
-        # otherwise we'll call unsafe_empty_inferred
-        stmts.append(_new_definition(func_ir, empty_func,
-                         ir.Global('unsafe_empty_inferred',
-                             unsafe_empty_inferred, loc=loc), loc))
-        array_kws = []
+        # this doesn't work in objmode as it's effectively untyped
+        if typed:
+            # otherwise we'll call unsafe_empty_inferred
+            stmts.append(_new_definition(func_ir, empty_func,
+                            ir.Global('unsafe_empty_inferred',
+                                unsafe_empty_inferred, loc=loc), loc))
+            array_kws = []
+        else:
+            raise GuardException
+
     # array_var = empty_func(size_tuple_var)
     stmts.append(_new_definition(func_ir, array_var,
                  ir.Expr.call(empty_func, (size_tuple_var,), list(array_kws), loc=loc), loc))
@@ -751,7 +867,7 @@ def _inline_arraycall(func_ir, cfg, visited, loop, swapped, enable_prange=False)
 
     # In append_block, change list_append into array assign
     for i in range(len(append_block.body)):
-        if append_block.body[i] == append_stmt:
+        if append_block.body[i] is append_stmt:
             debug_print("Replace append with SetItem")
             append_block.body[i] = ir.SetItem(target=array_var, index=index_var,
                                               value=append_stmt.value.args[0], loc=append_stmt.loc)
@@ -791,7 +907,7 @@ def _fix_nested_array(func_ir):
             arr = numba.unsafe.ndarray.empty_inferred(...).
         If it is arr = b[...], find array definition of b recursively.
         """
-        arr_def = func_ir.get_definition(arr)
+        arr_def = get_definition(func_ir, arr)
         _make_debug_print("find_array_def")(arr, arr_def)
         if isinstance(arr_def, ir.Expr):
             if guard(_find_unsafe_empty_inferred, func_ir, arr_def):
@@ -814,7 +930,7 @@ def _fix_nested_array(func_ir):
                 inst = body[i]
                 if isinstance(inst, ir.Assign):
                     defined.add(inst.target.name)
-                    if inst.value == expr:
+                    if inst.value is expr:
                         new_varlist = []
                         for var in varlist:
                             # var must be defined before this inst, or live
@@ -906,9 +1022,9 @@ class RewriteArrayOfConsts(rewrites.Rewrite):
     1D array creations from a constant list, and rewriting it into
     direct initialization of array elements without creating the list.
     '''
-    def __init__(self, pipeline, *args, **kws):
-        self.typingctx = pipeline.typingctx
-        super(RewriteArrayOfConsts, self).__init__(pipeline, *args, **kws)
+    def __init__(self, state, *args, **kws):
+        self.typingctx = state.typingctx
+        super(RewriteArrayOfConsts, self).__init__(*args, **kws)
 
     def match(self, func_ir, block, typemap, calltypes):
         if len(calltypes) == 0:
@@ -927,7 +1043,7 @@ def _inline_const_arraycall(block, func_ir, context, typemap, calltypes):
     """Look for array(list) call where list is a constant list created by build_list,
     and turn them into direct array creation and initialization, if the following
     conditions are met:
-      1. The build_list call immediate preceeds the array call;
+      1. The build_list call immediate precedes the array call;
       2. The list variable is no longer live after array call;
     If any condition check fails, no modification will be made.
     """
@@ -1026,76 +1142,102 @@ def _inline_const_arraycall(block, func_ir, context, typemap, calltypes):
         stmts.extend(dels)
         return True
 
-    # list_vars keep track of the variable created from the latest
-    # build_list instruction, as well as its synonyms.
-    list_vars = []
-    # dead_vars keep track of those in list_vars that are considered dead.
-    dead_vars = []
-    # list_items keep track of the elements used in build_list.
-    list_items = []
-    stmts = []
-    # dels keep track of the deletion of list_items, which will need to be
-    # moved after array initialization.
-    dels = []
-    modified = False
+    class State(object):
+        """
+        This class is used to hold the state in the following loop so as to make
+        it easy to reset the state of the variables tracking the various
+        statement kinds
+        """
+
+        def __init__(self):
+            # list_vars keep track of the variable created from the latest
+            # build_list instruction, as well as its synonyms.
+            self.list_vars = []
+            # dead_vars keep track of those in list_vars that are considered dead.
+            self.dead_vars = []
+            # list_items keep track of the elements used in build_list.
+            self.list_items = []
+            self.stmts = []
+            # dels keep track of the deletion of list_items, which will need to be
+            # moved after array initialization.
+            self.dels = []
+            # tracks if a modification has taken place
+            self.modified = False
+
+        def reset(self):
+            """
+            Resets the internal state of the variables used for tracking
+            """
+            self.list_vars = []
+            self.dead_vars = []
+            self.list_items = []
+            self.dels = []
+
+        def list_var_used(self, inst):
+            """
+            Returns True if the list being analysed is used between the
+            build_list and the array call.
+            """
+            return any([x.name in self.list_vars for x in inst.list_vars()])
+
+    state = State()
+
     for inst in block.body:
         if isinstance(inst, ir.Assign):
             if isinstance(inst.value, ir.Var):
-                if inst.value.name in list_vars:
-                    list_vars.append(inst.target.name)
-                    stmts.append(inst)
+                if inst.value.name in state.list_vars:
+                    state.list_vars.append(inst.target.name)
+                    state.stmts.append(inst)
                     continue
             elif isinstance(inst.value, ir.Expr):
                 expr = inst.value
                 if expr.op == 'build_list':
-                    list_vars = [inst.target.name]
-                    list_items = [x.name for x in expr.items]
-                    stmts.append(inst)
+                    # new build_list encountered, reset state
+                    state.reset()
+                    state.list_items = [x.name for x in expr.items]
+                    state.list_vars = [inst.target.name]
+                    state.stmts.append(inst)
                     continue
                 elif expr.op == 'call' and expr in calltypes:
                     arr_var = inst.target
                     if guard(inline_array, inst.target, expr,
-                                           stmts, list_vars, dels):
-                        modified = True
+                             state.stmts, state.list_vars, state.dels):
+                        state.modified = True
                         continue
         elif isinstance(inst, ir.Del):
             removed_var = inst.value
-            if removed_var in list_items:
-                dels.append(inst)
+            if removed_var in state.list_items:
+                state.dels.append(inst)
                 continue
-            elif removed_var in list_vars:
+            elif removed_var in state.list_vars:
                 # one of the list_vars is considered dead.
-                dead_vars.append(removed_var)
-                list_vars.remove(removed_var)
-                stmts.append(inst)
-                if list_vars == []:
+                state.dead_vars.append(removed_var)
+                state.list_vars.remove(removed_var)
+                state.stmts.append(inst)
+                if state.list_vars == []:
                     # if all list_vars are considered dead, we need to filter
                     # them out from existing stmts to completely remove
                     # build_list.
                     # Note that if a translation didn't take place, dead_vars
                     # will also be empty when we reach this point.
                     body = []
-                    for inst in stmts:
+                    for inst in state.stmts:
                         if ((isinstance(inst, ir.Assign) and
-                             inst.target.name in dead_vars) or
+                             inst.target.name in state.dead_vars) or
                              (isinstance(inst, ir.Del) and
-                             inst.value in dead_vars)):
+                             inst.value in state.dead_vars)):
                             continue
                         body.append(inst)
-                    stmts = body
-                    dead_vars = []
-                    modified = True
+                    state.stmts = body
+                    state.dead_vars = []
+                    state.modified = True
                     continue
-        stmts.append(inst)
+        state.stmts.append(inst)
 
         # If the list is used in any capacity between build_list and array
         # call, then we must call off the translation for this list because
         # it could be mutated and list_items would no longer be applicable.
-        list_var_used = any([ x.name in list_vars for x in inst.list_vars() ])
-        if list_var_used:
-            list_vars = []
-            dead_vars = []
-            list_items = []
-            dels = []
+        if state.list_var_used(inst):
+            state.reset()
 
-    return stmts if modified else None
+    return state.stmts if state.modified else None
