@@ -8,13 +8,15 @@ import linecache
 import pprint
 import re
 import sys
-import warnings
-from numba import config, errors
 import operator
+from types import FunctionType, BuiltinFunctionType
+from functools import total_ordering
 
+from numba import config, errors
 from .utils import BINOPS_TO_OPERATORS, INPLACE_BINOPS_TO_OPERATORS, UNARY_BUITINS_TO_OPERATORS, OPERATORS_TO_BUILTINS
 from .errors import (NotDefinedError, RedefinedError, VerificationError,
                      ConstantInferenceError)
+from .six import StringIO
 
 # terminal color markup
 _termcolor = errors.termcolor()
@@ -26,15 +28,33 @@ class Loc(object):
     """
     _defmatcher = re.compile('def\s+(\w+)\(.*')
 
-    def __init__(self, filename, line, col=None):
+    def __init__(self, filename, line, col=None, maybe_decorator=False):
+        """ Arguments:
+        filename - name of the file
+        line - line in file
+        col - column
+        maybe_decorator - Set to True if location is likely a jit decorator
+        """
         self.filename = filename
         self.line = line
         self.col = col
         self.lines = None # the source lines from the linecache
+        self.maybe_decorator = maybe_decorator
+
+    def __eq__(self, other):
+        # equivalence is solely based on filename, line and col
+        if type(self) is not type(other): return False
+        if self.filename != other.filename: return False
+        if self.line != other.line: return False
+        if self.col != other.col: return False
+        return True
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
 
     @classmethod
     def from_function_id(cls, func_id):
-        return cls(func_id.filename, func_id.firstlineno)
+        return cls(func_id.filename, func_id.firstlineno, maybe_decorator=True)
 
     def __repr__(self):
         return "Loc(filename=%s, line=%s, col=%s)" % (self.filename,
@@ -49,9 +69,11 @@ class Loc(object):
     def _find_definition(self):
         # try and find a def, go backwards from error line
         fn_name = None
-        lines = self._get_lines()
+        lines = self.get_lines()
         for x in reversed(lines[:self.line - 1]):
-            if 'def ' in x:
+            # the strip and startswith is to handle user code with commented out
+            # 'def' or use of 'def' in a docstring.
+            if x.strip().startswith('def '):
                 fn_name = x
                 break
 
@@ -65,7 +87,7 @@ class Loc(object):
             # Probably exec(<string>) or REPL.
             return None
 
-    def _get_lines(self):
+    def get_lines(self):
         if self.lines is None:
 
             self.lines = linecache.getlines(self._get_path())
@@ -88,10 +110,35 @@ class Loc(object):
 
     def strformat(self, nlines_up=2):
 
-        lines = self._get_lines()
+        lines = self.get_lines()
+
+        use_line = self.line
+
+        if self.maybe_decorator:
+            # try and sort out a better `loc`, if it's suspected that this loc
+            # points at a jit decorator by virtue of
+            # `__code__.co_firstlineno`
+
+            # get lines, add a dummy entry at the start as lines count from
+            # 1 but list index counts from 0
+            tmplines = [''] + lines
+
+            if lines and use_line and 'def ' not in tmplines[use_line]:
+                # look forward 10 lines, unlikely anyone managed to stretch
+                # a jit call declaration over >10 lines?!
+                min_line = max(0, use_line)
+                max_line = use_line + 10
+                selected = tmplines[min_line : max_line]
+                index = 0
+                for idx, x in enumerate(selected):
+                    if 'def ' in x:
+                        index = idx
+                        break
+                use_line = use_line + index
+
 
         ret = [] # accumulates output
-        if lines and self.line:
+        if lines and use_line:
 
             def count_spaces(string):
                 spaces = 0
@@ -103,7 +150,7 @@ class Loc(object):
             # this is often in places where exceptions are used for the purposes
             # of flow control. As a result max is in use to prevent slice from
             # `[negative: positive]`
-            selected = lines[max(0, self.line - nlines_up):self.line]
+            selected = lines[max(0, use_line - nlines_up):use_line]
 
             # see if selected contains a definition
             def_found = False
@@ -115,7 +162,7 @@ class Loc(object):
             if not def_found:
                 # try and find a def, go backwards from error line
                 fn_name = None
-                for x in reversed(lines[:self.line - 1]):
+                for x in reversed(lines[:use_line - 1]):
                     if 'def ' in x:
                         fn_name = x
                         break
@@ -137,7 +184,7 @@ class Loc(object):
             ret = "<source missing, REPL/exec in use?>"
 
         err = _termcolor.filename('\nFile "%s", line %d:')+'\n%s'
-        tmp = err % (self._get_path(), self.line, _termcolor.code(''.join(ret)))
+        tmp = err % (self._get_path(), use_line, _termcolor.code(''.join(ret)))
         return tmp
 
     def with_lineno(self, line, col=None):
@@ -146,11 +193,65 @@ class Loc(object):
         """
         return type(self)(self.filename, line, col)
 
+    def short(self):
+        """
+        Returns a short string
+        """
+        shortfilename = os.path.basename(self.filename)
+        return "%s:%s" % (shortfilename, self.line)
+
 
 # Used for annotating errors when source location is unknown.
 unknown_loc = Loc("unknown location", 0, 0)
 
 
+@total_ordering
+class SlotEqualityCheckMixin(object):
+    # some ir nodes are __dict__ free using __slots__ instead, this mixin
+    # should not trigger the unintended creation of __dict__.
+    __slots__ = tuple()
+
+    def __eq__(self, other):
+        if type(self) is type(other):
+            for name in self.__slots__:
+                if getattr(self, name) != getattr(other, name):
+                    return False
+            else:
+                return True
+        return False
+
+    def __le__(self, other):
+        return str(self) <= str(other)
+
+    def __hash__(self):
+        return id(self)
+
+
+@total_ordering
+class EqualityCheckMixin(object):
+    """ Mixin for basic equality checking """
+
+    def __eq__(self, other):
+        if type(self) is type(other):
+            def fixup(adict):
+                bad = ('loc', 'scope')
+                d = dict(adict)
+                for x in bad:
+                    d.pop(x, None)
+                return d
+            d1 = fixup(self.__dict__)
+            d2 = fixup(other.__dict__)
+            if d1 == d2:
+                return True
+        return False
+
+    def __le__(self, other):
+        return str(self) < str(other)
+
+    def __hash__(self):
+        return id(self)
+
+
 class VarMap(object):
     def __init__(self):
         self._con = {}
@@ -182,8 +283,23 @@ class VarMap(object):
     def __iter__(self):
         return self._con.iterkeys()
 
+    def __eq__(self, other):
+        if type(self) is type(other):
+            # check keys only, else __eq__ ref cycles, scope -> varmap -> var
+            return self._con.keys() == other._con.keys()
+        return False
 
-class Inst(object):
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+
+class AbstractRHS(object):
+    """Abstract base class for anything that can be the RHS of an assignment.
+    This class **does not** define any methods.
+    """
+
+
+class Inst(EqualityCheckMixin, AbstractRHS):
     """
     Base class for all IR instructions.
     """
@@ -254,6 +370,8 @@ class Expr(Inst):
     """
 
     def __init__(self, op, loc, **kws):
+        assert isinstance(op, str)
+        assert isinstance(loc, Loc)
         self.op = op
         self.loc = loc
         self._kws = kws
@@ -271,15 +389,21 @@ class Expr(Inst):
 
     @classmethod
     def binop(cls, fn, lhs, rhs, loc):
-        assert not isinstance(fn, str)
+        assert isinstance(fn, BuiltinFunctionType)
+        assert isinstance(lhs, Var)
+        assert isinstance(rhs, Var)
+        assert isinstance(loc, Loc)
         op = 'binop'
         return cls(op=op, loc=loc, fn=fn, lhs=lhs, rhs=rhs,
                    static_lhs=UNDEFINED, static_rhs=UNDEFINED)
 
     @classmethod
     def inplace_binop(cls, fn, immutable_fn, lhs, rhs, loc):
-        assert not isinstance(fn, str)
-        assert not isinstance(immutable_fn, str)
+        assert isinstance(fn, BuiltinFunctionType)
+        assert isinstance(immutable_fn, BuiltinFunctionType)
+        assert isinstance(lhs, Var)
+        assert isinstance(rhs, Var)
+        assert isinstance(loc, Loc)
         op = 'inplace_binop'
         return cls(op=op, loc=loc, fn=fn, immutable_fn=immutable_fn,
                    lhs=lhs, rhs=rhs,
@@ -287,73 +411,108 @@ class Expr(Inst):
 
     @classmethod
     def unary(cls, fn, value, loc):
+        assert isinstance(value, (str, Var, FunctionType))
+        assert isinstance(loc, Loc)
         op = 'unary'
         fn = UNARY_BUITINS_TO_OPERATORS.get(fn, fn)
         return cls(op=op, loc=loc, fn=fn, value=value)
 
     @classmethod
     def call(cls, func, args, kws, loc, vararg=None):
+        assert isinstance(func, (Var, Intrinsic))
+        assert isinstance(loc, Loc)
         op = 'call'
         return cls(op=op, loc=loc, func=func, args=args, kws=kws,
                    vararg=vararg)
 
     @classmethod
     def build_tuple(cls, items, loc):
+        assert isinstance(loc, Loc)
         op = 'build_tuple'
         return cls(op=op, loc=loc, items=items)
 
     @classmethod
     def build_list(cls, items, loc):
+        assert isinstance(loc, Loc)
         op = 'build_list'
         return cls(op=op, loc=loc, items=items)
 
     @classmethod
     def build_set(cls, items, loc):
+        assert isinstance(loc, Loc)
         op = 'build_set'
         return cls(op=op, loc=loc, items=items)
 
     @classmethod
     def build_map(cls, items, size, loc):
+        assert isinstance(loc, Loc)
         op = 'build_map'
         return cls(op=op, loc=loc, items=items, size=size)
 
     @classmethod
     def pair_first(cls, value, loc):
+        assert isinstance(value, Var)
         op = 'pair_first'
         return cls(op=op, loc=loc, value=value)
 
     @classmethod
     def pair_second(cls, value, loc):
+        assert isinstance(value, Var)
+        assert isinstance(loc, Loc)
         op = 'pair_second'
         return cls(op=op, loc=loc, value=value)
 
     @classmethod
     def getiter(cls, value, loc):
+        assert isinstance(value, Var)
+        assert isinstance(loc, Loc)
         op = 'getiter'
         return cls(op=op, loc=loc, value=value)
 
     @classmethod
     def iternext(cls, value, loc):
+        assert isinstance(value, Var)
+        assert isinstance(loc, Loc)
         op = 'iternext'
         return cls(op=op, loc=loc, value=value)
 
     @classmethod
     def exhaust_iter(cls, value, count, loc):
+        assert isinstance(value, Var)
+        assert isinstance(count, int)
+        assert isinstance(loc, Loc)
         op = 'exhaust_iter'
         return cls(op=op, loc=loc, value=value, count=count)
 
     @classmethod
     def getattr(cls, value, attr, loc):
+        assert isinstance(value, Var)
+        assert isinstance(attr, str)
+        assert isinstance(loc, Loc)
         op = 'getattr'
         return cls(op=op, loc=loc, value=value, attr=attr)
 
     @classmethod
     def getitem(cls, value, index, loc):
+        assert isinstance(value, Var)
+        assert isinstance(index, Var)
+        assert isinstance(loc, Loc)
         op = 'getitem'
         return cls(op=op, loc=loc, value=value, index=index)
 
+    @classmethod
+    def typed_getitem(cls, value, dtype, index, loc):
+        assert isinstance(value, Var)
+        assert isinstance(loc, Loc)
+        op = 'typed_getitem'
+        return cls(op=op, loc=loc, value=value, dtype=dtype,
+                   index=index)
+
     @classmethod
     def static_getitem(cls, value, index, index_var, loc):
+        assert isinstance(value, Var)
+        assert index_var is None or isinstance(index_var, Var)
+        assert isinstance(loc, Loc)
         op = 'static_getitem'
         return cls(op=op, loc=loc, value=value, index=index,
                    index_var=index_var)
@@ -363,6 +522,8 @@ class Expr(Inst):
         """
         A node for implicit casting at the return statement
         """
+        assert isinstance(value, Var)
+        assert isinstance(loc, Loc)
         op = 'cast'
         return cls(op=op, value=value, loc=loc)
 
@@ -371,6 +532,7 @@ class Expr(Inst):
         """
         A node for making a function object.
         """
+        assert isinstance(loc, Loc)
         op = 'make_function'
         return cls(op=op, name=name, code=code, closure=closure, defaults=defaults, loc=loc)
 
@@ -406,6 +568,10 @@ class SetItem(Stmt):
     """
 
     def __init__(self, target, index, value, loc):
+        assert isinstance(target, Var)
+        assert isinstance(index, Var)
+        assert isinstance(value, Var)
+        assert isinstance(loc, Loc)
         self.target = target
         self.index = index
         self.value = value
@@ -421,6 +587,11 @@ class StaticSetItem(Stmt):
     """
 
     def __init__(self, target, index, index_var, value, loc):
+        assert isinstance(target, Var)
+        assert not isinstance(index, Var)
+        assert isinstance(index_var, Var)
+        assert isinstance(value, Var)
+        assert isinstance(loc, Loc)
         self.target = target
         self.index = index
         self.index_var = index_var
@@ -437,6 +608,9 @@ class DelItem(Stmt):
     """
 
     def __init__(self, target, index, loc):
+        assert isinstance(target, Var)
+        assert isinstance(index, Var)
+        assert isinstance(loc, Loc)
         self.target = target
         self.index = index
         self.loc = loc
@@ -447,6 +621,10 @@ class DelItem(Stmt):
 
 class SetAttr(Stmt):
     def __init__(self, target, attr, value, loc):
+        assert isinstance(target, Var)
+        assert isinstance(attr, str)
+        assert isinstance(value, Var)
+        assert isinstance(loc, Loc)
         self.target = target
         self.attr = attr
         self.value = value
@@ -458,6 +636,9 @@ class SetAttr(Stmt):
 
 class DelAttr(Stmt):
     def __init__(self, target, attr, loc):
+        assert isinstance(target, Var)
+        assert isinstance(attr, str)
+        assert isinstance(loc, Loc)
         self.target = target
         self.attr = attr
         self.loc = loc
@@ -468,6 +649,10 @@ class DelAttr(Stmt):
 
 class StoreMap(Stmt):
     def __init__(self, dct, key, value, loc):
+        assert isinstance(dct, Var)
+        assert isinstance(key, Var)
+        assert isinstance(value, Var)
+        assert isinstance(loc, Loc)
         self.dct = dct
         self.key = key
         self.value = value
@@ -479,6 +664,8 @@ class StoreMap(Stmt):
 
 class Del(Stmt):
     def __init__(self, value, loc):
+        assert isinstance(value, str)
+        assert isinstance(loc, Loc)
         self.value = value
         self.loc = loc
 
@@ -490,6 +677,8 @@ class Raise(Terminator):
     is_exit = True
 
     def __init__(self, exception, loc):
+        assert exception is None or isinstance(exception, Var)
+        assert isinstance(loc, Loc)
         self.exception = exception
         self.loc = loc
 
@@ -509,6 +698,9 @@ class StaticRaise(Terminator):
     is_exit = True
 
     def __init__(self, exc_class, exc_args, loc):
+        assert exc_class is None or isinstance(exc_class, type)
+        assert isinstance(loc, Loc)
+        assert exc_args is None or isinstance(exc_args, tuple)
         self.exc_class = exc_class
         self.exc_args = exc_args
         self.loc = loc
@@ -526,6 +718,43 @@ class StaticRaise(Terminator):
         return []
 
 
+class TryRaise(Stmt):
+    """A raise statement inside a try-block
+    Similar to ``Raise`` but does not terminate.
+    """
+    def __init__(self, exception, loc):
+        assert exception is None or isinstance(exception, Var)
+        assert isinstance(loc, Loc)
+        self.exception = exception
+        self.loc = loc
+
+    def __str__(self):
+        return "try_raise %s" % self.exception
+
+
+class StaticTryRaise(Stmt):
+    """A raise statement inside a try-block.
+    Similar to ``StaticRaise`` but does not terminate.
+    """
+
+    def __init__(self, exc_class, exc_args, loc):
+        assert exc_class is None or isinstance(exc_class, type)
+        assert isinstance(loc, Loc)
+        assert exc_args is None or isinstance(exc_args, tuple)
+        self.exc_class = exc_class
+        self.exc_args = exc_args
+        self.loc = loc
+
+    def __str__(self):
+        if self.exc_class is None:
+            return "static_try_raise"
+        elif self.exc_args is None:
+            return "static_try_raise %s" % (self.exc_class,)
+        else:
+            return "static_try_raise %s(%s)" % (self.exc_class,
+                                     ", ".join(map(repr, self.exc_args)))
+
+
 class Return(Terminator):
     """
     Return to caller.
@@ -533,6 +762,8 @@ class Return(Terminator):
     is_exit = True
 
     def __init__(self, value, loc):
+        assert isinstance(value, Var), type(value)
+        assert isinstance(loc, Loc)
         self.value = value
         self.loc = loc
 
@@ -549,6 +780,7 @@ class Jump(Terminator):
     """
 
     def __init__(self, target, loc):
+        assert isinstance(loc, Loc)
         self.target = target
         self.loc = loc
 
@@ -565,6 +797,8 @@ class Branch(Terminator):
     """
 
     def __init__(self, cond, truebr, falsebr, loc):
+        assert isinstance(cond, Var)
+        assert isinstance(loc, Loc)
         self.cond = cond
         self.truebr = truebr
         self.falsebr = falsebr
@@ -582,6 +816,9 @@ class Assign(Stmt):
     Assign to a variable.
     """
     def __init__(self, value, target, loc):
+        assert isinstance(value, AbstractRHS)
+        assert isinstance(target, Var)
+        assert isinstance(loc, Loc)
         self.value = value
         self.target = target
         self.loc = loc
@@ -595,7 +832,10 @@ class Print(Stmt):
     Print some values.
     """
     def __init__(self, args, vararg, loc):
-        self.args = args
+        assert all(isinstance(x, Var) for x in args)
+        assert vararg is None or isinstance(vararg, Var)
+        assert isinstance(loc, Loc)
+        self.args = tuple(args)
         self.vararg = vararg
         # Constant-inferred arguments
         self.consts = {}
@@ -607,6 +847,8 @@ class Print(Stmt):
 
 class Yield(Inst):
     def __init__(self, value, loc, index):
+        assert isinstance(value, Var)
+        assert isinstance(loc, Loc)
         self.value = value
         self.loc = loc
         self.index = index
@@ -628,9 +870,11 @@ class EnterWith(Stmt):
         contextmanager : IR value
         begin, end : int
             The beginning and the ending offset of the with-body.
-        loc : int
+        loc : ir.Loc instance
             Source location
         """
+        assert isinstance(contextmanager, Var)
+        assert isinstance(loc, Loc)
         self.contextmanager = contextmanager
         self.begin = begin
         self.end = end
@@ -643,8 +887,11 @@ class EnterWith(Stmt):
         return [self.contextmanager]
 
 
-class Arg(object):
+class Arg(EqualityCheckMixin, AbstractRHS):
     def __init__(self, name, index, loc):
+        assert isinstance(name, str)
+        assert isinstance(index, int)
+        assert isinstance(loc, Loc)
         self.name = name
         self.index = index
         self.loc = loc
@@ -656,8 +903,9 @@ class Arg(object):
         raise ConstantInferenceError('%s' % self, loc=self.loc)
 
 
-class Const(object):
+class Const(EqualityCheckMixin, AbstractRHS):
     def __init__(self, value, loc, use_literal_type=True):
+        assert isinstance(loc, Loc)
         self.value = value
         self.loc = loc
         # Note: need better way to tell if this is a literal or not.
@@ -669,8 +917,10 @@ class Const(object):
     def infer_constant(self):
         return self.value
 
-class Global(object):
+
+class Global(EqualityCheckMixin, AbstractRHS):
     def __init__(self, name, value, loc):
+        assert isinstance(loc, Loc)
         self.name = name
         self.value = value
         self.loc = loc
@@ -687,13 +937,16 @@ class Global(object):
         return Global(self.name, self.value, copy.deepcopy(self.loc))
 
 
-class FreeVar(object):
+class FreeVar(EqualityCheckMixin, AbstractRHS):
     """
     A freevar, as loaded by LOAD_DECREF.
     (i.e. a variable defined in an enclosing non-global scope)
     """
 
     def __init__(self, index, name, value, loc):
+        assert isinstance(index, int)
+        assert isinstance(name, str)
+        assert isinstance(loc, Loc)
         # index inside __code__.co_freevars
         self.index = index
         # variable name
@@ -709,7 +962,7 @@ class FreeVar(object):
         return self.value
 
 
-class Var(object):
+class Var(EqualityCheckMixin, AbstractRHS):
     """
     Attributes
     -----------
@@ -722,12 +975,16 @@ class Var(object):
     """
 
     def __init__(self, scope, name, loc):
+        # NOTE: Use of scope=None should be removed.
+        assert scope is None or isinstance(scope, Scope)
+        assert isinstance(name, str)
+        assert isinstance(loc, Loc)
         self.scope = scope
         self.name = name
         self.loc = loc
 
     def __repr__(self):
-        return 'Var(%s, %s)' % (self.name, self.loc)
+        return 'Var(%s, %s)' % (self.name, self.loc.short())
 
     def __str__(self):
         return self.name
@@ -737,7 +994,7 @@ class Var(object):
         return self.name.startswith("$")
 
 
-class Intrinsic(object):
+class Intrinsic(EqualityCheckMixin):
     """
     A low-level "intrinsic" function.  Suitable as the callable of a "call"
     expression.
@@ -747,10 +1004,10 @@ class Intrinsic(object):
     The *type* is the equivalent Numba signature of calling the intrinsic.
     """
 
-    def __init__(self, name, type, args):
+    def __init__(self, name, type, args, loc=None):
         self.name = name
         self.type = type
-        self.loc = None
+        self.loc = loc
         self.args = args
 
     def __repr__(self):
@@ -760,7 +1017,7 @@ class Intrinsic(object):
         return self.name
 
 
-class Scope(object):
+class Scope(EqualityCheckMixin):
     """
     Attributes
     -----------
@@ -776,6 +1033,8 @@ class Scope(object):
     """
 
     def __init__(self, parent, loc):
+        assert parent is None or isinstance(parent, Scope)
+        assert isinstance(loc, Loc)
         self.parent = parent
         self.localvars = VarMap()
         self.loc = loc
@@ -814,7 +1073,6 @@ class Scope(object):
         if name in self.redefined:
             name = "%s.%d" % (name, self.redefined[name])
 
-        v = Var(scope=self, name=name, loc=loc)
         if name not in self.localvars:
             return self.define(name, loc)
         else:
@@ -852,12 +1110,14 @@ class Scope(object):
                                                           self.loc)
 
 
-class Block(object):
+class Block(EqualityCheckMixin):
     """A code block
 
     """
 
     def __init__(self, scope, loc):
+        assert isinstance(scope, Scope)
+        assert isinstance(loc, Loc)
         self.scope = scope
         self.body = []
         self.loc = loc
@@ -954,7 +1214,7 @@ class Block(object):
         return "<ir.Block at %s>" % (self.loc,)
 
 
-class Loop(object):
+class Loop(SlotEqualityCheckMixin):
     """Describes a loop-block
     """
     __slots__ = "entry", "exit"
@@ -968,7 +1228,7 @@ class Loop(object):
         return "Loop(entry=%s, exit=%s)" % args
 
 
-class With(object):
+class With(SlotEqualityCheckMixin):
     """Describes a with-block
     """
     __slots__ = "entry", "exit"
@@ -997,6 +1257,85 @@ class FunctionIR(object):
 
         self._reset_analysis_variables()
 
+    def equal_ir(self, other):
+        """ Checks that the IR contained within is equal to the IR in other.
+        Equality is defined by being equal in fundamental structure (blocks,
+        labels, IR node type and the order in which they are defined) and the
+        IR nodes being equal. IR node equality essentially comes down to
+        ensuring a node's `.__dict__` or `.__slots__` is equal, with the
+        exception of ignoring 'loc' and 'scope' entries. The upshot is that the
+        comparison is essentially location and scope invariant, but otherwise
+        behaves as unsurprisingly as possible.
+        """
+        if type(self) is type(other):
+            return self.blocks == other.blocks
+        return False
+
+    def diff_str(self, other):
+        """
+        Compute a human readable difference in the IR, returns a formatted
+        string ready for printing.
+        """
+        msg = []
+        for label, block in self.blocks.items():
+            other_blk = other.blocks.get(label, None)
+            if other_blk is not None:
+                if block != other_blk:
+                    msg.append(("Block %s differs" % label).center(80, '-'))
+                    # see if the instructions are just a permutation
+                    block_del = [x for x in block.body if isinstance(x, Del)]
+                    oth_del = [x for x in other_blk.body if isinstance(x, Del)]
+                    if block_del != oth_del:
+                        # this is a common issue, dels are all present, but
+                        # order shuffled.
+                        if sorted(block_del) == sorted(oth_del):
+                            msg.append(("Block %s contains the same dels but "
+                                        "their order is different") % label)
+                    if len(block.body) > len(other_blk.body):
+                        msg.append("This block contains more statements")
+                    elif len(block.body) < len(other_blk.body):
+                        msg.append("Other block contains more statements")
+
+                    # find the indexes where they don't match
+                    tmp = []
+                    for idx, stmts in enumerate(zip(block.body,
+                                                    other_blk.body)):
+                        b_s, o_s = stmts
+                        if b_s != o_s:
+                            tmp.append(idx)
+
+                    def get_pad(ablock, l):
+                        pointer = '-> '
+                        sp = len(pointer) * ' '
+                        pad = []
+                        nstmt = len(ablock)
+                        for i in range(nstmt):
+                            if i in tmp:
+                                item = pointer
+                            elif i >= l:
+                                item = pointer
+                            else:
+                                item = sp
+                            pad.append(item)
+                        return pad
+
+                    min_stmt_len = min(len(block.body), len(other_blk.body))
+
+                    with StringIO() as buf:
+                        it = [("self", block), ("other", other_blk)]
+                        for name, _block in it:
+                            buf.truncate(0)
+                            _block.dump(file=buf)
+                            stmts = buf.getvalue().splitlines()
+                            pad = get_pad(_block.body, min_stmt_len)
+                            title = ("%s: block %s" % (name, label))
+                            msg.append(title.center(80, '-'))
+                            msg.extend(["{0}{1}".format(a, b) for a, b in
+                                        zip(pad, stmts)])
+        if msg == []:
+            msg.append("IR is considered equivalent.")
+        return '\n'.join(msg)
+
     def _reset_analysis_variables(self):
         from . import consts
 
@@ -1085,12 +1424,57 @@ class FunctionIR(object):
                                % (name,))
             value = defs[0]
 
+    def get_assignee(self, rhs_value, in_blocks=None):
+        """
+        Finds the assignee for a given RHS value. If in_blocks is given the
+        search will be limited to the specified blocks.
+        """
+        if in_blocks is None:
+            blocks = self.blocks.values()
+        elif isinstance(in_blocks, int):
+            blocks = [self.blocks[in_blocks]]
+        else:
+            blocks = [self.blocks[blk] for blk in list(in_blocks)]
+
+        assert isinstance(rhs_value, AbstractRHS)
+
+        for blk in blocks:
+            for assign in blk.find_insts(Assign):
+                if assign.value == rhs_value:
+                    return assign.target
+
+        raise ValueError("Could not find an assignee for %s" % rhs_value)
+
+
     def dump(self, file=None):
+        from numba.six import StringIO
+        nofile = file is None
         # Avoid early bind of sys.stdout as default value
-        file = file or sys.stdout
+        file = file or StringIO()
         for offset, block in sorted(self.blocks.items()):
             print('label %s:' % (offset,), file=file)
             block.dump(file=file)
+        if nofile:
+            text = file.getvalue()
+            if config.HIGHLIGHT_DUMPS:
+                try:
+                    import pygments
+                except ImportError:
+                    msg = "Please install pygments to see highlighted dumps"
+                    raise ValueError(msg)
+                else:
+                    from pygments import highlight
+                    from pygments.lexers import DelphiLexer as lexer
+                    from pygments.formatters import Terminal256Formatter
+                    print(highlight(text, lexer(), Terminal256Formatter()))
+            else:
+                print(text)
+
+
+    def dump_to_string(self):
+        with StringIO() as sb:
+            self.dump(file=sb)
+            return sb.getvalue()
 
     def dump_generator_info(self, file=None):
         file = file or sys.stdout
@@ -1101,9 +1485,52 @@ class FunctionIR(object):
                   % (index, sorted(yp.live_vars), sorted(yp.weak_live_vars)),
                   file=file)
 
+    def render_dot(self, filename_prefix="numba_ir", include_ir=True):
+        """Render the CFG of the IR with GraphViz DOT via the
+        ``graphviz`` python binding.
+
+        Returns
+        -------
+        g : graphviz.Digraph
+            Use `g.view()` to open the graph in the default PDF application.
+        """
+
+        try:
+            import graphviz as gv
+        except ImportError:
+            raise ImportError(
+                "The feature requires `graphviz` but it is not available. "
+                "Please install with `pip install graphviz`"
+            )
+        g = gv.Digraph(
+            filename="{}{}.dot".format(
+                filename_prefix,
+                self.func_id.unique_name,
+            )
+        )
+        # Populate the nodes
+        for k, blk in self.blocks.items():
+            with StringIO() as sb:
+                blk.dump(sb)
+                label = sb.getvalue()
+            if include_ir:
+                label = ''.join(
+                    ['  {}\l'.format(x) for x in label.splitlines()],
+                )
+                label = "block {}\l".format(k) + label
+                g.node(str(k), label=label, shape='rect')
+            else:
+                label = "{}\l".format(k)
+                g.node(str(k), label=label, shape='circle')
+        # Populate the edges
+        for src, blk in self.blocks.items():
+            for dst in blk.terminator.get_targets():
+                g.edge(str(src), str(dst))
+        return g
+
 
 # A stub for undefined global reference
-class UndefinedType(object):
+class UndefinedType(EqualityCheckMixin):
 
     _singleton = None
 
@@ -1119,4 +1546,5 @@ class UndefinedType(object):
     def __repr__(self):
         return "Undefined"
 
+
 UNDEFINED = UndefinedType()
