@@ -9,6 +9,7 @@ import numpy
 import types as pytypes
 import collections
 import operator
+import warnings
 
 from llvmlite import ir as lir
 
@@ -19,7 +20,10 @@ from numba.typing.templates import signature, infer_global, AbstractTemplate
 from numba.targets.imputils import impl_ret_untracked
 from numba.analysis import (compute_live_map, compute_use_defs,
                             compute_cfg_from_blocks)
-from numba.errors import TypingError, UnsupportedError
+from numba.errors import (TypingError, UnsupportedError,
+                          NumbaPendingDeprecationWarning, NumbaWarning,
+                          feedback_details)
+
 import copy
 
 _unique_var_count = 0
@@ -256,25 +260,6 @@ def mk_loop_header(typemap, phi_var, calltypes, scope, loc):
     return header_block
 
 
-def find_op_typ(op, arg_typs):
-    for ft in typing.templates.builtin_registry.functions:
-        if ft.key == op:
-            try:
-                func_typ = types.Function(ft).get_call_type(typing.Context(),
-                                                            arg_typs, {})
-            except TypingError:
-                func_typ = None
-            if func_typ is not None:
-                return func_typ
-    else:
-        for f, ft in typing.templates.builtin_registry.globals:
-            if f == op:
-                return ft.get_call_type(typing.Context(),
-                                                    arg_typs, {})
-
-    raise RuntimeError("unknown array operation")
-
-
 def legalize_names(varnames):
     """returns a dictionary for conversion of variable names to legal
     parameter names.
@@ -316,6 +301,7 @@ def replace_var_names(blocks, namedict):
 def replace_var_callback(var, vardict):
     assert isinstance(var, ir.Var)
     while var.name in vardict.keys():
+        assert(vardict[var.name].name != var.name)
         new_var = vardict[var.name]
         var = ir.Var(new_var.scope, new_var.name, new_var.loc)
     return var
@@ -420,6 +406,8 @@ def visit_vars_inner(node, callback, cbdata):
         #     node.rhs.name = callback, cbdata.get(rhs, rhs)
         for arg in node._kws.keys():
             node._kws[arg] = visit_vars_inner(node._kws[arg], callback, cbdata)
+    elif isinstance(node, ir.Yield):
+        node.value = visit_vars_inner(node.value, callback, cbdata)
     return node
 
 
@@ -468,6 +456,35 @@ def find_max_label(blocks):
     return max_label
 
 
+def flatten_labels(blocks):
+    """makes the labels in range(0, len(blocks)), useful to compare CFGs
+    """
+    # first bulk move the labels out of the rewrite range
+    blocks = add_offset_to_labels(blocks, find_max_label(blocks) + 1)
+    # order them in topo order because it's easier to read
+    new_blocks = {}
+    topo_order = find_topo_order(blocks)
+    l_map = dict()
+    idx = 0
+    for x in topo_order:
+        l_map[x] = idx
+        idx += 1
+
+    for t_node in topo_order:
+        b = blocks[t_node]
+        # some parfor last blocks might be empty
+        term = None
+        if b.body:
+            term = b.body[-1]
+        if isinstance(term, ir.Jump):
+            b.body[-1] = ir.Jump(l_map[term.target], term.loc)
+        if isinstance(term, ir.Branch):
+            b.body[-1] = ir.Branch(term.cond, l_map[term.truebr],
+                                   l_map[term.falsebr], term.loc)
+        new_blocks[l_map[t_node]] = b
+    return new_blocks
+
+
 def remove_dels(blocks):
     """remove ir.Del nodes"""
     for block in blocks.values():
@@ -491,6 +508,21 @@ def remove_args(blocks):
     return
 
 
+def dead_code_elimination(func_ir, typemap=None, alias_map=None,
+                          arg_aliases=None):
+    """ Performs dead code elimination and leaves the IR in a valid state on
+    exit (ir.Dels are present in correct locations).
+    """
+    do_post_proc = False
+    while (remove_dead(func_ir.blocks, func_ir.arg_names, func_ir, typemap,
+                       alias_map, arg_aliases)):
+        do_post_proc = True
+
+    if do_post_proc:
+        post_proc = numba.postproc.PostProcessor(func_ir)
+        post_proc.run()
+
+
 def remove_dead(blocks, args, func_ir, typemap=None, alias_map=None, arg_aliases=None):
     """dead code elimination using liveness and CFG info.
     Returns True if something has been removed, or False if nothing is removed.
@@ -502,8 +534,11 @@ def remove_dead(blocks, args, func_ir, typemap=None, alias_map=None, arg_aliases
     if alias_map is None or arg_aliases is None:
         alias_map, arg_aliases = find_potential_aliases(blocks, args, typemap,
                                                         func_ir)
-    if config.DEBUG_ARRAY_OPT == 1:
-        print("alias map:", alias_map)
+    if config.DEBUG_ARRAY_OPT >= 1:
+        print("remove_dead alias map:", alias_map)
+        print("live_map:", live_map)
+        print("usemap:", usedefs.usemap)
+        print("defmap:", usedefs.defmap)
     # keep set for easier search
     alias_set = set(alias_map.keys())
 
@@ -511,11 +546,16 @@ def remove_dead(blocks, args, func_ir, typemap=None, alias_map=None, arg_aliases
     for label, block in blocks.items():
         # find live variables at each statement to delete dead assignment
         lives = {v.name for v in block.terminator.list_vars()}
+        if config.DEBUG_ARRAY_OPT >= 2:
+            print("remove_dead processing block", label, lives)
         # find live variables at the end of block
         for out_blk, _data in cfg.successors(label):
+            if config.DEBUG_ARRAY_OPT >= 2:
+                print("succ live_map", out_blk, live_map[out_blk])
             lives |= live_map[out_blk]
         removed |= remove_dead_block(block, lives, call_table, arg_aliases,
                                      alias_map, alias_set, func_ir, typemap)
+
     return removed
 
 
@@ -538,19 +578,24 @@ def remove_dead_block(block, lives, call_table, arg_aliases, alias_map,
     new_body = [block.terminator]
     # for each statement in reverse order, excluding terminator
     for stmt in reversed(block.body[:-1]):
+        if config.DEBUG_ARRAY_OPT >= 2:
+            print("remove_dead_block", stmt)
         # aliases of lives are also live
         alias_lives = set()
         init_alias_lives = lives & alias_set
         for v in init_alias_lives:
             alias_lives |= alias_map[v]
         lives_n_aliases = lives | alias_lives | arg_aliases
+
         # let external calls handle stmt if type matches
         if type(stmt) in remove_dead_extensions:
             f = remove_dead_extensions[type(stmt)]
-            stmt = f(stmt, lives, arg_aliases, alias_map, func_ir, typemap)
+            stmt = f(stmt, lives, lives_n_aliases, arg_aliases, alias_map, func_ir,
+                     typemap)
             if stmt is None:
                 removed = True
                 continue
+
         # ignore assignments that their lhs is not live or lhs==rhs
         if isinstance(stmt, ir.Assign):
             lhs = stmt.target
@@ -563,6 +608,12 @@ def remove_dead_block(block, lives, call_table, arg_aliases, alias_map,
                 removed = True
                 continue
             # TODO: remove other nodes like SetItem etc.
+
+        if isinstance(stmt, ir.Del):
+            if stmt.value not in lives:
+                removed = True
+                continue
+
         if isinstance(stmt, ir.SetItem):
             name = stmt.target.name
             if name not in lives_n_aliases:
@@ -607,7 +658,9 @@ def has_no_side_effect(rhs, lives, call_table):
             call_list == ['stencil', numba] or
             call_list == ['log', numpy] or
             call_list == ['dtype', numpy] or
-            call_list == [numba.array_analysis.wrap_index]):
+            call_list == [numba.array_analysis.wrap_index] or
+            call_list == [numba.special.prange] or
+            call_list == [numba.parfor.internal_prange]):
             return True
         elif (isinstance(call_list[0], numba.extending._Intrinsic) and
               (call_list[0]._name == 'empty_inferred' or
@@ -659,6 +712,13 @@ def is_pure(rhs, lives, call_table):
         return False
     return True
 
+def is_const_call(module_name, func_name):
+    # Returns True if there is no state in the given module changed by the given function.
+    if module_name == 'numpy':
+        if func_name in ['empty']:
+            return True
+    return False
+
 alias_analysis_extensions = {}
 alias_func_extensions = {}
 
@@ -696,6 +756,10 @@ def find_potential_aliases(blocks, args, typemap, func_ir, alias_map=None,
                 if (isinstance(expr, ir.Expr) and expr.op == 'getattr'
                         and expr.attr in ['T', 'ctypes', 'flat']):
                     _add_alias(lhs, expr.value.name, alias_map, arg_aliases)
+                # a = b.c.  a should alias b
+                if (isinstance(expr, ir.Expr) and expr.op == 'getattr'
+                        and expr.value.name in arg_aliases):
+                    _add_alias(lhs, expr.value.name, alias_map, arg_aliases)
                 # calls that can create aliases such as B = A.ravel()
                 if isinstance(expr, ir.Expr) and expr.op == 'call':
                     fdef = guard(find_callname, func_ir, expr, typemap)
@@ -784,7 +848,7 @@ def copy_propagate(blocks, typemap):
                                  | (in_copies[label] - kill_copies[label]))
         old_point = new_point
         new_point = copy.deepcopy(out_copies)
-    if config.DEBUG_ARRAY_OPT == 1:
+    if config.DEBUG_ARRAY_OPT >= 1:
         print("copy propagate out_copies:", out_copies)
     return in_copies, out_copies
 
@@ -859,7 +923,11 @@ def get_block_copies(blocks, typemap):
                     rhs = stmt.value.name
                     # copy is valid only if same type (see
                     # TestCFunc.test_locals)
-                    if typemap[lhs] == typemap[rhs]:
+                    # Some transformations can produce assignments of the
+                    # form A = A.  We don't put these mapping in the
+                    # copy propagation set because then you get cycles and
+                    # infinite loops in the replacement phase.
+                    if typemap[lhs] == typemap[rhs] and lhs != rhs:
                         assign_dict[lhs] = rhs
                         continue
                 if isinstance(stmt.value,
@@ -984,7 +1052,7 @@ def dprint_func_ir(func_ir, title, blocks=None):
     """Debug print function IR, with an optional blocks argument
     that may differ from the IR's original blocks.
     """
-    if config.DEBUG_ARRAY_OPT == 1:
+    if config.DEBUG_ARRAY_OPT >= 1:
         ir_blocks = func_ir.blocks
         func_ir.blocks = ir_blocks if blocks == None else blocks
         name = func_ir.func_id.func_qualname
@@ -998,7 +1066,7 @@ def find_topo_order(blocks, cfg = None):
     """find topological order of blocks such that true branches are visited
     first (e.g. for_break test in test_dataflow).
     """
-    if cfg == None:
+    if cfg is None:
         cfg = compute_cfg_from_blocks(blocks)
     post_order = []
     seen = set()
@@ -1039,7 +1107,7 @@ def get_call_table(blocks, call_table=None, reverse_call_table=None, topological
         order = find_topo_order(blocks)
     else:
         order = list(blocks.keys())
-    
+
     for label in reversed(order):
         for inst in reversed(blocks[label].body):
             if isinstance(inst, ir.Assign):
@@ -1067,6 +1135,13 @@ def get_call_table(blocks, call_table=None, reverse_call_table=None, topological
                     if lhs in reverse_call_table:
                         call_var = reverse_call_table[lhs]
                         call_table[call_var].append(rhs.value)
+                if isinstance(rhs, ir.Var):
+                    if lhs in call_table:
+                        call_table[lhs].append(rhs.name)
+                        reverse_call_table[rhs.name] = lhs
+                    if lhs in reverse_call_table:
+                        call_var = reverse_call_table[lhs]
+                        call_table[call_var].append(rhs.name)
             for T, f in call_table_extensions.items():
                 if isinstance(inst, T):
                     f(inst, call_table, reverse_call_table)
@@ -1292,7 +1367,7 @@ def merge_adjacent_blocks(blocks):
                 break
             next_block = blocks[next_label]
             # XXX: commented out since scope objects are not consistent
-            # thoughout the compiler. for example, pieces of code are compiled
+            # throughout the compiler. for example, pieces of code are compiled
             # and inlined on the fly without proper scope merge.
             # if block.scope != next_block.scope:
             #     break
@@ -1335,9 +1410,11 @@ def simplify(func_ir, typemap, calltypes):
         calltypes)
     restore_copy_var_names(func_ir.blocks, save_copies, typemap)
     # remove dead code to enable fusion
+    if config.DEBUG_ARRAY_OPT >= 1:
+        dprint_func_ir(func_ir, "after copy prop")
     remove_dead(func_ir.blocks, func_ir.arg_names, func_ir, typemap)
     func_ir.blocks = simplify_CFG(func_ir.blocks)
-    if config.DEBUG_ARRAY_OPT == 1:
+    if config.DEBUG_ARRAY_OPT >= 1:
         dprint_func_ir(func_ir, "after simplify")
 
 class GuardException(Exception):
@@ -1420,7 +1497,7 @@ def find_callname(func_ir, expr, typemap=None, definition_finder=get_definition)
                 if hasattr(callee_def.value, key):
                     value = getattr(callee_def.value, key)
                     break
-            if not value:
+            if not value or not isinstance(value, str):
                 raise GuardException
             attrs.append(value)
             def_val = callee_def.value
@@ -1482,7 +1559,7 @@ def find_const(func_ir, var):
     """
     require(isinstance(var, ir.Var))
     var_def = get_definition(func_ir, var)
-    require(isinstance(var_def, ir.Const))
+    require(isinstance(var_def, (ir.Const, ir.Global, ir.FreeVar)))
     return var_def.value
 
 def compile_to_numba_ir(mk_func, glbls, typingctx=None, arg_typs=None,
@@ -1495,8 +1572,8 @@ def compile_to_numba_ir(mk_func, glbls, typingctx=None, arg_typs=None,
     if typingctx and other typing inputs are available and update typemap and
     calltypes.
     """
-    from numba import compiler
-    # mk_func can be actual function or make_function node
+    from numba import typed_passes
+    # mk_func can be actual function or make_function node, or a njit function
     if hasattr(mk_func, 'code'):
         code = mk_func.code
     elif hasattr(mk_func, '__code__'):
@@ -1522,7 +1599,7 @@ def compile_to_numba_ir(mk_func, glbls, typingctx=None, arg_typs=None,
     # perform type inference if typingctx is available and update type
     # data structures typemap and calltypes
     if typingctx:
-        f_typemap, f_return_type, f_calltypes = compiler.type_inference_stage(
+        f_typemap, f_return_type, f_calltypes = typed_passes.type_inference_stage(
                 typingctx, f_ir, arg_typs, None)
         # remove argument entries like arg.a from typemap
         arg_names = [vname for vname in f_typemap if vname.startswith("arg.")]
@@ -1532,14 +1609,15 @@ def compile_to_numba_ir(mk_func, glbls, typingctx=None, arg_typs=None,
         calltypes.update(f_calltypes)
     return f_ir
 
-def get_ir_of_code(glbls, fcode):
+def _create_function_from_code_obj(fcode, func_env, func_arg, func_clo, glbls):
     """
-    Compile a code object to get its IR.
+    Creates a function from a code object. Args:
+    * fcode - the code object
+    * func_env - string for the freevar placeholders
+    * func_arg - string for the function args (e.g. "a, b, c, d=None")
+    * func_clo - string for the closure args
+    * glbls - the function globals
     """
-    nfree = len(fcode.co_freevars)
-    func_env = "\n".join(["  c_%d = None" % i for i in range(nfree)])
-    func_clo = ",".join(["c_%d" % i for i in range(nfree)])
-    func_arg = ",".join(["x_%d" % i for i in range(fcode.co_argcount)])
     func_text = "def g():\n%s\n  def f(%s):\n    return (%s)\n  return f" % (
         func_env, func_arg, func_clo)
     loc = {}
@@ -1568,8 +1646,23 @@ def get_ir_of_code(glbls, fcode):
             fcode.co_cellvars)
 
     f = loc['g']()
+    # replace the code body
     f.__code__ = fcode
     f.__name__ = fcode.co_name
+    return f
+
+def get_ir_of_code(glbls, fcode):
+    """
+    Compile a code object to get its IR.
+    """
+    nfree = len(fcode.co_freevars)
+    func_env = "\n".join(["  c_%d = None" % i for i in range(nfree)])
+    func_clo = ",".join(["c_%d" % i for i in range(nfree)])
+    func_arg = ",".join(["x_%d" % i for i in range(fcode.co_argcount)])
+
+    f = _create_function_from_code_obj(fcode, func_env, func_arg, func_clo,
+                                       glbls)
+
     from numba import compiler
     ir = compiler.run_frontend(f)
     # we need to run the before inference rewrite pass to normalize the IR
@@ -1577,15 +1670,15 @@ def get_ir_of_code(glbls, fcode):
     # for example, Raise nodes need to become StaticRaise before type inference
     class DummyPipeline(object):
         def __init__(self, f_ir):
-            self.typingctx = None
-            self.targetctx = None
-            self.args = None
-            self.func_ir = f_ir
-            self.typemap = None
-            self.return_type = None
-            self.calltypes = None
-    rewrites.rewrite_registry.apply('before-inference',
-                                    DummyPipeline(ir), ir)
+            self.state = compiler.StateDict()
+            self.state.typingctx = None
+            self.state.targetctx = None
+            self.state.args = None
+            self.state.func_ir = f_ir
+            self.state.typemap = None
+            self.state.return_type = None
+            self.state.calltypes = None
+    rewrites.rewrite_registry.apply('before-inference', DummyPipeline(ir).state)
     # call inline pass to handle cases like stencils and comprehensions
     swapped = {} # TODO: get this from diagnostics store
     inline_pass = numba.inline_closurecall.InlineClosureCallPass(
@@ -1789,7 +1882,7 @@ def find_global_value(func_ir, var):
     """Check if a variable is a global value, and return the value,
     or raise GuardException otherwise.
     """
-    dfn = func_ir.get_definition(var)
+    dfn = get_definition(func_ir, var)
     if isinstance(dfn, ir.Global):
         return dfn.value
 
@@ -1798,13 +1891,13 @@ def find_global_value(func_ir, var):
         try:
             val = getattr(prev_val, dfn.attr)
             return val
-        except KeyError:
+        except AttributeError:
             raise GuardException
 
     raise GuardException
 
 
-def raise_on_unsupported_feature(func_ir):
+def raise_on_unsupported_feature(func_ir, typemap):
     """
     Helper function to walk IR and raise if it finds op codes
     that are unsupported. Could be extended to cover IR sequences
@@ -1814,6 +1907,21 @@ def raise_on_unsupported_feature(func_ir):
     """
     gdb_calls = [] # accumulate calls to gdb/gdb_init
 
+    # issue 2195: check for excessively large tuples
+    for arg_name in func_ir.arg_names:
+        if arg_name in typemap and \
+           isinstance(typemap[arg_name], types.containers.UniTuple) and \
+           typemap[arg_name].count > 1000:
+            # Raise an exception when len(tuple) > 1000. The choice of this number (1000)
+            # was entirely arbitrary
+            msg = ("Tuple '{}' length must be smaller than 1000.\n"
+                   "Large tuples lead to the generation of a prohibitively large "
+                   "LLVM IR which causes excessive memory pressure "
+                   "and large compile times.\n"
+                   "As an alternative, the use of a 'list' is recommended in "
+                   "place of a 'tuple' as lists do not suffer from this problem.".format(arg_name))
+            raise UnsupportedError(msg, func_ir.loc)
+
     for blk in func_ir.blocks.values():
         for stmt in blk.find_insts(ir.Assign):
             # This raises on finding `make_function`
@@ -1846,7 +1954,7 @@ def raise_on_unsupported_feature(func_ir):
                             (use, expr)
                     raise UnsupportedError(msg, stmt.value.loc)
 
-            # this checks for gdb initilization calls, only one is permitted
+            # this checks for gdb initialization calls, only one is permitted
             if isinstance(stmt.value, (ir.Global, ir.FreeVar)):
                 val = stmt.value
                 val = getattr(val, 'value', None)
@@ -1862,6 +1970,43 @@ def raise_on_unsupported_feature(func_ir):
                 if found:
                     gdb_calls.append(stmt.loc) # report last seen location
 
+            # this checks that np.<type> was called if view is called
+            if isinstance(stmt.value, ir.Expr):
+                if stmt.value.op == 'getattr' and stmt.value.attr == 'view':
+                    var = stmt.value.value.name
+                    if isinstance(typemap[var], types.Array):
+                        continue
+                    df = func_ir.get_definition(var)
+                    cn = guard(find_callname, func_ir, df)
+                    if cn and cn[1] == 'numpy':
+                        ty = getattr(numpy, cn[0])
+                        if (numpy.issubdtype(ty, numpy.integer) or
+                                numpy.issubdtype(ty, numpy.floating)):
+                            continue
+
+                    vardescr = '' if var.startswith('$') else "'{}' ".format(var)
+                    raise TypingError(
+                        "'view' can only be called on NumPy dtypes, "
+                        "try wrapping the variable {}with 'np.<dtype>()'".
+                        format(vardescr), loc=stmt.loc)
+
+            # checks for globals that are also reflected
+            if isinstance(stmt.value, ir.Global):
+                ty = typemap[stmt.target.name]
+                msg = ("The use of a %s type, assigned to variable '%s' in "
+                       "globals, is not supported as globals are considered "
+                       "compile-time constants and there is no known way to "
+                       "compile a %s type as a constant.")
+                if (getattr(ty, 'reflected', False) or
+                    isinstance(ty, (types.DictType, types.ListType))):
+                    raise TypingError(msg % (ty, stmt.value.name, ty), loc=stmt.loc)
+
+            # checks for generator expressions (yield in use when func_ir has
+            # not been identified as a generator).
+            if isinstance(stmt.value, ir.Yield) and not func_ir.is_generator:
+                msg = "The use of generator expressions is unsupported."
+                raise UnsupportedError(msg, loc=stmt.loc)
+
     # There is more than one call to function gdb/gdb_init
     if len(gdb_calls) > 1:
         msg = ("Calling either numba.gdb() or numba.gdb_init() more than once "
@@ -1873,3 +2018,144 @@ def raise_on_unsupported_feature(func_ir):
                "nopython-mode\n\nConflicting calls found at:\n %s")
         buf = '\n'.join([x.strformat() for x in gdb_calls])
         raise UnsupportedError(msg % buf)
+
+
+def warn_deprecated(func_ir, typemap):
+    # first pass, just walk the type map
+    for name, ty in typemap.items():
+        # the Type Metaclass has a reflected member
+        if ty.reflected:
+            # if its an arg, report function call
+            if name.startswith('arg.'):
+                loc = func_ir.loc
+                arg = name.split('.')[1]
+                fname = func_ir.func_id.func_qualname
+                tyname = 'list' if isinstance(ty, types.List) else 'set'
+                url = ("http://numba.pydata.org/numba-doc/latest/reference/"
+                       "deprecation.html#deprecation-of-reflection-for-list-and"
+                       "-set-types")
+                msg = ("\nEncountered the use of a type that is scheduled for "
+                        "deprecation: type 'reflected %s' found for argument "
+                        "'%s' of function '%s'.\n\nFor more information visit "
+                        "%s" % (tyname, arg, fname, url))
+                warnings.warn(NumbaPendingDeprecationWarning(msg, loc=loc))
+
+
+def resolve_func_from_module(func_ir, node):
+    """
+    This returns the python function that is being getattr'd from a module in
+    some IR, it resolves import chains/submodules recursively. Should it not be
+    possible to find the python function being called None will be returned.
+
+    func_ir - the FunctionIR object
+    node - the IR node from which to start resolving (should be a `getattr`).
+    """
+    getattr_chain = []
+    def resolve_mod(mod):
+        if getattr(mod, 'op', False) == 'getattr':
+            getattr_chain.insert(0, mod.attr)
+            try:
+                mod = func_ir.get_definition(mod.value)
+            except KeyError: # multiple definitions
+                return None
+            return resolve_mod(mod)
+        elif isinstance(mod, (ir.Global, ir.FreeVar)):
+            if isinstance(mod.value, pytypes.ModuleType):
+                return mod
+        return None
+
+    mod = resolve_mod(node)
+    if mod is not None:
+        defn = mod.value
+        for x in getattr_chain:
+            defn = getattr(defn, x, False)
+            if not defn:
+                break
+        else:
+            return defn
+    else:
+        return None
+
+
+def check_and_legalize_ir(func_ir):
+    """
+    This checks that the IR presented is legal, warns and legalizes if not
+    """
+    orig_ir = func_ir.copy()
+    post_proc = numba.postproc.PostProcessor(func_ir)
+    post_proc.run()
+    msg = ("\nNumba has detected inconsistencies in its internal "
+           "representation of the code at %s. Numba can probably recover from "
+           "this problem and is attempting to do, however something inside "
+           "Numba needs fixing...\n%s\n") % (func_ir.loc, feedback_details)
+    if not func_ir.equal_ir(orig_ir):
+        msg +=  func_ir.diff_str(orig_ir)
+        warnings.warn(NumbaWarning(msg, loc=func_ir.loc))
+
+
+def convert_code_obj_to_function(code_obj, caller_ir):
+    """
+    Converts a code object from a `make_function.code` attr in the IR into a
+    python function, caller_ir is the FunctionIR of the caller and is used for
+    the resolution of freevars.
+    """
+    fcode = code_obj.code
+    nfree = len(fcode.co_freevars)
+
+    # try and resolve freevars if they are consts in the caller's IR
+    # these can be baked into the new function
+    freevars = []
+    for x in fcode.co_freevars:
+        # not using guard here to differentiate between multiple definition and
+        # non-const variable
+        try:
+            freevar_def = caller_ir.get_definition(x)
+        except KeyError:
+            msg = ("Cannot capture a constant value for variable '%s' as there "
+                   "are multiple definitions present." % x)
+            raise TypingError(msg, loc=code_obj.loc)
+        if isinstance(freevar_def, ir.Const):
+            freevars.append(freevar_def.value)
+        else:
+            msg = ("Cannot capture the non-constant value associated with "
+                   "variable '%s' in a function that will escape." % x)
+            raise TypingError(msg, loc=code_obj.loc)
+
+    func_env = "\n".join(["  c_%d = %s" % (i, x) for i, x in enumerate(freevars)])
+    func_clo = ",".join(["c_%d" % i for i in range(nfree)])
+    co_varnames = list(fcode.co_varnames)
+
+    # This is horrible. The code object knows about the number of args present
+    # it also knows the name of the args but these are bundled in with other
+    # vars in `co_varnames`. The make_function IR node knows what the defaults
+    # are, they are defined in the IR as consts. The following finds the total
+    # number of args (args + kwargs with defaults), finds the default values
+    # and infers the number of "kwargs with defaults" from this and then infers
+    # the number of actual arguments from that.
+    n_kwargs = 0
+    n_allargs = fcode.co_argcount
+    kwarg_defaults = caller_ir.get_definition(code_obj.defaults)
+    if kwarg_defaults is not None:
+        if isinstance(kwarg_defaults, tuple):
+            d = [caller_ir.get_definition(x).value for x in kwarg_defaults]
+            kwarg_defaults_tup = tuple(d)
+        else:
+            d = [caller_ir.get_definition(x).value
+                 for x in kwarg_defaults.items]
+            kwarg_defaults_tup = tuple(d)
+        n_kwargs = len(kwarg_defaults_tup)
+    nargs = n_allargs - n_kwargs
+
+    func_arg = ",".join(["%s" % (co_varnames[i]) for i in range(nargs)])
+    if n_kwargs:
+        kw_const = ["%s = %s" % (co_varnames[i + nargs], kwarg_defaults_tup[i])
+                    for i in range(n_kwargs)]
+        func_arg += ", "
+        func_arg += ", ".join(kw_const)
+
+    # globals are the same as those in the caller
+    glbls = caller_ir.func_id.func.__globals__
+
+    # create the function and return it
+    return _create_function_from_code_obj(fcode, func_env, func_arg, func_clo,
+                                          glbls)
