@@ -3,6 +3,7 @@ from __future__ import absolute_import, print_function
 from collections import OrderedDict
 import types as pytypes
 import inspect
+import operator
 
 from llvmlite import ir as llvmir
 
@@ -12,23 +13,15 @@ from numba import njit
 from numba.typing import templates
 from numba.datamodel import default_manager, models
 from numba.targets import imputils
-from numba import cgutils, utils
-from numba.config import PYVERSION
-from numba.six import exec_
-
-
-if PYVERSION >= (3, 3):
-    from collections.abc import Sequence
-else:
-    from collections import Sequence
-
-
+from numba import cgutils, utils, errors
+from numba.six import exec_, Sequence
 from . import _box
 
 
 ##############################################################################
 # Data model
 
+
 class InstanceModel(models.StructModel):
     def __init__(self, dmm, fe_typ):
         cls_data_ty = types.ClassDataType(fe_typ)
@@ -73,14 +66,18 @@ def ctor({args}):
 """
 
 
-def _getargs(fn):
+def _getargs(fn_sig):
     """
     Returns list of positional and keyword argument names in order.
     """
-    sig = utils.pysignature(fn)
-    params = sig.parameters
-    args = [k for k, v in params.items()
-            if (v.kind & v.POSITIONAL_OR_KEYWORD) == v.POSITIONAL_OR_KEYWORD]
+    params = fn_sig.parameters
+    args = []
+    for k, v in params.items():
+        if (v.kind & v.POSITIONAL_OR_KEYWORD) == v.POSITIONAL_OR_KEYWORD:
+            args.append(k)
+        else:
+            msg = "%s argument type unsupported in jitclass" % v.kind
+            raise errors.UnsupportedError(msg)
     return args
 
 
@@ -105,9 +102,11 @@ class JitClassType(type):
         Note the wrapper will only accept positional arguments.
         """
         init = cls.class_type.instance_type.methods['__init__']
+        init_sig = utils.pysignature(init)
         # get postitional and keyword arguments
         # offset by one to exclude the `self` arg
-        args = _getargs(init)[1:]
+        args = _getargs(init_sig)[1:]
+        cls._ctor_sig = init_sig
         ctor_source = _ctor_template.format(args=', '.join(args))
         glbls = {"__numba_cls_": cls}
         exec_(ctor_source, glbls)
@@ -120,7 +119,11 @@ class JitClassType(type):
         return False
 
     def __call__(cls, *args, **kwargs):
-        return cls._ctor(*args, **kwargs)
+        # The first argument of _ctor_sig is `cls`, which here
+        # is bound to None and then skipped when invoking the constructor.
+        bind = cls._ctor_sig.bind(None, *args, **kwargs)
+        bind.apply_defaults()
+        return cls._ctor(*bind.args[1:], **bind.kwargs)
 
 
 ##############################################################################
@@ -146,12 +149,14 @@ def _fix_up_private_attr(clsname, spec):
         out[k] = v
     return out
 
+
 def _add_linking_libs(context, call):
     """
     Add the required libs for the callable to allow inlining.
     """
-    for lib in getattr(call, "libs", ()):
-        context.active_code_library.add_linking_library(lib)
+    libs = getattr(call, "libs", ())
+    if libs:
+        context.add_linking_libs(libs)
 
 
 def register_class_type(cls, spec, class_ctor, builder):
@@ -296,7 +301,8 @@ class ClassBuilder(object):
         Register method implementations for the given instance type.
         """
         for meth in instance_type.jitmethods:
-            # There's no way to retrive the particular method name
+
+            # There's no way to retrieve the particular method name
             # inside the implementation function, so we have to register a
             # specific closure for each different name
             if meth not in self.implemented_methods:
@@ -304,18 +310,50 @@ class ClassBuilder(object):
                 self.implemented_methods.add(meth)
 
     def _implement_method(self, registry, attr):
-        @registry.lower((types.ClassInstanceType, attr),
-                        types.ClassInstanceType, types.VarArg(types.Any))
-        def imp(context, builder, sig, args):
-            instance_type = sig.args[0]
-            method = instance_type.jitmethods[attr]
-            disp_type = types.Dispatcher(method)
-            call = context.get_function(disp_type, sig)
-            out = call(builder, args)
-            _add_linking_libs(context, call)
-            return imputils.impl_ret_new_ref(context, builder,
-                                             sig.return_type, out)
+        # create a separate instance of imp method to avoid closure clashing
+        def get_imp():
+            def imp(context, builder, sig, args):
+                instance_type = sig.args[0]
+                method = instance_type.jitmethods[attr]
+                disp_type = types.Dispatcher(method)
+                call = context.get_function(disp_type, sig)
+                out = call(builder, args)
+                _add_linking_libs(context, call)
+                return imputils.impl_ret_new_ref(context, builder,
+                                                 sig.return_type, out)
+            return imp
 
+        def _getsetitem_gen(getset):
+            _dunder_meth = "__%s__" % getset
+            op = getattr(operator, getset)
+
+            @templates.infer_global(op)
+            class GetSetItem(templates.AbstractTemplate):
+                def generic(self, args, kws):
+                    instance = args[0]
+                    if isinstance(instance, types.ClassInstanceType) and \
+                            _dunder_meth in instance.jitmethods:
+                        meth = instance.jitmethods[_dunder_meth]
+                        disp_type = types.Dispatcher(meth)
+                        sig = disp_type.get_call_type(self.context, args, kws)
+                        return sig
+
+            # lower both {g,s}etitem and __{g,s}etitem__ to catch the calls
+            # from python and numba
+            imputils.lower_builtin((types.ClassInstanceType, _dunder_meth),
+                                   types.ClassInstanceType,
+                                   types.VarArg(types.Any))(get_imp())
+            imputils.lower_builtin(op,
+                                   types.ClassInstanceType,
+                                   types.VarArg(types.Any))(get_imp())
+
+        dunder_stripped = attr.strip('_')
+        if dunder_stripped in ("getitem", "setitem"):
+            _getsetitem_gen(dunder_stripped)
+        else:
+            registry.lower((types.ClassInstanceType, attr),
+                           types.ClassInstanceType,
+                           types.VarArg(types.Any))(get_imp())
 
 
 @templates.infer_getattr
@@ -352,7 +390,7 @@ class ClassAttribute(templates.AttributeTemplate):
 
 
 @ClassBuilder.class_impl_registry.lower_getattr_generic(types.ClassInstanceType)
-def attr_impl(context, builder, typ, value, attr):
+def get_attr_impl(context, builder, typ, value, attr):
     """
     Generic getattr() for @jitclass instances.
     """
@@ -380,7 +418,7 @@ def attr_impl(context, builder, typ, value, attr):
 
 
 @ClassBuilder.class_impl_registry.lower_setattr_generic(types.ClassInstanceType)
-def attr_impl(context, builder, sig, args, attr):
+def set_attr_impl(context, builder, sig, args, attr):
     """
     Generic setattr() for @jitclass instances.
     """
@@ -415,7 +453,8 @@ def attr_impl(context, builder, sig, args, attr):
         call(builder, (target, val))
         _add_linking_libs(context, call)
     else:
-        raise NotImplementedError('attribute {0!r} not implemented'.format(attr))
+        raise NotImplementedError(
+            'attribute {0!r} not implemented'.format(attr))
 
 
 def imp_dtor(context, module, instance_type):
@@ -444,7 +483,8 @@ def imp_dtor(context, module, instance_type):
     return dtor_fn
 
 
-@ClassBuilder.class_impl_registry.lower(types.ClassType, types.VarArg(types.Any))
+@ClassBuilder.class_impl_registry.lower(types.ClassType,
+                                        types.VarArg(types.Any))
 def ctor_impl(context, builder, sig, args):
     """
     Generic constructor (__new__) for jitclasses.
