@@ -7,10 +7,9 @@ import operator
 from functools import partial
 
 from llvmlite.llvmpy.core import Constant, Type, Builder
-from llvmlite import ir as llvmir
 
 from . import (_dynfunc, cgutils, config, funcdesc, generators, ir, types,
-               typing, utils)
+               typing, utils, ir_utils)
 from .errors import (LoweringError, new_error_context, TypingError,
                      LiteralTypingError)
 from .targets import removerefctpass
@@ -43,14 +42,16 @@ class Environment(_dynfunc.Environment):
         return _rebuild_env, (
             self.globals['__name__'],
             self.consts,
-            self.env_name
-            )
+            self.env_name,
+        )
 
     def __del__(self):
         if utils is None or utils.IS_PY3:
             return
         if _keepalive is None:
             return
+        if time is None or time.time is None:
+            return
         _keepalive.append((time.time(), self))
         if len(_keepalive) > 10:
             cur = time.time()
@@ -103,7 +104,7 @@ class BaseLower(object):
 
         # Specializes the target context as seen inside the Lowerer
         # This adds:
-        #  - environment: the python exceution environment
+        #  - environment: the python execution environment
         self.context = context.subtarget(environment=self.env,
                                          fndesc=self.fndesc)
 
@@ -158,17 +159,33 @@ class BaseLower(object):
         Called before lowering a block.
         """
 
+    def post_block(self, block):
+        """
+        Called after lowering a block.
+        """
+
     def return_exception(self, exc_class, exc_args=None, loc=None):
-        self.call_conv.return_user_exc(self.builder, exc_class, exc_args,
-                                       loc=loc,
-                                       func_name=self.func_ir.func_id.func_name)
+        """Propagate exception to the caller.
+        """
+        self.call_conv.return_user_exc(
+            self.builder, exc_class, exc_args,
+            loc=loc, func_name=self.func_ir.func_id.func_name,
+        )
+
+    def set_exception(self, exc_class, exc_args=None, loc=None):
+        """Set exception state in the current function.
+        """
+        self.call_conv.set_static_user_exc(
+            self.builder, exc_class, exc_args,
+            loc=loc, func_name=self.func_ir.func_id.func_name,
+        )
 
     def emit_environment_object(self):
         """Emit a pointer to hold the Environment object.
         """
         # Define global for the environment and initialize it to NULL
         envname = self.context.get_env_name(self.fndesc)
-        gvenv = self.context.declare_env_global(self.module, envname)
+        self.context.declare_env_global(self.module, envname)
 
     def lower(self):
         # Emit the Env into the module
@@ -187,7 +204,19 @@ class BaseLower(object):
 
         if config.DUMP_LLVM:
             print(("LLVM DUMP %s" % self.fndesc).center(80, '-'))
-            print(self.module)
+            if config.HIGHLIGHT_DUMPS:
+                try:
+                    from pygments import highlight
+                    from pygments.lexers import LlvmLexer as lexer
+                    from pygments.formatters import Terminal256Formatter
+                    print(highlight(self.module.__repr__(), lexer(),
+                                    Terminal256Formatter(
+                                        style='solarized-light')))
+                except ImportError:
+                    msg = "Please install pygments to see highlighted dumps"
+                    raise ValueError(msg)
+            else:
+                print(self.module)
             print('=' * 80)
 
         # Special optimization to remove NRT on functions that do not need it.
@@ -257,6 +286,7 @@ class BaseLower(object):
             with new_error_context('lowering "{inst}" at {loc}', inst=inst,
                                    loc=self.loc, errcls_=defaulterrcls):
                 self.lower_inst(inst)
+        self.post_block(block)
 
     def create_cpython_wrapper(self, release_gil=False):
         """
@@ -285,20 +315,42 @@ class BaseLower(object):
         if config.DEBUG_JIT:
             self.context.debug_print(self.builder, "DEBUGJIT: {0}".format(msg))
 
-    @property
-    def has_dynamic_globals(self):
-        """
-        If true, then can't cache LLVM module accross process calls.
-        """
-        return self.library.has_dynamic_globals
-
 
 # Dictionary mapping instruction class to its lowering function.
 lower_extensions = {}
 
+
 class Lower(BaseLower):
     GeneratorLower = generators.GeneratorLower
 
+    def pre_block(self, block):
+        from numba.unsafe import eh
+
+        super(Lower, self).pre_block(block)
+
+        # Detect if we are in a TRY block by looking for a call to
+        # `eh.exception_check`.
+        for call in block.find_exprs(op='call'):
+            defn = ir_utils.guard(
+                ir_utils.get_definition, self.func_ir, call.func,
+            )
+            if defn is not None and isinstance(defn, ir.Global):
+                if defn.value is eh.exception_check:
+                    if isinstance(block.terminator, ir.Branch):
+                        targetblk = self.blkmap[block.terminator.truebr]
+                        # NOTE: This hacks in an attribute for call_conv to
+                        #       pick up. This hack is no longer needed when
+                        #       all old-style implementations are gone.
+                        self.builder._in_try_block = {'target': targetblk}
+                        break
+
+    def post_block(self, block):
+        # Clean-up
+        try:
+            del self.builder._in_try_block
+        except AttributeError:
+            pass
+
     def lower_inst(self, inst):
         # Set debug location for all subsequent LL instructions
         self.debuginfo.mark_location(self.builder, self.loc)
@@ -345,7 +397,8 @@ class Lower(BaseLower):
             try:
                 impl = self.context.get_function('static_setitem', signature)
             except NotImplementedError:
-                return self.lower_setitem(inst.target, inst.index_var, inst.value, signature)
+                return self.lower_setitem(inst.target, inst.index_var,
+                                          inst.value, signature)
             else:
                 target = self.loadvar(inst.target.name)
                 value = self.loadvar(inst.value.name)
@@ -360,7 +413,13 @@ class Lower(BaseLower):
         elif isinstance(inst, ir.SetItem):
             signature = self.fndesc.calltypes[inst]
             assert signature is not None
-            return self.lower_setitem(inst.target, inst.index, inst.value, signature)
+            return self.lower_setitem(inst.target, inst.index, inst.value,
+                                      signature)
+
+        elif isinstance(inst, ir.StoreMap):
+            signature = self.fndesc.calltypes[inst]
+            assert signature is not None
+            return self.lower_setitem(inst.dct, inst.key, inst.value, signature)
 
         elif isinstance(inst, ir.DelItem):
             target = self.loadvar(inst.target.name)
@@ -371,7 +430,13 @@ class Lower(BaseLower):
 
             signature = self.fndesc.calltypes[inst]
             assert signature is not None
-            impl = self.context.get_function('delitem', signature)
+
+            op = operator.delitem
+            fnop = self.context.typing_context.resolve_value_type(op)
+            callsig = fnop.get_call_type(
+                self.context.typing_context, signature.args, {},
+            )
+            impl = self.context.get_function(fnop, callsig)
 
             assert targetty == signature.args[0]
             index = self.context.cast(self.builder, index, indexty,
@@ -402,6 +467,9 @@ class Lower(BaseLower):
         elif isinstance(inst, ir.StaticRaise):
             self.lower_static_raise(inst)
 
+        elif isinstance(inst, ir.StaticTryRaise):
+            self.lower_static_try_raise(inst)
+
         else:
             for _class, func in lower_extensions.items():
                 if isinstance(inst, _class):
@@ -418,7 +486,12 @@ class Lower(BaseLower):
         valuety = self.typeof(value_var.name)
         indexty = self.typeof(index_var.name)
 
-        impl = self.context.get_function('setitem', signature)
+        op = operator.setitem
+        fnop = self.context.typing_context.resolve_value_type(op)
+        callsig = fnop.get_call_type(
+            self.context.typing_context, signature.args, {},
+        )
+        impl = self.context.get_function(fnop, callsig)
 
         # Convert argument to match
         if isinstance(targetty, types.Optional):
@@ -441,6 +514,13 @@ class Lower(BaseLower):
         else:
             self.return_exception(inst.exc_class, inst.exc_args, loc=self.loc)
 
+    def lower_static_try_raise(self, inst):
+        if inst.exc_class is None:
+            # Reraise
+            self.set_exception(None, loc=self.loc)
+        else:
+            self.set_exception(inst.exc_class, inst.exc_args, loc=self.loc)
+
     def lower_assign(self, ty, inst):
         value = inst.value
         # In nopython mode, closure vars are frozen like globals
@@ -496,7 +576,7 @@ class Lower(BaseLower):
 
         # cast the local val to the type yielded
         yret = self.context.cast(self.builder, val, typ,
-                                    self.gentype.yield_type)
+                                 self.gentype.yield_type)
 
         # get the return repr of yielded value
         retval = self.context.get_return_value(self.builder, typ, yret)
@@ -511,7 +591,8 @@ class Lower(BaseLower):
 
     def lower_binop(self, resty, expr, op):
         # if op in utils.OPERATORS_TO_BUILTINS:
-        # map operator.the_op => the corresponding types.Function() TODO: is this looks dodgy ...
+        # map operator.the_op => the corresponding types.Function()
+        # TODO: is this looks dodgy ...
         op = self.context.typing_context.resolve_value_type(op)
 
         lhs = expr.lhs
@@ -538,7 +619,8 @@ class Lower(BaseLower):
                 return None
             try:
                 if isinstance(op, types.Function):
-                    static_sig = op.get_call_type(self.context.typing_context, tys, {})
+                    static_sig = op.get_call_type(self.context.typing_context,
+                                                  tys, {})
                 else:
                     static_sig = typing.signature(signature.return_type, *tys)
             except TypingError:
@@ -552,21 +634,21 @@ class Lower(BaseLower):
         res = try_static_impl(
             (_lit_or_omitted(static_lhs), _lit_or_omitted(static_rhs)),
             (static_lhs, static_rhs),
-            )
+        )
         if res is not None:
             return cast_result(res)
 
         res = try_static_impl(
             (_lit_or_omitted(static_lhs), rty),
             (static_lhs, rhs),
-            )
+        )
         if res is not None:
             return cast_result(res)
 
         res = try_static_impl(
             (lty, _lit_or_omitted(static_rhs)),
             (lhs, static_rhs),
-            )
+        )
         if res is not None:
             return cast_result(res)
 
@@ -583,8 +665,10 @@ class Lower(BaseLower):
         # Get implementation of getitem
         op = operator.getitem
         fnop = self.context.typing_context.resolve_value_type(op)
-        fnop.get_call_type(self.context.typing_context, signature.args, {})
-        impl = self.context.get_function(fnop, signature)
+        callsig = fnop.get_call_type(
+            self.context.typing_context, signature.args, {},
+        )
+        impl = self.context.get_function(fnop, callsig)
 
         argvals = (baseval, indexval)
         argtyps = (self.typeof(value.name),
@@ -691,169 +775,200 @@ class Lower(BaseLower):
             fnty = self.typeof(expr.func.name)
 
         if isinstance(fnty, types.ObjModeDispatcher):
-            self.init_pyapi()
-            # Acquire the GIL
-            gil_state = self.pyapi.gil_ensure()
-            # Fix types
-            argnames = [a.name for a in expr.args]
-            argtypes = [self.typeof(a) for a in argnames]
-            argvalues = [self.loadvar(a) for a in argnames]
-            for v, ty in zip(argvalues, argtypes):
-                # Because .from_native_value steal the reference
-                self.incref(ty, v)
-
-            argobjs = [self.pyapi.from_native_value(atyp, aval,
-                                                    self.env_manager)
-                       for atyp, aval in zip(argtypes, argvalues)]
-            # Make Call
-            entry_pt = fnty.dispatcher.compile(tuple(argtypes))
-            callee = self.context.add_dynamic_addr(self.builder,
-                                                   id(entry_pt),
-                                                   info="with_objectmode")
-            ret_obj = self.pyapi.call_function_objargs(callee, argobjs)
-            has_exception = cgutils.is_null(self.builder, ret_obj)
-            with self. builder.if_else(has_exception) as (then, orelse):
-                # Handles exception
-                # This branch must exit the function
-                with then:
-                    # Clean arg
-                    for obj in argobjs:
-                        self.pyapi.decref(obj)
-
-                    # Release the GIL
-                    self.pyapi.gil_release(gil_state)
-
-                    # Return and signal exception
-                    self.call_conv.return_exc(self.builder)
-
-                # Handles normal return
-                with orelse:
-                    # Fix output value
-                    native = self.pyapi.to_native_value(
-                        fnty.dispatcher.output_types,
-                        ret_obj,
-                        )
-                    output = native.value
-
-                    # Release objs
-                    self.pyapi.decref(ret_obj)
-                    for obj in argobjs:
-                        self.pyapi.decref(obj)
-
-                    # cleanup output
-                    if callable(native.cleanup):
-                        native.cleanup()
-
-                    # Release the GIL
-                    self.pyapi.gil_release(gil_state)
-
-                    # Error during unboxing
-                    with self.builder.if_then(native.is_error):
-                        self.call_conv.return_exc(self.builder)
-
-                    res = output
+            res = self._lower_call_ObjModeDispatcher(fnty, expr, signature)
 
         elif isinstance(fnty, types.ExternalFunction):
-            # Handle a named external function
-            self.debug_print("# external function")
-            argvals = self.fold_call_args(fnty, signature,
-                                          expr.args, expr.vararg, expr.kws)
-            fndesc = funcdesc.ExternalFunctionDescriptor(
-                fnty.symbol, fnty.sig.return_type, fnty.sig.args)
-            func = self.context.declare_external_function(self.builder.module,
-                                                          fndesc)
-            res = self.context.call_external_function(
-                self.builder, func, fndesc.argtypes, argvals)
-
-        elif isinstance(fnty, types.NumbaFunction):
-            # Handle a compiled Numba function
-            self.debug_print("# calling numba function")
-            argvals = self.fold_call_args(fnty, signature,
-                                          expr.args, expr.vararg, expr.kws)
-            res = self.context.call_internal(self.builder, fnty.fndesc,
-                                             fnty.sig, argvals)
+            res = self._lower_call_ExternalFunction(fnty, expr, signature)
 
         elif isinstance(fnty, types.ExternalFunctionPointer):
-            self.debug_print("# calling external function pointer")
-            argvals = self.fold_call_args(fnty, signature,
-                                          expr.args, expr.vararg, expr.kws)
-            # Handle a C function pointer
-            pointer = self.loadvar(expr.func.name)
-            # If the external function pointer uses libpython
-            if fnty.requires_gil:
-                self.init_pyapi()
-                # Acquire the GIL
-                gil_state = self.pyapi.gil_ensure()
-                # Make PyObjects
-                newargvals = []
-                pyvals = []
-                for exptyp, gottyp, aval in zip(fnty.sig.args, signature.args,
-                                                argvals):
-                    # Adjust argument values to pyobjects
-                    if exptyp == types.ffi_forced_object:
-                        self.incref(gottyp, aval)
-                        obj = self.pyapi.from_native_value(gottyp, aval,
-                                                           self.env_manager)
-                        newargvals.append(obj)
-                        pyvals.append(obj)
-                    else:
-                        newargvals.append(aval)
+            res = self._lower_call_ExternalFunctionPointer(
+                fnty, expr, signature)
 
-                # Call external function
-                res = self.context.call_function_pointer(self.builder, pointer,
-                                                         newargvals, fnty.cconv)
-                # Release PyObjects
-                for obj in pyvals:
+        elif isinstance(fnty, types.RecursiveCall):
+            res = self._lower_call_RecursiveCall(fnty, expr, signature)
+
+        else:
+            res = self._lower_call_normal(fnty, expr, signature)
+
+        # If lowering the call returned None, interpret that as returning dummy
+        # value if the return type of the function is void, otherwise there is
+        # a problem
+        if res is None:
+            if signature.return_type == types.void:
+                res = self.context.get_dummy_value()
+            else:
+                raise LoweringError(
+                    msg="non-void function returns None from implementation",
+                    loc=self.loc
+                )
+
+        return self.context.cast(self.builder, res, signature.return_type,
+                                 resty)
+
+    def _lower_call_ObjModeDispatcher(self, fnty, expr, signature):
+        self.init_pyapi()
+        # Acquire the GIL
+        gil_state = self.pyapi.gil_ensure()
+        # Fix types
+        argnames = [a.name for a in expr.args]
+        argtypes = [self.typeof(a) for a in argnames]
+        argvalues = [self.loadvar(a) for a in argnames]
+        for v, ty in zip(argvalues, argtypes):
+            # Because .from_native_value steal the reference
+            self.incref(ty, v)
+
+        argobjs = [self.pyapi.from_native_value(atyp, aval,
+                                                self.env_manager)
+                   for atyp, aval in zip(argtypes, argvalues)]
+        # Make Call
+        entry_pt = fnty.dispatcher.compile(tuple(argtypes))
+        callee = self.context.add_dynamic_addr(
+            self.builder,
+            id(entry_pt),
+            info="with_objectmode",
+        )
+        ret_obj = self.pyapi.call_function_objargs(callee, argobjs)
+        has_exception = cgutils.is_null(self.builder, ret_obj)
+        with self. builder.if_else(has_exception) as (then, orelse):
+            # Handles exception
+            # This branch must exit the function
+            with then:
+                # Clean arg
+                for obj in argobjs:
                     self.pyapi.decref(obj)
 
                 # Release the GIL
                 self.pyapi.gil_release(gil_state)
-            # If the external function pointer does NOT use libpython
-            else:
-                res = self.context.call_function_pointer(self.builder, pointer,
-                                                         argvals, fnty.cconv)
 
-        elif isinstance(fnty, types.RecursiveCall):
-            # Recursive call
-            argvals = self.fold_call_args(fnty, signature,
-                                          expr.args, expr.vararg, expr.kws)
-            qualprefix = fnty.overloads[signature.args]
-            mangler = self.context.mangler or default_mangler
-            mangled_name = mangler(qualprefix, signature.args)
-            # special case self recursion
-            if self.builder.function.name.startswith(mangled_name):
-                res = self.context.call_internal(self.builder, self.fndesc,
-                                                 signature, argvals)
-            else:
-                res = self.context.call_unresolved(self.builder, mangled_name,
-                                                   signature, argvals)
+                # Return and signal exception
+                self.call_conv.return_exc(self.builder)
 
+            # Handles normal return
+            with orelse:
+                # Fix output value
+                native = self.pyapi.to_native_value(
+                    fnty.dispatcher.output_types,
+                    ret_obj,
+                )
+                output = native.value
+
+                # Release objs
+                self.pyapi.decref(ret_obj)
+                for obj in argobjs:
+                    self.pyapi.decref(obj)
+
+                # cleanup output
+                if callable(native.cleanup):
+                    native.cleanup()
+
+                # Release the GIL
+                self.pyapi.gil_release(gil_state)
+
+                # Error during unboxing
+                with self.builder.if_then(native.is_error):
+                    self.call_conv.return_exc(self.builder)
+
+                return output
+
+    def _lower_call_ExternalFunction(self, fnty, expr, signature):
+        # Handle a named external function
+        self.debug_print("# external function")
+        argvals = self.fold_call_args(
+            fnty, signature, expr.args, expr.vararg, expr.kws,
+        )
+        fndesc = funcdesc.ExternalFunctionDescriptor(
+            fnty.symbol, fnty.sig.return_type, fnty.sig.args)
+        func = self.context.declare_external_function(
+            self.builder.module, fndesc)
+        return self.context.call_external_function(
+            self.builder, func, fndesc.argtypes, argvals,
+        )
+
+    def _lower_call_ExternalFunctionPointer(self, fnty, expr, signature):
+        # Handle a C function pointer
+        self.debug_print("# calling external function pointer")
+        argvals = self.fold_call_args(
+            fnty, signature, expr.args, expr.vararg, expr.kws,
+        )
+        pointer = self.loadvar(expr.func.name)
+        # If the external function pointer uses libpython
+        if fnty.requires_gil:
+            self.init_pyapi()
+            # Acquire the GIL
+            gil_state = self.pyapi.gil_ensure()
+            # Make PyObjects
+            newargvals = []
+            pyvals = []
+            for exptyp, gottyp, aval in zip(fnty.sig.args, signature.args,
+                                            argvals):
+                # Adjust argument values to pyobjects
+                if exptyp == types.ffi_forced_object:
+                    self.incref(gottyp, aval)
+                    obj = self.pyapi.from_native_value(
+                        gottyp, aval, self.env_manager,
+                    )
+                    newargvals.append(obj)
+                    pyvals.append(obj)
+                else:
+                    newargvals.append(aval)
+
+            # Call external function
+            res = self.context.call_function_pointer(
+                self.builder, pointer, newargvals, fnty.cconv,
+            )
+            # Release PyObjects
+            for obj in pyvals:
+                self.pyapi.decref(obj)
+
+            # Release the GIL
+            self.pyapi.gil_release(gil_state)
+        # If the external function pointer does NOT use libpython
         else:
-            # Normal function resolution
-            self.debug_print("# calling normal function: {0}".format(fnty))
-            self.debug_print("# signature: {0}".format(signature))
-            if (isinstance(expr.func, ir.Intrinsic) or
-                    isinstance(fnty, types.ObjModeDispatcher)):
-                argvals = expr.func.args
-            else:
-                argvals = self.fold_call_args(fnty, signature,
-                                              expr.args, expr.vararg, expr.kws)
-            impl = self.context.get_function(fnty, signature)
-            if signature.recvr:
-                # The "self" object is passed as the function object
-                # for bounded function
-                the_self = self.loadvar(expr.func.name)
-                # Prepend the self reference
-                argvals = [the_self] + list(argvals)
+            res = self.context.call_function_pointer(
+                self.builder, pointer, argvals, fnty.cconv,
+            )
+        return res
 
-            res = impl(self.builder, argvals, self.loc)
+    def _lower_call_RecursiveCall(self, fnty, expr, signature):
+        # Recursive call
+        argvals = self.fold_call_args(
+            fnty, signature, expr.args, expr.vararg, expr.kws,
+        )
+        qualprefix = fnty.overloads[signature.args]
+        mangler = self.context.mangler or default_mangler
+        mangled_name = mangler(qualprefix, signature.args)
+        # special case self recursion
+        if self.builder.function.name.startswith(mangled_name):
+            res = self.context.call_internal(
+                self.builder, self.fndesc, signature, argvals,
+            )
+        else:
+            res = self.context.call_unresolved(
+                self.builder, mangled_name, signature, argvals,
+            )
+        return res
 
-            libs = getattr(impl, "libs", ())
-            for lib in libs:
-                self.library.add_linking_library(lib)
+    def _lower_call_normal(self, fnty, expr, signature):
+        # Normal function resolution
+        self.debug_print("# calling normal function: {0}".format(fnty))
+        self.debug_print("# signature: {0}".format(signature))
+        if (isinstance(expr.func, ir.Intrinsic) or
+                isinstance(fnty, types.ObjModeDispatcher)):
+            argvals = expr.func.args
+        else:
+            argvals = self.fold_call_args(
+                fnty, signature, expr.args, expr.vararg, expr.kws,
+            )
+        impl = self.context.get_function(fnty, signature)
+        if signature.recvr:
+            # The "self" object is passed as the function object
+            # for bounded function
+            the_self = self.loadvar(expr.func.name)
+            # Prepend the self reference
+            argvals = [the_self] + list(argvals)
 
-        return self.context.cast(self.builder, res, signature.return_type,
-                                 resty)
+        res = impl(self.builder, argvals, self.loc)
+        return res
 
     def lower_expr(self, resty, expr):
         if expr.op == 'binop':
@@ -869,9 +984,10 @@ class Lower(BaseLower):
         elif expr.op == 'unary':
             val = self.loadvar(expr.value.name)
             typ = self.typeof(expr.value.name)
+            func_ty = self.context.typing_context.resolve_value_type(expr.fn)
             # Get function
             signature = self.fndesc.calltypes[expr]
-            impl = self.context.get_function(expr.fn, signature)
+            impl = self.context.get_function(func_ty, signature)
             # Convert argument to match
             val = self.context.cast(self.builder, val, typ, signature.args[0])
             res = impl(self.builder, [val])
@@ -989,12 +1105,13 @@ class Lower(BaseLower):
                 resty,
                 self.typeof(expr.value.name),
                 _lit_or_omitted(expr.index),
-                )
+            )
             try:
                 # Both get_function() and the returned implementation can
                 # raise NotImplementedError if the types aren't supported
                 impl = self.context.get_function("static_getitem", signature)
-                return impl(self.builder, (self.loadvar(expr.value.name), expr.index))
+                return impl(self.builder,
+                            (self.loadvar(expr.value.name), expr.index))
             except NotImplementedError:
                 if expr.index_var is None:
                     raise
@@ -1003,7 +1120,15 @@ class Lower(BaseLower):
                 signature = self.fndesc.calltypes[expr]
                 return self.lower_getitem(resty, expr, expr.value,
                                           expr.index_var, signature)
-
+        elif expr.op == "typed_getitem":
+            signature = typing.signature(
+                resty,
+                self.typeof(expr.value.name),
+                self.typeof(expr.index.name),
+            )
+            impl = self.context.get_function("typed_getitem", signature)
+            return impl(self.builder, (self.loadvar(expr.value.name),
+                        self.loadvar(expr.index.name)))
         elif expr.op == "getitem":
             signature = self.fndesc.calltypes[expr]
             return self.lower_getitem(resty, expr, expr.value, expr.index,
@@ -1021,7 +1146,8 @@ class Lower(BaseLower):
         elif expr.op == "build_list":
             itemvals = [self.loadvar(i.name) for i in expr.items]
             itemtys = [self.typeof(i.name) for i in expr.items]
-            castvals = [self.context.cast(self.builder, val, fromty, resty.dtype)
+            castvals = [self.context.cast(self.builder, val, fromty,
+                                          resty.dtype)
                         for val, fromty in zip(itemvals, itemtys)]
             return self.context.build_list(self.builder, resty, castvals)
 
@@ -1030,10 +1156,28 @@ class Lower(BaseLower):
             items = expr.items[::-1]
             itemvals = [self.loadvar(i.name) for i in items]
             itemtys = [self.typeof(i.name) for i in items]
-            castvals = [self.context.cast(self.builder, val, fromty, resty.dtype)
+            castvals = [self.context.cast(self.builder, val, fromty,
+                                          resty.dtype)
                         for val, fromty in zip(itemvals, itemtys)]
             return self.context.build_set(self.builder, resty, castvals)
 
+        elif expr.op == "build_map":
+            items = expr.items
+            keys, values = [], []
+            key_types, value_types = [], []
+            for k, v in items:
+                key = self.loadvar(k.name)
+                keytype = self.typeof(k.name)
+                val = self.loadvar(v.name)
+                valtype = self.typeof(v.name)
+                keys.append(key)
+                values.append(val)
+                key_types.append(keytype)
+                value_types.append(valtype)
+            return self.context.build_map(self.builder, resty,
+                                          list(zip(key_types, value_types)),
+                                          list(zip(keys, values)))
+
         elif expr.op == "cast":
             val = self.loadvar(expr.value.name)
             ty = self.typeof(expr.value.name)
@@ -1087,9 +1231,11 @@ class Lower(BaseLower):
         # Store variable
         ptr = self.getvar(name)
         if value.type != ptr.type.pointee:
-            msg = ("Storing {value.type} to ptr of {ptr.type.pointee} ('{name}'). "
-                   "FE type {fetype}").format(value=value, ptr=ptr,
-                                              fetype=fetype, name=name)
+            msg = ("Storing {value.type} to ptr of {ptr.type.pointee} "
+                   "('{name}'). FE type {fetype}").format(value=value,
+                                                          ptr=ptr,
+                                                          fetype=fetype,
+                                                          name=name)
             raise AssertionError(msg)
 
         self.builder.store(value, ptr)
@@ -1117,7 +1263,8 @@ class Lower(BaseLower):
         # Is user variable?
         is_uservar = not name.startswith('$')
         # Allocate space for variable
-        aptr = cgutils.alloca_once(self.builder, lltype, name=name, zfill=True)
+        aptr = cgutils.alloca_once(self.builder, lltype,
+                                   name=name, zfill=False)
         if is_uservar:
             # Emit debug info for user variable
             sizeof = self.context.get_abi_sizeof(lltype)
