@@ -24,10 +24,10 @@ class RewriteArrayExprs(rewrites.Rewrite):
     rewriting those expressions to a single operation that will expand
     into something similar to a ufunc call.
     '''
-    def __init__(self, pipeline, *args, **kws):
-        super(RewriteArrayExprs, self).__init__(pipeline, *args, **kws)
+    def __init__(self, state, *args, **kws):
+        super(RewriteArrayExprs, self).__init__(state, *args, **kws)
         # Install a lowering hook if we are using this rewrite.
-        special_ops = self.pipeline.targetctx.special_ops
+        special_ops = state.targetctx.special_ops
         if 'arrayexpr' not in special_ops:
             special_ops['arrayexpr'] = _lower_array_expr
 
@@ -367,7 +367,6 @@ def _lower_array_expr(lowerer, expr):
         ast.fix_missing_locations(ast_module)
 
     # 2. Compile the AST module and extract the Python function.
-
     code_obj = compile(ast_module, expr_filename, 'exec')
     six.exec_(code_obj, namespace)
     impl = namespace[expr_name]
@@ -379,6 +378,8 @@ def _lower_array_expr(lowerer, expr):
     outer_sig = expr.ty(*(lowerer.typeof(name) for name in expr_args))
     inner_sig_args = []
     for argty in outer_sig.args:
+        if isinstance(argty, types.Optional):
+            argty = argty.type
         if isinstance(argty, types.Array):
             inner_sig_args.append(argty.dtype)
         else:
