@@ -10,6 +10,7 @@ from functools import reduce
 
 import numpy as np
 
+from numba.six import exec_
 from numba.utils import longint
 from numba.utils import IS_PY3
 from numba.npyufunc.ufuncbuilder import _BaseUFuncBuilder, parse_identity
@@ -17,16 +18,6 @@ from numba import sigutils, types
 from numba.typing import signature
 from numba.npyufunc.sigparse import parse_signature
 
-if IS_PY3:
-    def _exec(codestr, glbls):
-        exec(codestr, glbls)
-else:
-    eval(compile("""
-def _exec(codestr, glbls):
-    exec codestr in glbls
-""",
-                 "<_exec>", "exec"))
-
 
 def _broadcast_axis(a, b):
     """
@@ -106,12 +97,13 @@ class UFuncMechanism(object):
             if isinstance(arg, np.ndarray):
                 self.arrays[i] = arg
             elif self.is_device_array(arg):
-                self.arrays[i] = arg
+                self.arrays[i] = self.as_device_array(arg)
             elif isinstance(arg, (int, longint, float, complex, np.number)):
                 # Is scalar
                 self.scalarpos.append(i)
             else:
-                raise TypeError("argument #%d has invalid type" % (i + 1,))
+                raise TypeError("argument #%d has invalid type of %s" \
+                % (i + 1, type(arg) ))
 
     def _fill_argtypes(self):
         """
@@ -227,6 +219,14 @@ class UFuncMechanism(object):
         """
         return False
 
+    def as_device_array(self, obj):
+        """Convert the `obj` to a device array
+        Override in subclass
+
+        Default implementation is an identity function
+        """
+        return obj
+
     def broadcast_device(self, ary, shape):
         """Handles ondevice broadcasting
 
@@ -259,6 +259,10 @@ class UFuncMechanism(object):
 
         outshape = args[0].shape
 
+        # Adjust output value
+        if out is not None and cr.is_device_array(out):
+            out = cr.as_device_array(out)
+
         def attempt_ravel(a):
             if cr.SUPPORT_DEVICE_SLICING:
                 raise NotImplementedError
@@ -360,7 +364,13 @@ class DeviceVectorize(_BaseUFuncBuilder):
     def __init__(self, func, identity=None, cache=False, targetoptions={}):
         if cache:
             raise TypeError("caching is not supported")
-        assert not targetoptions
+        for opt in targetoptions:
+            if opt == 'nopython':
+                warnings.warn("nopython kwarg for cuda target is redundant",
+                              RuntimeWarning)
+            else:
+                fmt = "cuda vectorize target does not support option: '%s'"
+                raise KeyError(fmt % opt)
         self.py_func = func
         self.identity = parse_identity(identity)
         # { arg_dtype: (return_dtype), cudakernel }
@@ -393,7 +403,7 @@ class DeviceVectorize(_BaseUFuncBuilder):
         corefn, return_type = self._compile_core(devfnsig)
         glbl = self._get_globals(corefn)
         sig = signature(types.void, *([a[:] for a in args] + [return_type[:]]))
-        _exec(kernelsource, glbl)
+        exec_(kernelsource, glbl)
 
         stager = glbl['__vectorized_%s' % funcname]
         kernel = self._compile_kernel(stager, sig)
@@ -462,17 +472,17 @@ class DeviceGUFuncVectorize(_BaseUFuncBuilder):
 
         indims = [len(x) for x in self.inputsig]
         outdims = [len(x) for x in self.outputsig]
+        args, return_type = sigutils.normalize_signature(sig)
 
         funcname = self.py_func.__name__
         src = expand_gufunc_template(self._kernel_template, indims,
-                                     outdims, funcname)
+                                     outdims, funcname, args)
 
         glbls = self._get_globals(sig)
 
-        _exec(src, glbls)
+        exec_(src, glbls)
         fnobj = glbls['__gufunc_{name}'.format(name=funcname)]
 
-        args, return_type = sigutils.normalize_signature(sig)
         outertys = list(_determine_gufunc_outer_types(args, indims + outdims))
         kernel = self._compile_kernel(fnobj, sig=tuple(outertys))
 
@@ -496,17 +506,18 @@ def _determine_gufunc_outer_types(argtys, dims):
             yield types.Array(dtype=at, ndim=1, layout='A')
 
 
-def expand_gufunc_template(template, indims, outdims, funcname):
+def expand_gufunc_template(template, indims, outdims, funcname, argtypes):
     """Expand gufunc source template
     """
     argdims = indims + outdims
     argnames = ["arg{0}".format(i) for i in range(len(argdims))]
     checkedarg = "min({0})".format(', '.join(["{0}.shape[0]".format(a)
                                               for a in argnames]))
-    inputs = [_gen_src_for_indexing(aref, adims, _gen_src_for_input_indexing)
-              for aref, adims in zip(argnames, indims)]
-    outputs = [_gen_src_for_indexing(aref, adims, _gen_src_for_output_indexing)
-               for aref, adims in zip(argnames[len(indims):], outdims)]
+    inputs = [_gen_src_for_indexing(aref, adims, atype)
+              for aref, adims, atype in zip(argnames, indims, argtypes)]
+    outputs = [_gen_src_for_indexing(aref, adims, atype)
+               for aref, adims, atype in zip(argnames[len(indims):], outdims,
+                                             argtypes[len(indims):])]
     argitems = inputs + outputs
     src = template.format(name=funcname, args=', '.join(argnames),
                           checkedarg=checkedarg,
@@ -514,28 +525,23 @@ def expand_gufunc_template(template, indims, outdims, funcname):
     return src
 
 
-def _gen_src_for_indexing(aref, adims, gen_sliced):
-    return "{aref}[{sliced}]".format(aref=aref, sliced=gen_sliced(adims))
+def _gen_src_for_indexing(aref, adims, atype):
+    return "{aref}[{sliced}]".format(aref=aref,
+                                     sliced=_gen_src_index(adims, atype))
 
 
-def _gen_src_for_input_indexing(adims):
+def _gen_src_index(adims, atype):
     if adims > 0:
-        return _gen_src_for_array_indexing(adims)
+        return ','.join(['__tid__'] + [':'] * adims)
+    elif isinstance(atype, types.Array) and atype.ndim - 1 == adims:
+        # Special case for 0-nd in shape-signature but
+        # 1d array in type signature.
+        # Slice it so that the result has the same dimension.
+        return '__tid__:(__tid__ + 1)'
     else:
         return '__tid__'
 
 
-def _gen_src_for_output_indexing(adims):
-    if adims > 0:
-        return _gen_src_for_array_indexing(adims)
-    else:
-        return '__tid__:(__tid__ + 1)'
-
-
-def _gen_src_for_array_indexing(adims):
-    return ','.join(['__tid__'] + [':'] * adims)
-
-
 class GUFuncEngine(object):
     '''Determine how to broadcast and execute a gufunc
     base on input shape and signature
@@ -641,7 +647,8 @@ class GenerializedUFunc(object):
         assert self.engine.nout == 1, "only support single output"
 
     def __call__(self, *args, **kws):
-        callsteps = self._call_steps(args, kws)
+        callsteps = self._call_steps(self.engine.nin, self.engine.nout,
+                                     args, kws)
         callsteps.prepare_inputs()
         indtypes, schedule, outdtype, kernel = self._schedule(
             callsteps.norm_inputs, callsteps.output)
@@ -741,22 +748,38 @@ class GUFuncCallSteps(object):
         '_need_device_conversion',
     ]
 
-    def __init__(self, args, kwargs):
+    def __init__(self, nin, nout, args, kwargs):
+        if nout > 1:
+            raise ValueError('multiple output is not supported')
         self.args = args
         self.kwargs = kwargs
 
+        user_output_is_device = False
         self.output = self.kwargs.get('out')
+        if self.output is not None:
+            user_output_is_device = self.is_device_array(self.output)
+            if user_output_is_device:
+                self.output = self.as_device_array(self.output)
         self._is_device_array = [self.is_device_array(a) for a in self.args]
-        self._need_device_conversion = not any(self._is_device_array)
+        self._need_device_conversion = (not any(self._is_device_array) and
+                                        not user_output_is_device)
 
         # Normalize inputs
         inputs = []
         for a, isdev in zip(self.args, self._is_device_array):
             if isdev:
-                inputs.append(a)
+                inputs.append(self.as_device_array(a))
             else:
-                inputs.append(np.array(a))
-        self.norm_inputs = inputs
+                inputs.append(np.asarray(a))
+        self.norm_inputs = inputs[:nin]
+        # Check if there are extra arguments for outputs.
+        unused_inputs = inputs[nin:]
+        if unused_inputs:
+            if self.output is not None:
+                raise ValueError("cannot specify 'out' as both a positional "
+                                 "and keyword argument")
+            else:
+                [self.output] = unused_inputs
 
     def adjust_input_types(self, indtypes):
         """
@@ -812,6 +835,9 @@ class GUFuncCallSteps(object):
     def is_device_array(self, obj):
         raise NotImplementedError
 
+    def as_device_array(self, obj):
+        return obj
+
     def to_device(self, hostary):
         raise NotImplementedError
 
