@@ -18,7 +18,8 @@ from numba.ir_utils import (add_offset_to_labels, replace_var_names,
                             rename_labels, get_name_var_table, visit_vars_inner,
                             get_definition, guard, find_callname,
                             get_call_table, is_pure, get_np_ufunc_typ,
-                            get_unused_var_name, find_potential_aliases)
+                            get_unused_var_name, find_potential_aliases,
+                            is_const_call)
 from numba.analysis import (compute_use_defs, compute_live_map,
                             compute_dead_maps, compute_cfg_from_blocks)
 from ..typing import signature
@@ -26,9 +27,11 @@ from numba import config
 from numba.targets.cpu import ParallelOptions
 from numba.six import exec_
 from numba.parfor import print_wrapped, ensure_parallel_support
+import types as pytypes
+import operator
 
 import warnings
-from ..errors import ParallelSafetyWarning
+from ..errors import NumbaParallelSafetyWarning
 
 
 def _lower_parfor_parallel(lowerer, parfor):
@@ -93,7 +96,7 @@ def _lower_parfor_parallel(lowerer, parfor):
     parfor_output_arrays = numba.parfor.get_parfor_outputs(
         parfor, parfor.params)
     parfor_redvars, parfor_reddict = numba.parfor.get_parfor_reductions(
-        parfor, parfor.params, lowerer.fndesc.calltypes)
+        lowerer.func_ir, parfor, parfor.params, lowerer.fndesc.calltypes)
 
     # init reduction array allocation here.
     nredvars = len(parfor_redvars)
@@ -293,6 +296,8 @@ def _lower_parfor_parallel(lowerer, parfor):
             name = parfor_redvars[i]
             redarr = redarrs[name]
             redvar_typ = lowerer.fndesc.typemap[name]
+            if config.DEBUG_ARRAY_OPT:
+                print("post-gufunc reduction:", name, redarr, redvar_typ)
 
             if config.DEBUG_ARRAY_OPT_RUNTIME:
                 res_print_str = "res_print"
@@ -334,7 +339,7 @@ def _lower_parfor_parallel(lowerer, parfor):
                 lowerer.lower_inst(init_assign)
 
                 if config.DEBUG_ARRAY_OPT_RUNTIME:
-                    res_print_str = "one_res_print"
+                    res_print_str = "res_print1 for thread " + str(j) + ":"
                     strconsttyp = types.StringLiteral(res_print_str)
                     lhs = ir.Var(scope, mk_unique_var("str_const"), loc)
                     assign_lhs = ir.Assign(value=ir.Const(value=res_print_str, loc=loc),
@@ -342,14 +347,15 @@ def _lower_parfor_parallel(lowerer, parfor):
                     typemap[lhs.name] = strconsttyp
                     lowerer.lower_inst(assign_lhs)
 
-                    res_print = ir.Print(args=[lhs, index_var, oneelem, init_var],
+                    res_print = ir.Print(args=[lhs, index_var, oneelem, init_var, ir.Var(scope, name, loc)],
                                          vararg=None, loc=loc)
                     lowerer.fndesc.calltypes[res_print] = signature(types.none,
                                                              typemap[lhs.name],
                                                              typemap[index_var.name],
                                                              typemap[oneelem.name],
-                                                             typemap[init_var.name])
-                    print("res_print", res_print)
+                                                             typemap[init_var.name],
+                                                             typemap[name])
+                    print("res_print1", res_print)
                     lowerer.lower_inst(res_print)
 
                 # generate code for combining reduction variable with thread output
@@ -364,8 +370,19 @@ def _lower_parfor_parallel(lowerer, parfor):
                     if isinstance(inst, ir.Assign):
                         rhs = inst.value
                         # We probably need to generalize this since it only does substitutions in
-                        # inplace_ginops.
-                        if isinstance(rhs, ir.Expr) and rhs.op == 'inplace_binop' and rhs.rhs.name == init_var.name:
+                        # inplace_binops.
+                        if (isinstance(rhs, ir.Expr) and rhs.op == 'inplace_binop' and
+                            rhs.rhs.name == init_var.name):
+                            if config.DEBUG_ARRAY_OPT:
+                                print("Adding call to reduction", rhs)
+                            if rhs.fn == operator.isub:
+                                rhs.fn = operator.iadd
+                                rhs.immutable_fn = operator.add
+                            if rhs.fn == operator.itruediv or rhs.fn == operator.ifloordiv:
+                                rhs.fn = operator.imul
+                                rhs.immutable_fn = operator.mul
+                            if config.DEBUG_ARRAY_OPT:
+                                print("After changing sub to add or div to mul", rhs)
                             # Get calltype of rhs.
                             ct = lowerer.fndesc.calltypes[rhs]
                             assert(len(ct.args) == 2)
@@ -373,12 +390,38 @@ def _lower_parfor_parallel(lowerer, parfor):
                             ctargs = (ct.args[0], redvar_typ)
                             # Update the signature of the call.
                             ct = ct.replace(args=ctargs)
-                            # Remove so we can re-insrt since calltypes is unique dict.
+                            # Remove so we can re-insert since calltypes is unique dict.
                             lowerer.fndesc.calltypes.pop(rhs)
                             # Add calltype back in for the expr with updated signature.
                             lowerer.fndesc.calltypes[rhs] = ct
                     lowerer.lower_inst(inst)
+                    if isinstance(inst, ir.Assign) and name == inst.target.name:
+                        break
 
+                    if config.DEBUG_ARRAY_OPT_RUNTIME:
+                        res_print_str = "res_print2 for thread " + str(j) + ":"
+                        strconsttyp = types.StringLiteral(res_print_str)
+                        lhs = ir.Var(scope, mk_unique_var("str_const"), loc)
+                        assign_lhs = ir.Assign(value=ir.Const(value=res_print_str, loc=loc),
+                                               target=lhs, loc=loc)
+                        typemap[lhs.name] = strconsttyp
+                        lowerer.lower_inst(assign_lhs)
+
+                        res_print = ir.Print(args=[lhs, index_var, oneelem, init_var, ir.Var(scope, name, loc)],
+                                             vararg=None, loc=loc)
+                        lowerer.fndesc.calltypes[res_print] = signature(types.none,
+                                                                 typemap[lhs.name],
+                                                                 typemap[index_var.name],
+                                                                 typemap[oneelem.name],
+                                                                 typemap[init_var.name],
+                                                                 typemap[name])
+                        print("res_print2", res_print)
+                        lowerer.lower_inst(res_print)
+
+
+        # Cleanup reduction variable
+        for v in redarrs.values():
+            lowerer.lower_inst(ir.Del(v.name, loc=loc))
     # Restore the original typemap of the function that was replaced temporarily at the
     # Beginning of this function.
     lowerer.fndesc.typemap = orig_typemap
@@ -477,39 +520,110 @@ def unwrap_loop_body(loop_body):
     last_label = max(loop_body.keys())
     loop_body[last_label].body = loop_body[last_label].body[:-1]
 
-def compute_def_once_block(block, def_once, def_more, getattr_taken):
+def add_to_def_once_sets(a_def, def_once, def_more):
+    '''If the variable is already defined more than once, do nothing.
+       Else if defined exactly once previously then transition this
+       variable to the defined more than once set (remove it from
+       def_once set and add to def_more set).
+       Else this must be the first time we've seen this variable defined
+       so add to def_once set.
+    '''
+    if a_def in def_more:
+        pass
+    elif a_def in def_once:
+        def_more.add(a_def)
+        def_once.remove(a_def)
+    else:
+        def_once.add(a_def)
+
+def compute_def_once_block(block, def_once, def_more, getattr_taken, typemap, module_assigns):
+    '''Effect changes to the set of variables defined once or more than once
+       for a single block.
+       block - the block to process
+       def_once - set of variable names known to be defined exactly once
+       def_more - set of variable names known to be defined more than once
+       getattr_taken - dict mapping variable name to tuple of object and attribute taken
+       module_assigns - dict mapping variable name to the Global that they came from
+    '''
+    # The only "defs" occur in assignments, so find such instructions.
     assignments = block.find_insts(ir.Assign)
+    # For each assignment...
     for one_assign in assignments:
+        # Get the LHS/target of the assignment.
         a_def = one_assign.target.name
-        if a_def in def_more:
-            pass
-        elif a_def in def_once:
-            def_more.add(a_def)
-            def_once.remove(a_def)
-        else:
-            def_once.add(a_def)
+        # Add variable to def sets.
+        add_to_def_once_sets(a_def, def_once, def_more)
 
         rhs = one_assign.value
+        if isinstance(rhs, ir.Global):
+            # Remember assignments of the form "a = Global(...)"
+            # Is this a module?
+            if isinstance(rhs.value, pytypes.ModuleType):
+                module_assigns[a_def] = rhs.value.__name__
         if isinstance(rhs, ir.Expr) and rhs.op == 'getattr' and rhs.value.name in def_once:
-            getattr_taken[a_def] = rhs.value.name
+            # Remember assignments of the form "a = b.c"
+            getattr_taken[a_def] = (rhs.value.name, rhs.attr)
         if isinstance(rhs, ir.Expr) and rhs.op == 'call' and rhs.func.name in getattr_taken:
-            base_obj = getattr_taken[rhs.func.name]
-            def_more.add(base_obj)
-            def_once.remove(base_obj)
+            # If "a" is being called then lookup the getattr definition of "a"
+            # as above, getting the module variable "b" (base_obj)
+            # and the attribute "c" (base_attr).
+            base_obj, base_attr = getattr_taken[rhs.func.name]
+            if base_obj in module_assigns:
+                # If we know the definition of the module variable then get the module
+                # name from module_assigns.
+                base_mod_name = module_assigns[base_obj]
+                if not is_const_call(base_mod_name, base_attr):
+                    # Calling a method on an object could modify the object and is thus
+                    # like a def of that object.  We call is_const_call to see if this module/attribute
+                    # combination is known to not modify the module state.  If we don't know that
+                    # the combination is safe then we have to assume there could be a modification to
+                    # the module and thus add the module variable as defined more than once.
+                    add_to_def_once_sets(base_obj, def_once, def_more)
+            else:
+                # Assume the worst and say that base_obj could be modified by the call.
+                add_to_def_once_sets(base_obj, def_once, def_more)
+        if isinstance(rhs, ir.Expr) and rhs.op == 'call':
+            # If a mutable object is passed to a function, then it may be changed and
+            # therefore can't be hoisted.
+            # For each argument to the function...
+            for argvar in rhs.args:
+                # Get the argument's type.
+                if isinstance(argvar, ir.Var):
+                    argvar = argvar.name
+                avtype = typemap[argvar]
+                # If that type doesn't have a mutable attribute or it does and it's set to
+                # not mutable then this usage is safe for hoisting.
+                if getattr(avtype, 'mutable', False):
+                    # Here we have a mutable variable passed to a function so add this variable
+                    # to the def lists.
+                    add_to_def_once_sets(argvar, def_once, def_more)
 
-def compute_def_once_internal(loop_body, def_once, def_more, getattr_taken):
+def compute_def_once_internal(loop_body, def_once, def_more, getattr_taken, typemap, module_assigns):
+    '''Compute the set of variables defined exactly once in the given set of blocks
+       and use the given sets for storing which variables are defined once, more than
+       once and which have had a getattr call on them.
+    '''
+    # For each block...
     for label, block in loop_body.items():
-        compute_def_once_block(block, def_once, def_more, getattr_taken)
+        # Scan this block and effect changes to def_once, def_more, and getattr_taken
+        # based on the instructions in that block.
+        compute_def_once_block(block, def_once, def_more, getattr_taken, typemap, module_assigns)
+        # Have to recursively process parfors manually here.
         for inst in block.body:
             if isinstance(inst, parfor.Parfor):
-                compute_def_once_block(inst.init_block, def_once, def_more, getattr_taken)
-                compute_def_once_internal(inst.loop_body, def_once, def_more, getattr_taken)
+                # Recursively compute for the parfor's init block.
+                compute_def_once_block(inst.init_block, def_once, def_more, getattr_taken, typemap, module_assigns)
+                # Recursively compute for the parfor's loop body.
+                compute_def_once_internal(inst.loop_body, def_once, def_more, getattr_taken, typemap, module_assigns)
 
-def compute_def_once(loop_body):
-    def_once = set()
-    def_more = set()
+def compute_def_once(loop_body, typemap):
+    '''Compute the set of variables defined exactly once in the given set of blocks.
+    '''
+    def_once = set()   # set to hold variables defined exactly once
+    def_more = set()   # set to hold variables defined more than once
     getattr_taken = {}
-    compute_def_once_internal(loop_body, def_once, def_more, getattr_taken)
+    module_assigns = {}
+    compute_def_once_internal(loop_body, def_once, def_more, getattr_taken, typemap, module_assigns)
     return def_once
 
 def find_vars(var, varset):
@@ -518,13 +632,21 @@ def find_vars(var, varset):
     return var
 
 def _hoist_internal(inst, dep_on_param, call_table, hoisted, not_hoisted,
-                    typemap):
+                    typemap, stored_arrays):
+    if inst.target.name in stored_arrays:
+        not_hoisted.append((inst, "stored array"))
+        if config.DEBUG_ARRAY_OPT >= 1:
+            print("Instruction", inst, " could not be hoisted because the created array is stored.")
+        return False
+
     uses = set()
     visit_vars_inner(inst.value, find_vars, uses)
     diff = uses.difference(dep_on_param)
+    if config.DEBUG_ARRAY_OPT >= 1:
+        print("_hoist_internal:", inst, "uses:", uses, "diff:", diff)
     if len(diff) == 0 and is_pure(inst.value, None, call_table):
-        if config.DEBUG_ARRAY_OPT == 1:
-            print("Will hoist instruction", inst)
+        if config.DEBUG_ARRAY_OPT >= 1:
+            print("Will hoist instruction", inst, typemap[inst.target.name])
         hoisted.append(inst)
         if not isinstance(typemap[inst.target.name], types.npytypes.Array):
             dep_on_param += [inst.target.name]
@@ -532,57 +654,72 @@ def _hoist_internal(inst, dep_on_param, call_table, hoisted, not_hoisted,
     else:
         if len(diff) > 0:
             not_hoisted.append((inst, "dependency"))
-            if config.DEBUG_ARRAY_OPT == 1:
+            if config.DEBUG_ARRAY_OPT >= 1:
                 print("Instruction", inst, " could not be hoisted because of a dependency.")
         else:
             not_hoisted.append((inst, "not pure"))
-            if config.DEBUG_ARRAY_OPT == 1:
+            if config.DEBUG_ARRAY_OPT >= 1:
                 print("Instruction", inst, " could not be hoisted because it isn't pure.")
     return False
 
-def find_setitems_block(setitems, block):
+def find_setitems_block(setitems, itemsset, block, typemap):
     for inst in block.body:
         if isinstance(inst, ir.StaticSetItem) or isinstance(inst, ir.SetItem):
             setitems.add(inst.target.name)
+            # If we store a non-mutable object into an array then that is safe to hoist.
+            # If the stored object is mutable and you hoist then multiple entries in the
+            # outer array could reference the same object and changing one index would then
+            # change other indices.
+            if getattr(typemap[inst.value.name], "mutable", False):
+                itemsset.add(inst.value.name)
         elif isinstance(inst, parfor.Parfor):
-            find_setitems_block(setitems, inst.init_block)
-            find_setitems_body(setitems, inst.loop_body)
+            find_setitems_block(setitems, itemsset, inst.init_block, typemap)
+            find_setitems_body(setitems, itemsset, inst.loop_body, typemap)
 
-def find_setitems_body(setitems, loop_body):
+def find_setitems_body(setitems, itemsset, loop_body, typemap):
+    """
+      Find the arrays that are written into (goes into setitems) and the
+      mutable objects (mostly arrays) that are written into other arrays
+      (goes into itemsset).
+    """
     for label, block in loop_body.items():
-        find_setitems_block(setitems, block)
+        find_setitems_block(setitems, itemsset, block, typemap)
 
 def hoist(parfor_params, loop_body, typemap, wrapped_blocks):
     dep_on_param = copy.copy(parfor_params)
     hoisted = []
     not_hoisted = []
 
-    def_once = compute_def_once(loop_body)
+    # Compute the set of variable defined exactly once in the loop body.
+    def_once = compute_def_once(loop_body, typemap)
     (call_table, reverse_call_table) = get_call_table(wrapped_blocks)
 
     setitems = set()
-    find_setitems_body(setitems, loop_body)
+    itemsset = set()
+    find_setitems_body(setitems, itemsset, loop_body, typemap)
     dep_on_param = list(set(dep_on_param).difference(setitems))
+    if config.DEBUG_ARRAY_OPT >= 1:
+        print("hoist - def_once:", def_once, "setitems:", setitems, "itemsset:", itemsset, "dep_on_param:", dep_on_param, "parfor_params:", parfor_params)
 
     for label, block in loop_body.items():
         new_block = []
         for inst in block.body:
             if isinstance(inst, ir.Assign) and inst.target.name in def_once:
                 if _hoist_internal(inst, dep_on_param, call_table,
-                                   hoisted, not_hoisted, typemap):
+                                   hoisted, not_hoisted, typemap, itemsset):
                     # don't add this instruction to the block since it is
                     # hoisted
                     continue
             elif isinstance(inst, parfor.Parfor):
                 new_init_block = []
-                if config.DEBUG_ARRAY_OPT == 1:
+                if config.DEBUG_ARRAY_OPT >= 1:
                     print("parfor")
                     inst.dump()
                 for ib_inst in inst.init_block.body:
                     if (isinstance(ib_inst, ir.Assign) and
                         ib_inst.target.name in def_once):
                         if _hoist_internal(ib_inst, dep_on_param, call_table,
-                                           hoisted, not_hoisted, typemap):
+                                           hoisted, not_hoisted, typemap, itemsset):
                             # don't add this instuction to the block since it is hoisted
                             continue
                     new_init_block.append(ib_inst)
@@ -615,6 +752,28 @@ def redarraytype_to_sig(redarraytyp):
     assert isinstance(redarraytyp, types.npytypes.Array)
     return types.npytypes.Array(redarraytyp.dtype, max(1, redarraytyp.ndim - 1), redarraytyp.layout)
 
+def legalize_names_with_typemap(names, typemap):
+    """ We use ir_utils.legalize_names to replace internal IR variable names
+        containing illegal characters (e.g. period) with a legal character
+        (underscore) so as to create legal variable names.
+        The original variable names are in the typemap so we also
+        need to add the legalized name to the typemap as well.
+    """
+    outdict = legalize_names(names)
+    # For each pair in the dict of legalized names...
+    for x, y in outdict.items():
+        # If the name had some legalization change to it...
+        if x != y:
+            # Set the type of the new name the same as the type of the old name.
+            typemap[y] = typemap[x]
+    return outdict
+
+def to_scalar_from_0d(x):
+    if isinstance(x, types.ArrayCompatible):
+        if x.ndim == 0:
+            return x.dtype
+    return x
+
 def _create_gufunc_for_parfor_body(
         lowerer,
         parfor,
@@ -657,7 +816,7 @@ def _create_gufunc_for_parfor_body(
     # Get all parfor reduction vars, and operators.
     typemap = lowerer.fndesc.typemap
     parfor_redvars, parfor_reddict = numba.parfor.get_parfor_reductions(
-        parfor, parfor_params, lowerer.fndesc.calltypes)
+        lowerer.func_ir, parfor, parfor_params, lowerer.fndesc.calltypes)
     # Compute just the parfor inputs as a set difference.
     parfor_inputs = sorted(
         list(
@@ -667,15 +826,13 @@ def _create_gufunc_for_parfor_body(
 
     races = races.difference(set(parfor_redvars))
     for race in races:
-        warnings.warn_explicit("Variable %s used in parallel loop may be written "
-                      "to simultaneously by multiple workers and may result "
-                      "in non-deterministic or unintended results." % race,
-                      ParallelSafetyWarning,
-                      loc.filename,
-                      loc.line)
+        msg = ("Variable %s used in parallel loop may be written "
+               "to simultaneously by multiple workers and may result "
+               "in non-deterministic or unintended results." % race)
+        warnings.warn(NumbaParallelSafetyWarning(msg, loc))
     replace_var_with_array(races, loop_body, typemap, lowerer.fndesc.calltypes)
 
-    if config.DEBUG_ARRAY_OPT == 1:
+    if config.DEBUG_ARRAY_OPT >= 1:
         print("parfor_params = ", parfor_params, " ", type(parfor_params))
         print("parfor_outputs = ", parfor_outputs, " ", type(parfor_outputs))
         print("parfor_inputs = ", parfor_inputs, " ", type(parfor_inputs))
@@ -699,7 +856,7 @@ def _create_gufunc_for_parfor_body(
     # Reorder all the params so that inputs go first then outputs.
     parfor_params = parfor_inputs + parfor_outputs + parfor_redarrs
 
-    if config.DEBUG_ARRAY_OPT == 1:
+    if config.DEBUG_ARRAY_OPT >= 1:
         print("parfor_params = ", parfor_params, " ", type(parfor_params))
         print("loop_indices = ", loop_indices, " ", type(loop_indices))
         print("loop_body = ", loop_body, " ", type(loop_body))
@@ -707,8 +864,8 @@ def _create_gufunc_for_parfor_body(
 
     # Some Var are not legal parameter names so create a dict of potentially illegal
     # param name to guaranteed legal name.
-    param_dict = legalize_names(parfor_params + parfor_redvars)
-    if config.DEBUG_ARRAY_OPT == 1:
+    param_dict = legalize_names_with_typemap(parfor_params + parfor_redvars, typemap)
+    if config.DEBUG_ARRAY_OPT >= 1:
         print(
             "param_dict = ",
             sorted(
@@ -718,10 +875,10 @@ def _create_gufunc_for_parfor_body(
 
     # Some loop_indices are not legal parameter names so create a dict of potentially illegal
     # loop index to guaranteed legal name.
-    ind_dict = legalize_names(loop_indices)
+    ind_dict = legalize_names_with_typemap(loop_indices, typemap)
     # Compute a new list of legal loop index names.
     legal_loop_indices = [ind_dict[v] for v in loop_indices]
-    if config.DEBUG_ARRAY_OPT == 1:
+    if config.DEBUG_ARRAY_OPT >= 1:
         print("ind_dict = ", sorted(ind_dict.items()), " ", type(ind_dict))
         print(
             "legal_loop_indices = ",
@@ -733,7 +890,7 @@ def _create_gufunc_for_parfor_body(
             print("pd type = ", typemap[pd], " ", type(typemap[pd]))
 
     # Get the types of each parameter.
-    param_types = [typemap[v] for v in parfor_params]
+    param_types = [to_scalar_from_0d(typemap[v]) for v in parfor_params]
     # Calculate types of args passed to gufunc.
     func_arg_types = [typemap[v] for v in (parfor_inputs + parfor_outputs)] + parfor_red_arg_types
 
@@ -760,7 +917,7 @@ def _create_gufunc_for_parfor_body(
     loop_body_var_table = get_name_var_table(loop_body)
     sentinel_name = get_unused_var_name("__sentinel__", loop_body_var_table)
 
-    if config.DEBUG_ARRAY_OPT == 1:
+    if config.DEBUG_ARRAY_OPT >= 1:
         print(
             "legal parfor_params = ",
             parfor_params,
@@ -1059,10 +1216,10 @@ def call_parallel_gufunc(lowerer, cres, gu_signature, outer_sig, expr_args, expr
     '''
     context = lowerer.context
     builder = lowerer.builder
-    library = lowerer.library
 
-    from .parallel import (ParallelGUFuncBuilder, build_gufunc_wrapper,
-                           get_thread_count, _launch_threads)
+    from .parallel import (build_gufunc_wrapper,
+                           get_thread_count,
+                           _launch_threads)
 
     if config.DEBUG_ARRAY_OPT:
         print("make_parallel_loop")
@@ -1082,8 +1239,9 @@ def call_parallel_gufunc(lowerer, cres, gu_signature, outer_sig, expr_args, expr
     # These are necessary for build_gufunc_wrapper to find external symbols
     _launch_threads()
 
-    wrapper_ptr, env, wrapper_name = build_gufunc_wrapper(llvm_func, cres, sin,
-                                                          sout, {})
+    info = build_gufunc_wrapper(llvm_func, cres, sin, sout,
+                                cache=False, is_parfors=True)
+    wrapper_name = info.name
     cres.library._ensure_finalized()
 
     if config.DEBUG_ARRAY_OPT:
@@ -1125,8 +1283,7 @@ def call_parallel_gufunc(lowerer, cres, gu_signature, outer_sig, expr_args, expr
     sizeof_intp = context.get_abi_sizeof(intp_t)
 
     # Prepare sched, first pop it out of expr_args, outer_sig, and gu_signature
-    sched_name = expr_args.pop(0)
-    sched_typ = outer_sig.args[0]
+    expr_args.pop(0)
     sched_sig = sin.pop(0)
 
     if config.DEBUG_ARRAY_OPT:
@@ -1346,7 +1503,7 @@ def call_parallel_gufunc(lowerer, cres, gu_signature, outer_sig, expr_args, expr
             sizeof = context.get_abi_sizeof(typ)
             # Set stepsize to the size of that dtype.
             stepsize = context.get_constant(types.intp, sizeof)
-            if red_stride != None:
+            if red_stride is not None:
                 for rs in red_stride:
                     stepsize = builder.mul(stepsize, rs)
         else:
@@ -1363,22 +1520,17 @@ def call_parallel_gufunc(lowerer, cres, gu_signature, outer_sig, expr_args, expr
 
     # ----------------------------------------------------------------------------
     # prepare data
-    data = builder.inttoptr(zero, byte_ptr_t)
+    data = cgutils.get_null_value(byte_ptr_t)
 
     fnty = lc.Type.function(lc.Type.void(), [byte_ptr_ptr_t, intp_ptr_t,
                                              intp_ptr_t, byte_ptr_t])
-    # Use the dynamic address of the kernel so the backend knows this module
-    # cannot be cached.
-    fnptr = cres.target_context.add_dynamic_addr(
-        builder,
-        wrapper_ptr,
-        info="parallel gufunc kernel",
-        )
-    fn = builder.bitcast(fnptr, fnty.as_pointer())
+
+    fn = builder.module.get_or_insert_function(fnty, name=wrapper_name)
+    context.active_code_library.add_linking_library(info.library)
 
     if config.DEBUG_ARRAY_OPT:
         cgutils.printf(builder, "before calling kernel %p\n", fn)
-    result = builder.call(fn, [args, shapes, steps, data])
+    builder.call(fn, [args, shapes, steps, data])
     if config.DEBUG_ARRAY_OPT:
         cgutils.printf(builder, "after calling kernel %p\n", fn)
 
@@ -1387,5 +1539,4 @@ def call_parallel_gufunc(lowerer, cres, gu_signature, outer_sig, expr_args, expr
         only_elem_ptr = builder.gep(rv_arg, [context.get_constant(types.intp, 0)])
         builder.store(builder.load(only_elem_ptr), lowerer.getvar(k))
 
-    scope = init_block.scope
-    loc = init_block.loc
+    context.active_code_library.add_linking_library(cres.library)
