@@ -1,6 +1,7 @@
 typedef struct opaque_thread * thread_pointer;
 
-enum QUEUE_STATE {
+enum QUEUE_STATE
+{
     /*
     The queue has 4 states:
 
@@ -12,10 +13,6 @@ enum QUEUE_STATE {
     IDLE = 0, READY, RUNNING, DONE
 };
 
-/* Launch new thread */
-static
-thread_pointer numba_new_thread(void *worker, void *arg);
-
 /* Launch `count` number of threads and create the associated thread queue.
 Must invoke once before each add_task() is used.
 *Warning* queues memory are leaked at interpreter tear down!
@@ -36,3 +33,24 @@ void synchronize(void);
 /* Signal worker threads that tasks are added and it is ready to run */
 static
 void ready(void);
+
+/* parallel for loop with 1d tiling.
+
+ Args:
+
+ fn - function pointer to the gufunc
+
+ These are from the Python: args, dimensions, steps, data = lfunc.args
+ args - gufunc args
+ dims - gufunc dims
+ steps - gufunc steps
+ data - gufunc data
+
+ inner_ndim - inner dimension of the gufunc
+ array_count - the number of arrays in the signature (Python: len(sig.args) + 1)
+ the +1 is for the output array.
+
+ */
+static void
+parallel_for(void *fn, char **args, size_t *dims, size_t *steps, void *data,\
+             size_t inner_ndim, size_t array_count);
