@@ -6,7 +6,9 @@ import re
 
 import numpy as np
 
-from . import errors, types, config, npdatetime, utils
+from . import errors, types, config, utils
+# re-export
+from numba.cgutils import is_nonelike   # noqa: F401
 
 
 version = tuple(map(int, np.__version__.split('.')[:2]))
@@ -136,17 +138,62 @@ def as_dtype(nbtype):
         letter = _as_dtype_letters[type(nbtype)]
         return np.dtype('%s%d' % (letter, nbtype.count))
     if isinstance(nbtype, types.Record):
-        return nbtype.dtype
+        return as_struct_dtype(nbtype)
     if isinstance(nbtype, types.EnumMember):
         return as_dtype(nbtype.dtype)
     if isinstance(nbtype, types.npytypes.DType):
         return as_dtype(nbtype.dtype)
     if isinstance(nbtype, types.NumberClass):
         return as_dtype(nbtype.dtype)
+    if isinstance(nbtype, types.NestedArray):
+        spec = (as_dtype(nbtype.dtype), tuple(nbtype.shape))
+        return np.dtype(spec)
     raise NotImplementedError("%r cannot be represented as a Numpy dtype"
                               % (nbtype,))
 
 
+def as_struct_dtype(rec):
+    """Convert Numba Record type to NumPy structured dtype
+    """
+    assert isinstance(rec, types.Record)
+    names = []
+    formats = []
+    offsets = []
+    titles = []
+    # Fill the fields if they are not a title.
+    for k, t in rec.members:
+        if not rec.is_title(k):
+            names.append(k)
+            formats.append(as_dtype(t))
+            offsets.append(rec.offset(k))
+            titles.append(rec.fields[k].title)
+
+    fields = {
+        'names': names,
+        'formats': formats,
+        'offsets': offsets,
+        'itemsize': rec.size,
+        'titles': titles,
+    }
+    _check_struct_alignment(rec, fields)
+    return np.dtype(fields, align=rec.aligned)
+
+
+def _check_struct_alignment(rec, fields):
+    """Check alignment compatibility with Numpy"""
+    if rec.aligned:
+        for k, dt in zip(fields['names'], fields['formats']):
+            llvm_align = rec.alignof(k)
+            npy_align = dt.alignment
+            if llvm_align is not None and npy_align != llvm_align:
+                msg = (
+                    'NumPy is using a different alignment ({}) '
+                    'than Numba/LLVM ({}) for {}. '
+                    'This is likely a NumPy bug.'
+                )
+                raise ValueError(msg.format(npy_align, llvm_align, dt))
+
+
 def is_arrayscalar(val):
     return np.dtype(type(val)) in FROM_DTYPE
 
@@ -160,7 +207,8 @@ def map_arrayscalar_type(val):
         try:
             dtype = np.dtype(type(val))
         except TypeError:
-            raise NotImplementedError("no corresponding numpy dtype for %r" % type(val))
+            raise NotImplementedError("no corresponding numpy dtype "
+                                      "for %r" % type(val))
     return from_dtype(dtype)
 
 
@@ -187,12 +235,11 @@ def select_array_wrapper(inputs):
     An index into *inputs* is returned.
     """
     max_prio = float('-inf')
-    selected_input = None
     selected_index = None
     for index, ty in enumerate(inputs):
         # Ties are broken by choosing the first winner, as in Numpy
-        if isinstance(ty, types.ArrayCompatible) and ty.array_priority > max_prio:
-            selected_input = ty
+        if (isinstance(ty, types.ArrayCompatible) and
+                ty.array_priority > max_prio):
             selected_index = index
             max_prio = ty.array_priority
 
@@ -252,7 +299,7 @@ def supported_ufunc_loop(ufunc, loop):
         supported_types = '?bBhHiIlLqQfd'
         # check if all the types involved in the ufunc loop are
         # supported in this mode
-        supported_loop =  all(t in supported_types for t in loop_types)
+        supported_loop = all(t in supported_types for t in loop_types)
 
     return supported_loop
 
@@ -396,15 +443,24 @@ def _is_aligned_struct(struct):
 
 
 def from_struct_dtype(dtype):
+    """Convert a NumPy structured dtype to Numba Record type
+    """
     if dtype.hasobject:
         raise TypeError("Do not support dtype containing object")
 
-    fields = {}
-
+    fields = []
     for name, info in dtype.fields.items():
-        # *info* may have 3 element if it has a "title", which can be ignored
+        # *info* may have 3 element
         [elemdtype, offset] = info[:2]
-        fields[name] = from_dtype(elemdtype), offset
+        title = info[2] if len(info) == 3 else None
+
+        ty = from_dtype(elemdtype)
+        infos = {
+            'type': ty,
+            'offset': offset,
+            'title': title,
+        }
+        fields.append((name, infos))
 
     # Note: dtype.alignment is not consistent.
     #       It is different after passing into a recarray.
@@ -412,7 +468,7 @@ def from_struct_dtype(dtype):
     size = dtype.itemsize
     aligned = _is_aligned_struct(dtype)
 
-    return types.Record(str(dtype.descr), fields, size, aligned, dtype)
+    return types.Record(fields, size, aligned)
 
 
 def _get_bytes_buffer(ptr, nbytes):
@@ -424,6 +480,7 @@ def _get_bytes_buffer(ptr, nbytes):
     arrty = ctypes.c_byte * nbytes
     return arrty.from_address(ptr)
 
+
 def _get_array_from_ptr(ptr, nbytes, dtype):
     return np.frombuffer(_get_bytes_buffer(ptr, nbytes), dtype)
 
@@ -541,3 +598,11 @@ def is_fortran(dims, strides, itemsize):
             return False
         ax += 1
     return True
+
+
+def type_can_asarray(arr):
+    """ Returns True if the type of 'arr' is supported by the Numba `np.asarray`
+    implementation, False otherwise.
+    """
+    ok = (types.Array, types.Sequence, types.Tuple, types.Number, types.Boolean)
+    return isinstance(arr, ok)
