@@ -15,22 +15,17 @@ import logging
 from contextlib import contextmanager
 
 from collections import defaultdict, deque
-from numba.utils import total_ordering
+from functools import total_ordering
 from numba import mviewbuf
 from numba import utils
 from numba import config
 from .error import HsaSupportError, HsaDriverError, HsaApiError
 from . import enums, enums_ext, drvapi
 from numba.utils import longint as long
+from numba.six import Sequence
 import numpy as np
 
 
-if config.PYVERSION >= (3, 3):
-    from collections.abc import Sequence
-else:
-    from collections import Sequence
-
-
 _logger = logging.getLogger(__name__)
 
 
@@ -867,7 +862,7 @@ class Queue(object):
 
         # synchronous if no signal was provided
         if signal is None:
-            _logger.info('wait for sychronous kernel to complete')
+            _logger.info('wait for synchronous kernel to complete')
             timeout = 10
             if not s.wait_until_ne_one(timeout=timeout):
                 msg = "Kernel timed out after {timeout} second"
@@ -897,7 +892,7 @@ class Signal(object):
     def __init__(self, signal_id):
         self._id = signal_id
         self._as_parameter_ = self._id
-        utils.finalize(self, hsa.hsa_signal_destroy, self._id)
+        weakref.finalize(self, hsa.hsa_signal_destroy, self._id)
 
     def load_relaxed(self):
         return hsa.hsa_signal_load_relaxed(self._id)
@@ -990,7 +985,7 @@ class Program(object):
         check_fptr_return(ret)
 
         self._as_parameter_ = self._id
-        utils.finalize(self, self._ftabl.hsa_ext_program_destroy,
+        weakref.finalize(self, self._ftabl.hsa_ext_program_destroy,
                        self._id)
 
     def add_module(self, module):
@@ -1018,7 +1013,7 @@ class CodeObject(object):
     def __init__(self, code_object):
         self._id = code_object
         self._as_parameter_ = self._id
-        utils.finalize(self, hsa.hsa_code_object_destroy, self._id)
+        weakref.finalize(self, hsa.hsa_code_object_destroy, self._id)
 
 
 class Executable(object):
@@ -1030,7 +1025,7 @@ class Executable(object):
                                   ctypes.byref(ex))
         self._id = ex
         self._as_parameter_ = self._id
-        utils.finalize(self, hsa.hsa_executable_destroy, self._id)
+        weakref.finalize(self, hsa.hsa_executable_destroy, self._id)
 
     def load(self, agent, code_object):
         hsa.hsa_executable_load_code_object(self._id, agent._id,
