@@ -34,6 +34,7 @@ NRT_meminfo_new_from_pyobject(void *data, PyObject *ownerobj) {
     return NRT_MemInfo_new(data, dummy_size, pyobject_dtor, ownerobj);
 }
 
+
 /*
  * A Python object wrapping a NRT meminfo.
  */
@@ -43,6 +44,7 @@ typedef struct {
     NRT_MemInfo *meminfo;
 } MemInfoObject;
 
+
 static
 int MemInfo_init(MemInfoObject *self, PyObject *args, PyObject *kwds) {
     static char *keywords[] = {"ptr", NULL};
@@ -218,6 +220,35 @@ static PyTypeObject MemInfoType = {
 };
 
 
+/*
+Return a MemInfo* as a MemInfoObject*
+The NRT reference to the MemInfo is borrowed.
+*/
+NUMBA_EXPORT_FUNC(MemInfoObject*)
+NRT_meminfo_as_pyobject(NRT_MemInfo *meminfo) {
+    MemInfoObject *mi;
+    PyObject *addr;
+
+    addr = PyLong_FromVoidPtr(meminfo);
+    if (!addr) return NULL;
+    mi = (MemInfoObject*)PyObject_CallFunctionObjArgs((PyObject *)&MemInfoType, addr, NULL);
+    Py_DECREF(addr);
+    if (!mi) return NULL;
+    return mi;
+}
+
+
+/*
+Return a MemInfo* from a MemInfoObject*
+A new reference is returned.
+*/
+NUMBA_EXPORT_FUNC(NRT_MemInfo*)
+NRT_meminfo_from_pyobject(MemInfoObject *miobj) {
+    NRT_MemInfo_acquire(miobj->meminfo);
+    return miobj->meminfo;
+}
+
+
 /*
  * Array adaptor code
  */
