@@ -14,7 +14,7 @@ import llvmlite.llvmpy.core as lc
 from llvmlite.llvmpy.core import Type, Constant, LLVMException
 import llvmlite.binding as ll
 
-from numba import types, utils, cgutils, typing, funcdesc, debuginfo
+from numba import types, utils, cgutils, typing, funcdesc, debuginfo, config
 from numba import _dynfunc, _helperlib
 from numba.compiler_lock import global_compiler_lock
 from numba.pythonapi import PythonAPI
@@ -107,7 +107,7 @@ class OverloadSelector(object):
 
     def _match_arglist(self, formal_args, actual_args):
         """
-        Returns True if the the signature is "matching".
+        Returns True if the signature is "matching".
         A formal signature is "matching" if the actual signature matches exactly
         or if the formal signature is a compatible generic signature.
         """
@@ -181,7 +181,7 @@ class BaseContext(object):
 
     Most objects are lowered as plain-old-data structure in the generated
     llvm.  They are passed around by reference (a pointer to the structure).
-    Only POD structure can life across function boundaries by copying the
+    Only POD structure can live across function boundaries by copying the
     data.
     """
     # True if the target requires strict alignment
@@ -197,7 +197,15 @@ class BaseContext(object):
     DIBuilder = debuginfo.DIBuilder
 
     # Bound checking
-    enable_boundcheck = False
+    @property
+    def enable_boundscheck(self):
+        if config.BOUNDSCHECK is not None:
+            return config.BOUNDSCHECK
+        return self._boundscheck
+
+    @enable_boundscheck.setter
+    def enable_boundscheck(self, value):
+        self._boundscheck = value
 
     # NRT
     enable_nrt = False
@@ -215,9 +223,9 @@ class BaseContext(object):
     allow_dynamic_globals = False
 
     # Fast math flags
-    enable_fastmath = False
+    fastmath = False
 
-    # python exceution environment
+    # python execution environment
     environment = None
 
     # the function descriptor
@@ -244,6 +252,8 @@ class BaseContext(object):
         self._pid = None
         self._codelib_stack = []
 
+        self._boundscheck = False
+
         self.data_model_manager = datamodel.default_manager
 
         # Initialize
@@ -261,8 +271,8 @@ class BaseContext(object):
         """
         # Populate built-in registry
         from . import (arraymath, enumimpl, iterators, linalg, numbers,
-                       optional, polynomial, rangeobj, slicing, smartarray,
-                       tupleobj, gdb_hook)
+                       optional, polynomial, rangeobj, slicing, tupleobj,
+                       gdb_hook, hashing, heapq, literal)
         try:
             from . import npdatetime
         except NotImplementedError:
@@ -275,7 +285,7 @@ class BaseContext(object):
 
     def load_additional_registries(self):
         """
-        Load target-specific registries.  Can be overriden by subclasses.
+        Load target-specific registries.  Can be overridden by subclasses.
         """
 
     def mangler(self, name, types):
@@ -285,7 +295,7 @@ class BaseContext(object):
         return funcdesc.default_mangler(name, types)
 
     def get_env_name(self, fndesc):
-        """Get the environment name given a FunctionDescriptior.
+        """Get the environment name given a FunctionDescriptor.
 
         Use this instead of the ``fndesc.env_name`` so that the target-context
         can provide necessary mangling of the symbol to meet ABI requirements.
@@ -335,7 +345,7 @@ class BaseContext(object):
                 raise NameError("unknown option {0!r}".format(k))
             setattr(obj, k, v)
         if obj.codegen() is not self.codegen():
-            # We can't share functions accross different codegens
+            # We can't share functions across different codegens
             obj.cached_internal_func = {}
         return obj
 
@@ -560,7 +570,9 @@ class BaseContext(object):
     def get_generator_impl(self, genty):
         """
         """
-        return self._generators[genty][1]
+        res = self._generators[genty][1]
+        self.add_linking_libs(getattr(res, 'libs', ()))
+        return res
 
     def get_bound_function(self, builder, obj, ty):
         assert self.get_value_type(ty) == obj.type
@@ -704,8 +716,11 @@ class BaseContext(object):
         assert ty is not None
         cav = self.cast(builder, av, at, ty)
         cbv = self.cast(builder, bv, bt, ty)
-        cmpsig = typing.signature(types.boolean, ty, ty)
-        cmpfunc = self.get_function(key, cmpsig)
+        fnty = self.typing_context.resolve_value_type(key)
+        # the sig is homogeneous in the unified casted type
+        cmpsig = fnty.get_call_type(self.typing_context, (ty, ty), {})
+        cmpfunc = self.get_function(fnty, cmpsig)
+        self.add_linking_libs(getattr(cmpfunc, 'libs', ()))
         return cmpfunc(builder, (cav, cbv))
 
     def make_optional_none(self, builder, valtype):
@@ -723,7 +738,9 @@ class BaseContext(object):
         """
         Return the truth value of a value of the given Numba type.
         """
-        impl = self.get_function(bool, typing.signature(types.boolean, typ))
+        fnty = self.typing_context.resolve_value_type(bool)
+        sig = fnty.get_call_type(self.typing_context, (typ,), {})
+        impl = self.get_function(fnty, sig)
         return impl(builder, (val,))
 
     def get_c_value(self, builder, typ, name, dllimport=False):
@@ -815,15 +832,14 @@ class BaseContext(object):
                                             locals=locals)
 
             # Allow inlining the function inside callers.
-            codegen.add_linking_library(cres.library)
+            self.active_code_library.add_linking_library(cres.library)
             return cres
 
     def compile_subroutine(self, builder, impl, sig, locals={}, flags=None,
                            caching=True):
         """
         Compile the function *impl* for the given *sig* (in nopython mode).
-        Return a placeholder object that's callable from another Numba
-        function.
+        Return an instance of CompileResult.
 
         If *caching* evaluates True, the function keeps the compiled function
         for reuse in *.cached_internal_func*.
@@ -841,22 +857,20 @@ class BaseContext(object):
             cres = self._compile_subroutine_no_cache(builder, impl, sig,
                                                      locals=locals,
                                                      flags=flags)
-            lib = cres.library
-            ty = types.NumbaFunction(cres.fndesc, sig)
-            self.cached_internal_func[cache_key] = ty, lib
+            self.cached_internal_func[cache_key] = cres
 
-        ty, lib = self.cached_internal_func[cache_key]
+        cres = self.cached_internal_func[cache_key]
         # Allow inlining the function inside callers.
-        self.active_code_library.add_linking_library(lib)
-        return ty
+        self.active_code_library.add_linking_library(cres.library)
+        return cres
 
     def compile_internal(self, builder, impl, sig, args, locals={}):
         """
         Like compile_subroutine(), but also call the function with the given
         *args*.
         """
-        ty = self.compile_subroutine(builder, impl, sig, locals)
-        return self.call_internal(builder, ty.fndesc, sig, args)
+        cres = self.compile_subroutine(builder, impl, sig, locals)
+        return self.call_internal(builder, cres.fndesc, sig, args)
 
     def call_internal(self, builder, fndesc, sig, args):
         """
@@ -1107,6 +1121,13 @@ class BaseContext(object):
         finally:
             self._codelib_stack.pop()
 
+    def add_linking_libs(self, libs):
+        """Add iterable of linking libraries to the *active_code_library*.
+        """
+        colib = self.active_code_library
+        for lib in libs:
+            colib.add_linking_library(lib)
+
 
 class _wrap_impl(object):
     """
@@ -1116,19 +1137,21 @@ class _wrap_impl(object):
     """
 
     def __init__(self, imp, context, sig):
-        self._imp = _wrap_missing_loc(imp)
+        self._callable = _wrap_missing_loc(imp)
+        self._imp = self._callable()
         self._context = context
         self._sig = sig
 
     def __call__(self, builder, args, loc=None):
-        return self._imp(self._context, builder, self._sig, args, loc=loc)
+        res = self._imp(self._context, builder, self._sig, args, loc=loc)
+        self._context.add_linking_libs(getattr(self, 'libs', ()))
+        return res
 
     def __getattr__(self, item):
         return getattr(self._imp, item)
 
     def __repr__(self):
-        return "<wrapped %s>" % self._imp
-
+        return "<wrapped %s>" % repr(self._callable)
 
 def _has_loc(fn):
     """Does function *fn* take ``loc`` argument?
@@ -1137,27 +1160,36 @@ def _has_loc(fn):
     return 'loc' in sig.parameters
 
 
-def _wrap_missing_loc(fn):
-    """Wrap function for missing ``loc`` keyword argument.
-    Otherwise, return the original *fn*.
-    """
-    if not _has_loc(fn):
-        def wrapper(*args, **kwargs):
-            kwargs.pop('loc')     # drop unused loc
-            return fn(*args, **kwargs)
+class _wrap_missing_loc(object):
 
-        # Copy the following attributes from the wrapped.
-        # Following similar implementation as functools.wraps but
-        # ignore attributes if not available (i.e fix py2.7)
-        attrs = '__name__', 'libs'
-        for attr in attrs:
-            try:
-                val = getattr(fn, attr)
-            except AttributeError:
-                pass
-            else:
-                setattr(wrapper, attr, val)
+    def __init__(self, fn):
+        self.func = fn # store this to help with debug
 
-        return wrapper
-    else:
-        return fn
+    def __call__(self):
+        """Wrap function for missing ``loc`` keyword argument.
+        Otherwise, return the original *fn*.
+        """
+        fn = self.func
+        if not _has_loc(fn):
+            def wrapper(*args, **kwargs):
+                kwargs.pop('loc')     # drop unused loc
+                return fn(*args, **kwargs)
+
+            # Copy the following attributes from the wrapped.
+            # Following similar implementation as functools.wraps but
+            # ignore attributes if not available (i.e fix py2.7)
+            attrs = '__name__', 'libs'
+            for attr in attrs:
+                try:
+                    val = getattr(fn, attr)
+                except AttributeError:
+                    pass
+                else:
+                    setattr(wrapper, attr, val)
+
+            return wrapper
+        else:
+            return fn
+
+    def __repr__(self):
+        return "<wrapped %s>" % self.func
