@@ -11,11 +11,15 @@ from numba.callwrapper import PyCallWrapper
 from .base import BaseContext, PYOBJECT
 from numba import utils, cgutils, types
 from numba.utils import cached_property
-from numba.targets import callconv, codegen, externals, intrinsics, listobj, setobj
+from numba.targets import (
+    callconv, codegen, externals, intrinsics, listobj, setobj, dictimpl,
+)
 from .options import TargetOptions
 from numba.runtime import rtsys
 from numba.compiler_lock import global_compiler_lock
+import numba.entrypoints
 from . import fastmathpass
+from .cpu_options import ParallelOptions, FastMathOptions, InlineOptions
 
 
 # Keep those structures in sync with _dynfunc.c.
@@ -72,6 +76,9 @@ class CPUContext(BaseContext):
         self.install_registry(randomimpl.registry)
         self.install_registry(randomimpl.registry)
 
+        # load 3rd party extensions
+        numba.entrypoints.init_all()
+
     @property
     def target_data(self):
         return self._internal_codegen.target_data
@@ -127,9 +134,15 @@ class CPUContext(BaseContext):
         """
         return setobj.build_set(self, builder, set_type, items)
 
+    def build_map(self, builder, dict_type, item_types, items):
+        from numba import dictobject
+
+        return dictobject.build_map(self, builder, dict_type, item_types, items)
+
+
     def post_lowering(self, mod, library):
-        if self.enable_fastmath:
-            fastmathpass.rewrite_module(mod)
+        if self.fastmath:
+            fastmathpass.rewrite_module(mod, self.fastmath)
 
         if self.is32bit:
             # 32-bit machine needs to replace all 64-bit div/rem to avoid
@@ -186,35 +199,6 @@ class CPUContext(BaseContext):
         return self.get_abi_sizeof(self.get_value_type(aryty))
 
 
-class ParallelOptions(object):
-    """
-    Options for controlling auto parallelization.
-    """
-    def __init__(self, value):
-        if isinstance(value, bool):
-            self.enabled = value
-            self.comprehension = value
-            self.reduction = value
-            self.setitem = value
-            self.numpy = value
-            self.stencil = value
-            self.fusion = value
-            self.prange = value
-        elif isinstance(value, dict):
-            self.enabled = True
-            self.comprehension = value.pop('comprehension', True)
-            self.reduction = value.pop('reduction', True)
-            self.setitem = value.pop('setitem', True)
-            self.numpy = value.pop('numpy', True)
-            self.stencil = value.pop('stencil', True)
-            self.fusion = value.pop('fusion', True)
-            self.prange = value.pop('prange', True)
-            if value:
-                raise NameError("Unrecognized parallel options: %s" % value.keys())
-        else:
-            raise ValueError("Expect parallel option to be either a bool or a dict")
-
-
 # ----------------------------------------------------------------------------
 # TargetOptions
 
@@ -224,14 +208,15 @@ class CPUTargetOptions(TargetOptions):
         "nogil": bool,
         "forceobj": bool,
         "looplift": bool,
-        "boundcheck": bool,
+        "boundscheck": bool,
         "debug": bool,
         "_nrt": bool,
         "no_rewrites": bool,
         "no_cpython_wrapper": bool,
-        "fastmath": bool,
+        "fastmath": FastMathOptions,
         "error_model": str,
         "parallel": ParallelOptions,
+        "inline": InlineOptions,
     }
 
 
