@@ -34,7 +34,7 @@ ll_intp_p = intp_t.as_pointer()
 
 
 # fortran int type, this needs to match the F_INT C declaration in
-# _lapack.c and is present to accomodate potential future 64bit int
+# _lapack.c and is present to accommodate potential future 64bit int
 # based LAPACK use.
 F_INT_nptype = np.int32
 F_INT_nbtype = types.int32
@@ -1502,7 +1502,7 @@ def _system_check_non_empty_impl(a, b):
         return twoD_impl
 
 
-def _lstsq_residual(b, n, rhs):
+def _lstsq_residual(b, n, nrhs):
     """
     Compute the residual from the 'b' scratch space.
     """
@@ -1510,7 +1510,7 @@ def _lstsq_residual(b, n, rhs):
 
 
 @overload(_lstsq_residual)
-def _lstsq_residual_impl(b, n, rhs):
+def _lstsq_residual_impl(b, n, nrhs):
     ndim = b.ndim
     dtype = b.dtype
     real_dtype = np_support.as_dtype(getattr(dtype, "underlying_float", dtype))
@@ -1798,7 +1798,7 @@ def pinv_impl(a, rcond=1.e-15):
         # gain a few % performance increase:
         # * A is destroyed by the SVD algorithm from LAPACK so a copy is
         #   required, this memory is exactly the right size in which to return
-        #   the pseudo-inverse and so can be resued for this purpose.
+        #   the pseudo-inverse and so can be reused for this purpose.
         # * The pseudo-inverse of S can be applied to either V or U^H, this
         #   then leaves a GEMM operation to compute the inverse via either:
         #   A^+ = (V*(S^+))*U^H
@@ -2192,10 +2192,10 @@ def _get_norm_impl(a, ord_flag):
 
         # handle "ord" being "None", must be done separately
         if ord_flag in (None, types.none):
-            def oneD_impl(a, order=None):
+            def oneD_impl(a, ord=None):
                 return _oneD_norm_2(a)
         else:
-            def oneD_impl(a, order=None):
+            def oneD_impl(a, ord=None):
                 n = len(a)
 
                 # Shortcut to handle zero length arrays
@@ -2211,9 +2211,9 @@ def _get_norm_impl(a, ord_flag):
                 # This is the same as for ord=="None" but because
                 # we have to handle "None" specially this condition
                 # is separated
-                if order == 2:
+                if ord == 2:
                     return _oneD_norm_2(a)
-                elif order == np.inf:
+                elif ord == np.inf:
                     # max(abs(a))
                     ret = abs(a[0])
                     for k in range(1, n):
@@ -2222,7 +2222,7 @@ def _get_norm_impl(a, ord_flag):
                             ret = val
                     return ret
 
-                elif order == -np.inf:
+                elif ord == -np.inf:
                     # min(abs(a))
                     ret = abs(a[0])
                     for k in range(1, n):
@@ -2231,7 +2231,7 @@ def _get_norm_impl(a, ord_flag):
                             ret = val
                     return ret
 
-                elif order == 0:
+                elif ord == 0:
                     # sum(a != 0)
                     ret = 0.0
                     for k in range(n):
@@ -2239,7 +2239,7 @@ def _get_norm_impl(a, ord_flag):
                             ret += 1.
                     return ret
 
-                elif order == 1:
+                elif ord == 1:
                     # sum(abs(a))
                     ret = 0.0
                     for k in range(n):
@@ -2250,8 +2250,8 @@ def _get_norm_impl(a, ord_flag):
                     # sum(abs(a)**ord)**(1./ord)
                     ret = 0.0
                     for k in range(n):
-                        ret += abs(a[k])**order
-                    return ret**(1. / order)
+                        ret += abs(a[k])**ord
+                    return ret**(1. / ord)
         return oneD_impl
 
     elif a.ndim == 2:
@@ -2277,7 +2277,7 @@ def _get_norm_impl(a, ord_flag):
 
             # Compute the Frobenius norm, this is the L2,2 induced norm of `A`
             # which is the L2-norm of A.ravel() and so can be computed via BLAS
-            def twoD_impl(a, order=None):
+            def twoD_impl(a, ord=None):
                 n = a.size
                 if n == 0:
                     # reshape() currently doesn't support zero-sized arrays
@@ -2288,7 +2288,7 @@ def _get_norm_impl(a, ord_flag):
             # max value for this dtype
             max_val = np.finfo(np_ret_type.type).max
 
-            def twoD_impl(a, order=None):
+            def twoD_impl(a, ord=None):
                 n = a.shape[-1]
                 m = a.shape[-2]
 
@@ -2299,7 +2299,7 @@ def _get_norm_impl(a, ord_flag):
                 if a.size == 0:
                     return 0.0
 
-                if order == np.inf:
+                if ord == np.inf:
                     # max of sum of abs across rows
                     # max(sum(abs(a)), axis=1)
                     global_max = 0.
@@ -2311,7 +2311,7 @@ def _get_norm_impl(a, ord_flag):
                             global_max = tmp
                     return global_max
 
-                elif order == -np.inf:
+                elif ord == -np.inf:
                     # min of sum of abs across rows
                     # min(sum(abs(a)), axis=1)
                     global_min = max_val
@@ -2322,7 +2322,7 @@ def _get_norm_impl(a, ord_flag):
                         if tmp < global_min:
                             global_min = tmp
                     return global_min
-                elif order == 1:
+                elif ord == 1:
                     # max of sum of abs across cols
                     # max(sum(abs(a)), axis=0)
                     global_max = 0.
@@ -2334,7 +2334,7 @@ def _get_norm_impl(a, ord_flag):
                             global_max = tmp
                     return global_max
 
-                elif order == -1:
+                elif ord == -1:
                     # min of sum of abs across cols
                     # min(sum(abs(a)), axis=0)
                     global_min = max_val
@@ -2348,10 +2348,10 @@ def _get_norm_impl(a, ord_flag):
 
                 # Results via SVD, singular values are sorted on return
                 # by definition.
-                elif order == 2:
+                elif ord == 2:
                     # max SV
                     return _compute_singular_values(a)[0]
-                elif order == -2:
+                elif ord == -2:
                     # min SV
                     return _compute_singular_values(a)[-1]
                 else:
@@ -2377,46 +2377,43 @@ def cond_impl(a, p=None):
 
     _check_linalg_matrix(a, "cond")
 
-    def _get_cond_impl(a, p):
-        # handle the p==None case separately for type inference to work ok
-        if p in (None, types.none):
-            def cond_none_impl(a, p=None):
-                s = _compute_singular_values(a)
-                return s[0] / s[-1]
-            return cond_none_impl
+    def impl(a, p=None):
+        # This is extracted for performance, numpy does approximately:
+        # `condition = norm(a) * norm(inv(a))`
+        # in the cases of `p == 2` or `p ==-2` singular values are used
+        # for computing norms. This costs numpy an svd of `a` then an
+        # inversion of `a` and another svd of `a`.
+        # Below is a different approach, which also gives a more
+        # accurate answer as there is no inversion involved.
+        # Recall that the singular values of an inverted matrix are the
+        # reciprocal of singular values of the original matrix.
+        # Therefore calling `svd(a)` once yields all the information
+        # needed about both `a` and `inv(a)` without the cost or
+        # potential loss of accuracy incurred through inversion.
+        # For the case of `p == 2`, the result is just the ratio of
+        # `largest singular value/smallest singular value`, and for the
+        # case of `p==-2` the result is simply the
+        # `smallest singular value/largest singular value`.
+        # As a result of this, numba accepts non-square matrices as
+        # input when p==+/-2 as well as when p==None.
+        if p == 2 or p == -2 or p is None:
+            s = _compute_singular_values(a)
+            if p == 2 or p is None:
+                r = np.divide(s[0], s[-1])
+            else:
+                r = np.divide(s[-1], s[0])
+        else:  # cases np.inf, -np.inf, 1, -1
+            norm_a = np.linalg.norm(a, p)
+            norm_inv_a = np.linalg.norm(np.linalg.inv(a), p)
+            r = norm_a * norm_inv_a
+        # NumPy uses a NaN mask, if the input has a NaN, it will return NaN,
+        # Numba calls ban NaN through the use of _check_finite_matrix but this
+        # catches cases where NaN occurs through floating point use
+        if np.isnan(r):
+            return np.inf
         else:
-            def cond_not_none_impl(a, p):
-                # This is extracted for performance, numpy does approximately:
-                # `condition = norm(a) * norm(inv(a))`
-                # in the cases of `p == 2` or `p ==-2` singular values are used
-                # for computing norms. This costs numpy an svd of `a` then an
-                # inversion of `a` and another svd of `a`.
-                # Below is a different approach, which also gives a more
-                # accurate answer as there is no inversion involved.
-                # Recall that the singular values of an inverted matrix are the
-                # reciprocal of singular values of the original matrix.
-                # Therefore calling `svd(a)` once yields all the information
-                # needed about both `a` and `inv(a)` without the cost or
-                # potential loss of accuracy incurred through inversion.
-                # For the case of `p == 2`, the result is just the ratio of
-                # `largest singular value/smallest singular value`, and for the
-                # case of `p==-2` the result is simply the
-                # `smallest singular value/largest singular value`.
-                # As a result of this, numba accepts non-square matrices as
-                # input when p==+/-2 as well as when p==None.
-                if p == 2 or p == -2:
-                    s = _compute_singular_values(a)
-                    if p == 2:
-                        return s[0] / s[-1]
-                    else:
-                        return s[-1] / s[0]
-                else:  # cases np.inf, -np.inf, 1, -1
-                    norm_a = np.linalg.norm(a, p)
-                    norm_inv_a = np.linalg.norm(np.linalg.inv(a), p)
-                    return norm_a * norm_inv_a
-            return cond_not_none_impl
-
-    return _get_cond_impl(a, p)
+            return r
+    return impl
 
 
 @register_jitable
@@ -2465,7 +2462,7 @@ def matrix_rank_impl(a, tol=None):
                 return _get_rank_from_singular_values(s, t)
             return _2d_tol_none_impl
         else:
-            def _2d_tol_not_none_impl(a, tol):
+            def _2d_tol_not_none_impl(a, tol=None):
                 s = _compute_singular_values(a)
                 return _get_rank_from_singular_values(s, tol)
             return _2d_tol_not_none_impl
@@ -2485,7 +2482,7 @@ def matrix_rank_impl(a, tol=None):
             # lead to a reported rank of 0 whereas a tol of 1e-15 should lead
             # to a reported rank of 1, numpy reports 1 regardless.
             # The code below replicates the numpy behaviour.
-            def _1d_matrix_rank_impl(a, tol):
+            def _1d_matrix_rank_impl(a, tol=None):
                 for k in range(len(a)):
                     if a[k] != 0.:
                         return 1
@@ -2578,14 +2575,14 @@ def matrix_power_impl(a, n):
 
 
 @overload(np.trace)
-def matrix_trace_impl(a, offset=types.int_):
+def matrix_trace_impl(a, offset=0):
     """
     Computes the trace of an array.
     """
 
     _check_linalg_matrix(a, "trace", la_prefix=False)
 
-    if not isinstance(offset, types.Integer):
+    if not isinstance(offset, (int, types.Integer)):
         raise TypeError("integer argument expected, got %s" % offset)
 
     def matrix_trace_impl(a, offset=0):
@@ -2686,7 +2683,11 @@ else:
 def _kron_normaliser_impl(x):
     # makes x into a 2d array
     if isinstance(x, types.Array):
-        if x.ndim == 2:
+        if x.layout not in ('C', 'F'):
+            raise TypingError("np.linalg.kron only supports 'C' or 'F' layout "
+                              "input arrays. Receieved an input of "
+                              "layout '{}'.".format(x.layout))
+        elif x.ndim == 2:
             @register_jitable
             def nrm_shape(x):
                 xn = x.shape[-1]
