@@ -8,7 +8,7 @@ import operator
 from llvmlite.llvmpy.core import Type, Constant
 import llvmlite.llvmpy.core as lc
 
-from numba import npdatetime, types, cgutils
+from numba import npdatetime, types, cgutils, numpy_support
 from .imputils import lower_builtin, lower_constant, impl_ret_untracked
 from ..utils import IS_PY3
 
@@ -26,44 +26,52 @@ def scale_by_constant(builder, val, factor):
     """
     return builder.mul(val, Constant.int(TIMEDELTA64, factor))
 
+
 def unscale_by_constant(builder, val, factor):
     """
     Divide *val* by the constant *factor*.
     """
     return builder.sdiv(val, Constant.int(TIMEDELTA64, factor))
 
+
 def add_constant(builder, val, const):
     """
     Add constant *const* to *val*.
     """
     return builder.add(val, Constant.int(TIMEDELTA64, const))
 
+
 def scale_timedelta(context, builder, val, srcty, destty):
     """
     Scale the timedelta64 *val* from *srcty* to *destty*
     (both numba.types.NPTimedelta instances)
     """
-    factor = npdatetime.get_timedelta_conversion_factor(srcty.unit, destty.unit)
+    factor = npdatetime.get_timedelta_conversion_factor(
+        srcty.unit, destty.unit)
     if factor is None:
         # This can happen when using explicit output in a ufunc.
         raise NotImplementedError("cannot convert timedelta64 from %r to %r"
                                   % (srcty.unit, destty.unit))
     return scale_by_constant(builder, val, factor)
 
+
 def normalize_timedeltas(context, builder, left, right, leftty, rightty):
     """
     Scale either *left* or *right* to the other's unit, in order to have
     homogeneous units.
     """
-    factor = npdatetime.get_timedelta_conversion_factor(leftty.unit, rightty.unit)
+    factor = npdatetime.get_timedelta_conversion_factor(
+        leftty.unit, rightty.unit)
     if factor is not None:
         return scale_by_constant(builder, left, factor), right
-    factor = npdatetime.get_timedelta_conversion_factor(rightty.unit, leftty.unit)
+    factor = npdatetime.get_timedelta_conversion_factor(
+        rightty.unit, leftty.unit)
     if factor is not None:
         return left, scale_by_constant(builder, right, factor)
     # Typing should not let this happen, except on == and != operators
     raise RuntimeError("cannot normalize %r and %r" % (leftty, rightty))
 
+
 def alloc_timedelta_result(builder, name='ret'):
     """
     Allocate a NaT-initialized datetime64 (or timedelta64) result slot.
@@ -72,6 +80,7 @@ def alloc_timedelta_result(builder, name='ret'):
     builder.store(NAT, ret)
     return ret
 
+
 def alloc_boolean_result(builder, name='ret'):
     """
     Allocate an uninitialized boolean result slot.
@@ -79,12 +88,14 @@ def alloc_boolean_result(builder, name='ret'):
     ret = cgutils.alloca_once(builder, Type.int(1), name=name)
     return ret
 
+
 def is_not_nat(builder, val):
     """
     Return a predicate which is true if *val* is not NaT.
     """
     return builder.icmp(lc.ICMP_NE, val, NAT)
 
+
 def are_not_nat(builder, vals):
     """
     Return a predicate which is true if all of *vals* are not NaT.
@@ -95,6 +106,7 @@ def are_not_nat(builder, vals):
         pred = builder.and_(pred, is_not_nat(builder, val))
     return pred
 
+
 def make_constant_array(vals):
     consts = [Constant.int(TIMEDELTA64, v) for v in vals]
     return Constant.array(TIMEDELTA64, consts)
@@ -120,14 +132,16 @@ def datetime_constant(context, builder, ty, pyval):
 
 @lower_builtin(operator.pos, types.NPTimedelta)
 def timedelta_pos_impl(context, builder, sig, args):
-    res =args[0]
+    res = args[0]
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
+
 @lower_builtin(operator.neg, types.NPTimedelta)
 def timedelta_neg_impl(context, builder, sig, args):
     res = builder.neg(args[0])
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
+
 @lower_builtin(abs, types.NPTimedelta)
 def timedelta_abs_impl(context, builder, sig, args):
     val, = args
@@ -140,6 +154,7 @@ def timedelta_abs_impl(context, builder, sig, args):
     res = builder.load(ret)
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
+
 def timedelta_sign_impl(context, builder, sig, args):
     """
     np.sign(timedelta64)
@@ -148,12 +163,12 @@ def timedelta_sign_impl(context, builder, sig, args):
     ret = alloc_timedelta_result(builder)
     zero = Constant.int(TIMEDELTA64, 0)
     with builder.if_else(builder.icmp(lc.ICMP_SGT, val, zero)
-                        ) as (gt_zero, le_zero):
+                         ) as (gt_zero, le_zero):
         with gt_zero:
             builder.store(Constant.int(TIMEDELTA64, 1), ret)
         with le_zero:
             with builder.if_else(builder.icmp(lc.ICMP_EQ, val, zero)
-                                ) as (eq_zero, lt_zero):
+                                 ) as (eq_zero, lt_zero):
                 with eq_zero:
                     builder.store(Constant.int(TIMEDELTA64, 0), ret)
                 with lt_zero:
@@ -161,6 +176,7 @@ def timedelta_sign_impl(context, builder, sig, args):
     res = builder.load(ret)
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
+
 @lower_builtin(operator.add, *TIMEDELTA_BINOP_SIG)
 @lower_builtin(operator.iadd, *TIMEDELTA_BINOP_SIG)
 def timedelta_add_impl(context, builder, sig, args):
@@ -174,6 +190,7 @@ def timedelta_add_impl(context, builder, sig, args):
     res = builder.load(ret)
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
+
 @lower_builtin(operator.sub, *TIMEDELTA_BINOP_SIG)
 @lower_builtin(operator.isub, *TIMEDELTA_BINOP_SIG)
 def timedelta_sub_impl(context, builder, sig, args):
@@ -211,20 +228,22 @@ def _timedelta_times_number(context, builder, td_arg, td_type,
 @lower_builtin(operator.imul, types.NPTimedelta, types.Float)
 def timedelta_times_number(context, builder, sig, args):
     res = _timedelta_times_number(context, builder,
-                                   args[0], sig.args[0], args[1], sig.args[1],
-                                   sig.return_type)
+                                  args[0], sig.args[0], args[1], sig.args[1],
+                                  sig.return_type)
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
+
 @lower_builtin(operator.mul, types.Integer, types.NPTimedelta)
 @lower_builtin(operator.imul, types.Integer, types.NPTimedelta)
 @lower_builtin(operator.mul, types.Float, types.NPTimedelta)
 @lower_builtin(operator.imul, types.Float, types.NPTimedelta)
 def number_times_timedelta(context, builder, sig, args):
     res = _timedelta_times_number(context, builder,
-                                   args[1], sig.args[1], args[0], sig.args[0],
-                                   sig.return_type)
+                                  args[1], sig.args[1], args[0], sig.args[0],
+                                  sig.return_type)
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
+
 @lower_builtin(operator.truediv, types.NPTimedelta, types.Integer)
 @lower_builtin(operator.itruediv, types.NPTimedelta, types.Integer)
 @lower_builtin(operator.floordiv, types.NPTimedelta, types.Integer)
@@ -249,17 +268,22 @@ def timedelta_over_number(context, builder, sig, args):
             val = builder.sdiv(td_arg, number_arg)
         # The scaling is required for ufuncs np.*divide() with an explicit
         # output in a different unit.
-        val = scale_timedelta(context, builder, val, sig.args[0], sig.return_type)
+        val = scale_timedelta(context, builder, val,
+                              sig.args[0], sig.return_type)
         builder.store(val, ret)
     res = builder.load(ret)
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
 
 if not IS_PY3:
-    lower_builtin(operator.div, types.NPTimedelta, types.Integer)(timedelta_over_number)
-    lower_builtin(operator.idiv, types.NPTimedelta, types.Integer)(timedelta_over_number)
-    lower_builtin(operator.div, types.NPTimedelta, types.Float)(timedelta_over_number)
-    lower_builtin(operator.idiv, types.NPTimedelta, types.Float)(timedelta_over_number)
+    lower_builtin(operator.div, types.NPTimedelta,
+                  types.Integer)(timedelta_over_number)
+    lower_builtin(operator.idiv, types.NPTimedelta,
+                  types.Integer)(timedelta_over_number)
+    lower_builtin(operator.div, types.NPTimedelta,
+                  types.Float)(timedelta_over_number)
+    lower_builtin(operator.idiv, types.NPTimedelta,
+                  types.Float)(timedelta_over_number)
 
 
 @lower_builtin(operator.truediv, *TIMEDELTA_BINOP_SIG)
@@ -282,10 +306,90 @@ def timedelta_over_timedelta(context, builder, sig, args):
 
 if not IS_PY3:
     lower_builtin(operator.div, *TIMEDELTA_BINOP_SIG)(timedelta_over_timedelta)
-    lower_builtin(operator.idiv, *TIMEDELTA_BINOP_SIG)(timedelta_over_timedelta)
+    lower_builtin(operator.idiv, *
+                  TIMEDELTA_BINOP_SIG)(timedelta_over_timedelta)
+
+if numpy_support.version >= (1, 16):
+    # np 1.16 added support for:
+    # * np.floor_divide on mm->q
+    # * np.remainder on mm->m
+
+    @lower_builtin(operator.floordiv, *TIMEDELTA_BINOP_SIG)
+    def timedelta_floor_div_timedelta(context, builder, sig, args):
+        [va, vb] = args
+        [ta, tb] = sig.args
+        ll_ret_type = context.get_value_type(sig.return_type)
+        not_nan = are_not_nat(builder, [va, vb])
+        ret = cgutils.alloca_once(builder, ll_ret_type, name='ret')
+        zero = Constant.int(ll_ret_type, 0)
+        one = Constant.int(ll_ret_type, 1)
+        builder.store(zero, ret)
+        with cgutils.if_likely(builder, not_nan):
+            va, vb = normalize_timedeltas(context, builder, va, vb, ta, tb)
+            # is the denominator zero or NaT?
+            denom_ok = builder.not_(builder.icmp_signed('==', vb, zero))
+            with cgutils.if_likely(builder, denom_ok):
+                # is either arg negative?
+                vaneg = builder.icmp_signed('<', va, zero)
+                neg = builder.or_(vaneg, builder.icmp_signed('<', vb, zero))
+                with builder.if_else(neg) as (then, otherwise):
+                    with then:  # one or more value negative
+                        with builder.if_else(vaneg) as (negthen, negotherwise):
+                            with negthen:
+                                top = builder.sub(va, one)
+                                div = builder.sdiv(top, vb)
+                                builder.store(div, ret)
+                            with negotherwise:
+                                top = builder.add(va, one)
+                                div = builder.sdiv(top, vb)
+                                builder.store(div, ret)
+                    with otherwise:
+                        div = builder.sdiv(va, vb)
+                        builder.store(div, ret)
+        res = builder.load(ret)
+        return impl_ret_untracked(context, builder, sig.return_type, res)
+
+    if not IS_PY3:
+        lower_builtin(operator.idiv, *
+                      TIMEDELTA_BINOP_SIG)(timedelta_floor_div_timedelta)
+
+    def timedelta_mod_timedelta(context, builder, sig, args):
+        # inspired by https://github.com/numpy/numpy/blob/fe8072a12d65e43bd2e0b0f9ad67ab0108cc54b3/numpy/core/src/umath/loops.c.src#L1424
+        # alg is basically as `a % b`:
+        # if a or b is NaT return NaT
+        # elseif b is 0 return NaT
+        # else pretend a and b are int and do pythonic int modulus
+
+        [va, vb] = args
+        [ta, tb] = sig.args
+        not_nan = are_not_nat(builder, [va, vb])
+        ll_ret_type = context.get_value_type(sig.return_type)
+        ret = alloc_timedelta_result(builder)
+        builder.store(NAT, ret)
+        zero = Constant.int(ll_ret_type, 0)
+        with cgutils.if_likely(builder, not_nan):
+            va, vb = normalize_timedeltas(context, builder, va, vb, ta, tb)
+            # is the denominator zero or NaT?
+            denom_ok = builder.not_(builder.icmp_signed('==', vb, zero))
+            with cgutils.if_likely(builder, denom_ok):
+                # is either arg negative?
+                vapos = builder.icmp_signed('>', va, zero)
+                vbpos = builder.icmp_signed('>', vb, zero)
+                rem = builder.srem(va, vb)
+                cond = builder.or_(builder.and_(vapos, vbpos),
+                                   builder.icmp_signed('==', rem, zero))
+                with builder.if_else(cond) as (then, otherwise):
+                    with then:
+                        builder.store(rem, ret)
+                    with otherwise:
+                        builder.store(builder.add(rem, vb), ret)
+
+        res = builder.load(ret)
+        return impl_ret_untracked(context, builder, sig.return_type, res)
 
 # Comparison operators on timedelta64
 
+
 def _create_timedelta_comparison_impl(ll_op, default_value):
     def impl(context, builder, sig, args):
         [va, vb] = args
@@ -294,15 +398,24 @@ def _create_timedelta_comparison_impl(ll_op, default_value):
         with builder.if_else(are_not_nat(builder, [va, vb])) as (then, otherwise):
             with then:
                 try:
-                    norm_a, norm_b = normalize_timedeltas(context, builder, va, vb, ta, tb)
+                    norm_a, norm_b = normalize_timedeltas(
+                        context, builder, va, vb, ta, tb)
                 except RuntimeError:
                     # Cannot normalize units => the values are unequal (except if NaT)
                     builder.store(default_value, ret)
                 else:
                     builder.store(builder.icmp(ll_op, norm_a, norm_b), ret)
             with otherwise:
-                # No scaling when comparing NaTs
-                builder.store(builder.icmp(ll_op, va, vb), ret)
+                if numpy_support.version < (1, 16):
+                    # No scaling when comparing NaTs
+                    builder.store(builder.icmp(ll_op, va, vb), ret)
+                else:
+                    # NumPy >= 1.16 switched to NaT ==/>=/>/</<= NaT being
+                    # False and NaT != <anything, including NaT> being True
+                    if ll_op == lc.ICMP_NE:
+                        builder.store(cgutils.true_bit, ret)
+                    else:
+                        builder.store(cgutils.false_bit, ret)
         res = builder.load(ret)
         return impl_ret_untracked(context, builder, sig.return_type, res)
 
@@ -316,32 +429,39 @@ def _create_timedelta_ordering_impl(ll_op):
         ret = alloc_boolean_result(builder)
         with builder.if_else(are_not_nat(builder, [va, vb])) as (then, otherwise):
             with then:
-                norm_a, norm_b = normalize_timedeltas(context, builder, va, vb, ta, tb)
+                norm_a, norm_b = normalize_timedeltas(
+                    context, builder, va, vb, ta, tb)
                 builder.store(builder.icmp(ll_op, norm_a, norm_b), ret)
             with otherwise:
-                # No scaling when comparing NaT with something else
-                # (i.e. NaT is <= everything else, since it's the smallest
-                #  int64 value)
-                builder.store(builder.icmp(ll_op, va, vb), ret)
+                if numpy_support.version < (1, 16):
+                    # No scaling when comparing NaT with something else
+                    # (i.e. NaT is <= everything else, since it's the smallest
+                    #  int64 value)
+                    builder.store(builder.icmp(ll_op, va, vb), ret)
+                else:
+                    # NumPy >= 1.16 switched to NaT >=/>/</<= NaT being False
+                    builder.store(cgutils.false_bit, ret)
         res = builder.load(ret)
         return impl_ret_untracked(context, builder, sig.return_type, res)
 
     return impl
 
 
-timedelta_eq_timedelta_impl = _create_timedelta_comparison_impl(lc.ICMP_EQ, cgutils.false_bit)
-timedelta_ne_timedelta_impl = _create_timedelta_comparison_impl(lc.ICMP_NE, cgutils.true_bit)
+timedelta_eq_timedelta_impl = _create_timedelta_comparison_impl(
+    lc.ICMP_EQ, cgutils.false_bit)
+timedelta_ne_timedelta_impl = _create_timedelta_comparison_impl(
+    lc.ICMP_NE, cgutils.true_bit)
 timedelta_lt_timedelta_impl = _create_timedelta_ordering_impl(lc.ICMP_SLT)
 timedelta_le_timedelta_impl = _create_timedelta_ordering_impl(lc.ICMP_SLE)
 timedelta_gt_timedelta_impl = _create_timedelta_ordering_impl(lc.ICMP_SGT)
 timedelta_ge_timedelta_impl = _create_timedelta_ordering_impl(lc.ICMP_SGE)
 
 for op_, func in [(operator.eq, timedelta_eq_timedelta_impl),
-                 (operator.ne, timedelta_ne_timedelta_impl),
-                 (operator.lt,  timedelta_lt_timedelta_impl),
-                 (operator.le, timedelta_le_timedelta_impl),
-                 (operator.gt,  timedelta_gt_timedelta_impl),
-                 (operator.ge, timedelta_ge_timedelta_impl)]:
+                  (operator.ne, timedelta_ne_timedelta_impl),
+                  (operator.lt,  timedelta_lt_timedelta_impl),
+                  (operator.le, timedelta_le_timedelta_impl),
+                  (operator.gt,  timedelta_gt_timedelta_impl),
+                  (operator.ge, timedelta_ge_timedelta_impl)]:
     lower_builtin(op_, *TIMEDELTA_BINOP_SIG)(func)
 
 
@@ -362,6 +482,7 @@ def is_leap_year(builder, year_val):
     return builder.and_(multiple_of_4,
                         builder.or_(not_multiple_of_100, multiple_of_400))
 
+
 def year_to_days(builder, year_val):
     """
     Given a year *year_val* (offset to 1970), return the number of days
@@ -374,7 +495,7 @@ def year_to_days(builder, year_val):
     days = scale_by_constant(builder, year_val, 365)
     # Adjust for leap years
     with builder.if_else(cgutils.is_neg_int(builder, year_val)) \
-        as (if_neg, if_pos):
+            as (if_neg, if_pos):
         with if_pos:
             # At or after 1970:
             # 1968 is the closest leap year before 1970.
@@ -466,7 +587,8 @@ def convert_datetime_for_arith(builder, dt_val, src_unit, dest_unit):
     Convert datetime *dt_val* from *src_unit* to *dest_unit*.
     """
     # First partial conversion to days or weeks, if necessary.
-    dt_val, dt_unit = reduce_datetime_for_unit(builder, dt_val, src_unit, dest_unit)
+    dt_val, dt_unit = reduce_datetime_for_unit(
+        builder, dt_val, src_unit, dest_unit)
     # Then multiply by the remaining constant factor.
     dt_factor = npdatetime.get_timedelta_conversion_factor(dt_unit, dest_unit)
     if dt_factor is None:
@@ -483,55 +605,61 @@ def _datetime_timedelta_arith(ll_op_name):
         with cgutils.if_likely(builder, are_not_nat(builder, [dt_arg, td_arg])):
             dt_arg = convert_datetime_for_arith(builder, dt_arg,
                                                 dt_unit, ret_unit)
-            td_factor = npdatetime.get_timedelta_conversion_factor(td_unit, ret_unit)
+            td_factor = npdatetime.get_timedelta_conversion_factor(
+                td_unit, ret_unit)
             td_arg = scale_by_constant(builder, td_arg, td_factor)
             ret_val = getattr(builder, ll_op_name)(dt_arg, td_arg)
             builder.store(ret_val, ret)
         return builder.load(ret)
     return impl
 
+
 _datetime_plus_timedelta = _datetime_timedelta_arith('add')
 _datetime_minus_timedelta = _datetime_timedelta_arith('sub')
 
 # datetime64 + timedelta64
 
+
 @lower_builtin(operator.add, types.NPDatetime, types.NPTimedelta)
 @lower_builtin(operator.iadd, types.NPDatetime, types.NPTimedelta)
 def datetime_plus_timedelta(context, builder, sig, args):
     dt_arg, td_arg = args
     dt_type, td_type = sig.args
     res = _datetime_plus_timedelta(context, builder,
-                                    dt_arg, dt_type.unit,
-                                    td_arg, td_type.unit,
-                                    sig.return_type.unit)
+                                   dt_arg, dt_type.unit,
+                                   td_arg, td_type.unit,
+                                   sig.return_type.unit)
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
+
 @lower_builtin(operator.add, types.NPTimedelta, types.NPDatetime)
 @lower_builtin(operator.iadd, types.NPTimedelta, types.NPDatetime)
 def timedelta_plus_datetime(context, builder, sig, args):
     td_arg, dt_arg = args
     td_type, dt_type = sig.args
     res = _datetime_plus_timedelta(context, builder,
-                                    dt_arg, dt_type.unit,
-                                    td_arg, td_type.unit,
-                                    sig.return_type.unit)
+                                   dt_arg, dt_type.unit,
+                                   td_arg, td_type.unit,
+                                   sig.return_type.unit)
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
 # datetime64 - timedelta64
 
+
 @lower_builtin(operator.sub, types.NPDatetime, types.NPTimedelta)
 @lower_builtin(operator.isub, types.NPDatetime, types.NPTimedelta)
 def datetime_minus_timedelta(context, builder, sig, args):
     dt_arg, td_arg = args
     dt_type, td_type = sig.args
     res = _datetime_minus_timedelta(context, builder,
-                                     dt_arg, dt_type.unit,
-                                     td_arg, td_type.unit,
-                                     sig.return_type.unit)
+                                    dt_arg, dt_type.unit,
+                                    td_arg, td_type.unit,
+                                    sig.return_type.unit)
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
 # datetime64 - datetime64
 
+
 @lower_builtin(operator.sub, types.NPDatetime, types.NPDatetime)
 def datetime_minus_datetime(context, builder, sig, args):
     va, vb = args
@@ -550,6 +678,7 @@ def datetime_minus_datetime(context, builder, sig, args):
 
 # datetime64 comparisons
 
+
 def _create_datetime_comparison_impl(ll_op):
     def impl(context, builder, sig, args):
         va, vb = args
@@ -560,13 +689,21 @@ def _create_datetime_comparison_impl(ll_op):
         ret = alloc_boolean_result(builder)
         with builder.if_else(are_not_nat(builder, [va, vb])) as (then, otherwise):
             with then:
-                norm_a = convert_datetime_for_arith(builder, va, unit_a, ret_unit)
-                norm_b = convert_datetime_for_arith(builder, vb, unit_b, ret_unit)
+                norm_a = convert_datetime_for_arith(
+                    builder, va, unit_a, ret_unit)
+                norm_b = convert_datetime_for_arith(
+                    builder, vb, unit_b, ret_unit)
                 ret_val = builder.icmp(ll_op, norm_a, norm_b)
                 builder.store(ret_val, ret)
             with otherwise:
-                # No scaling when comparing NaTs
-                ret_val = builder.icmp(ll_op, va, vb)
+                if numpy_support.version < (1, 16):
+                    # No scaling when comparing NaTs
+                    ret_val = builder.icmp(ll_op, va, vb)
+                else:
+                    if ll_op == lc.ICMP_NE:
+                        ret_val = cgutils.true_bit
+                    else:
+                        ret_val = cgutils.false_bit
                 builder.store(ret_val, ret)
         res = builder.load(ret)
         return impl_ret_untracked(context, builder, sig.return_type, res)
