@@ -136,7 +136,8 @@ class _ArrayHelper(namedtuple('_ArrayHelper', ('context', 'builder',
         return _ArrayIndexingHelper(self, indices)
 
     def _load_effective_address(self, indices):
-        return cgutils.get_item_pointer2(self.builder,
+        return cgutils.get_item_pointer2(self.context,
+                                         self.builder,
                                          data=self.data,
                                          shape=self.shape,
                                          strides=self.strides,
@@ -161,6 +162,14 @@ def _prepare_argument(ctxt, bld, inp, tyinp, where='input operand'):
     _ScalarHelper or _ArrayHelper) class to handle the argument.
     using the polymorphic interface of the Helper classes, scalar
     and array cases can be handled with the same code"""
+
+    # first un-Optional Optionals
+    if isinstance(tyinp, types.Optional):
+        oty = tyinp
+        tyinp = tyinp.type
+        inp = ctxt.cast(bld, inp, oty, tyinp)
+
+    # then prepare the arg for a concrete instance
     if isinstance(tyinp, types.ArrayCompatible):
         ary     = ctxt.make_array(tyinp)(ctxt, bld, inp)
         shape   = cgutils.unpack_tuple(bld, ary.shape, tyinp.ndim)
@@ -487,7 +496,7 @@ def register_binary_operator_kernel(op, kernel, inplace=False):
         # The visible signature is (A, B) -> A
         # The implementation's signature (with explicit output)
         # is (A, B, A) -> A
-        args = args + (args[0],)
+        args = tuple(args) + (args[0],)
         sig = typing.signature(sig.return_type, *sig.args + (sig.args[0],))
         return numpy_ufunc_kernel(context, builder, sig, args, kernel,
                                   explicit_output=True)
