@@ -44,15 +44,15 @@ class TargetOptions(object):
         if kws.pop('looplift', True):
             flags.set("enable_looplift")
 
-        if kws.pop('boundcheck', False):
-            flags.set("boundcheck")
+        if kws.pop('boundscheck', False):
+            flags.set("boundscheck")
 
         if kws.pop('_nrt', True):
             flags.set("nrt")
 
         if kws.pop('debug', config.DEBUGINFO_DEFAULT):
             flags.set("debuginfo")
-            flags.set("boundcheck")
+            flags.set("boundscheck")
 
         if kws.pop('nogil', False):
             flags.set("release_gil")
@@ -66,15 +66,17 @@ class TargetOptions(object):
         if 'parallel' in kws:
             flags.set('auto_parallel', kws.pop('parallel'))
 
-        if kws.pop('fastmath', False):
-            flags.set('fastmath')
+        if 'fastmath' in kws:
+            flags.set('fastmath', kws.pop('fastmath'))
 
         if 'error_model' in kws:
             flags.set('error_model', kws.pop('error_model'))
 
+        if 'inline' in kws:
+            flags.set('inline', kws.pop('inline'))
+
         flags.set("enable_pyobject_looplift")
 
         if kws:
             # Unread options?
             raise NameError("Unrecognized options: %s" % kws.keys())
-
