@@ -14,7 +14,7 @@ from numba import types, cgutils, typing
 from numba.targets.imputils import (lower_builtin, lower_cast,
                                     iternext_impl, impl_ret_borrowed,
                                     impl_ret_new_ref, impl_ret_untracked,
-                                    for_iter, call_len)
+                                    for_iter, call_len, RefType)
 from numba.utils import cached_property
 from . import quicksort, slicing
 
@@ -58,8 +58,10 @@ def get_hash_value(context, builder, typ, value):
     """
     Compute the hash of the given value.
     """
-    sig = typing.signature(types.intp, typ)
-    fn = context.get_function(hash, sig)
+    typingctx = context.typing_context
+    fnty = typingctx.resolve_value_type(hash)
+    sig = fnty.get_call_type(typingctx, (typ,), {})
+    fn = context.get_function(fnty, sig)
     h = fn(builder, (value,))
     # Fixup reserved values
     is_ok = is_hash_used(context, builder, h)
@@ -474,6 +476,56 @@ class SetInstance(object):
         h = get_hash_value(context, builder, self._ty.dtype, item)
         self._add_key(payload, item, h, do_resize)
 
+    def add_pyapi(self, pyapi, item, do_resize=True):
+        """A version of .add for use inside functions following Python calling
+        convention.
+        """
+        context = self._context
+        builder = self._builder
+
+        payload = self.payload
+        h = self._pyapi_get_hash_value(pyapi, context, builder, item)
+        self._add_key(payload, item, h, do_resize)
+
+    def _pyapi_get_hash_value(self, pyapi, context, builder, item):
+        """Python API compatible version of `get_hash_value()`.
+        """
+        def emit_wrapper(resty, argtypes):
+            # Because `get_hash_value()` could raise a nopython exception,
+            # we need to wrap it in a function that has nopython
+            # calling convention.
+
+            fnty = context.call_conv.get_function_type(resty, argtypes)
+            mod = builder.module
+            fn = ir.Function(
+                mod, fnty, name=mod.get_unique_name('.set_hash_item'),
+            )
+            fn.linkage = 'internal'
+            inner_builder = ir.IRBuilder(fn.append_basic_block())
+            [inner_item] = context.call_conv.decode_arguments(
+                inner_builder, argtypes, fn,
+            )
+            # Call get_hash_value()
+            h = get_hash_value(
+                context, inner_builder, self._ty.dtype, inner_item,
+            )
+            context.call_conv.return_value(inner_builder, h)
+            return fn
+
+        argtypes = [self._ty.dtype]
+        resty = types.intp
+        fn = emit_wrapper(resty, argtypes)
+        # Call wrapper function
+        status, retval = context.call_conv.call_function(
+            builder, fn, resty, argtypes, [item],
+        )
+        # Handle return status
+        with builder.if_then(builder.not_(status.is_ok), likely=False):
+            # Raise nopython exception as a Python exception
+            context.call_conv.raise_error(builder, pyapi, status)
+            builder.ret(pyapi.get_null_object())
+        return retval
+
     def contains(self, item):
         context = self._context
         builder = self._builder
@@ -1163,7 +1215,7 @@ def getiter_set(context, builder, sig, args):
     return impl_ret_borrowed(context, builder, sig.return_type, inst.value)
 
 @lower_builtin('iternext', types.SetIter)
-@iternext_impl
+@iternext_impl(RefType.BORROWED)
 def iternext_listiter(context, builder, sig, args, result):
     inst = SetIterInstance(context, builder, sig.args[0], args[0])
     inst.iternext(result)
