@@ -8,15 +8,23 @@ import operator
 
 from .imputils import (lower_builtin, lower_getattr_generic, lower_cast,
                        lower_constant,
-                       iternext_impl, impl_ret_borrowed, impl_ret_untracked)
+                       iternext_impl, impl_ret_borrowed, impl_ret_untracked,
+                       RefType)
 from .. import typing, types, cgutils
-from ..extending import overload_method
+from ..extending import overload_method, overload, intrinsic
 
 
 @lower_builtin(types.NamedTupleClass, types.VarArg(types.Any))
 def namedtuple_constructor(context, builder, sig, args):
     # A namedtuple has the same representation as a regular tuple
-    res = context.make_tuple(builder, sig.return_type, args)
+    # the arguments need casting (lower_cast) from the types in the ctor args
+    # to those in the ctor return type, this is to handle cases such as a
+    # literal present in the args, but a type present in the return type.
+    newargs = []
+    for i, arg in enumerate(args):
+        casted = context.cast(builder, arg, sig.args[i], sig.return_type[i])
+        newargs.append(casted)
+    res = context.make_tuple(builder, sig.return_type, tuple(newargs))
     # The tuple's contents are borrowed
     return impl_ret_borrowed(context, builder, sig.return_type, res)
 
@@ -48,6 +56,7 @@ def tuple_cmp_ordered(context, builder, op, sig, args):
     builder.position_at_end(bbend)
     return builder.load(res)
 
+
 @lower_builtin(operator.eq, types.BaseTuple, types.BaseTuple)
 def tuple_eq(context, builder, sig, args):
     tu, tv = sig.args
@@ -88,30 +97,7 @@ def tuple_ge(context, builder, sig, args):
     res = tuple_cmp_ordered(context, builder, operator.ge, sig, args)
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
-@lower_builtin(hash, types.BaseTuple)
-def hash_tuple(context, builder, sig, args):
-    tupty, = sig.args
-    tup, = args
-    lty = context.get_value_type(sig.return_type)
-
-    h = ir.Constant(lty, 0x345678)
-    mult = ir.Constant(lty, 1000003)
-    n = ir.Constant(lty, len(tupty))
-
-    for i, ty in enumerate(tupty.types):
-        # h = h * mult
-        h = builder.mul(h, mult)
-        val = builder.extract_value(tup, i)
-        hash_impl = context.get_function(hash,
-                                         typing.signature(sig.return_type, ty))
-        h_val = hash_impl(builder, (val,))
-        # h = h ^ hash(val)
-        h = builder.xor(h, h_val)
-        # Perturb: mult = mult + len(tup)
-        mult = builder.add(mult, n)
-
-    return h
-
+# for hashing see hashing.py
 
 @lower_getattr_generic(types.BaseNamedTuple)
 def namedtuple_getattr(context, builder, typ, value, attr):
@@ -131,7 +117,9 @@ def unituple_constant(context, builder, ty, pyval):
     """
     consts = [context.get_constant_generic(builder, ty.dtype, v)
               for v in pyval]
-    return ir.ArrayType(consts[0].type, len(consts))(consts)
+    return impl_ret_borrowed(
+        context, builder, ty, cgutils.pack_array(builder, consts),
+    )
 
 @lower_constant(types.Tuple)
 @lower_constant(types.NamedTuple)
@@ -141,7 +129,9 @@ def unituple_constant(context, builder, ty, pyval):
     """
     consts = [context.get_constant_generic(builder, ty.types[i], v)
               for i, v in enumerate(pyval)]
-    return ir.Constant.literal_struct(consts)
+    return impl_ret_borrowed(
+        context, builder, ty, cgutils.pack_struct(builder, consts),
+    )
 
 
 #------------------------------------------------------------------------------
@@ -167,7 +157,7 @@ def getiter_unituple(context, builder, sig, args):
 
 
 @lower_builtin('iternext', types.UniTupleIter)
-@iternext_impl
+@iternext_impl(RefType.BORROWED)
 def iternext_unituple(context, builder, sig, args, result):
     [tupiterty] = sig.args
     [tupiter] = args
@@ -188,13 +178,104 @@ def iternext_unituple(context, builder, sig, args, result):
                                        types.intp)
         getitem_out = getitem_unituple(context, builder, getitem_sig,
                                        [tup, idx])
+        # As a iternext_impl function, this will incref the yieled value.
+        # We need to release the new reference from getitem_unituple.
+        if context.enable_nrt:
+            context.nrt.decref(builder, tupiterty.container.dtype, getitem_out)
         result.yield_(getitem_out)
         nidx = builder.add(idx, context.get_constant(types.intp, 1))
         builder.store(nidx, iterval.index)
 
 
+@lower_builtin('typed_getitem', types.BaseTuple, types.Any)
+def getitem_typed(context, builder, sig, args):
+    tupty, _ = sig.args
+    tup, idx = args
+    errmsg_oob = ("tuple index out of range",)
+
+    if len(tupty) == 0:
+        # Empty tuple.
+
+        # Always branch and raise IndexError
+        with builder.if_then(cgutils.true_bit):
+            context.call_conv.return_user_exc(builder, IndexError,
+                                              errmsg_oob)
+        # This is unreachable in runtime,
+        # but it exists to not terminate the current basicblock.
+        res = context.get_constant_null(sig.return_type)
+        return impl_ret_untracked(context, builder,
+                                  sig.return_type, res)
+    else:
+        # The tuple is not empty
+
+        bbelse = builder.append_basic_block("typed_switch.else")
+        bbend = builder.append_basic_block("typed_switch.end")
+        switch = builder.switch(idx, bbelse)
+
+        with builder.goto_block(bbelse):
+            context.call_conv.return_user_exc(builder, IndexError,
+                                            errmsg_oob)
+
+        lrtty = context.get_value_type(sig.return_type)
+        voidptrty = context.get_value_type(types.voidptr)
+        with builder.goto_block(bbend):
+            phinode = builder.phi(voidptrty)
+
+        for i in range(tupty.count):
+            ki = context.get_constant(types.intp, i)
+            bbi = builder.append_basic_block("typed_switch.%d" % i)
+            switch.add_case(ki, bbi)
+            # handle negative indexing, create case (-tuple.count + i) to
+            # reference same block as i
+            kin = context.get_constant(types.intp, -tupty.count + i)
+            switch.add_case(kin, bbi)
+            with builder.goto_block(bbi):
+                value = builder.extract_value(tup, i)
+                # Dragon warning...
+                # The fact the code has made it this far suggests that type
+                # inference decided whatever was being done with the item pulled
+                # from the tuple was legitimate, it is not the job of lowering
+                # to argue about that. However, here lies a problem, the tuple
+                # lowering is implemented as a switch table with each case
+                # writing to a phi node slot that is returned. The type of this
+                # phi node slot needs to be "correct" for the current type but
+                # it also needs to survive stores being made to it from the
+                # other cases that will in effect never run. To do this a stack
+                # slot is made for each case for the specific type and then cast
+                # to a void pointer type, this is then added as an incoming on
+                # the phi node, at the end of the switch the phi node is then
+                # cast back to the required return type for this typed_getitem.
+                # The only further complication is that if the value is not a
+                # pointer then the void* juggle won't work so a cast is made
+                # prior to store, again, that type inference has permitted it
+                # suggests this is safe.
+                # End Dragon warning...
+                DOCAST = context.typing_context.unify_types(sig.args[0][i],
+                                        sig.return_type) == sig.return_type
+                if DOCAST:
+                    value_slot = builder.alloca(lrtty,
+                                                name="TYPED_VALUE_SLOT%s" % i)
+                    casted = context.cast(builder, value, sig.args[0][i],
+                                        sig.return_type)
+                    builder.store(casted, value_slot)
+                else:
+                    value_slot = builder.alloca(value.type,
+                                                name="TYPED_VALUE_SLOT%s" % i)
+                    builder.store(value, value_slot)
+                phinode.add_incoming(builder.bitcast(value_slot, voidptrty),
+                                     bbi)
+                builder.branch(bbend)
+
+        builder.position_at_end(bbend)
+        res = builder.bitcast(phinode, lrtty.as_pointer())
+        res = builder.load(res)
+        return impl_ret_borrowed(context, builder, sig.return_type, res)
+
+
 @lower_builtin(operator.getitem, types.UniTuple, types.intp)
+@lower_builtin(operator.getitem, types.UniTuple, types.uintp)
 @lower_builtin(operator.getitem, types.NamedUniTuple, types.intp)
+@lower_builtin(operator.getitem, types.NamedUniTuple, types.uintp)
 def getitem_unituple(context, builder, sig, args):
     tupty, _ = sig.args
     tup, idx = args
