@@ -1,6 +1,7 @@
 """
 Assorted utilities for use in tests.
 """
+from __future__ import print_function
 
 import cmath
 import contextlib
@@ -14,14 +15,24 @@ import subprocess
 import sys
 import tempfile
 import time
+import io
+import ctypes
+import multiprocessing as mp
+import warnings
+from contextlib import contextmanager
 
 import numpy as np
+try:
+    import scipy
+except ImportError:
+    scipy = None
 
 from numba import config, errors, typing, utils, numpy_support, testing
 from numba.compiler import compile_extra, compile_isolated, Flags, DEFAULT_FLAGS
 from numba.targets import cpu
 import numba.unittest_support as unittest
 from numba.runtime import rtsys
+from numba.six import PY2
 
 
 enable_pyobj_flags = Flags()
@@ -43,6 +54,21 @@ _windows_py27 = (sys.platform.startswith('win32') and
 _32bit = sys.maxsize <= 2 ** 32
 _reason = 'parfors not supported'
 skip_parfors_unsupported = unittest.skipIf(_32bit or _windows_py27, _reason)
+skip_py38_or_later = unittest.skipIf(
+    utils.PYVERSION >= (3, 8),
+    "unsupported on py3.8 or later"
+)
+skip_tryexcept_unsupported = unittest.skipIf(
+    utils.PYVERSION < (3, 7),
+    "try-except unsupported on py3.6 or earlier"
+)
+skip_tryexcept_supported = unittest.skipIf(
+    utils.PYVERSION >= (3, 7),
+    "try-except supported on py3.7 or later"
+)
+
+_msg = "SciPy needed for test"
+skip_unless_scipy = unittest.skipIf(scipy is None, _msg)
 
 
 class CompilationCache(object):
@@ -145,7 +171,8 @@ class TestCase(unittest.TestCase):
 
 
     _bool_types = (bool, np.bool_)
-    _exact_typesets = [_bool_types, utils.INT_TYPES, (str,), (np.integer,), (utils.text_type), ]
+    _exact_typesets = [_bool_types, utils.INT_TYPES, (str,), (np.integer,),
+                       (utils.text_type), (bytes, np.bytes_)]
     _approx_typesets = [(float,), (complex,), (np.inexact)]
     _sequence_typesets = [(tuple, list)]
     _float_types = (float, np.floating)
@@ -397,6 +424,12 @@ class TestCase(unittest.TestCase):
         if isinstance(first, self._complex_types):
             _assertNumberEqual(first.real, second.real, delta)
             _assertNumberEqual(first.imag, second.imag, delta)
+        elif isinstance(first, (np.timedelta64, np.datetime64)):
+            # Since Np 1.16 NaT == NaT is False, so special comparison needed
+            if numpy_support.version >= (1, 16) and np.isnat(first):
+                self.assertEqual(np.isnat(first), np.isnat(second))
+            else:
+                _assertNumberEqual(first, second, delta)
         else:
             _assertNumberEqual(first, second, delta)
 
@@ -413,6 +446,14 @@ class TestCase(unittest.TestCase):
         self.assertPreciseEqual(got, expected)
         return got, expected
 
+    if PY2:
+        @contextmanager
+        def subTest(self, *args, **kwargs):
+            """A stub TestCase.subTest backport.
+            This implementation is a no-op.
+            """
+            yield
+
 
 class SerialMixin(object):
     """Mixin to mark test for serial execution.
@@ -481,7 +522,14 @@ def tweak_code(func, codestring=None, consts=None):
         codestring = co.co_code
     if consts is None:
         consts = co.co_consts
-    if sys.version_info >= (3,):
+    if sys.version_info >= (3, 8):
+        new_code = tp(co.co_argcount, co.co_posonlyargcount,
+                      co.co_kwonlyargcount, co.co_nlocals,
+                      co.co_stacksize, co.co_flags, codestring,
+                      consts, co.co_names, co.co_varnames,
+                      co.co_filename, co.co_name, co.co_firstlineno,
+                      co.co_lnotab)
+    elif sys.version_info >= (3,):
         new_code = tp(co.co_argcount, co.co_kwonlyargcount, co.co_nlocals,
                       co.co_stacksize, co.co_flags, codestring,
                       consts, co.co_names, co.co_varnames,
@@ -674,3 +722,103 @@ def forbid_codegen():
         for (obj, attrname), value in old.items():
             setattr(obj, attrname, value)
 
+
+# For details about redirection of file-descriptor, read
+# https://eli.thegreenplace.net/2015/redirecting-all-kinds-of-stdout-in-python/
+
+@contextlib.contextmanager
+def redirect_fd(fd):
+    """
+    Temporarily redirect *fd* to a pipe's write end and return a file object
+    wrapping the pipe's read end.
+    """
+
+    from numba import _helperlib
+    libnumba = ctypes.CDLL(_helperlib.__file__)
+
+    libnumba._numba_flush_stdout()
+    save = os.dup(fd)
+    r, w = os.pipe()
+    try:
+        os.dup2(w, fd)
+        yield io.open(r, "r")
+    finally:
+        libnumba._numba_flush_stdout()
+        os.close(w)
+        os.dup2(save, fd)
+        os.close(save)
+
+
+def redirect_c_stdout():
+    """Redirect C stdout
+    """
+    fd = sys.__stdout__.fileno()
+    return redirect_fd(fd)
+
+
+def run_in_new_process_caching(func, cache_dir_prefix=__name__, verbose=True):
+    """Spawn a new process to run `func` with a temporary cache directory.
+
+    The childprocess's stdout and stderr will be captured and redirected to
+    the current process's stdout and stderr.
+
+    Returns
+    -------
+    ret : dict
+        exitcode: 0 for success. 1 for exception-raised.
+        stdout: str
+        stderr: str
+    """
+    ctx = mp.get_context('spawn')
+    qout = ctx.Queue()
+    cache_dir = temp_directory(cache_dir_prefix)
+    with override_env_config('NUMBA_CACHE_DIR', cache_dir):
+        proc = ctx.Process(target=_remote_runner, args=[func, qout])
+        proc.start()
+        proc.join()
+        stdout = qout.get_nowait()
+        stderr = qout.get_nowait()
+        if verbose and stdout.strip():
+            print()
+            print('STDOUT'.center(80, '-'))
+            print(stdout)
+        if verbose and stderr.strip():
+            print(file=sys.stderr)
+            print('STDERR'.center(80, '-'), file=sys.stderr)
+            print(stderr, file=sys.stderr)
+    return {
+        'exitcode': proc.exitcode,
+        'stdout': stdout,
+        'stderr': stderr,
+    }
+
+
+def _remote_runner(fn, qout):
+    """Used by `run_in_new_process_caching()`
+    """
+    with captured_stderr() as stderr:
+        with captured_stdout() as stdout:
+            try:
+                fn()
+            except Exception:
+                traceback.print_exc()
+                exitcode = 1
+            else:
+                exitcode = 0
+        qout.put(stdout.getvalue())
+    qout.put(stderr.getvalue())
+    sys.exit(exitcode)
+
+class CheckWarningsMixin(object):
+    @contextlib.contextmanager
+    def check_warnings(self, messages, category=RuntimeWarning):
+        with warnings.catch_warnings(record=True) as catch:
+            warnings.simplefilter("always")
+            yield
+        found = 0
+        for w in catch:
+            for m in messages:
+                if m in str(w.message):
+                    self.assertEqual(w.category, category)
+                    found += 1
+        self.assertEqual(found, len(messages))
