@@ -7,13 +7,23 @@ import sys
 from collections import namedtuple
 
 from numba import unittest_support as unittest
-from numba import njit, typeof, types, typing, typeof, ir, utils, bytecode
+from numba import (njit, typeof, types, typing, typeof, ir, utils, bytecode,
+    jitclass, prange, postproc)
 from .support import TestCase, tag
 from numba.array_analysis import EquivSet, ArrayAnalysis
-from numba.compiler import Pipeline, Flags, _PipelineManager
+from numba.compiler import Compiler, Flags, PassManager
 from numba.targets import cpu, registry
 from numba.numpy_support import version as numpy_version
 from numba.ir_utils import remove_dead
+from numba.untyped_passes import (ExtractByteCode, TranslateByteCode, FixupArgs,
+                             IRProcessing, DeadBranchPrune,
+                             RewriteSemanticConstants, GenericRewrites,
+                             WithLifting, PreserveIR, InlineClosureLikes)
+
+from numba.typed_passes import (NopythonTypeInference, AnnotateTypes,
+                                NopythonRewrites, IRLegalization)
+
+from numba.compiler_machinery import FunctionPass, PassManager, register_pass
 
 # for parallel tests, marking that Windows with Python 2.7 is not supported
 _windows_py27 = (sys.platform.startswith('win32') and
@@ -22,6 +32,15 @@ _32bit = sys.maxsize <= 2 ** 32
 _reason = 'parfors not supported'
 skip_unsupported = unittest.skipIf(_32bit or _windows_py27, _reason)
 
+
+# test class for #3700
+@jitclass([('L', types.int32), ('T', types.int32)])
+class ExampleClass3700(object):
+    def __init__(self, n):
+        self.L = n
+        self.T = n + 1
+
+
 class TestEquivSet(TestCase):
 
     """
@@ -63,7 +82,27 @@ class TestEquivSet(TestCase):
         self.assertFalse(r.is_equiv('c', 'd'))
 
 
-class ArrayAnalysisTester(Pipeline):
+@register_pass(analysis_only=False, mutates_CFG=True)
+class ArrayAnalysisPass(FunctionPass):
+    _name = "array_analysis_pass"
+
+    def __init__(self):
+        FunctionPass.__init__(self)
+
+    def run_pass(self, state):
+        state.array_analysis = ArrayAnalysis(state.typingctx, state.func_ir,
+                                             state.type_annotation.typemap,
+                                             state.type_annotation.calltypes)
+        state.array_analysis.run(state.func_ir.blocks)
+        post_proc = postproc.PostProcessor(state.func_ir)
+        post_proc.run()
+        state.func_ir_copies.append(state.func_ir.copy())
+        if state.test_idempotence and len(state.func_ir_copies) > 1:
+            state.test_idempotence(state.func_ir_copies)
+        return False
+
+
+class ArrayAnalysisTester(Compiler):
 
     @classmethod
     def mk_pipeline(cls, args, return_type=None, flags=None, locals={},
@@ -82,53 +121,46 @@ class ArrayAnalysisTester(Pipeline):
         """
         Populate and run compiler pipeline
         """
-        self.func_id = bytecode.FunctionIdentity.from_function(func)
+        self.state.func_id = bytecode.FunctionIdentity.from_function(func)
+        ExtractByteCode().run_pass(self.state)
 
-        try:
-            bc = self.extract_bytecode(self.func_id)
-        except BaseException as e:
-            raise e
+        self.state.lifted = ()
+        self.state.lifted_from = None
+        state = self.state
+        state.func_ir_copies = []
+        state.test_idempotence = test_idempotence
 
-        self.bc = bc
-        self.lifted = ()
-        self.lifted_from = None
+        name = 'array_analysis_testing'
+        pm = PassManager(name)
+        pm.add_pass(TranslateByteCode, "analyzing bytecode")
+        pm.add_pass(FixupArgs, "fix up args")
+        pm.add_pass(IRProcessing, "processing IR")
+        # pre typing
+        if not state.flags.no_rewrites:
+            pm.add_pass(GenericRewrites, "nopython rewrites")
+            pm.add_pass(RewriteSemanticConstants, "rewrite semantic constants")
+            pm.add_pass(DeadBranchPrune, "dead branch pruning")
+        pm.add_pass(InlineClosureLikes,
+                    "inline calls to locally defined closures")
+        # typing
+        pm.add_pass(NopythonTypeInference, "nopython frontend")
+        pm.add_pass(AnnotateTypes, "annotate types")
 
-        pm = _PipelineManager()
+        if not state.flags.no_rewrites:
+            pm.add_pass(NopythonRewrites, "nopython rewrites")
 
-        pm.create_pipeline("nopython")
-        if self.func_ir is None:
-            pm.add_stage(self.stage_analyze_bytecode, "analyzing bytecode")
-        pm.add_stage(self.stage_process_ir, "processing IR")
-        if not self.flags.no_rewrites:
-            if self.status.can_fallback:
-                pm.add_stage(
-                    self.stage_preserve_ir, "preserve IR for fallback")
-            pm.add_stage(self.stage_generic_rewrites, "nopython rewrites")
-        pm.add_stage(
-            self.stage_inline_pass, "inline calls to locally defined closures")
-        pm.add_stage(self.stage_nopython_frontend, "nopython frontend")
-        pm.add_stage(self.stage_annotate_type, "annotate type")
-        if not self.flags.no_rewrites:
-            pm.add_stage(self.stage_nopython_rewrites, "nopython rewrites")
-        func_ir_copies = []
-
-        def stage_array_analysis():
-            self.array_analysis = ArrayAnalysis(self.typingctx, self.func_ir,
-                                                self.type_annotation.typemap,
-                                                self.type_annotation.calltypes)
-            self.array_analysis.run(self.func_ir.blocks)
-            func_ir_copies.append(self.func_ir.copy())
-            if test_idempotence and len(func_ir_copies) > 1:
-                test_idempotence(func_ir_copies)
-
-        pm.add_stage(stage_array_analysis, "analyze array equivalences")
+        # Array Analysis pass
+        pm.add_pass(ArrayAnalysisPass, "array analysis")
         if test_idempotence:
             # Do another pass of array analysis to test idempotence
-            pm.add_stage(stage_array_analysis, "analyze array equivalences")
+            pm.add_pass(ArrayAnalysisPass, "idempotence array analysis")
+        # legalise
+        pm.add_pass(IRLegalization, "ensure IR is legal prior to lowering")
 
+        # partial compile
         pm.finalize()
-        res = pm.run(self.status)
-        return self.array_analysis
+        pm.run(state)
+        return state.array_analysis
 
 
 class TestArrayAnalysis(TestCase):
@@ -327,6 +359,29 @@ class TestArrayAnalysis(TestCase):
                                asserts=[self.with_assert('A', 'B'),
                                         self.without_assert('C', 'D')])
 
+        def test_11():
+            a = np.ones(5)
+            b = np.ones(5)
+            c = a[1:]
+            d = b[:-1]
+            e = len(c)
+            f = len(d)
+            return e == f
+        self._compile_and_test(test_11, (),
+                               equivs=[self.with_equiv('e', 'f')])
+
+        def test_12():
+            a = np.ones(25).reshape((5,5))
+            b = np.ones(25).reshape((5,5))
+            c = a[1:,:]
+            d = b[:-1,:]
+            e = c.shape[0]
+            f = d.shape[0]
+            g = len(d)
+            return e == f
+        self._compile_and_test(test_12, (),
+                               equivs=[self.with_equiv('e', 'f', 'g')])
+
         def test_tup_arg(T):
             T2 = T
             return T2[0]
@@ -524,7 +579,7 @@ class TestArrayAnalysis(TestCase):
             E = B + C
             return E
         self._compile_and_test(test_7, (types.intp,),
-                               asserts=[self.without_assert('B', 'C')],
+                               asserts=[self.with_assert('B', 'C')],
                                idempotent=False)
 
         def test_8(m):
@@ -552,7 +607,8 @@ class TestArrayAnalysis(TestCase):
                                equivs=[self.without_equiv('B', 'C'),
                                        self.with_equiv('A', 'm'),
                                        self.with_equiv('B', 'D'),
-                                       self.with_equiv('F', 'D'),],)
+                                       self.with_equiv('F', 'D'),],
+                               idempotent=False)
 
     def test_numpy_calls(self):
         def test_zeros(n):
@@ -904,5 +960,54 @@ class TestArrayAnalysisParallelRequired(TestCase):
 
         self.assertEqual(njit(test_impl, parallel=True)(), test_impl())
 
+    @skip_unsupported
+    def test_array_T_issue_3700(self):
+
+        def test_impl(t_obj, X):
+            for i in prange(t_obj.T):
+                X[i] = i
+            return X.sum()
+
+        n = 5
+        t_obj = ExampleClass3700(n)
+        X1 = np.zeros(t_obj.T)
+        X2 = np.zeros(t_obj.T)
+        self.assertEqual(
+            njit(test_impl, parallel=True)(t_obj, X1), test_impl(t_obj, X2))
+
+    @skip_unsupported
+    def test_slice_shape_issue_3380(self):
+        # these tests shouldn't throw error in array analysis
+        def test_impl1():
+            a = slice(None, None)
+            return True
+
+        self.assertEqual(njit(test_impl1, parallel=True)(), test_impl1())
+
+        def test_impl2(A, a):
+            b = a
+            return A[b]
+
+        A = np.arange(10)
+        a = slice(None)
+        np.testing.assert_array_equal(
+            njit(test_impl2, parallel=True)(A, a), test_impl2(A, a))
+
+    @skip_unsupported
+    def test_slice_dtype_issue_5056(self):
+        # see issue 5056
+
+        @njit(parallel=True)
+        def test_impl(data):
+            N = data.shape[0]
+            sums = np.zeros(N)
+            for i in prange(N):
+                sums[i] = np.sum(data[np.int32(0):np.int32(1)])
+            return sums
+
+        data = np.arange(10.)
+        np.testing.assert_array_equal(test_impl(data), test_impl.py_func(data))
+
+
 if __name__ == '__main__':
     unittest.main()
