@@ -6,8 +6,8 @@ import numpy as np
 
 from numba import njit, vectorize
 from numba import unittest_support as unittest
-from numba import compiler, typing, typeof, ir, utils
-from numba.compiler import Pipeline, _PipelineManager, Flags
+from numba import compiler, typing, typeof, ir, utils, types
+from numba.compiler import Compiler, Flags
 from numba.targets import cpu
 from .support import MemoryLeakMixin, TestCase
 
@@ -75,7 +75,7 @@ def distance_matrix(vectors):
     return result
 
 
-class RewritesTester(Pipeline):
+class RewritesTester(Compiler):
     @classmethod
     def mk_pipeline(cls, args, return_type=None, flags=None, locals={},
                     library=None, typing_context=None, target_context=None):
@@ -137,9 +137,9 @@ class TestArrayExpressions(MemoryLeakMixin, TestCase):
         np.testing.assert_array_equal(expected, actual)
         np.testing.assert_array_equal(control, actual)
 
-        ir0 = control_pipeline.func_ir.blocks
-        ir1 = test_pipeline.func_ir.blocks
-        ir2 = control_pipeline2.func_ir.blocks
+        ir0 = control_pipeline.state.func_ir.blocks
+        ir1 = test_pipeline.state.func_ir.blocks
+        ir2 = control_pipeline2.state.func_ir.blocks
         self.assertEqual(len(ir0), len(ir1))
         self.assertEqual(len(ir0), len(ir2))
         # The rewritten IR should be smaller than the original.
@@ -282,8 +282,8 @@ class TestArrayExpressions(MemoryLeakMixin, TestCase):
         scalar optimizations such as rewriting `x ** 2`.
         """
         ns = self._test_cube_function()
-        self._assert_total_rewrite(ns.control_pipeline.func_ir.blocks,
-                                   ns.test_pipeline.func_ir.blocks,
+        self._assert_total_rewrite(ns.control_pipeline.state.func_ir.blocks,
+                                   ns.test_pipeline.state.func_ir.blocks,
                                    trivial=True)
 
     def test_complicated_expr(self):
@@ -293,8 +293,8 @@ class TestArrayExpressions(MemoryLeakMixin, TestCase):
         array expressions.
         '''
         ns = self._test_root_function()
-        self._assert_total_rewrite(ns.control_pipeline.func_ir.blocks,
-                                   ns.test_pipeline.func_ir.blocks)
+        self._assert_total_rewrite(ns.control_pipeline.state.func_ir.blocks,
+                                   ns.test_pipeline.state.func_ir.blocks)
 
     def test_common_subexpressions(self, fn=neg_root_common_subexpr):
         '''
@@ -302,8 +302,8 @@ class TestArrayExpressions(MemoryLeakMixin, TestCase):
         subexpressions properly.
         '''
         ns = self._test_root_function(fn)
-        ir0 = ns.control_pipeline.func_ir.blocks
-        ir1 = ns.test_pipeline.func_ir.blocks
+        ir0 = ns.control_pipeline.state.func_ir.blocks
+        ir1 = ns.test_pipeline.state.func_ir.blocks
         self.assertEqual(len(ir0), len(ir1))
         self.assertGreater(len(ir0[0].body), len(ir1[0].body))
         self.assertEqual(len(list(self._get_array_exprs(ir0[0].body))), 0)
@@ -356,24 +356,24 @@ class TestArrayExpressions(MemoryLeakMixin, TestCase):
         np.testing.assert_array_almost_equal(expected, control)
         np.testing.assert_array_almost_equal(expected, actual)
 
-        self._assert_total_rewrite(control_pipeline.func_ir.blocks,
-                                   test_pipeline.func_ir.blocks)
+        self._assert_total_rewrite(control_pipeline.state.func_ir.blocks,
+                                   test_pipeline.state.func_ir.blocks)
 
     def test_cmp_op(self):
         '''
         Verify that comparison operators are supported by the rewriter.
         '''
         ns = self._test_root_function(are_roots_imaginary)
-        self._assert_total_rewrite(ns.control_pipeline.func_ir.blocks,
-                                   ns.test_pipeline.func_ir.blocks)
+        self._assert_total_rewrite(ns.control_pipeline.state.func_ir.blocks,
+                                   ns.test_pipeline.state.func_ir.blocks)
 
     def test_explicit_output(self):
         """
         Check that ufunc calls with explicit outputs are not rewritten.
         """
         ns = self._test_explicit_output_function(explicit_output)
-        self._assert_no_rewrite(ns.control_pipeline.func_ir.blocks,
-                                ns.test_pipeline.func_ir.blocks)
+        self._assert_no_rewrite(ns.control_pipeline.state.func_ir.blocks,
+                                ns.test_pipeline.state.func_ir.blocks)
 
 
 class TestRewriteIssues(MemoryLeakMixin, TestCase):
@@ -486,5 +486,133 @@ class TestSemantics(MemoryLeakMixin, unittest.TestCase):
         np.testing.assert_array_equal(expect, got)
 
 
+class TestOptionals(MemoryLeakMixin, unittest.TestCase):
+    """ Tests the arrival and correct lowering of Optional types at a arrayexpr
+    derived ufunc, see #3972"""
+
+    def test_optional_scalar_type(self):
+
+        @njit
+        def arr_expr(x, y):
+            return x + y
+
+        @njit
+        def do_call(x, y):
+            if y > 0:
+                z = None
+            else:
+                z = y
+            return arr_expr(x, z)
+
+        args = (np.arange(5), -1.2)
+
+        # check result
+        res = do_call(*args)
+        expected = do_call.py_func(*args)
+        np.testing.assert_allclose(res, expected)
+
+        # check type
+        s = arr_expr.signatures
+        oty = s[0][1]
+        self.assertTrue(isinstance(oty, types.Optional))
+        self.assertTrue(isinstance(oty.type, types.Float))
+
+
+    def test_optional_array_type(self):
+
+        @njit
+        def arr_expr(x, y):
+            return x + y
+
+        @njit
+        def do_call(x, y):
+            if y[0] > 0:
+                z = None
+            else:
+                z = y
+            return arr_expr(x, z)
+
+        args = (np.arange(5), np.arange(5.))
+
+        # check result
+        res = do_call(*args)
+        expected = do_call.py_func(*args)
+        np.testing.assert_allclose(res, expected)
+
+        # check type
+        s = arr_expr.signatures
+        oty = s[0][1]
+        self.assertTrue(isinstance(oty, types.Optional))
+        self.assertTrue(isinstance(oty.type, types.Array))
+        self.assertTrue(isinstance(oty.type.dtype, types.Float))
+
+
+class TestOptionalsExceptions(MemoryLeakMixin, unittest.TestCase):
+    # same as above, but the Optional resolves to None and TypeError's
+
+    def test_optional_scalar_type_exception_on_none(self):
+
+        self.disable_leak_check()
+
+        @njit
+        def arr_expr(x, y):
+            return x + y
+
+        @njit
+        def do_call(x, y):
+            if y > 0:
+                z = None
+            else:
+                z = y
+            return arr_expr(x, z)
+
+        args = (np.arange(5), 1.0)
+
+        # check result
+        with self.assertRaises(TypeError) as raises:
+            do_call(*args)
+
+        self.assertIn("expected float64, got None", str(raises.exception))
+
+        # check type
+        s = arr_expr.signatures
+        oty = s[0][1]
+        self.assertTrue(isinstance(oty, types.Optional))
+        self.assertTrue(isinstance(oty.type, types.Float))
+
+    def test_optional_array_type_exception_on_none(self):
+
+        self.disable_leak_check()
+
+        @njit
+        def arr_expr(x, y):
+            return x + y
+
+        @njit
+        def do_call(x, y):
+            if y[0] > 0:
+                z = None
+            else:
+                z = y
+            return arr_expr(x, z)
+
+        args = (np.arange(5), np.arange(1., 5.))
+
+        # check result
+        with self.assertRaises(TypeError) as raises:
+            do_call(*args)
+
+        excstr = str(raises.exception)
+        self.assertIn("expected array(float64,", excstr)
+        self.assertIn("got None", excstr)
+
+        # check type
+        s = arr_expr.signatures
+        oty = s[0][1]
+        self.assertTrue(isinstance(oty, types.Optional))
+        self.assertTrue(isinstance(oty.type, types.Array))
+        self.assertTrue(isinstance(oty.type.dtype, types.Float))
+
+
 if __name__ == "__main__":
     unittest.main()
