@@ -2,6 +2,7 @@ from __future__ import division
 
 from itertools import product, cycle, permutations
 import sys
+import warnings
 
 import numpy as np
 
@@ -11,10 +12,10 @@ from numba.compiler import compile_isolated
 from numba.errors import TypingError, LoweringError
 from numba.numpy_support import (as_dtype, strict_ufunc_typing,
                                  version as numpy_version)
-from .support import TestCase, CompilationCache, MemoryLeak, MemoryLeakMixin, tag
+from .support import (TestCase, CompilationCache, MemoryLeak, MemoryLeakMixin,
+                      tag)
 from .matmul_usecase import needs_blas
 
-
 def np_around_array(arr, decimals, out):
     np.around(arr, decimals, out)
 
@@ -86,6 +87,42 @@ def array_take(arr, indices):
 def array_take_kws(arr, indices, axis):
     return arr.take(indices, axis=axis)
 
+def np_arange_1(arg0):
+    return np.arange(arg0)
+
+def np_arange_2(arg0, arg1):
+    return np.arange(arg0, arg1)
+
+def np_arange_3(arg0, arg1, arg2):
+    return np.arange(arg0, arg1, arg2)
+
+def np_arange_4(arg0, arg1, arg2, arg3):
+    return np.arange(arg0, arg1, arg2, arg3)
+
+def np_arange_1_stop(arg0, stop):
+    return np.arange(arg0, stop=stop)
+
+def np_arange_1_step(arg0, step):
+    return np.arange(arg0, step=step)
+
+def np_arange_1_dtype(arg0, dtype):
+    return np.arange(arg0, dtype=dtype)
+
+def np_arange_2_step(arg0, arg1, step):
+    return np.arange(arg0, arg1, step=step)
+
+def np_arange_2_dtype(arg0, arg1, dtype):
+    return np.arange(arg0, arg1, dtype=dtype)
+
+def np_arange_start_stop(start, stop):
+    return np.arange(start=start, stop=stop)
+
+def np_arange_start_stop_step(start, stop, step):
+    return np.arange(start=start, stop=stop, step=step)
+
+def np_arange_start_stop_step_dtype(start, stop, step, dtype):
+    return np.arange(start=start, stop=stop, step=step, dtype=dtype)
+
 def array_fill(arr, val):
     return arr.fill(val)
 
@@ -148,9 +185,18 @@ def array_itemset(a, v):
 def array_sum(a, *args):
     return a.sum(*args)
 
-def array_sum_kws(a, axis):
+def array_sum_axis_kws(a, axis):
     return a.sum(axis=axis)
 
+def array_sum_dtype_kws(a, dtype):
+    return a.sum(dtype=dtype)
+
+def array_sum_axis_dtype_kws(a, dtype, axis):
+    return a.sum(axis=axis, dtype=dtype)
+
+def array_sum_axis_dtype_pos(a, a1, a2):
+    return a.sum(a1, a2)
+
 def array_sum_const_multi(arr, axis):
     # use np.sum with different constant args multiple times to check
     # for internal compile cache to see if constant-specialization is
@@ -165,6 +211,11 @@ def array_sum_const_multi(arr, axis):
     e = np.sum(arr, axis=-1)
     return a, b, c, d, e
 
+def array_sum_const_axis_neg_one(a, axis):
+    # use .sum with -1 axis, this is for use with 1D arrays where the above
+    # "const_multi" variant would raise errors
+    return a.sum(axis=-1)
+
 def array_cumsum(a, *args):
     return a.cumsum(*args)
 
@@ -414,6 +465,11 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
         arr = np.arange(16, dtype=np.int32)[::2]
         check(arr, np.uint64)
 
+        # check read only attr does not get copied
+        arr = np.arange(16, dtype=np.int32)
+        arr.flags.writeable = False
+        check(arr, np.int32)
+
         # Invalid conversion
         dt = np.dtype([('x', np.int8)])
         with self.assertTypingError() as raises:
@@ -482,7 +538,6 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
         arr = np.array([0]).reshape(())
         check_arr(arr)
 
-
     def test_array_transpose(self):
         self.check_layout_dependent_func(array_transpose)
 
@@ -521,7 +576,7 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
     def test_np_frombuffer_allocated(self):
         self.check_np_frombuffer_allocated(np_frombuffer_allocated)
 
-    def test_np_frombuffer_allocated(self):
+    def test_np_frombuffer_allocated2(self):
         self.check_np_frombuffer_allocated(np_frombuffer_allocated_dtype)
 
     def check_nonzero(self, pyfunc):
@@ -557,6 +612,10 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
             arr = np.array([v]).reshape(())
             check_arr(arr)
 
+        if sys.version_info >= (3,):
+            arr = np.array(["Hello", "", "world"])
+            check_arr(arr)
+
     def test_array_nonzero(self):
         self.check_nonzero(array_nonzero)
 
@@ -595,7 +654,7 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
             cres = compile_isolated(pyfunc, (typeof(arr), typeof(x), typeof(y)))
             expected = pyfunc(arr, x, y)
             got = cres.entry_point(arr, x, y)
-            # Contiguity of result varies accross Numpy versions, only
+            # Contiguity of result varies across Numpy versions, only
             # check contents. NumPy 1.11+ seems to stabilize.
             if numpy_version < (1, 11):
                 self.assertEqual(got.dtype, expected.dtype)
@@ -639,6 +698,230 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
         for x in (0, 1, True, False, 2.5, 0j):
             check_scal(x)
 
+    def test_np_where_3_broadcast_x_y_scalar(self):
+        pyfunc = np_where_3
+        cfunc = jit(nopython=True)(pyfunc)
+
+        def check_ok(args):
+            expected = pyfunc(*args)
+            got = cfunc(*args)
+            self.assertPreciseEqual(got, expected)
+
+        def a_variations():
+            a = np.linspace(-2, 4, 20)
+            self.random.shuffle(a)
+            yield a
+            yield a.reshape(2, 5, 2)
+            yield a.reshape(4, 5, order='F')
+            yield a.reshape(2, 5, 2)[::-1]
+
+        for a in a_variations():
+            params = (a > 0, 0, 1)
+            check_ok(params)
+
+            params = (a < 0, np.nan, 1 + 4j)
+            check_ok(params)
+
+            params = (a > 1, True, False)
+            check_ok(params)
+
+    def test_np_where_3_broadcast_x_or_y_scalar(self):
+        pyfunc = np_where_3
+        cfunc = jit(nopython=True)(pyfunc)
+
+        def check_ok(args):
+            condition, x, y = args
+
+            expected = pyfunc(condition, x, y)
+            got = cfunc(condition, x, y)
+            self.assertPreciseEqual(got, expected)
+
+            # swap x and y
+            expected = pyfunc(condition, y, x)
+            got = cfunc(condition, y, x)
+            self.assertPreciseEqual(got, expected)
+
+        def array_permutations():
+            x = np.arange(9).reshape(3, 3)
+            yield x
+            yield x * 1.1
+            yield np.asfortranarray(x)
+            yield x[::-1]
+            yield np.linspace(-10, 10, 60).reshape(3, 4, 5) * 1j
+
+        def scalar_permutations():
+            yield 0
+            yield 4.3
+            yield np.nan
+            yield True
+            yield 8 + 4j
+
+        for x in array_permutations():
+            for y in scalar_permutations():
+                x_mean = np.mean(x)
+                condition = x > x_mean
+                params = (condition, x, y)
+                check_ok(params)
+
+    def test_arange_1_arg(self):
+
+        all_pyfuncs = (
+            np_arange_1,
+            lambda x: np.arange(x, 10),
+            lambda x: np.arange(7, step=abs(x))
+        )
+
+        for pyfunc in all_pyfuncs:
+            pyfunc = np_arange_1
+            cfunc = jit(nopython=True)(pyfunc)
+
+            def check_ok(arg0):
+                expected = pyfunc(arg0)
+                got = cfunc(arg0)
+                np.testing.assert_allclose(expected, got)
+
+            check_ok(0)
+            check_ok(1)
+            check_ok(4)
+            check_ok(5.5)
+            check_ok(-3)
+            check_ok(np.complex(4, 4))
+            check_ok(np.int8(0))
+
+    def test_arange_2_arg(self):
+        def check_ok(arg0, arg1, pyfunc, cfunc):
+            expected = pyfunc(arg0, arg1)
+            got = cfunc(arg0, arg1)
+            np.testing.assert_allclose(expected, got)
+
+        all_pyfuncs = (
+            np_arange_2,
+            np_arange_start_stop,
+            np_arange_1_stop,
+            np_arange_1_step,
+            lambda x, y: np.arange(x, y, 5),
+            lambda x, y: np.arange(2, y, step=x),
+        )
+
+        for pyfunc in all_pyfuncs:
+            cfunc = jit(nopython=True)(pyfunc)
+
+            check_ok(-1, 5, pyfunc, cfunc)
+            check_ok(-8, -1, pyfunc, cfunc)
+            check_ok(4, 0.5, pyfunc, cfunc)
+            check_ok(0.5, 4, pyfunc, cfunc)
+            check_ok(np.complex(1, 1), np.complex(4, 4), pyfunc, cfunc)
+            check_ok(np.complex(4, 4), np.complex(1, 1), pyfunc, cfunc)
+            check_ok(3, None, pyfunc, cfunc)
+
+        pyfunc = np_arange_1_dtype
+        cfunc = jit(nopython=True)(pyfunc)
+
+        check_ok(5, np.float32, pyfunc, cfunc)
+        check_ok(2.0, np.int32, pyfunc, cfunc)
+        check_ok(10, np.complex128, pyfunc, cfunc)
+        check_ok(np.complex64(10), np.complex128, pyfunc, cfunc)
+        check_ok(7, None, pyfunc, cfunc)
+        check_ok(np.int8(0), None, pyfunc, cfunc)
+
+    def test_arange_3_arg(self):
+        windows64 = sys.platform.startswith('win32') and sys.maxsize > 2 ** 32
+
+        def check_ok(arg0, arg1, arg2, pyfunc, cfunc, check_dtype=False):
+            expected = pyfunc(arg0, arg1, arg2)
+            got = cfunc(arg0, arg1, arg2)
+            np.testing.assert_allclose(expected, got)
+            # windows 64 cannot differentiate between a python int and a
+            # np.int64 which means the result from numba is int64 more often
+            # than in NumPy.
+            if not windows64:
+                self.assertEqual(expected.dtype, got.dtype)
+
+        for pyfunc in (np_arange_3, np_arange_2_step, np_arange_start_stop_step):
+            cfunc = jit(nopython=True)(pyfunc)
+
+            check_ok(0, 5, 1, pyfunc, cfunc)
+            check_ok(-8, -1, 3, pyfunc, cfunc)
+            check_ok(0, -10, -2, pyfunc, cfunc)
+            check_ok(0.5, 4, 2, pyfunc, cfunc)
+            check_ok(0, 1, 0.1, pyfunc, cfunc)
+            check_ok(0, np.complex(4, 4), np.complex(1, 1), pyfunc, cfunc)
+            check_ok(3, 6, None, pyfunc, cfunc)
+            check_ok(3, None, None, pyfunc, cfunc)
+            check_ok(np.int8(0), np.int8(5), np.int8(1), pyfunc, cfunc)
+            check_ok(np.int8(0), np.int16(5), np.int32(1), pyfunc, cfunc)
+            # check upcasting logic, this matters most on windows
+            i8 = np.int8
+            check_ok(i8(0), i8(5), i8(1), pyfunc, cfunc, True) # C int
+            check_ok(np.int64(0), i8(5), i8(1), pyfunc, cfunc, True) # int64
+
+        pyfunc = np_arange_2_dtype
+        cfunc = jit(nopython=True)(pyfunc)
+
+        check_ok(1, 5, np.float32, pyfunc, cfunc)
+        check_ok(2.0, 8, np.int32, pyfunc, cfunc)
+        check_ok(-2, 10, np.complex128, pyfunc, cfunc)
+        check_ok(3, np.complex64(10), np.complex128, pyfunc, cfunc)
+        check_ok(1, 7, None, pyfunc, cfunc)
+        check_ok(np.int8(0), np.int32(5), None, pyfunc, cfunc, True)
+
+    def test_arange_4_arg(self):
+        for pyfunc in (np_arange_4, np_arange_start_stop_step_dtype):
+            cfunc = jit(nopython=True)(pyfunc)
+
+            def check_ok(arg0, arg1, arg2, arg3):
+                expected = pyfunc(arg0, arg1, arg2, arg3)
+                got = cfunc(arg0, arg1, arg2, arg3)
+                np.testing.assert_allclose(expected, got)
+
+            check_ok(0, 5, 1, np.float64)
+            check_ok(-8, -1, 3, np.int32)
+            check_ok(0, -10, -2, np.float32)
+            check_ok(0.5, 4, 2, None)
+            check_ok(0, 1, 0.1, np.complex128)
+            check_ok(0, np.complex(4, 4), np.complex(1, 1), np.complex128)
+            check_ok(3, 6, None, None)
+            check_ok(3, None, None, None)
+
+    def test_arange_throws(self):
+        # Exceptions leak references
+        self.disable_leak_check()
+
+        bad_funcs_1 = [
+            lambda x: np.arange(stop=x),
+            lambda x: np.arange(step=x),
+            lambda x: np.arange(dtype=x),
+        ]
+        bad_funcs_2 = [
+            lambda x, y: np.arange(stop=x, step=y),
+            lambda x, y: np.arange(stop=x, dtype=y),
+        ]
+
+        for pyfunc in bad_funcs_1:
+            with self.assertRaises(TypingError) as raises:
+                cfunc = jit(nopython=True)(pyfunc)
+                cfunc(2)
+        for pyfunc in bad_funcs_2:
+            with self.assertRaises(TypingError) as raises:
+                cfunc = jit(nopython=True)(pyfunc)
+                cfunc(2, 6)
+
+        # check step size = 0, this is nonsense
+        pyfunc = np_arange_3
+        cfunc = jit(nopython=True)(pyfunc)
+        for f in (pyfunc, cfunc,):
+            for inputs in [(1, np.int16(2), 0), (1, 2, 0)]:
+                # there's a different error depending on whether any of the
+                # input values are np scalars
+                permitted_errors = (ZeroDivisionError, ValueError)
+                with self.assertRaises(permitted_errors) as raises:
+                    # this will raise RuntimeWarning's about zero division
+                    with warnings.catch_warnings():
+                        warnings.simplefilter("ignore")
+                        f(*inputs)
+                    self.assertIn("Maximum allowed size exceeded",
+                                str(raises.exception))
+
     def test_item(self):
         pyfunc = array_item
         cfunc = jit(nopython=True)(pyfunc)
@@ -696,22 +979,178 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
         check_err(np.array([]))
 
     def test_sum(self):
+        """ test sum over a whole range of dtypes, no axis or dtype parameter
+        """
         pyfunc = array_sum
         cfunc = jit(nopython=True)(pyfunc)
-        # OK
-        a = np.ones((7, 6, 5, 4, 3))
-        self.assertPreciseEqual(pyfunc(a), cfunc(a))
-        # OK
-        self.assertPreciseEqual(pyfunc(a, 0), cfunc(a, 0))
+        all_dtypes = [np.float64, np.float32, np.int64, np.int32,
+                      np.complex64, np.complex128, np.uint32, np.uint64, np.timedelta64]
+        all_test_arrays = [
+            [np.ones((7, 6, 5, 4, 3), arr_dtype),
+             np.ones(1, arr_dtype),
+             np.ones((7, 3), arr_dtype) * -5]
+            for arr_dtype in all_dtypes]
 
-    def test_sum_kws(self):
-        pyfunc = array_sum_kws
+        for arr_list in all_test_arrays:
+            for arr in arr_list:
+                with self.subTest("Test np.sum with {} input ".format(arr.dtype)):
+                    self.assertPreciseEqual(pyfunc(arr), cfunc(arr))
+
+    def test_sum_axis_kws1(self):
+        """ test sum with axis parameter over a whole range of dtypes  """
+        pyfunc = array_sum_axis_kws
         cfunc = jit(nopython=True)(pyfunc)
+        all_dtypes = [np.float64, np.float32, np.int64, np.uint64, np.complex64,
+                      np.complex128]
+        # timedelta test cannot be enabled until issue #4540 is fixed
+        # all_dtypes += [np.timedelta64]
+        all_test_arrays = [
+            [np.ones((7, 6, 5, 4, 3), arr_dtype),
+             np.ones(1, arr_dtype),
+             np.ones((7, 3), arr_dtype) * -5]
+            for arr_dtype in all_dtypes]
+
+        for arr_list in all_test_arrays:
+            for arr in arr_list:
+                for axis in (0, 1, 2):
+                    if axis > len(arr.shape)-1:
+                        continue
+                    with self.subTest("Testing np.sum(axis) with {} "
+                                      "input ".format(arr.dtype)):
+                        self.assertPreciseEqual(pyfunc(arr, axis=axis),
+                                                cfunc(arr, axis=axis))
+
+    def test_sum_axis_kws2(self):
+        """  testing uint32 and int32 separately
+
+        uint32 and int32 must be tested separately because Numpy's current
+        behaviour is different in 64bits Windows (accumulates as int32)
+        and 64bits Linux (accumulates as int64), while Numba has decided to always
+        accumulate as int64, when the OS is 64bits. No testing has been done
+        for behaviours in 32 bits platforms.
+        """
+        pyfunc = array_sum_axis_kws
+        cfunc = jit(nopython=True)(pyfunc)
+        all_dtypes = [np.int32, np.uint32]
+        # expected return dtypes in Numba
+        out_dtypes = {np.dtype('int32'): np.int64, np.dtype('uint32'): np.uint64,
+                      np.dtype('int64'): np.int64}
+        # timedelta test cannot be enabled until issue #4540 is fixed
+        # all_dtypes += [np.timedelta64]
+        all_test_arrays = [
+            [np.ones((7, 6, 5, 4, 3), arr_dtype),
+             np.ones(1, arr_dtype),
+             np.ones((7, 3), arr_dtype) * -5]
+            for arr_dtype in all_dtypes]
+
+        for arr_list in all_test_arrays:
+            for arr in arr_list:
+                for axis in (0, 1, 2):
+                    if axis > len(arr.shape)-1:
+                        continue
+                    with self.subTest("Testing np.sum(axis) with {} "
+                                      "input ".format(arr.dtype)):
+                        npy_res = pyfunc(arr, axis=axis)
+                        numba_res = cfunc(arr, axis=axis)
+                        if isinstance(numba_res, np.ndarray):
+                            self.assertPreciseEqual(
+                                npy_res.astype(out_dtypes[arr.dtype]),
+                                numba_res.astype(out_dtypes[arr.dtype]))
+                        else:
+                            # the results are scalars
+                            self.assertEqual(npy_res, numba_res)
+
+    def test_sum_dtype_kws(self):
+        """ test sum with dtype parameter over a whole range of dtypes """
+        pyfunc = array_sum_dtype_kws
+        cfunc = jit(nopython=True)(pyfunc)
+        all_dtypes = [np.float64, np.float32, np.int64, np.int32, np.uint32,
+                      np.uint64, np.complex64, np.complex128]
+        # timedelta test cannot be enabled until issue #4540 is fixed
+        # all_dtypes += [np.timedelta64]
+        all_test_arrays = [
+            [np.ones((7, 6, 5, 4, 3), arr_dtype),
+             np.ones(1, arr_dtype),
+             np.ones((7, 3), arr_dtype) * -5]
+            for arr_dtype in all_dtypes]
+
+        out_dtypes = {np.dtype('float64'): [np.float64],
+                      np.dtype('float32'): [np.float64, np.float32],
+                      np.dtype('int64'): [np.float64, np.int64, np.float32],
+                      np.dtype('int32'): [np.float64, np.int64, np.float32, np.int32],
+                      np.dtype('uint32'): [np.float64, np.int64, np.float32],
+                      np.dtype('uint64'): [np.float64, np.int64],
+                      np.dtype('complex64'): [np.complex64, np.complex128],
+                      np.dtype('complex128'): [np.complex128],
+                      np.dtype('timedelta64'): [np.timedelta64]}
+
+        for arr_list in all_test_arrays:
+            for arr in arr_list:
+                for out_dtype in out_dtypes[arr.dtype]:
+                    subtest_str = ("Testing np.sum with {} input and {} output"
+                                   .format(arr.dtype, out_dtype))
+                    with self.subTest(subtest_str):
+                        self.assertPreciseEqual(pyfunc(arr, dtype=out_dtype),
+                                                cfunc(arr, dtype=out_dtype))
+
+    def test_sum_axis_dtype_kws(self):
+        """ test sum with axis and dtype parameters over a whole range of dtypes """
+        pyfunc = array_sum_axis_dtype_kws
+        cfunc = jit(nopython=True)(pyfunc)
+        all_dtypes = [np.float64, np.float32, np.int64, np.int32, np.uint32,
+                      np.uint64, np.complex64, np.complex128]
+        all_test_arrays = [
+            [np.ones((7, 6, 5, 4, 3), arr_dtype),
+             np.ones(1, arr_dtype),
+             np.ones((7, 3), arr_dtype) * -5]
+            for arr_dtype in all_dtypes]
+
+        out_dtypes = {np.dtype('float64'): [np.float64],
+                      np.dtype('float32'): [np.float64, np.float32],
+                      np.dtype('int64'): [np.float64, np.int64, np.float32],
+                      np.dtype('int32'): [np.float64, np.int64, np.float32, np.int32],
+                      np.dtype('uint32'): [np.float64, np.int64, np.float32],
+                      np.dtype('uint64'): [np.float64, np.uint64],
+                      np.dtype('complex64'): [np.complex64, np.complex128],
+                      np.dtype('complex128'): [np.complex128],
+                      np.dtype('timedelta64'): [np.timedelta64]}
+
+        for arr_list in all_test_arrays:
+            for arr in arr_list:
+                for out_dtype in out_dtypes[arr.dtype]:
+                    for axis in (0, 1, 2):
+                        if axis > len(arr.shape) - 1:
+                            continue
+                        subtest_str = ("Testing np.sum with {} input and {} output "
+                                       .format(arr.dtype, out_dtype))
+                        with self.subTest(subtest_str):
+                            py_res = pyfunc(arr, axis=axis, dtype=out_dtype)
+                            nb_res = cfunc(arr, axis=axis, dtype=out_dtype)
+                            self.assertPreciseEqual(py_res, nb_res)
+
+    def test_sum_axis_dtype_pos_arg(self):
+        """ testing that axis and dtype inputs work when passed as positional """
+        pyfunc = array_sum_axis_dtype_pos
+        cfunc = jit(nopython=True)(pyfunc)
+        dtype = np.float64
         # OK
         a = np.ones((7, 6, 5, 4, 3))
-        self.assertPreciseEqual(pyfunc(a, axis=1), cfunc(a, axis=1))
-        # OK
-        self.assertPreciseEqual(pyfunc(a, axis=2), cfunc(a, axis=2))
+        self.assertPreciseEqual(pyfunc(a, 1, dtype),
+                                cfunc(a,  1, dtype))
+
+        self.assertPreciseEqual(pyfunc(a, 2, dtype),
+                                cfunc(a, 2, dtype))
+
+    def test_sum_1d_kws(self):
+        # check 1d reduces to scalar
+        pyfunc = array_sum_axis_kws
+        cfunc = jit(nopython=True)(pyfunc)
+        a = np.arange(10.)
+        self.assertPreciseEqual(pyfunc(a, axis=0), cfunc(a, axis=0))
+        pyfunc = array_sum_const_axis_neg_one
+        cfunc = jit(nopython=True)(pyfunc)
+        a = np.arange(10.)
+        self.assertPreciseEqual(pyfunc(a, axis=-1), cfunc(a, axis=-1))
 
     def test_sum_const(self):
         pyfunc = array_sum_const_multi
@@ -808,7 +1247,7 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
         test_indices.append(np.array([1, 5, 1, 11, 3]))
         test_indices.append(np.array([[1, 5, 1], [11, 3, 0]], order='F'))
         test_indices.append(np.array([[[1, 5, 1], [11, 3, 0]]]))
-        test_indices.append(np.array([[[[1, 5]], [[11, 0]],[[1, 2]]]]))
+        test_indices.append(np.array([[[[1, 5]], [[11, 0]], [[1, 2]]]]))
         test_indices.append([1, 5, 1, 11, 3])
         test_indices.append((1, 5, 1))
         test_indices.append(((1, 5, 1), (11, 3, 2)))
@@ -828,7 +1267,7 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
                            [szA], [-szA - 1]]
         for x in illegal_indices:
             with self.assertRaises(IndexError):
-                cfunc(A, x) # oob raises
+                cfunc(A, x) #  oob raises
 
         # check float indexing raises
         with self.assertRaises(TypingError):
