@@ -12,6 +12,7 @@ import warnings
 import inspect
 import pickle
 import weakref
+from itertools import chain
 
 try:
     import jinja2
@@ -26,7 +27,7 @@ except ImportError:
 import numpy as np
 
 from numba import unittest_support as unittest
-from numba import utils, jit, generated_jit, types, typeof
+from numba import utils, jit, generated_jit, types, typeof, errors
 from numba import _dispatcher
 from numba.compiler import compile_isolated
 from numba.errors import NumbaWarning
@@ -44,6 +45,7 @@ import llvmlite.binding as ll
 
 _is_armv7l = platform.machine() == 'armv7l'
 
+
 def dummy(x):
     return x
 
@@ -111,6 +113,7 @@ class BaseTest(TestCase):
         f = jit(**self.jit_args)(pyfunc)
         return f, check
 
+
 def check_access_is_preventable():
     # This exists to check whether it is possible to prevent access to
     # a file/directory through the use of `chmod 500`. If a user has
@@ -132,7 +135,8 @@ def check_access_is_preventable():
             f.write('check2')
     except (OSError, IOError) as e:
         # Check that the cause of the exception is due to access/permission
-        # as per https://github.com/conda/conda/blob/4.5.0/conda/gateways/disk/permissions.py#L35-L37
+        # as per
+        # https://github.com/conda/conda/blob/4.5.0/conda/gateways/disk/permissions.py#L35-L37  # noqa: E501
         eno = getattr(e, 'errno', None)
         if eno in (errno.EACCES, errno.EPERM):
             # errno reports access/perm fail so access prevention via
@@ -143,6 +147,7 @@ def check_access_is_preventable():
         shutil.rmtree(test_dir)
     return ret
 
+
 _access_preventable = check_access_is_preventable()
 _access_msg = "Cannot create a directory to which writes are preventable"
 skip_bad_access = unittest.skipUnless(_access_preventable, _access_msg)
@@ -221,7 +226,7 @@ class TestDispatcher(BaseTest):
         def wrapper():
             try:
                 self.assertEqual(foo(1), 2)
-            except BaseException as e:
+            except Exception as e:
                 errors.append(e)
 
         threads = [threading.Thread(target=wrapper) for i in range(16)]
@@ -261,15 +266,17 @@ class TestDispatcher(BaseTest):
         # as the actual argument types.
         self.assertRegexpMatches(
             str(cm.exception),
-            r"Ambiguous overloading for <function add [^>]*> \(float64, float64\):\n"
+            r"Ambiguous overloading for <function add [^>]*> "
+            r"\(float64, float64\):\n"
             r"\(float32, float64\) -> float64\n"
             r"\(float64, float32\) -> float64"
-            )
+        )
         # The integer signature is not part of the best matches
         self.assertNotIn("int64", str(cm.exception))
 
     def test_signature_mismatch(self):
-        tmpl = "Signature mismatch: %d argument types given, but function takes 2 arguments"
+        tmpl = ("Signature mismatch: %d argument types given, but function "
+                "takes 2 arguments")
         with self.assertRaises(TypeError) as cm:
             jit("()")(add)
         self.assertIn(tmpl % 0, str(cm.exception))
@@ -393,7 +400,8 @@ class TestDispatcher(BaseTest):
 
         self.assertIs(foo, foo_rebuilt)
 
-        # do we get the same object even if we delete all the explict references?
+        # do we get the same object even if we delete all the explicit
+        # references?
         id_orig = id(foo_rebuilt)
         del foo
         del foo_rebuilt
@@ -408,7 +416,7 @@ class TestDispatcher(BaseTest):
         self.assertEqual(memo_size, len(memo))
 
         # show that deserializing creates a new object
-        newer_foo = pickle.loads(serialized_foo)
+        pickle.loads(serialized_foo)
         self.assertIs(ref(), None)
 
     @needs_lapack
@@ -543,6 +551,34 @@ class TestDispatcher(BaseTest):
         check("F_contig_aligned", F_contig_aligned)
         check("C_contig_misaligned", C_contig_misaligned)
 
+    def test_dispatch_recompiles_for_scalars(self):
+        # for context #3612, essentially, compiling a lambda x:x for a
+        # numerically wide type (everything can be converted to a complex128)
+        # and then calling again with e.g. an int32 would lead to the int32
+        # being converted to a complex128 whereas it ought to compile an int32
+        # specialization.
+        def foo(x):
+            return x
+
+        # jit and compile on dispatch for 3 scalar types, expect 3 signatures
+        jitfoo = jit(nopython=True)(foo)
+        jitfoo(np.complex128(1 + 2j))
+        jitfoo(np.int32(10))
+        jitfoo(np.bool_(False))
+        self.assertEqual(len(jitfoo.signatures), 3)
+        expected_sigs = [(types.complex128,), (types.int32,), (types.bool_,)]
+        self.assertEqual(jitfoo.signatures, expected_sigs)
+
+        # now jit with signatures so recompilation is forbidden
+        # expect 1 signature and type conversion
+        jitfoo = jit([(types.complex128,)], nopython=True)(foo)
+        jitfoo(np.complex128(1 + 2j))
+        jitfoo(np.int32(10))
+        jitfoo(np.bool_(False))
+        self.assertEqual(len(jitfoo.signatures), 1)
+        expected_sigs = [(types.complex128,)]
+        self.assertEqual(jitfoo.signatures, expected_sigs)
+
 
 class TestSignatureHandling(BaseTest):
     """
@@ -647,7 +683,6 @@ class TestGeneratedDispatcher(TestCase):
         self.assertEqual(f(1j, 42), 42 + 1j)
         self.assertEqual(f(x=1j, y=7), 7 + 1j)
 
-
     @tag('important')
     def test_generated_dtype(self):
         f = generated_jit(nopython=True)(dtype_generated_usecase)
@@ -666,12 +701,14 @@ class TestGeneratedDispatcher(TestCase):
         # Mismatching # of arguments
         with self.assertRaises(TypeError) as raises:
             f(1j)
-        self.assertIn("should be compatible with signature '(x, y=5)', but has signature '(x)'",
+        self.assertIn("should be compatible with signature '(x, y=5)', "
+                      "but has signature '(x)'",
                       str(raises.exception))
         # Mismatching defaults
         with self.assertRaises(TypeError) as raises:
             f(1)
-        self.assertIn("should be compatible with signature '(x, y=5)', but has signature '(x, y=6)'",
+        self.assertIn("should be compatible with signature '(x, y=5)', "
+                      "but has signature '(x, y=6)'",
                       str(raises.exception))
 
 
@@ -762,7 +799,9 @@ class TestDispatcherMethods(TestCase):
             wrapper = "{}{}".format(len(wrapper), wrapper)
         module_name = __name__.split('.', 1)[0]
         module_len = len(module_name)
-        prefix = r'^digraph "CFG for \'_ZN{}{}{}'.format(wrapper, module_len, module_name)
+        prefix = r'^digraph "CFG for \'_ZN{}{}{}'.format(wrapper,
+                                                         module_len,
+                                                         module_name)
         self.assertRegexpMatches(str(cfg), prefix)
         # .display() requires an optional dependency on `graphviz`.
         # just test for the attribute without running it.
@@ -832,6 +871,31 @@ class TestDispatcherMethods(TestCase):
         # Exercise the method
         foo.inspect_types(utils.StringIO())
 
+        # Test output
+        expected = str(foo.overloads[foo.signatures[0]].type_annotation)
+        with captured_stdout() as out:
+            foo.inspect_types()
+        assert expected in out.getvalue()
+
+    def test_inspect_types_with_signature(self):
+        @jit
+        def foo(a):
+            return a + 1
+
+        foo(1)
+        foo(1.0)
+        # Inspect all signatures
+        with captured_stdout() as total:
+            foo.inspect_types()
+        # Inspect first signature
+        with captured_stdout() as first:
+            foo.inspect_types(signature=foo.signatures[0])
+        # Inspect second signature
+        with captured_stdout() as second:
+            foo.inspect_types(signature=foo.signatures[1])
+
+        self.assertEqual(total.getvalue(), first.getvalue() + second.getvalue())
+
     @unittest.skipIf(jinja2 is None, "please install the 'jinja2' package")
     @unittest.skipIf(pygments is None, "please install the 'pygments' package")
     def test_inspect_types_pretty(self):
@@ -860,6 +924,19 @@ class TestDispatcherMethods(TestCase):
         self.assertIn("`file` must be None if `pretty=True`",
                       str(raises.exception))
 
+    def test_get_annotation_info(self):
+        @jit
+        def foo(a):
+            return a + 1
+
+        foo(1)
+        foo(1.3)
+
+        expected = dict(chain.from_iterable(foo.get_annotation_info(i).items()
+                                            for i in foo.signatures))
+        result = foo.get_annotation_info()
+        self.assertEqual(expected, result)
+
     def test_issue_with_array_layout_conflict(self):
         """
         This test an issue with the dispatcher when an array that is both
@@ -974,7 +1051,8 @@ class BaseCacheUsecasesTest(BaseCacheTest):
                                  stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         out, err = popen.communicate()
         if popen.returncode != 0:
-            raise AssertionError("process failed with code %s: stderr follows\n%s\n"
+            raise AssertionError("process failed with code %s: "
+                                 "stderr follows\n%s\n"
                                  % (popen.returncode, err.decode()))
 
     def check_module(self, mod):
@@ -1105,9 +1183,8 @@ class TestCache(BaseCacheUsecasesTest):
             self.check_pycache(0)
 
         self.assertEqual(len(w), 1)
-        self.assertEqual(str(w[0].message),
-                         'Cannot cache compiled function "looplifted" '
-                         'as it uses lifted loops')
+        self.assertIn('Cannot cache compiled function "looplifted" '
+                      'as it uses lifted loops', str(w[0].message))
 
     def test_big_array(self):
         # Code references big array globals cannot be cached
@@ -1128,16 +1205,18 @@ class TestCache(BaseCacheUsecasesTest):
         # a warning.
         mod = self.import_module()
 
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter('always', NumbaWarning)
+        for f in [mod.use_c_sin, mod.use_c_sin_nest1, mod.use_c_sin_nest2]:
+            with warnings.catch_warnings(record=True) as w:
+                warnings.simplefilter('always', NumbaWarning)
 
-            f = mod.use_c_sin
-            self.assertPreciseEqual(f(0.0), 0.0)
-            self.check_pycache(0)
+                self.assertPreciseEqual(f(0.0), 0.0)
+                self.check_pycache(0)
 
-        self.assertEqual(len(w), 1)
-        self.assertIn('Cannot cache compiled function "use_c_sin"',
-                      str(w[0].message))
+            self.assertEqual(len(w), 1)
+            self.assertIn(
+                'Cannot cache compiled function "{}"'.format(f.__name__),
+                str(w[0].message),
+            )
 
     def test_closure(self):
         mod = self.import_module()
@@ -1242,7 +1321,8 @@ class TestCache(BaseCacheUsecasesTest):
             self.assertIsNone(locator)
 
             sys.frozen = True
-            # returns a cache locator object, only works when executable is frozen
+            # returns a cache locator object, only works when the executable
+            # is frozen
             locator = _UserWideCacheLocator.from_function(function, source)
             self.assertIsInstance(locator, _UserWideCacheLocator)
 
@@ -1258,7 +1338,7 @@ class TestCache(BaseCacheUsecasesTest):
         mod = self.import_module()
         f = mod.add_usecase
         # Remove this function's cache files at the end, to avoid accumulation
-        # accross test calls.
+        # across test calls.
         self.addCleanup(shutil.rmtree, f.stats.cache_path, ignore_errors=True)
 
         self.assertPreciseEqual(f(2, 3), 6)
@@ -1372,7 +1452,7 @@ class TestSequentialParForsCache(BaseCacheUsecasesTest):
         f = mod.parfor_usecase
         ary = np.ones(10)
         self.assertPreciseEqual(f(ary), ary * ary + ary)
-        dynamic_globals = [cres.has_dynamic_globals
+        dynamic_globals = [cres.library.has_dynamic_globals
                            for cres in f.overloads.values()]
         self.assertEqual(dynamic_globals, [False])
         self.check_pycache(2)  # 1 index, 1 data
@@ -1613,6 +1693,56 @@ def cache_file_collision_tester(q, tempdir, modname_bar1, modname_bar2):
     q.put(r2)
 
 
+class TestCacheMultipleFilesWithSignature(unittest.TestCase):
+    # Regression test for https://github.com/numba/numba/issues/3658
+
+    _numba_parallel_test_ = False
+
+    source_text_file1 = """
+from file2 import function2
+"""
+    source_text_file2 = """
+from numba import njit
+
+@njit('float64(float64)', cache=True)
+def function1(x):
+    return x
+
+@njit('float64(float64)', cache=True)
+def function2(x):
+    return x
+"""
+
+    def setUp(self):
+        self.tempdir = temp_directory('test_cache_file_loc')
+
+        self.file1 = os.path.join(self.tempdir, 'file1.py')
+        with open(self.file1, 'w') as fout:
+            print(self.source_text_file1, file=fout)
+
+        self.file2 = os.path.join(self.tempdir, 'file2.py')
+        with open(self.file2, 'w') as fout:
+            print(self.source_text_file2, file=fout)
+
+    def tearDown(self):
+        shutil.rmtree(self.tempdir)
+
+    def test_caching_mutliple_files_with_signature(self):
+        # Execute file1.py
+        popen = subprocess.Popen([sys.executable, self.file1],
+                                 stdout=subprocess.PIPE,
+                                 stderr=subprocess.PIPE)
+        out, err = popen.communicate()
+        self.assertEqual(popen.returncode, 0)
+
+        # Execute file2.py
+        popen = subprocess.Popen([sys.executable, self.file2],
+                                 stdout=subprocess.PIPE,
+                                 stderr=subprocess.PIPE)
+        out, err = popen.communicate()
+        self.assertEqual(popen.returncode, 0)
+
+
 class TestDispatcherFunctionBoundaries(TestCase):
     def test_pass_dispatcher_as_arg(self):
         # Test that a Dispatcher object can be pass as argument
@@ -1718,5 +1848,101 @@ class TestBoxingDefaultError(unittest.TestCase):
         self.assertRegexpMatches(str(raises.exception), pat)
 
 
+class TestNoRetryFailedSignature(unittest.TestCase):
+    """Test that failed-to-compile signatures are not recompiled.
+    """
+
+    def run_test(self, func):
+        fcom = func._compiler
+        self.assertEqual(len(fcom._failed_cache), 0)
+        # expected failure because `int` has no `__getitem__`
+        with self.assertRaises(errors.TypingError):
+            func(1)
+        self.assertEqual(len(fcom._failed_cache), 1)
+        # retry
+        with self.assertRaises(errors.TypingError):
+            func(1)
+        self.assertEqual(len(fcom._failed_cache), 1)
+        # retry with double
+        with self.assertRaises(errors.TypingError):
+            func(1.0)
+        self.assertEqual(len(fcom._failed_cache), 2)
+
+    def test_direct_call(self):
+        @jit(nopython=True)
+        def foo(x):
+            return x[0]
+
+        self.run_test(foo)
+
+    def test_nested_call(self):
+        @jit(nopython=True)
+        def bar(x):
+            return x[0]
+
+        @jit(nopython=True)
+        def foobar(x):
+            bar(x)
+
+        @jit(nopython=True)
+        def foo(x):
+            return bar(x) + foobar(x)
+
+        self.run_test(foo)
+
+    def test_error_count(self):
+        def check(field, would_fail):
+            # Slightly modified from the reproducer in issue #4117.
+            # Before the patch, the compilation time of the failing case is
+            # much longer than of the successful case. This can be detected
+            # by the number of times `trigger()` is visited.
+            k = 10
+            counter = {'c': 0}
+
+            @generated_jit
+            def trigger(x):
+                # Keep track of every visit
+                counter['c'] += 1
+                if would_fail:
+                    raise errors.TypingError("invoke_failed")
+                return lambda x: x
+
+            @jit(nopython=True)
+            def ident(out, x):
+                pass
+
+            def chain_assign(fs, inner=ident):
+                tab_head, tab_tail = fs[-1], fs[:-1]
+
+                @jit(nopython=True)
+                def assign(out, x):
+                    inner(out, x)
+                    out[0] += tab_head(x)
+
+                if tab_tail:
+                    return chain_assign(tab_tail, assign)
+                else:
+                    return assign
+
+            chain = chain_assign((trigger,) * k)
+            out = np.ones(2)
+            if would_fail:
+                with self.assertRaises(errors.TypingError) as raises:
+                    chain(out, 1)
+                self.assertIn('invoke_failed', str(raises.exception))
+            else:
+                chain(out, 1)
+
+            # Returns the visit counts
+            return counter['c']
+
+        ct_ok = check('a', False)
+        ct_bad = check('c', True)
+        # `trigger()` is visited exactly once for both successful and failed
+        # compilation.
+        self.assertEqual(ct_ok, 1)
+        self.assertEqual(ct_bad, 1)
+
+
 if __name__ == '__main__':
     unittest.main()
