@@ -12,11 +12,13 @@ import re
 import numpy as np
 
 from numba import unittest_support as unittest
-from numba import jit, types, errors, typing, compiler
+from numba import njit, jit, types, errors, typing, compiler
+from numba.typed_passes import type_inference_stage
 from numba.targets.registry import cpu_target
 from numba.compiler import compile_isolated
 from .support import (TestCase, captured_stdout, tag, temp_directory,
                       override_config)
+from numba.errors import LoweringError
 
 from numba.extending import (typeof_impl, type_callable,
                              lower_builtin, lower_cast,
@@ -32,6 +34,8 @@ from numba.extending import (typeof_impl, type_callable,
 from numba.typing.templates import (
     ConcreteTemplate, signature, infer, infer_global, AbstractTemplate)
 
+_IS_PY3 = sys.version_info >= (3,)
+
 # Pandas-like API implementation
 from .pdlike_usecase import Index, Series
 
@@ -115,8 +119,7 @@ def unbox_index(typ, obj, c):
 def func1(x=None):
     raise NotImplementedError
 
-@type_callable(func1)
-def type_func1(context):
+def type_func1_(context):
     def typer(x=None):
         if x in (None, types.none):
             # 0-arg or 1-arg with None
@@ -127,6 +130,10 @@ def type_func1(context):
 
     return typer
 
+
+type_func1 = type_callable(func1)(type_func1_)
+
+
 @lower_builtin(func1)
 @lower_builtin(func1, types.none)
 def func1_nullary(context, builder, sig, args):
@@ -259,6 +266,14 @@ def overload_add_dummy(arg1, arg2):
         return dummy_add_impl
 
 
+@overload(operator.delitem)
+def overload_dummy_delitem(obj, idx):
+    if isinstance(obj, MyDummyType) and isinstance(idx, types.Integer):
+        def dummy_delitem_impl(obj, idx):
+            print('del', obj, idx)
+        return dummy_delitem_impl
+
+
 @overload(operator.getitem)
 def overload_dummy_getitem(obj, idx):
     if isinstance(obj, MyDummyType) and isinstance(idx, types.Integer):
@@ -267,6 +282,18 @@ def overload_dummy_getitem(obj, idx):
         return dummy_getitem_impl
 
 
+@overload(operator.setitem)
+def overload_dummy_setitem(obj, idx, val):
+    if all([
+        isinstance(obj, MyDummyType),
+        isinstance(idx, types.Integer),
+        isinstance(val, types.Integer)
+    ]):
+        def dummy_setitem_impl(obj, idx, val):
+            print(idx, val)
+        return dummy_setitem_impl
+
+
 def call_add_operator(arg1, arg2):
     return operator.add(arg1, arg2)
 
@@ -294,10 +321,18 @@ def call_iadd_binop(arg1, arg2):
     return arg1
 
 
+def call_delitem(obj, idx):
+    del obj[idx]
+
+
 def call_getitem(obj, idx):
     return obj[idx]
 
 
+def call_setitem(obj, idx, val):
+    obj[idx] = val
+
+
 @overload_method(MyDummyType, 'length')
 def overload_method_length(arg):
     def imp(arg):
@@ -380,6 +415,18 @@ def mk_func_test_impl():
     mk_func_input(lambda a: a)
 
 
+# -----------------------------------------------------------------------
+
+
+@overload(np.exp)
+def overload_np_exp(obj):
+    if isinstance(obj, MyDummyType):
+        def imp(obj):
+            # Returns a constant if a MyDummyType is seen
+            return 0xdeadbeef
+        return imp
+
+
 class TestLowLevelExtending(TestCase):
     """
     Test the low-level two-tier extension API.
@@ -405,6 +452,10 @@ class TestLowLevelExtending(TestCase):
         cr = compile_isolated(pyfunc, (types.float64,))
         self.assertPreciseEqual(cr.entry_point(18.0), 6.0)
 
+    def test_type_callable_keeps_function(self):
+        self.assertIs(type_func1, type_func1_)
+        self.assertIsNotNone(type_func1)
+
     def test_cast_mydummy(self):
         pyfunc = get_dummy
         cr = compile_isolated(pyfunc, (), types.float64)
@@ -416,7 +467,7 @@ class TestLowLevelExtending(TestCase):
         test_ir = compiler.run_frontend(mk_func_test_impl)
         typingctx = cpu_target.typing_context
         typingctx.refresh()
-        typemap, _, _ = compiler.type_inference_stage(
+        typemap, _, _ = type_inference_stage(
             typingctx, test_ir, (), None)
         self.assertTrue(any(isinstance(a, types.MakeFunctionLiteral)
                             for a in typemap.values()))
@@ -620,11 +671,43 @@ class TestHighLevelExtending(TestCase):
         # this will call add(Number, Number) as MyDummy implicitly casts to Number
         self.assertPreciseEqual(cfunc(MyDummy(), MyDummy()), 84)
 
+    def test_delitem(self):
+        pyfunc = call_delitem
+        cfunc = jit(nopython=True)(pyfunc)
+        obj = MyDummy()
+        e = None
+
+        with captured_stdout() as out:
+            try:
+                cfunc(obj, 321)
+            except Exception as exc:
+                e = exc
+
+        if e is not None:
+            raise e
+        self.assertEqual(out.getvalue(), 'del hello! 321\n')
+
     def test_getitem(self):
         pyfunc = call_getitem
         cfunc = jit(nopython=True)(pyfunc)
         self.assertPreciseEqual(cfunc(MyDummy(), 321), 321 + 123)
 
+    def test_setitem(self):
+        pyfunc = call_setitem
+        cfunc = jit(nopython=True)(pyfunc)
+        obj = MyDummy()
+        e = None
+
+        with captured_stdout() as out:
+            try:
+                cfunc(obj, 321, 123)
+            except Exception as exc:
+                e = exc
+
+        if e is not None:
+            raise e
+        self.assertEqual(out.getvalue(), '321 123\n')
+
     def test_no_cpython_wrapper(self):
         """
         Test overloading whose return value cannot be represented in CPython.
@@ -644,6 +727,323 @@ class TestHighLevelExtending(TestCase):
         expectmsg = "cannot convert native Module"
         self.assertIn(expectmsg, errmsg)
 
+    def test_typing_vs_impl_signature_mismatch_handling(self):
+        """
+        Tests that an overload which has a differing typing and implementing
+        signature raises an exception.
+        """
+        def gen_ol(impl=None):
+
+            def myoverload(a, b, c, kw=None):
+                pass
+
+            @overload(myoverload)
+            def _myoverload_impl(a, b, c, kw=None):
+                return impl
+
+            @jit(nopython=True)
+            def foo(a, b, c, d):
+                myoverload(a, b, c, kw=d)
+
+            return foo
+
+        sentinel = "Typing and implementation arguments differ in"
+
+        # kwarg value is different
+        def impl1(a, b, c, kw=12):
+            if a > 10:
+                return 1
+            else:
+                return -1
+
+        with self.assertRaises(errors.TypingError) as e:
+            gen_ol(impl1)(1, 2, 3, 4)
+        msg = str(e.exception)
+        self.assertIn(sentinel, msg)
+        self.assertIn("keyword argument default values", msg)
+        if _IS_PY3:
+            self.assertIn('<Parameter "kw=12">', msg)
+            self.assertIn('<Parameter "kw=None">', msg)
+
+        # kwarg name is different
+        def impl2(a, b, c, kwarg=None):
+            if a > 10:
+                return 1
+            else:
+                return -1
+
+        with self.assertRaises(errors.TypingError) as e:
+            gen_ol(impl2)(1, 2, 3, 4)
+        msg = str(e.exception)
+        self.assertIn(sentinel, msg)
+        self.assertIn("keyword argument names", msg)
+        if _IS_PY3:
+            self.assertIn('<Parameter "kwarg=None">', msg)
+            self.assertIn('<Parameter "kw=None">', msg)
+
+        # arg name is different
+        def impl3(z, b, c, kw=None):
+            if a > 10:
+                return 1
+            else:
+                return -1
+
+        with self.assertRaises(errors.TypingError) as e:
+            gen_ol(impl3)(1, 2, 3, 4)
+        msg = str(e.exception)
+        self.assertIn(sentinel, msg)
+        self.assertIn("argument names", msg)
+        self.assertFalse("keyword" in msg)
+        if _IS_PY3:
+            self.assertIn('<Parameter "a">', msg)
+            self.assertIn('<Parameter "z">', msg)
+
+        # impl4/5 has invalid syntax for python < 3
+        if _IS_PY3:
+            from .overload_usecases import impl4, impl5
+            with self.assertRaises(errors.TypingError) as e:
+                gen_ol(impl4)(1, 2, 3, 4)
+            msg = str(e.exception)
+            self.assertIn(sentinel, msg)
+            self.assertIn("argument names", msg)
+            self.assertFalse("keyword" in msg)
+            self.assertIn("First difference: 'z'", msg)
+
+            with self.assertRaises(errors.TypingError) as e:
+                gen_ol(impl5)(1, 2, 3, 4)
+            msg = str(e.exception)
+            self.assertIn(sentinel, msg)
+            self.assertIn("argument names", msg)
+            self.assertFalse("keyword" in msg)
+            self.assertIn('<Parameter "a">', msg)
+            self.assertIn('<Parameter "z">', msg)
+
+        # too many args
+        def impl6(a, b, c, d, e, kw=None):
+            if a > 10:
+                return 1
+            else:
+                return -1
+
+        with self.assertRaises(errors.TypingError) as e:
+            gen_ol(impl6)(1, 2, 3, 4)
+        msg = str(e.exception)
+        self.assertIn(sentinel, msg)
+        self.assertIn("argument names", msg)
+        self.assertFalse("keyword" in msg)
+        if _IS_PY3:
+            self.assertIn('<Parameter "d">', msg)
+            self.assertIn('<Parameter "e">', msg)
+
+        # too few args
+        def impl7(a, b, kw=None):
+            if a > 10:
+                return 1
+            else:
+                return -1
+
+        with self.assertRaises(errors.TypingError) as e:
+            gen_ol(impl7)(1, 2, 3, 4)
+        msg = str(e.exception)
+        self.assertIn(sentinel, msg)
+        self.assertIn("argument names", msg)
+        self.assertFalse("keyword" in msg)
+        if _IS_PY3:
+            self.assertIn('<Parameter "c">', msg)
+
+        # too many kwargs
+        def impl8(a, b, c, kw=None, extra_kwarg=None):
+            if a > 10:
+                return 1
+            else:
+                return -1
+
+        with self.assertRaises(errors.TypingError) as e:
+            gen_ol(impl8)(1, 2, 3, 4)
+        msg = str(e.exception)
+        self.assertIn(sentinel, msg)
+        self.assertIn("keyword argument names", msg)
+        if _IS_PY3:
+            self.assertIn('<Parameter "extra_kwarg=None">', msg)
+
+        # too few kwargs
+        def impl9(a, b, c):
+            if a > 10:
+                return 1
+            else:
+                return -1
+
+        with self.assertRaises(errors.TypingError) as e:
+            gen_ol(impl9)(1, 2, 3, 4)
+        msg = str(e.exception)
+        self.assertIn(sentinel, msg)
+        self.assertIn("keyword argument names", msg)
+        if _IS_PY3:
+            self.assertIn('<Parameter "kw=None">', msg)
+
+    @unittest.skipUnless(_IS_PY3, "Python 3+ only syntax")
+    def test_typing_vs_impl_signature_mismatch_handling_var_positional(self):
+        """
+        Tests that an overload which has a differing typing and implementing
+        signature raises an exception and uses VAR_POSITIONAL (*args) in typing
+        """
+        def myoverload(a, kw=None):
+            pass
+
+        from .overload_usecases import var_positional_impl
+        overload(myoverload)(var_positional_impl)
+
+
+        @jit(nopython=True)
+        def foo(a, b):
+            return myoverload(a, b, 9, kw=11)
+
+        with self.assertRaises(errors.TypingError) as e:
+            foo(1, 5)
+        msg = str(e.exception)
+        self.assertIn("VAR_POSITIONAL (e.g. *args) argument kind", msg)
+        self.assertIn("offending argument name is '*star_args_token'", msg)
+
+    def test_typing_vs_impl_signature_mismatch_handling_var_keyword(self):
+        """
+        Tests that an overload which uses **kwargs (VAR_KEYWORD)
+        """
+
+        def gen_ol(impl, strict=True):
+
+            def myoverload(a, kw=None):
+                pass
+
+            overload(myoverload, strict=strict)(impl)
+
+            @jit(nopython=True)
+            def foo(a, b):
+                return myoverload(a, kw=11)
+
+            return foo
+
+        # **kwargs in typing
+        def ol1(a, **kws):
+            def impl(a, kw=10):
+                return a
+            return impl
+
+        gen_ol(ol1, False)(1, 2) # no error if strictness not enforced
+        with self.assertRaises(errors.TypingError) as e:
+            gen_ol(ol1)(1, 2)
+        msg = str(e.exception)
+        self.assertIn("use of VAR_KEYWORD (e.g. **kwargs) is unsupported", msg)
+        self.assertIn("offending argument name is '**kws'", msg)
+
+        # **kwargs in implementation
+        def ol2(a, kw=0):
+            def impl(a, **kws):
+                return a
+            return impl
+
+        with self.assertRaises(errors.TypingError) as e:
+            gen_ol(ol2)(1, 2)
+        msg = str(e.exception)
+        self.assertIn("use of VAR_KEYWORD (e.g. **kwargs) is unsupported", msg)
+        self.assertIn("offending argument name is '**kws'", msg)
+
+    def test_overload_method_kwargs(self):
+        # Issue #3489
+        @overload_method(types.Array, 'foo')
+        def fooimpl(arr, a_kwarg=10):
+            def impl(arr, a_kwarg=10):
+                return a_kwarg
+            return impl
+
+        @njit
+        def bar(A):
+            return A.foo(), A.foo(20), A.foo(a_kwarg=30)
+
+        Z = np.arange(5)
+
+        self.assertEqual(bar(Z), (10, 20, 30))
+
+    def test_overload_method_literal_unpack(self):
+        # Issue #3683
+        @overload_method(types.Array, 'litfoo')
+        def litfoo(arr, val):
+            # Must be an integer
+            if isinstance(val, types.Integer):
+                # Must not be literal
+                if not isinstance(val, types.Literal):
+                    def impl(arr, val):
+                        return val
+                    return impl
+
+        @njit
+        def bar(A):
+            return A.litfoo(0xcafe)
+
+        A = np.zeros(1)
+        bar(A)
+        self.assertEqual(bar(A), 0xcafe)
+
+    def test_overload_ufunc(self):
+        # Issue #4133.
+        # Use an extended type (MyDummyType) to use with a customized
+        # ufunc (np.exp).
+        @njit
+        def test():
+            return np.exp(mydummy)
+
+        self.assertEqual(test(), 0xdeadbeef)
+
+    def test_overload_method_stararg(self):
+        @overload_method(MyDummyType, "method_stararg")
+        def _ov_method_stararg(obj, val, val2, *args):
+            def get(obj, val, val2, *args):
+                return (val, val2, args)
+
+            return get
+
+        @njit
+        def foo(obj, *args):
+            # Test with expanding stararg
+            return obj.method_stararg(*args)
+
+        obj = MyDummy()
+        self.assertEqual(foo(obj, 1, 2), (1, 2, ()))
+        self.assertEqual(foo(obj, 1, 2, 3), (1, 2, (3,)))
+        self.assertEqual(foo(obj, 1, 2, 3, 4), (1, 2, (3, 4)))
+
+        @njit
+        def bar(obj):
+            # Test with explicit argument
+            return (
+                obj.method_stararg(1, 2),
+                obj.method_stararg(1, 2, 3),
+                obj.method_stararg(1, 2, 3, 4),
+            )
+
+        self.assertEqual(
+            bar(obj),
+            (
+                (1, 2, ()),
+                (1, 2, (3,)),
+                (1, 2, (3, 4))
+            ),
+        )
+
+        # Check cases that put tuple type into stararg
+        # NOTE: the expected result has an extra tuple because of stararg.
+        self.assertEqual(
+            foo(obj, 1, 2, (3,)),
+            (1, 2, ((3,),)),
+        )
+        self.assertEqual(
+            foo(obj, 1, 2, (3, 4)),
+            (1, 2, ((3, 4),)),
+        )
+        self.assertEqual(
+            foo(obj, 1, 2, (3, (4, 5))),
+            (1, 2, ((3, (4, 5)),)),
+        )
+
 
 def _assert_cache_stats(cfunc, expect_hit, expect_misses):
     hit = cfunc._cache_hits[cfunc.signatures[0]]
@@ -703,6 +1103,45 @@ def run_caching_overload_method(q, cache_dir):
         _assert_cache_stats(cfunc, 1, 0)
 
 class TestIntrinsic(TestCase):
+    def test_void_return(self):
+        """
+        Verify that returning a None from codegen function is handled
+        automatically for void functions, otherwise raise exception.
+        """
+
+        @intrinsic
+        def void_func(typingctx, a):
+            sig = types.void(types.int32)
+            def codegen(context, builder, signature, args):
+                pass  # do nothing, return None, should be turned into
+                      # dummy value
+
+            return sig, codegen
+
+        @intrinsic
+        def non_void_func(typingctx, a):
+            sig = types.int32(types.int32)
+            def codegen(context, builder, signature, args):
+                pass # oops, should be returning a value here, raise exception
+            return sig, codegen
+
+        @jit(nopython=True)
+        def call_void_func():
+            void_func(1)
+            return 0
+
+        @jit(nopython=True)
+        def call_non_void_func():
+            non_void_func(1)
+            return 0
+
+        # void func should work
+        self.assertEqual(call_void_func(), 0)
+        # not void function should raise exception
+        with self.assertRaises(LoweringError) as e:
+            call_non_void_func()
+        self.assertIn('non-void function returns None', e.exception.msg)
+
     def test_ll_pointer_cast(self):
         """
         Usecase test: custom reinterpret cast to turn int values to pointers
@@ -735,7 +1174,7 @@ class TestIntrinsic(TestCase):
         def unsafe_get_ctypes_pointer(src):
             raise NotImplementedError("not callable from python")
 
-        @overload(unsafe_get_ctypes_pointer)
+        @overload(unsafe_get_ctypes_pointer, strict=False)
         def array_impl_unsafe_get_ctypes_pointer(arrtype):
             if isinstance(arrtype, types.Array):
                 unsafe_cast = unsafe_caster(types.CPointer(arrtype.dtype))
@@ -911,5 +1350,99 @@ class TestImportCythonFunction(unittest.TestCase):
         self.assertEqual(msg, str(raises.exception))
 
 
+@overload_method(MyDummyType, 'method_jit_option_check_nrt',
+                 jit_options={'_nrt': True})
+def ov_method_jit_option_check_nrt(obj):
+    def imp(obj):
+        return np.arange(10)
+    return imp
+
+
+@overload_method(MyDummyType, 'method_jit_option_check_no_nrt',
+                 jit_options={'_nrt': False})
+def ov_method_jit_option_check_no_nrt(obj):
+    def imp(obj):
+        return np.arange(10)
+    return imp
+
+
+
+@overload_attribute(MyDummyType, 'attr_jit_option_check_nrt',
+                    jit_options={'_nrt': True})
+def ov_attr_jit_option_check_nrt(obj):
+    def imp(obj):
+        return np.arange(10)
+    return imp
+
+
+@overload_attribute(MyDummyType, 'attr_jit_option_check_no_nrt',
+                    jit_options={'_nrt': False})
+def ov_attr_jit_option_check_no_nrt(obj):
+    def imp(obj):
+        return np.arange(10)
+    return imp
+
+
+class TestJitOptionsNoNRT(TestCase):
+    # Test overload*(jit_options={...}) by turning off _nrt
+
+    def check_error_no_nrt(self, func, *args, **kwargs):
+        # Check that the compilation fails with a complaint about dynamic array
+        msg = ("Only accept returning of array passed into "
+               "the function as argument")
+        with self.assertRaises(errors.TypingError) as raises:
+            func(*args, **kwargs)
+        self.assertIn(msg, str(raises.exception))
+
+    def no_nrt_overload_check(self, flag):
+        def dummy():
+            return np.arange(10)
+
+        @overload(dummy, jit_options={'_nrt': flag})
+        def ov_dummy():
+            def dummy():
+                return np.arange(10)
+            return dummy
+
+        @njit
+        def foo():
+            return dummy()
+
+        if flag:
+            self.assertPreciseEqual(foo(), np.arange(10))
+        else:
+            self.check_error_no_nrt(foo)
+
+    def test_overload_no_nrt(self):
+        self.no_nrt_overload_check(True)
+        self.no_nrt_overload_check(False)
+
+    def test_overload_method_no_nrt(self):
+        @njit
+        def udt(x):
+            return x.method_jit_option_check_nrt()
+
+        self.assertPreciseEqual(udt(mydummy), np.arange(10))
+
+        @njit
+        def udt(x):
+            return x.method_jit_option_check_no_nrt()
+
+        self.check_error_no_nrt(udt, mydummy)
+
+    def test_overload_attribute_no_nrt(self):
+        @njit
+        def udt(x):
+            return x.attr_jit_option_check_nrt
+
+        self.assertPreciseEqual(udt(mydummy), np.arange(10))
+
+        @njit
+        def udt(x):
+            return x.attr_jit_option_check_no_nrt
+
+        self.check_error_no_nrt(udt, mydummy)
+
+
 if __name__ == '__main__':
     unittest.main()
