@@ -3,9 +3,11 @@ from __future__ import print_function
 import itertools
 
 import numba.unittest_support as unittest
-from numba.controlflow import CFGraph, Loop
+from numba.controlflow import CFGraph, ControlFlowAnalysis
 from numba.compiler import compile_isolated, Flags
 from numba import types
+from numba.bytecode import FunctionIdentity, ByteCode
+from numba.utils import IS_PY3
 from .support import TestCase, tag
 
 enable_pyobj_flags = Flags()
@@ -24,12 +26,14 @@ def for_loop_usecase1(x, y):
         result += i
     return result
 
+
 def for_loop_usecase2(x, y):
     result = 0
     for i, j in enumerate(range(x, y, -1)):
         result += i * j
     return result
 
+
 def for_loop_usecase3(x, y):
     result = 0
     lst = [x, y]
@@ -37,6 +41,7 @@ def for_loop_usecase3(x, y):
         result += i
     return result
 
+
 def for_loop_usecase4(x, y):
     result = 0
     for i in range(10):
@@ -44,6 +49,7 @@ def for_loop_usecase4(x, y):
             result += 1
     return result
 
+
 def for_loop_usecase5(x, y):
     result = 0
     for i in range(x):
@@ -52,6 +58,7 @@ def for_loop_usecase5(x, y):
             break
     return result
 
+
 def for_loop_usecase6(x, y):
     result = 0
     for i in range(x):
@@ -60,6 +67,7 @@ def for_loop_usecase6(x, y):
         result += 1
     return result
 
+
 def for_loop_usecase7(x, y):
     for i in range(x):
         x = 0
@@ -69,12 +77,14 @@ def for_loop_usecase7(x, y):
             pass
     return 0
 
+
 def for_loop_usecase8(x, y):
     result = 0
     for i in range(x, y, y - x + 1):
         result += 1
     return result
 
+
 def for_loop_usecase9(x, y):
     z = 0
     for i in range(x):
@@ -88,6 +98,7 @@ def for_loop_usecase9(x, y):
 
     return z
 
+
 def for_loop_usecase10(x, y):
     for i in range(x):
         if i == y:
@@ -106,12 +117,14 @@ def while_loop_usecase1(x, y):
         i += 1
     return result
 
+
 def while_loop_usecase2(x, y):
     result = 0
     while result != x:
         result += 1
     return result
 
+
 def while_loop_usecase3(x, y):
     result = 0
     i = 0
@@ -123,6 +136,7 @@ def while_loop_usecase3(x, y):
             j += 1
     return result
 
+
 def while_loop_usecase4(x, y):
     result = 0
     while True:
@@ -131,6 +145,7 @@ def while_loop_usecase4(x, y):
             break
     return result
 
+
 def while_loop_usecase5(x, y):
     result = 0
     while result < x:
@@ -140,6 +155,7 @@ def while_loop_usecase5(x, y):
         result += 1
     return result
 
+
 def ifelse_usecase1(x, y):
     if x > 0:
         pass
@@ -149,6 +165,7 @@ def ifelse_usecase1(x, y):
         pass
     return True
 
+
 def ifelse_usecase2(x, y):
     if x > y:
         return 1
@@ -157,6 +174,7 @@ def ifelse_usecase2(x, y):
     else:
         return 3
 
+
 def ifelse_usecase3(x, y):
     if x > 0:
         if y > 0:
@@ -170,6 +188,7 @@ def ifelse_usecase3(x, y):
     else:
         return 0
 
+
 def ifelse_usecase4(x, y):
     if x == y:
         return 1
@@ -194,6 +213,14 @@ def double_infinite_loop(x, y):
 
     return i, L
 
+
+def try_except_usecase():
+    try:
+        pass
+    except Exception:
+        pass
+
+
 class TestFlowControl(TestCase):
 
     def run_test(self, pyfunc, x_operands, y_operands,
@@ -454,25 +481,24 @@ class TestCFGraph(TestCase):
                 if z:
                     return ...
         """
-        g = self.from_adj_list(
-            {0: [7],
-             7: [10, 60],
-             10: [13],
-             13: [20],
-             20: [56, 23],
-             23: [32, 44],
-             32: [20],
-             44: [20],
-             56: [57],
-             57: [7],
-             60: [61],
-             61: [68],
-             68: [87, 71],
-             71: [80, 68],
-             80: [],
-             87: [88],
-             88: []}
-            )
+        g = self.from_adj_list({0: [7],
+                                7: [10, 60],
+                                10: [13],
+                                13: [20],
+                                20: [56, 23],
+                                23: [32, 44],
+                                32: [20],
+                                44: [20],
+                                56: [57],
+                                57: [7],
+                                60: [61],
+                                61: [68],
+                                68: [87, 71],
+                                71: [80, 68],
+                                80: [],
+                                87: [88],
+                                88: []}
+                               )
         g.set_entry_point(0)
         g.process()
         return g
@@ -670,46 +696,46 @@ class TestCFGraph(TestCase):
         g = self.multiple_exits()
         doms = g.dominators()
         self.check_dominators(doms,
-            {0: [0],
-             7: [0, 7],
-             10: [0, 7, 10],
-             19: [0, 7, 10, 19],
-             23: [0, 7, 10, 23],
-             29: [0, 7, 10, 23, 29],
-             36: [0, 7, 36],
-             37: [0, 7, 37],
-             })
+                              {0: [0],
+                               7: [0, 7],
+                               10: [0, 7, 10],
+                               19: [0, 7, 10, 19],
+                               23: [0, 7, 10, 23],
+                               29: [0, 7, 10, 23, 29],
+                               36: [0, 7, 36],
+                               37: [0, 7, 37],
+                               })
         g = self.multiple_loops()
         doms = g.dominators()
         self.check_dominators(doms,
-            {0: [0],
-             7: [0, 7],
-             10: [0, 10, 7],
-             13: [0, 10, 13, 7],
-             20: [0, 10, 20, 13, 7],
-             23: [0, 20, 23, 7, 10, 13],
-             32: [32, 0, 20, 23, 7, 10, 13],
-             44: [0, 20, 23, 7, 10, 44, 13],
-             56: [0, 20, 7, 56, 10, 13],
-             57: [0, 20, 7, 56, 57, 10, 13],
-             60: [0, 60, 7],
-             61: [0, 60, 61, 7],
-             68: [0, 68, 60, 61, 7],
-             71: [0, 68, 71, 7, 60, 61],
-             80: [80, 0, 68, 71, 7, 60, 61],
-             87: [0, 68, 87, 7, 60, 61],
-             88: [0, 68, 87, 88, 7, 60, 61]
-             })
+                              {0: [0],
+                               7: [0, 7],
+                               10: [0, 10, 7],
+                               13: [0, 10, 13, 7],
+                               20: [0, 10, 20, 13, 7],
+                               23: [0, 20, 23, 7, 10, 13],
+                               32: [32, 0, 20, 23, 7, 10, 13],
+                               44: [0, 20, 23, 7, 10, 44, 13],
+                               56: [0, 20, 7, 56, 10, 13],
+                               57: [0, 20, 7, 56, 57, 10, 13],
+                               60: [0, 60, 7],
+                               61: [0, 60, 61, 7],
+                               68: [0, 68, 60, 61, 7],
+                               71: [0, 68, 71, 7, 60, 61],
+                               80: [80, 0, 68, 71, 7, 60, 61],
+                               87: [0, 68, 87, 7, 60, 61],
+                               88: [0, 68, 87, 88, 7, 60, 61]
+                               })
         g = self.infinite_loop1()
         doms = g.dominators()
         self.check_dominators(doms,
-            {0: [0],
-             6: [0, 6],
-             10: [0, 10],
-             13: [0, 10, 13],
-             19: [0, 10, 19, 13],
-             26: [0, 10, 13, 26],
-             })
+                              {0: [0],
+                               6: [0, 6],
+                               10: [0, 10],
+                               13: [0, 10, 13],
+                               19: [0, 10, 19, 13],
+                               26: [0, 10, 13, 26],
+                               })
 
     def test_post_dominators_loopless(self):
         def eq_(d, l):
@@ -733,36 +759,36 @@ class TestCFGraph(TestCase):
         g = self.multiple_exits()
         doms = g.post_dominators()
         self.check_dominators(doms,
-            {0: [0, 7],
-             7: [7],
-             10: [10],
-             19: [19],
-             23: [23],
-             29: [29, 37],
-             36: [36, 37],
-             37: [37],
-            })
+                              {0: [0, 7],
+                               7: [7],
+                               10: [10],
+                               19: [19],
+                               23: [23],
+                               29: [29, 37],
+                               36: [36, 37],
+                               37: [37],
+                               })
         g = self.multiple_loops()
         doms = g.post_dominators()
         self.check_dominators(doms,
-            {0: [0, 60, 68, 61, 7],
-             7: [60, 68, 61, 7],
-             10: [68, 7, 10, 13, 20, 56, 57, 60, 61],
-             13: [68, 7, 13, 20, 56, 57, 60, 61],
-             20: [20, 68, 7, 56, 57, 60, 61],
-             23: [68, 7, 20, 23, 56, 57, 60, 61],
-             32: [32, 68, 7, 20, 56, 57, 60, 61],
-             44: [68, 7, 44, 20, 56, 57, 60, 61],
-             56: [68, 7, 56, 57, 60, 61],
-             57: [57, 60, 68, 61, 7],
-             60: [60, 68, 61],
-             61: [68, 61],
-             68: [68],
-             71: [71],
-             80: [80],
-             87: [88, 87],
-             88: [88]
-             })
+                              {0: [0, 60, 68, 61, 7],
+                               7: [60, 68, 61, 7],
+                               10: [68, 7, 10, 13, 20, 56, 57, 60, 61],
+                               13: [68, 7, 13, 20, 56, 57, 60, 61],
+                               20: [20, 68, 7, 56, 57, 60, 61],
+                               23: [68, 7, 20, 23, 56, 57, 60, 61],
+                               32: [32, 68, 7, 20, 56, 57, 60, 61],
+                               44: [68, 7, 44, 20, 56, 57, 60, 61],
+                               56: [68, 7, 56, 57, 60, 61],
+                               57: [57, 60, 68, 61, 7],
+                               60: [60, 68, 61],
+                               61: [68, 61],
+                               68: [68],
+                               71: [71],
+                               80: [80],
+                               87: [88, 87],
+                               88: [88]
+                               })
 
     def test_post_dominators_infinite_loops(self):
         # Post-dominators with infinite loops need special care
@@ -770,21 +796,92 @@ class TestCFGraph(TestCase):
         g = self.infinite_loop1()
         doms = g.post_dominators()
         self.check_dominators(doms,
-            {0: [0],
-             6: [6],
-             10: [10, 13],
-             13: [13],
-             19: [19],
-             26: [26],
-             })
+                              {0: [0],
+                               6: [6],
+                               10: [10, 13],
+                               13: [13],
+                               19: [19],
+                               26: [26],
+                               })
         g = self.infinite_loop2()
         doms = g.post_dominators()
         self.check_dominators(doms,
-            {0: [0, 3],
-             3: [3],
-             9: [9],
-             16: [16],
-             })
+                              {0: [0, 3],
+                               3: [3],
+                               9: [9],
+                               16: [16],
+                               })
+
+    def test_dominator_tree(self):
+        def check(graph, expected):
+            domtree = graph.dominator_tree()
+            self.assertEqual(domtree, expected)
+
+        check(self.loopless1(),
+              {0: {12, 18, 21}, 12: set(), 18: set(), 21: set()})
+        check(self.loopless2(),
+              {12: set(), 18: set(), 21: {34, 42}, 34: set(), 42: set(),
+               99: {18, 12, 21}})
+        check(self.loopless1_dead_nodes(),
+              {0: {12, 18, 21}, 12: set(), 18: set(), 21: set()})
+        check(self.multiple_loops(),
+              {0: {7}, 7: {10, 60}, 60: {61}, 61: {68}, 68: {71, 87},
+               87: {88}, 88: set(), 71: {80}, 80: set(), 10: {13},
+               13: {20}, 20: {56, 23}, 23: {32, 44}, 44: set(),
+               32: set(), 56: {57}, 57: set()})
+        check(self.multiple_exits(),
+              {0: {7}, 7: {10, 36, 37}, 36: set(), 10: {19, 23},
+               23: {29}, 29: set(), 37: set(), 19: set()})
+        check(self.infinite_loop1(),
+              {0: {10, 6}, 6: set(), 10: {13}, 13: {26, 19}, 19: set(),
+               26: set()})
+        check(self.infinite_loop2(),
+              {0: {3}, 3: {16, 9}, 9: set(), 16: set()})
+
+    def test_immediate_dominators(self):
+        def check(graph, expected):
+            idoms = graph.immediate_dominators()
+            self.assertEqual(idoms, expected)
+
+        check(self.loopless1(),
+              {0: 0, 12: 0, 18: 0, 21: 0})
+        check(self.loopless2(),
+              {18: 99, 12: 99, 21: 99, 42: 21, 34: 21, 99: 99})
+        check(self.loopless1_dead_nodes(),
+              {0: 0, 12: 0, 18: 0, 21: 0})
+        check(self.multiple_loops(),
+              {0: 0, 7: 0, 10: 7, 13: 10, 20: 13, 23: 20,
+               32: 23, 44: 23, 56: 20, 57: 56, 60: 7, 61: 60,
+               68: 61, 71: 68, 80: 71, 87: 68, 88: 87})
+        check(self.multiple_exits(),
+              {0:0, 7: 0, 10: 7, 19: 10, 23: 10, 29: 23, 36: 7, 37: 7})
+        check(self.infinite_loop1(),
+              {0: 0, 6: 0, 10: 0, 13: 10, 19: 13, 26: 13})
+        check(self.infinite_loop2(),
+              {0: 0, 3: 0, 9: 3, 16: 3})
+
+    def test_dominance_frontier(self):
+        def check(graph, expected):
+            df = graph.dominance_frontier()
+            self.assertEqual(df, expected)
+
+        check(self.loopless1(),
+              {0: set(), 12: {21}, 18: {21}, 21: set()})
+        check(self.loopless2(),
+              {18: {21}, 12: {21}, 21: set(), 42: set(), 34: set(), 99: set()})
+        check(self.loopless1_dead_nodes(),
+              {0: set(), 12: {21}, 18: {21}, 21: set()})
+        check(self.multiple_loops(),
+              {0: set(), 7: {7}, 10: {7}, 13: {7}, 20: {20, 7}, 23: {20},
+               32: {20}, 44: {20}, 56: {7}, 57: {7}, 60: set(), 61: set(),
+               68: {68}, 71: {68}, 80: set(), 87: set(), 88: set()})
+        check(self.multiple_exits(),
+              {0: set(), 7: {7}, 10: {37, 7}, 19: set(),
+               23: {37, 7}, 29: {37}, 36: {37}, 37: set()})
+        check(self.infinite_loop1(),
+              {0: set(), 6: set(), 10: set(), 13: {13}, 19: {13}, 26: {13}})
+        check(self.infinite_loop2(),
+              {0: set(), 3: {3}, 9: {3}, 16: {3}})
 
     def test_backbone_loopless(self):
         for g in [self.loopless1(), self.loopless1_dead_nodes()]:
@@ -872,6 +969,275 @@ class TestCFGraph(TestCase):
         for node in [7, 10, 23]:
             self.assertEqual(g.in_loops(node), [loop])
 
+    def test_equals(self):
+
+        def get_new():
+            g = self.from_adj_list({0: [18, 12], 12: [21], 18: [21], 21: []})
+            g.set_entry_point(0)
+            g.process()
+            return g
+
+        x = get_new()
+        y = get_new()
+
+        # identical
+        self.assertEqual(x, y)
+
+        # identical but defined in a different order
+        g = self.from_adj_list({0: [12, 18], 18: [21], 21: [], 12: [21]})
+        g.set_entry_point(0)
+        g.process()
+        self.assertEqual(x, g)
+
+        # different entry point
+        z = get_new()
+        z.set_entry_point(18)
+        z.process()
+        self.assertNotEqual(x, z)
+
+        # extra node/edge, same entry point
+        z = self.from_adj_list({0: [18, 12], 12: [21], 18: [21], 21: [22],
+                                22: []})
+        z.set_entry_point(0)
+        z.process()
+        self.assertNotEqual(x, z)
+
+        # same nodes, different edges
+        a = self.from_adj_list({0: [18, 12], 12: [0], 18: []})
+        a.set_entry_point(0)
+        a.process()
+        z = self.from_adj_list({0: [18, 12], 12: [18], 18: []})
+        z.set_entry_point(0)
+        z.process()
+        self.assertNotEqual(a, z)
+
+
+class TestRealCodeDomFront(TestCase):
+    """Test IDOM and DOMFRONT computation on real python bytecode.
+    Note: there will be less testing on IDOM (esp in loop) because of
+    the extra blocks inserted by the interpreter.  But, testing on DOMFRONT
+    (which depends on IDOM) is easier.
+
+    Testing is done by associating names to basicblock by using globals of
+    the pattern "SET_BLOCK_<name>", which are scanned by
+    `.get_cfa_and_namedblocks` into *namedblocks* dictionary.  That way, we
+    can check that a block of a certain name is a IDOM or DOMFRONT of another
+    named block.
+    """
+    def cfa(self, bc):
+        cfa = ControlFlowAnalysis(bc)
+        cfa.run()
+        return cfa
+
+    def get_cfa_and_namedblocks(self, fn):
+        fid = FunctionIdentity.from_function(fn)
+        bc = ByteCode(func_id=fid)
+        cfa = self.cfa(bc)
+        namedblocks = self._scan_namedblocks(bc, cfa)
+
+        #### To debug, uncomment below
+        # print(bc.dump())
+        # print("IDOMS")
+        # for k, v in sorted(cfa.graph.immediate_dominators().items()):
+        #     print('{} -> {}'.format(k, v))
+        # print("DOMFRONT")
+        # for k, vs in sorted(cfa.graph.dominance_frontier().items()):
+        #     print('{} -> {}'.format(k, vs))
+        # print(namedblocks)
+        # cfa.graph.render_dot().view()
+
+        return cfa, namedblocks
+
+    def _scan_namedblocks(self, bc, cfa):
+        """Scan namedblocks as denoted by a LOAD_GLOBAL bytecode referring
+        to global variables with the pattern "SET_BLOCK_<name>", where "<name>"
+        would be the name for the current block.
+        """
+        namedblocks = {}
+        blocks = sorted([x.offset for x in cfa.iterblocks()])
+        prefix = 'SET_BLOCK_'
+
+        for inst in bc:
+            # Find LOAD_GLOBAL that refers to "SET_BLOCK_<name>"
+            if inst.opname == 'LOAD_GLOBAL':
+                gv = bc.co_names[inst.arg]
+                if gv.startswith(prefix):
+                    name = gv[len(prefix):]
+                    # Find the block where this instruction resides
+                    for s, e in zip(blocks, blocks[1:] + [blocks[-1] + 1]):
+                        if s <= inst.offset < e:
+                            break
+                    else:
+                        raise AssertionError('unreachable loop')
+                    blkno = s
+                    namedblocks[name] = blkno
+        return namedblocks
+
+    def test_loop(self):
+        def foo(n):
+            c = 0
+            SET_BLOCK_A                     # noqa: F821
+            i = 0
+            while SET_BLOCK_B0:             # noqa: F821
+                SET_BLOCK_B1                # noqa: F821
+                c += 1
+                i += 1
+            SET_BLOCK_C                     # noqa: F821
+            return c
+
+        cfa, blkpts = self.get_cfa_and_namedblocks(foo)
+        idoms = cfa.graph.immediate_dominators()
+
+        self.assertEqual(blkpts['B0'], idoms[blkpts['B1']])
+
+        domfront = cfa.graph.dominance_frontier()
+        self.assertFalse(domfront[blkpts['A']])
+        self.assertFalse(domfront[blkpts['C']])
+        self.assertEqual({blkpts['B0']}, domfront[blkpts['B1']])
+
+    def test_loop_nested_and_break(self):
+        def foo(n):
+            SET_BLOCK_A                     # noqa: F821
+            while SET_BLOCK_B0:             # noqa: F821
+                SET_BLOCK_B1                # noqa: F821
+                while SET_BLOCK_C0:         # noqa: F821
+                    SET_BLOCK_C1            # noqa: F821
+                    if SET_BLOCK_D0:        # noqa: F821
+                        SET_BLOCK_D1        # noqa: F821
+                        break
+                    elif n:
+                        SET_BLOCK_D2        # noqa: F821
+                    SET_BLOCK_E             # noqa: F821
+                SET_BLOCK_F                 # noqa: F821
+            SET_BLOCK_G                     # noqa: F821
+
+        cfa, blkpts = self.get_cfa_and_namedblocks(foo)
+        idoms = cfa.graph.immediate_dominators()
+        self.assertEqual(blkpts['D0'], blkpts['C1'])
+        self.assertEqual(blkpts['C0'], idoms[blkpts['C1']])
+
+        domfront = cfa.graph.dominance_frontier()
+        self.assertFalse(domfront[blkpts['A']])
+        self.assertFalse(domfront[blkpts['G']])
+        self.assertEqual({blkpts['B0']}, domfront[blkpts['B1']])
+        # 2 domfront members for C1
+        # C0 because of the loop; F because of the break.
+        self.assertEqual({blkpts['C0'], blkpts['F']}, domfront[blkpts['C1']])
+        self.assertEqual({blkpts['F']}, domfront[blkpts['D1']])
+        self.assertEqual({blkpts['E']}, domfront[blkpts['D2']])
+        self.assertEqual({blkpts['C0']}, domfront[blkpts['E']])
+        self.assertEqual({blkpts['B0']}, domfront[blkpts['F']])
+        self.assertEqual({blkpts['B0']}, domfront[blkpts['B0']])
+
+    def test_if_else(self):
+        def foo(a, b):
+            c = 0
+            SET_BLOCK_A           # noqa: F821
+            if a < b:
+                SET_BLOCK_B       # noqa: F821
+                c = 1
+            elif SET_BLOCK_C0:    # noqa: F821
+                SET_BLOCK_C1      # noqa: F821
+                c = 2
+            else:
+                SET_BLOCK_D       # noqa: F821
+                c = 3
+
+            SET_BLOCK_E           # noqa: F821
+            if a % b == 0:
+                SET_BLOCK_F       # noqa: F821
+                c += 1
+            SET_BLOCK_G           # noqa: F821
+            return c
+
+        cfa, blkpts = self.get_cfa_and_namedblocks(foo)
+
+        idoms = cfa.graph.immediate_dominators()
+        self.assertEqual(blkpts['A'], idoms[blkpts['B']])
+        self.assertEqual(blkpts['A'], idoms[blkpts['C0']])
+        self.assertEqual(blkpts['C0'], idoms[blkpts['C1']])
+        self.assertEqual(blkpts['C0'], idoms[blkpts['D']])
+        self.assertEqual(blkpts['A'], idoms[blkpts['E']])
+        self.assertEqual(blkpts['E'], idoms[blkpts['F']])
+        self.assertEqual(blkpts['E'], idoms[blkpts['G']])
+
+        domfront = cfa.graph.dominance_frontier()
+        self.assertFalse(domfront[blkpts['A']])
+        self.assertFalse(domfront[blkpts['E']])
+        self.assertFalse(domfront[blkpts['G']])
+        self.assertEqual({blkpts['E']}, domfront[blkpts['B']])
+        self.assertEqual({blkpts['E']}, domfront[blkpts['C0']])
+        self.assertEqual({blkpts['E']}, domfront[blkpts['C1']])
+        self.assertEqual({blkpts['E']}, domfront[blkpts['D']])
+        self.assertEqual({blkpts['G']}, domfront[blkpts['F']])
+
+    def test_if_else_nested(self):
+        def foo():
+            if SET_BLOCK_A0:                # noqa: F821
+                SET_BLOCK_A1                # noqa: F821
+                if SET_BLOCK_B0:            # noqa: F821
+                    SET_BLOCK_B1            # noqa: F821
+                    a = 0
+                else:
+                    if SET_BLOCK_C0:        # noqa: F821
+                        SET_BLOCK_C1        # noqa: F821
+                        a = 1
+                    else:
+                        SET_BLOCK_C2        # noqa: F821
+                        a = 2
+                    SET_BLOCK_D             # noqa: F821
+                SET_BLOCK_E                 # noqa: F821
+            SET_BLOCK_F                     # noqa: F821
+            return a
+
+        cfa, blkpts = self.get_cfa_and_namedblocks(foo)
+
+        idoms = cfa.graph.immediate_dominators()
+        self.assertEqual(blkpts['A0'], idoms[blkpts['A1']])
+        self.assertEqual(blkpts['A1'], idoms[blkpts['B1']])
+        self.assertEqual(blkpts['A1'], idoms[blkpts['C0']])
+        self.assertEqual(blkpts['C0'], idoms[blkpts['D']])
+        self.assertEqual(blkpts['A1'], idoms[blkpts['E']])
+        self.assertEqual(blkpts['A0'], idoms[blkpts['F']])
+
+        domfront = cfa.graph.dominance_frontier()
+        self.assertFalse(domfront[blkpts['A0']])
+        self.assertFalse(domfront[blkpts['F']])
+        self.assertEqual({blkpts['E']}, domfront[blkpts['B1']])
+        self.assertEqual({blkpts['D']}, domfront[blkpts['C1']])
+        self.assertEqual({blkpts['E']}, domfront[blkpts['D']])
+        self.assertEqual({blkpts['F']}, domfront[blkpts['E']])
+
+    def test_infinite_loop(self):
+        def foo():
+            SET_BLOCK_A                     # noqa: F821
+            while True:  # infinite loop
+                if SET_BLOCK_B:             # noqa: F821
+                    SET_BLOCK_C             # noqa: F821
+                    return
+                SET_BLOCK_D                 # noqa: F821
+            SET_BLOCK_E                     # noqa: F821
+
+        # Whether infinite loop (i.g. while True) are optimized out.
+        infinite_loop_opt_out = bool(IS_PY3)
+
+        cfa, blkpts = self.get_cfa_and_namedblocks(foo)
+
+        idoms = cfa.graph.immediate_dominators()
+        if infinite_loop_opt_out:
+            self.assertNotIn(blkpts['E'], idoms)
+        self.assertEqual(blkpts['B'], idoms[blkpts['C']])
+        self.assertEqual(blkpts['B'], idoms[blkpts['D']])
+
+        domfront = cfa.graph.dominance_frontier()
+        if infinite_loop_opt_out:
+            self.assertNotIn(blkpts['E'], domfront)
+        self.assertFalse(domfront[blkpts['A']])
+        self.assertFalse(domfront[blkpts['C']])
+        if infinite_loop_opt_out:
+            self.assertEqual({blkpts['B']}, domfront[blkpts['B']])
+            self.assertEqual({blkpts['B']}, domfront[blkpts['D']])
+
 
 if __name__ == '__main__':
     unittest.main()
