@@ -6,6 +6,7 @@ from itertools import product, cycle
 import sys
 import warnings
 from numbers import Number, Integral
+import platform
 
 import numpy as np
 
@@ -15,6 +16,8 @@ from numba.numpy_support import version as numpy_version
 from .support import TestCase, tag
 from .matmul_usecase import matmul_usecase, needs_matmul, needs_blas
 
+_is_armv7l = platform.machine() == 'armv7l'
+
 try:
     import scipy.linalg.cython_lapack
     has_lapack = True
@@ -66,10 +69,10 @@ class TestProduct(TestCase):
         Check performance warning(s) for non-contiguity.
         """
         with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter('always', errors.PerformanceWarning)
+            warnings.simplefilter('always', errors.NumbaPerformanceWarning)
             yield
         self.assertGreaterEqual(len(w), 1)
-        self.assertIs(w[0].category, errors.PerformanceWarning)
+        self.assertIs(w[0].category, errors.NumbaPerformanceWarning)
         self.assertIn("faster on contiguous arrays", str(w[0].message))
         self.assertEqual(w[0].filename, pyfunc.__code__.co_filename)
         # This works because our functions are one-liners
@@ -82,10 +85,33 @@ class TestProduct(TestCase):
             self.assertPreciseEqual(got, expected, ignore_sign_on_zero=True)
             del got, expected
 
+
+    def _aligned_copy(self, arr):
+        # This exists for armv7l because NumPy wants aligned arrays for the
+        # `out` arg of functions, but np.empty/np.copy doesn't seem to always
+        # produce them, in particular for complex dtypes
+        size = (arr.size + 1) * arr.itemsize + 1
+        datasize = arr.size * arr.itemsize
+        tmp = np.empty(size, dtype=np.uint8)
+        for i in range(arr.itemsize + 1):
+            new = tmp[i : i + datasize].view(dtype=arr.dtype)
+            if new.flags.aligned:
+                break
+        else:
+            raise Exception("Could not obtain aligned array")
+        if arr.flags.c_contiguous:
+            new = np.reshape(new, arr.shape, order='C')
+        else:
+            new = np.reshape(new, arr.shape, order='F')
+        new[:] = arr[:]
+        assert new.flags.aligned
+        return new
+
     def check_func_out(self, pyfunc, cfunc, args, out):
+        copier = self._aligned_copy if _is_armv7l else np.copy
         with self.assertNoNRTLeak():
-            expected = np.copy(out)
-            got = np.copy(out)
+            expected = copier(out)
+            got = copier(out)
             self.assertIs(pyfunc(*args, out=expected), expected)
             self.assertIs(cfunc(*args, out=got), got)
             self.assertPreciseEqual(got, expected, ignore_sign_on_zero=True)
@@ -499,6 +525,20 @@ class TestLinalgBase(TestCase):
 
         return Q
 
+    def shape_with_0_input(self, *args):
+        """
+        returns True if an input argument has a dimension that is zero
+        and Numpy version is < 1.13, else False. This is due to behaviour
+        changes in handling dimension zero arrays:
+        https://github.com/numpy/numpy/issues/10573
+        """
+        if numpy_version < (1, 13):
+            for x in args:
+                if isinstance(x, np.ndarray):
+                    if 0 in x.shape:
+                        return True
+        return False
+
     def assert_error(self, cfunc, args, msg, err=ValueError):
         with self.assertRaises(err) as raises:
             cfunc(*args)
@@ -585,6 +625,10 @@ class TestLinalgBase(TestCase):
         msg = "Invalid norm order for matrices."
         self.assert_error(cfunc, args, msg, ValueError)
 
+    def assert_raise_on_empty(self, cfunc, args):
+        msg = 'Arrays cannot be empty'
+        self.assert_error(cfunc, args, msg, np.linalg.LinAlgError)
+
 
 class TestTestLinalgBase(TestCase):
     """
@@ -733,6 +777,9 @@ class TestLinalgInv(TestLinalgBase):
             a = self.specific_sample_matrix((n, n), dtype, order)
             check(a)
 
+        # 0 dimensioned matrix
+        check(np.empty((0, 0)))
+
         # Non square matrix
         self.assert_non_square(cfunc, (np.ones((2, 3)),))
 
@@ -777,6 +824,11 @@ class TestLinalgCholesky(TestLinalgBase):
         cfunc = jit(nopython=True)(cholesky_matrix)
 
         def check(a):
+            if self.shape_with_0_input(a):
+                # has shape with 0 on input, numpy will fail,
+                # just make sure Numba runs without error
+                cfunc(a)
+                return
             expected = cholesky_matrix(a)
             got = cfunc(a)
             use_reconstruction = False
@@ -810,6 +862,9 @@ class TestLinalgCholesky(TestLinalgBase):
             a = self.sample_matrix(n, dtype, order)
             check(a)
 
+        # 0 dimensioned matrix
+        check(np.empty((0, 0)))
+
         rn = "cholesky"
         # Non square matrices
         self.assert_non_square(cfunc, (np.ones((2, 3), dtype=np.float64),))
@@ -856,6 +911,11 @@ class TestLinalgEigenSystems(TestLinalgBase):
         cfunc = jit(nopython=True)(func)
 
         def check(a):
+            if self.shape_with_0_input(a):
+                # has shape with 0 on input, numpy will fail,
+                # just make sure Numba runs without error
+                cfunc(a)
+                return
             expected = func(a)
             got = cfunc(a)
             # check that the returned tuple is same length
@@ -954,6 +1014,10 @@ class TestLinalgEigenSystems(TestLinalgBase):
 
         # Test both a real and complex type as the impls are different
         for ty in [np.float32, np.complex64]:
+
+            # 0 dimensioned matrix
+            check(np.empty((0, 0), dtype=ty))
+
             # Non square matrices
             self.assert_non_square(cfunc, (np.ones((2, 3), dtype=ty),))
 
@@ -1105,6 +1169,10 @@ class TestLinalgSvd(TestLinalgBase):
         self.assert_no_nan_or_inf(cfunc,
                                   (np.array([[1., 2., ], [np.inf, np.nan]],
                                             dtype=np.float64),))
+        # empty
+        for sz in [(0, 1), (1, 0), (0, 0)]:
+            args = (np.empty(sz), True)
+            self.assert_raise_on_empty(cfunc, args)
 
 
 class TestLinalgQr(TestLinalgBase):
@@ -1197,6 +1265,10 @@ class TestLinalgQr(TestLinalgBase):
                                   (np.array([[1., 2., ], [np.inf, np.nan]],
                                             dtype=np.float64),))
 
+        # empty
+        for sz in [(0, 1), (1, 0), (0, 0)]:
+            self.assert_raise_on_empty(cfunc, (np.empty(sz),))
+
 
 class TestLinalgSystems(TestLinalgBase):
     """
@@ -1408,6 +1480,18 @@ class TestLinalgLstsq(TestLinalgSystems):
                 inner_test_loop_fn(A, dt,
                                    rcond=approx_half_rank_rcond)
 
+        # check empty arrays
+        empties = [
+        [(0, 1), (1,)], # empty A, valid b
+        [(1, 0), (1,)], # empty A, valid b
+        [(1, 1), (0,)], # valid A, empty 1D b 
+        [(1, 1), (1, 0)],  # valid A, empty 2D b 
+        ]
+
+        for A, b in empties:
+            args = (np.empty(A), np.empty(b))
+            self.assert_raise_on_empty(cfunc, args)
+
         # Test input validation
         ok = np.array([[1., 2.], [3., 4.]], dtype=np.float64)
 
@@ -1525,6 +1609,9 @@ class TestLinalgSolve(TestLinalgSystems):
                 tmp = B[:, 0].copy(order=b_order)
                 check(A, tmp)
 
+        # check empty
+        cfunc(np.empty((0, 0)), np.empty((0,)))
+
         # Test input validation
         ok = np.array([[1., 0.], [0., 1.]], dtype=np.float64)
 
@@ -1584,6 +1671,11 @@ class TestLinalgPinv(TestLinalgBase):
         cfunc = jit(nopython=True)(pinv_matrix)
 
         def check(a, **kwargs):
+            if self.shape_with_0_input(a):
+                # has shape with 0 on input, numpy will fail,
+                # just make sure Numba runs without error
+                cfunc(a, **kwargs)
+                return
             expected = pinv_matrix(a, **kwargs)
             got = cfunc(a, **kwargs)
 
@@ -1673,6 +1765,10 @@ class TestLinalgPinv(TestLinalgBase):
                 approx_half_rank_rcond = minmn * rcond
                 check(a, rcond=approx_half_rank_rcond)
 
+        # check empty
+        for sz in [(0, 1), (1, 0)]:
+            check(np.empty(sz))
+
         rn = "pinv"
 
         # Wrong dtype
@@ -1698,6 +1794,11 @@ class TestLinalgDetAndSlogdet(TestLinalgBase):
     """
 
     def check_det(self, cfunc, a, **kwargs):
+        if self.shape_with_0_input(a):
+            # has shape with 0 on input, numpy will fail,
+            # just make sure Numba runs without error
+            cfunc(a, **kwargs)
+            return
         expected = det_matrix(a, **kwargs)
         got = cfunc(a, **kwargs)
 
@@ -1711,6 +1812,11 @@ class TestLinalgDetAndSlogdet(TestLinalgBase):
             cfunc(a, **kwargs)
 
     def check_slogdet(self, cfunc, a, **kwargs):
+        if self.shape_with_0_input(a):
+            # has shape with 0 on input, numpy will fail,
+            # just make sure Numba runs without error
+            cfunc(a, **kwargs)
+            return
         expected = slogdet_matrix(a, **kwargs)
         got = cfunc(a, **kwargs)
 
@@ -1762,6 +1868,9 @@ class TestLinalgDetAndSlogdet(TestLinalgBase):
             a = np.zeros((3, 3), dtype=dtype)
             check(cfunc, a)
 
+        # check empty
+        check(cfunc, np.empty((0, 0)))
+
         # Wrong dtype
         self.assert_wrong_dtype(rn, cfunc,
                                 (np.ones((2, 2), dtype=np.int32),))
@@ -1833,13 +1942,6 @@ class TestLinalgNorm(TestLinalgSystems):
             a = self.sample_vector(10, dtype)[::3]
             check(a, ord=nrm_type)
 
-        # check that numba returns zero for empty arrays. Numpy returns zero
-        # for most norm types and raises ValueError for +/-np.inf.
-        for dtype, nrm_type, order in \
-                product(self.dtypes, nrm_types, 'FC'):
-            a = np.array([], dtype=dtype, order=order)
-            self.assertEqual(cfunc(a, nrm_type), 0.0)
-
         # Check 2D inputs:
         # test: column vector, tall, wide, square, row vector
         # prime sizes
@@ -1869,10 +1971,14 @@ class TestLinalgNorm(TestLinalgSystems):
             check(a[1, 4::3], ord=nrm_type)
 
         # check that numba returns zero for empty arrays. Numpy returns zero
-        # for some norm types and raises a variety of errors for others.
+        # for most norm types and raises ValueError for +/-np.inf.
+        # there is not a great deal of consistency in Numpy's response so
+        # it is not being emulated in Numba
         for dtype, nrm_type, order in \
                 product(self.dtypes, nrm_types, 'FC'):
-            a = np.array([[]], dtype=dtype, order=order)
+            a = np.empty((0,), dtype=dtype, order=order)
+            self.assertEqual(cfunc(a, nrm_type), 0.0)
+            a = np.empty((0, 0), dtype=dtype, order=order)
             self.assertEqual(cfunc(a, nrm_type), 0.0)
 
         rn = "norm"
@@ -1942,15 +2048,31 @@ class TestLinalgCond(TestLinalgBase):
             a = self.specific_sample_matrix(size, dtype, order)
             check(a)
 
+        # empty
+        for sz in [(0, 1), (1, 0), (0, 0)]:
+            self.assert_raise_on_empty(cfunc, (np.empty(sz),))
+
+        # singular systems to trip divide-by-zero
+        # only for np > 1.14, before this norm was computed via inversion which
+        # will fail with numpy.linalg.linalg.LinAlgError: Singular matrix
+        if numpy_version > (1, 14):
+            x = np.array([[1, 0], [0, 0]], dtype=np.float64)
+            check(x)
+            check(x, p=2)
+            x = np.array([[0, 0], [0, 0]], dtype=np.float64)
+            check(x, p=-2)
+
         # try an ill-conditioned system with 2-norm, make sure np raises an
         # overflow warning as the result is `+inf` and that the result from
         # numba matches.
         with warnings.catch_warnings():
             a = np.array([[1.e308, 0], [0, 0.1]], dtype=np.float64)
-            warnings.simplefilter("error", RuntimeWarning)
-            self.assertRaisesRegexp(RuntimeWarning,
-                                    'overflow encountered in.*',
-                                    check, a)
+            if numpy_version < (1, 15):
+                # overflow warning is silenced in np >= 1.15
+                warnings.simplefilter("error", RuntimeWarning)
+                self.assertRaisesRegexp(RuntimeWarning,
+                                        'overflow encountered in.*',
+                                        check, a)
             warnings.simplefilter("ignore", RuntimeWarning)
             check(a)
 
@@ -2058,6 +2180,11 @@ class TestLinalgMatrixRank(TestLinalgSystems):
             self.assertEqual(cfunc(a), 1)
             check(a)
 
+        # empty
+        for sz in [(0, 1), (1, 0), (0, 0)]:
+            for tol in [None, 1e-13]:
+                self.assert_raise_on_empty(cfunc, (np.empty(sz), tol))
+
         rn = "matrix_rank"
 
         # Wrong dtype
@@ -2113,6 +2240,8 @@ class TestLinalgMatrixPower(TestLinalgBase):
                 product(sizes, powers, self.dtypes, 'FC'):
             a = self.specific_sample_matrix(size, dtype, order)
             check(a, pwr)
+            a = np.empty((0, 0), dtype=dtype, order=order)
+            check(a, pwr)
 
         rn = "matrix_power"
 
@@ -2124,6 +2253,11 @@ class TestLinalgMatrixPower(TestLinalgBase):
         self.assert_wrong_dtype(rn, cfunc,
                                 (np.ones((2, 2), dtype=np.int32), 1))
 
+        # non square system
+        args = (np.ones((3, 5)), 1)
+        msg = 'input must be a square array'
+        self.assert_error(cfunc, args, msg)
+
         # Dimension issue
         self.assert_wrong_dimensions(rn, cfunc,
                                      (np.ones(10, dtype=np.float64), 1))
@@ -2185,6 +2319,10 @@ class TestTrace(TestLinalgBase):
             check(a, offset=offset)
             if offset == 0:
                 check(a)
+            a = np.empty((0, 0), dtype=dtype, order=order)
+            check(a, offset=offset)
+            if offset == 0:
+                check(a)
 
         rn = "trace"
 
@@ -2310,6 +2448,10 @@ class TestBasics(TestLinalgSystems):  # TestLinalgSystems for 1d test
 
         self._assert_wrong_dim("kron", cfunc)
 
+        args = (np.empty(10)[::2], np.empty(10)[::2])
+        msg = "only supports 'C' or 'F' layout"
+        self.assert_error(cfunc, args, msg, err=errors.TypingError)
+
 
 if __name__ == '__main__':
     unittest.main()
