@@ -5,6 +5,7 @@ import contextlib
 import itertools
 import math
 import sys
+import ctypes as ct
 import numpy as np
 
 from numba.compiler import compile_isolated, Flags
@@ -694,6 +695,11 @@ class TestLists(MemoryLeakMixin, TestCase):
         # Fail in malloc()
         with self.assertRaises(MemoryError):
             cfunc(1, 2**58)
+        if sys.platform.startswith('darwin'):
+            libc = ct.CDLL('libc.dylib')
+            libc.printf("###Please ignore the above error message i.e. \
+can't allocate region. It is in fact the purpose of this test to \
+request more memory than can be provided###\n".encode("UTF-8"))
         # Overflow size computation when multiplying by item size
         with self.assertRaises(MemoryError):
             cfunc(1, 2**62)
@@ -826,7 +832,7 @@ class TestUnboxing(MemoryLeakMixin, TestCase):
             cfunc(lst)
         if utils.IS_PY3:
             msg = ("can't unbox heterogeneous list: "
-                   "tuple({0} x 1) != tuple({0} x 2)")
+                   "UniTuple({0} x 1) != UniTuple({0} x 2)")
             self.assertEqual(str(raises.exception), msg.format(types.intp))
         else:
             self.assertEqual(
