@@ -37,6 +37,7 @@ def lift2(x):
 
 def lift3(x):
     # Output variable from the loop
+    _ = object()
     a = np.arange(5, dtype=np.int64)
     c = 0
     for i in range(a.shape[0]):
@@ -45,6 +46,7 @@ def lift3(x):
 
 def lift4(x):
     # Output two variables from the loop
+    _ = object()
     a = np.arange(5, dtype=np.int64)
     c = 0
     d = 0
@@ -54,6 +56,7 @@ def lift4(x):
     return c + d
 
 def lift5(x):
+    _ = object()
     a = np.arange(4)
     for i in range(a.shape[0]):
         # Inner has a break statement
@@ -86,13 +89,14 @@ def reject1(x):
 
 
 def reject_gen1(x):
+    _ = object()
     a = np.arange(4)
     for i in range(a.shape[0]):
         # Inner is a generator => cannot loop-lift
         yield a[i]
 
 def reject_gen2(x):
-    # Outer needs object mode because of np.empty()
+    _ = object()
     a = np.arange(3)
     for i in range(a.size):
         # Middle has a yield => cannot loop-lift
@@ -106,8 +110,8 @@ def reject_gen2(x):
 def reject_npm1(x):
     a = np.empty(3, dtype=np.int32)
     for i in range(a.size):
-        # Inner uses np.arange() => cannot loop-lift unless
-        # enable_pyobject_looplift is enabled.
+        # Inner uses object() => cannot loop-lift
+        _ = object()
         a[i] = np.arange(i + 1)[i]
 
     return a
@@ -484,7 +488,7 @@ class TestLoopLiftingInAction(MemoryLeakMixin, TestCase):
         """
         https://github.com/numba/numba/issues/2179
 
-        Looplifting transformation is using the wrong verion of variable `h`.
+        Looplifting transformation is using the wrong version of variable `h`.
         """
         from numba import jit
 
@@ -532,6 +536,52 @@ class TestLoopLiftingInAction(MemoryLeakMixin, TestCase):
         # Ensure that is really a new overload for the lifted loop
         self.assertEqual(len(lifted.signatures), 2)
 
+    def test_lift_listcomp_block0(self):
+
+        def foo(X):
+            [y for y in (1,)]
+            for x in (1,):
+                pass
+            return X
+
+        # this is not nice, if you have 2+? liftable loops with one of them
+        # being list comp and in block 0 and force objmode compilation is set,
+        # in py27 this leads to a BUILD_LIST that is a lifting candidate with an
+        # entry of block 0, this is a problem as the loop lift prelude would be
+        # written to block -1 and havoc ensues. Therefore block 0 loop lifts
+        # are banned under this set of circumstances.
+
+        # check all compile and execute
+        from numba import jit
+        f = jit()(foo)
+        f(1)
+        self.assertEqual(f.overloads[f.signatures[0]].lifted, ())
+
+        f = jit(forceobj=True)(foo)
+        f(1)
+        self.assertEqual(len(f.overloads[f.signatures[0]].lifted), 1)
+
+    def test_lift_objectmode_issue_4223(self):
+        from numba import jit
+
+        @jit
+        def foo(a, b, c, d, x0, y0, n):
+            xs, ys = np.zeros(n), np.zeros(n)
+            xs[0], ys[0] = x0, y0
+            for i in np.arange(n-1):
+                xs[i+1] = np.sin(a * ys[i]) + c * np.cos(a * xs[i])
+                ys[i+1] = np.sin(b * xs[i]) + d * np.cos(b * ys[i])
+            object() # ensure object mode
+            return xs, ys
+
+        kwargs = dict(a=1.7, b=1.7, c=0.6, d=1.2, x0=0, y0=0, n=200)
+        got = foo(**kwargs)
+        expected = foo.py_func(**kwargs)
+        self.assertPreciseEqual(got[0], expected[0])
+        self .assertPreciseEqual(got[1], expected[1])
+        [lifted] = foo.overloads[foo.signatures[0]].lifted
+        self.assertEqual(len(lifted.nopython_signatures), 1)
+
 
 if __name__ == '__main__':
     unittest.main()
