@@ -236,6 +236,9 @@ class TestCC(BasePYCCTest):
         self.check_compile_for_cpu("host")
 
     @tag('important')
+    @unittest.skipIf(sys.platform == 'darwin' and
+                     sys.version_info[:2] == (3, 8),
+                     'distutils incorrectly using gcc on python 3.8 builds')
     def test_compile_helperlib(self):
         with self.check_cc_compiled(self._test_module.cc_helperlib) as lib:
             res = lib.power(2, 7)
@@ -301,6 +304,14 @@ class TestCC(BasePYCCTest):
                 """ % dict(has_blas=has_blas)
             self.check_cc_compiled_in_subprocess(lib, code)
 
+    def test_c_extension_usecase(self):
+        # Test C-extensions
+        with self.check_cc_compiled(self._test_module.cc_nrt) as lib:
+            arr = np.arange(128, dtype=np.intp)
+            got = lib.dict_usecase(arr)
+            expect = arr * arr
+            self.assertPreciseEqual(got, expect)
+
 
 @_skip_missing_compilers
 class TestDistutilsSupport(TestCase):
