@@ -262,6 +262,22 @@ class TestRandom(BaseTest):
         self._check_random_seed(numpy_seed, jit_nullary("np.random.sample"))
         self._check_random_seed(numpy_seed, jit_nullary("np.random.rand"))
 
+    def _check_random_sized(self, seedfunc, randomfunc):
+        # Our seed() mimicks Numpy's.
+        r = np.random.RandomState()
+        for i in [0, 1, 125, 2**32 - 1]:
+            # Need to cast to a C-sized int (for Numpy <= 1.7)
+            r.seed(np.uint32(i))
+            seedfunc(i)
+            for n in range(10):
+                self.assertPreciseEqual(randomfunc(n), r.uniform(0.0, 1.0, n))
+
+    def test_numpy_random_sized(self):
+        self._check_random_sized(numpy_seed, jit_unary("np.random.random_sample"))
+        self._check_random_sized(numpy_seed, jit_unary("np.random.ranf"))
+        self._check_random_sized(numpy_seed, jit_unary("np.random.sample"))
+        self._check_random_sized(numpy_seed, jit_unary("np.random.rand"))
+
     def test_independent_generators(self):
         # PRNGs for Numpy and Python are independent.
         N = 10
@@ -1480,7 +1496,12 @@ class TestProcesses(ConcurrencyBaseTest):
                 q.put(e)
                 raise
 
-        procs = [multiprocessing.Process(target=target)
+        if hasattr(multiprocessing, 'get_context'):
+            # The test works only in fork context.
+            mpc = multiprocessing.get_context('fork')
+        else:
+            mpc = multiprocessing
+        procs = [mpc.Process(target=target)
                  for i in range(nprocs)]
         for p in procs:
             p.start()
