@@ -7,7 +7,7 @@ import sys
 import numpy
 
 from numba.compiler import compile_isolated
-from numba import types
+from numba import types, utils, jit, njit
 from .support import tag
 
 
@@ -54,6 +54,22 @@ def range_iter_len1(a):
 def range_iter_len2(a):
     return range_iter_len(iter(a))
 
+def range_attrs(start, stop, step):
+    r1 = range(start)
+    r2 = range(start, stop)
+    r3 = range(start, stop, step)
+    tmp = []
+    for r in (r1, r2, r3):
+        tmp.append((r.start, r.stop, r.step))
+    return tmp
+
+def range_contains(val, start, stop, step):
+    r1 = range(start)
+    r2 = range(start, stop)
+    r3 = range(start, stop, step)
+    return [val in r for r in (r1, r2, r3)]
+
+
 class TestRange(unittest.TestCase):
 
     @tag('important')
@@ -131,7 +147,7 @@ class TestRange(unittest.TestCase):
             for args in arglist:
                 args_ = tuple(typ(x) for x in args)
                 self.assertEqual(cfunc(*args_), pyfunc(*args_))
- 
+
     @tag('important')
     def test_range_iter_len1(self):
         range_func = range_len1
@@ -152,6 +168,57 @@ class TestRange(unittest.TestCase):
         arglist = [1, 2, 3, 4, 5]
         self.assertEqual(cfunc(arglist), len(arglist))
 
+    @tag('important')
+    @unittest.skipUnless(utils.IS_PY3, "range() attrs are Py3 only")
+    def test_range_attrs(self):
+        pyfunc = range_attrs
+        arglist = [(0, 0, 1),
+                   (0, -1, 1),
+                   (-1, 1, 1),
+                   (-1, 4, 1),
+                   (-1, 4, 10),
+                   (5, -5, -2),]
+
+        cres = compile_isolated(pyfunc, (types.int64,) * 3)
+        cfunc = cres.entry_point
+        for arg in arglist:
+            self.assertEqual(cfunc(*arg), pyfunc(*arg))
+
+    @tag('important')
+    def test_range_contains(self):
+        pyfunc = range_contains
+        arglist = [(0, 0, 1),
+                   (-1, 0, 1),
+                   (1, 0, -1),
+                   (0, -1, 1),
+                   (0, 1, -1),
+                   (-1, 1, 1),
+                   (-1, 4, 1),
+                   (-1, 4, 10),
+                   (5, -5, -2),]
+
+        bool_vals = [True, False]
+        int_vals = [-10, -6, -5, -4, -2, -1, 0,
+                     1, 2, 4, 5, 6, 10]
+        float_vals = [-1.1, -1.0, 0.0, 1.0, 1.1]
+        complex_vals = [1 + 0j, 1 + 1j, 1.1 + 0j, 1.0 + 1.1j]
+
+        vallist = (bool_vals + int_vals + float_vals
+                + complex_vals)
+
+        cfunc = njit(pyfunc)
+        for arg in arglist:
+            for val in vallist:
+                self.assertEqual(cfunc(val, *arg), pyfunc(val, *arg))
+
+        non_numeric_vals = [{'a': 1}, [1, ], 'abc', (1,)]
+
+        cfunc_obj = jit(pyfunc, forceobj=True)
+        for arg in arglist:
+            for val in non_numeric_vals:
+                self.assertEqual(cfunc_obj(val, *arg), pyfunc(val, *arg))
+
+
+
 if __name__ == '__main__':
     unittest.main()
-
