@@ -3,65 +3,82 @@ from __future__ import print_function, division, absolute_import
 import sys
 
 import numpy as np
+import ctypes
 from numba import jit, numpy_support, types
 from numba import unittest_support as unittest
 from numba.compiler import compile_isolated
 from numba.itanium_mangler import mangle_type
 from numba.utils import IS_PY3
+from numba.config import IS_WIN32
+from numba.numpy_support import version as numpy_version
 from .support import tag
 
 
 def get_a(ary, i):
     return ary[i].a
 
+
 def get_b(ary, i):
     return ary[i].b
 
+
 def get_c(ary, i):
     return ary[i].c
 
+
 def make_getitem(item):
     # This also exercises constant lookup from a closure variable
     def get_xx(ary, i):
         return ary[i][item]
     return get_xx
 
+
 # Issue #1664: constant index lookup should fall back to regular getitem
 def get_zero_a(ary, _unused):
     return ary[0].a
 
+
 getitem_a = make_getitem('a')
 getitem_b = make_getitem('b')
 getitem_c = make_getitem('c')
 
+
 def get_a_subarray(ary, i):
     return ary.a[i]
 
+
 def get_b_subarray(ary, i):
     return ary.b[i]
 
+
 def get_c_subarray(ary, i):
     return ary.c[i]
 
+
 def get_a_zero(ary, _unused):
     return ary.a[0]
 
+
 def make_getitem_subarray(item):
     # This also exercises constant lookup from a closure variable
     def get_xx_subarray(ary, i):
         return ary[item][i]
     return get_xx_subarray
 
+
 getitem_a_subarray = make_getitem_subarray('a')
 getitem_b_subarray = make_getitem_subarray('b')
 getitem_c_subarray = make_getitem_subarray('c')
 
+
 def get_two_arrays_a(ary1, ary2, i):
     return ary1[i].a + ary2[i].a
 
+
 def get_two_arrays_b(ary1, ary2, i):
     return ary1[i].b + ary2[i].b
 
+
 def get_two_arrays_c(ary1, ary2, i):
     return ary1[i].c + ary2[i].c
 
@@ -73,39 +90,49 @@ def get_two_arrays_distinct(ary1, ary2, i):
 def set_a(ary, i, v):
     ary[i].a = v
 
+
 def set_b(ary, i, v):
     ary[i].b = v
 
+
 def set_c(ary, i, v):
     ary[i].c = v
 
+
 def make_setitem(item):
     def set_xx(ary, i, v):
         ary[i][item] = v
     return set_xx
 
+
 setitem_a = make_setitem('a')
 setitem_b = make_setitem('b')
 setitem_c = make_setitem('c')
 
+
 def set_a_subarray(ary, i, v):
     ary.a[i] = v
 
+
 def set_b_subarray(ary, i, v):
     ary.b[i] = v
 
+
 def set_c_subarray(ary, i, v):
     ary.c[i] = v
 
+
 def make_setitem_subarray(item):
     def set_xx_subarray(ary, i, v):
         ary[item][i] = v
     return set_xx_subarray
 
+
 setitem_a_subarray = make_setitem('a')
 setitem_b_subarray = make_setitem('b')
 setitem_c_subarray = make_setitem('c')
 
+
 def set_record(ary, i, j):
     ary[i] = ary[j]
 
@@ -175,6 +202,7 @@ def record_write_array(ary):
     ary.h[0] = 3.0
     ary.h[1] = 4.0
 
+
 def record_write_2d_array(ary):
     ary.i = 3
     ary.j[0, 0] = 5.0
@@ -188,15 +216,19 @@ def record_write_2d_array(ary):
 def record_read_array0(ary):
     return ary.h[0]
 
+
 def record_read_array1(ary):
     return ary.h[1]
 
+
 def record_read_2d_array00(ary):
     return ary.j[0,0]
 
+
 def record_read_2d_array10(ary):
     return ary.j[1,0]
 
+
 def record_read_2d_array01(ary):
     return ary.j[0,1]
 
@@ -204,6 +236,7 @@ def record_read_2d_array01(ary):
 def record_read_first_arr(ary):
     return ary.k[2, 2]
 
+
 def record_read_second_arr(ary):
     return ary.l[2, 2]
 
@@ -243,6 +276,99 @@ recordwith2arrays = np.dtype([('k', np.int32, (10, 20)),
 recordwithcharseq = np.dtype([('m', np.int32),
                               ('n', 'S5')])
 
+
+class TestRecordDtypeMakeCStruct(unittest.TestCase):
+    def test_two_scalars(self):
+
+        class Ref(ctypes.Structure):
+            _fields_ = [
+                ('apple', ctypes.c_int32),
+                ('orange', ctypes.c_float),
+            ]
+
+        ty = types.Record.make_c_struct([
+            ('apple', types.int32),
+            ('orange', types.float32),
+        ])
+        # Correct offsets
+        self.assertEqual(len(ty), 2)
+        self.assertEqual(ty.offset('apple'), Ref.apple.offset)
+        self.assertEqual(ty.offset('orange'), Ref.orange.offset)
+        # Correct size
+        self.assertEqual(ty.size, ctypes.sizeof(Ref))
+        # Is aligned
+        dtype = ty.dtype
+        self.assertTrue(dtype.isalignedstruct)
+
+    def test_three_scalars(self):
+
+        class Ref(ctypes.Structure):
+            _fields_ = [
+                ('apple', ctypes.c_int32),
+                ('mango', ctypes.c_int8),
+                ('orange', ctypes.c_float),
+            ]
+
+        ty = types.Record.make_c_struct([
+            ('apple', types.int32),
+            ('mango', types.int8),
+            ('orange', types.float32),
+        ])
+        # Correct offsets
+        self.assertEqual(len(ty), 3)
+        self.assertEqual(ty.offset('apple'), Ref.apple.offset)
+        self.assertEqual(ty.offset('mango'), Ref.mango.offset)
+        self.assertEqual(ty.offset('orange'), Ref.orange.offset)
+        # Correct size
+        self.assertEqual(ty.size, ctypes.sizeof(Ref))
+        # Is aligned
+        dtype = ty.dtype
+        self.assertTrue(dtype.isalignedstruct)
+
+    def test_complex_struct(self):
+        class Complex(ctypes.Structure):
+            _fields_ = [
+                ('real', ctypes.c_double),
+                ('imag', ctypes.c_double),
+            ]
+
+        class Ref(ctypes.Structure):
+            _fields_ = [
+                ('apple', ctypes.c_int32),
+                ('mango', Complex),
+            ]
+
+        ty = types.Record.make_c_struct([
+            ('apple', types.intc),
+            ('mango', types.complex128),
+        ])
+        # Correct offsets
+        self.assertEqual(len(ty), 2)
+        self.assertEqual(ty.offset('apple'), Ref.apple.offset)
+        self.assertEqual(ty.offset('mango'), Ref.mango.offset)
+        # Correct size
+        self.assertEqual(ty.size, ctypes.sizeof(Ref))
+        # Is aligned?
+        # NumPy version < 1.16 misalign complex-128 types to 16bytes.
+        # (it seems to align on windows?!)
+        if numpy_version >= (1, 16) or IS_WIN32:
+            dtype = ty.dtype
+            self.assertTrue(dtype.isalignedstruct)
+        else:
+            with self.assertRaises(ValueError) as raises:
+                dtype = ty.dtype
+            # get numpy alignment
+            npalign = np.dtype(np.complex128).alignment
+            # llvm should align to alignment of double.
+            llalign = np.dtype(np.double).alignment
+            self.assertIn(
+                ("NumPy is using a different alignment ({}) "
+                 "than Numba/LLVM ({}) for complex128. "
+                 "This is likely a NumPy bug.").format(npalign, llalign),
+                str(raises.exception),
+            )
+
+
 class TestRecordDtype(unittest.TestCase):
 
     def _createSampleArrays(self):
@@ -309,7 +435,8 @@ class TestRecordDtype(unittest.TestCase):
         rec = numpy_support.from_dtype(recordtype)
         cfunc = self.get_cfunc(pyfunc, (rec[:], types.intp))
         for i in range(self.refsample1d.size):
-            self.assertEqual(pyfunc(self.refsample1d, i), cfunc(self.nbsample1d, i))
+            self.assertEqual(pyfunc(self.refsample1d, i),
+                             cfunc(self.nbsample1d, i))
 
     @tag('important')
     def test_get_a(self):
@@ -340,7 +467,7 @@ class TestRecordDtype(unittest.TestCase):
         cfunc = self.get_cfunc(pyfunc, (rec[:], rec[:], types.intp))
         for i in range(self.refsample1d.size):
             self.assertEqual(pyfunc(self.refsample1d, self.refsample1d3, i),
-                              cfunc(self.nbsample1d, self.nbsample1d3, i))
+                             cfunc(self.nbsample1d, self.nbsample1d3, i))
 
     def test_two_distinct_arrays(self):
         '''
@@ -437,7 +564,7 @@ class TestRecordDtype(unittest.TestCase):
         nbval = self.nbsample1d.copy()[0]
         attrs = 'abc'
         valtypes = types.float64, types.int16, types.complex64
-        values = 1.23, 12345, 123+456j
+        values = 1.23, 12345, 123 + 456j
         old_refcnt = sys.getrefcount(nbval)
 
         for attr, valtyp, val in zip(attrs, valtypes, values):
@@ -461,7 +588,7 @@ class TestRecordDtype(unittest.TestCase):
             got = cfunc(*args)
             try:
                 self.assertEqual(expected, got)
-            except AssertionError as e:
+            except AssertionError:
                 # On ARM, a LLVM misoptimization can produce buggy code,
                 # see https://llvm.org/bugs/show_bug.cgi?id=24669
                 import llvmlite.binding as ll
@@ -483,7 +610,6 @@ class TestRecordDtype(unittest.TestCase):
     def test_record_args_reverse(self):
         self._test_record_args(True)
 
-
     def test_two_records(self):
         '''
         Testing the use of two scalar records of the same type
@@ -505,7 +631,6 @@ class TestRecordDtype(unittest.TestCase):
             got = cfunc(nbval1, nbval2)
             self.assertEqual(expected, got)
 
-
     def test_two_distinct_records(self):
         '''
         Testing the use of two scalar records of differing type
@@ -521,7 +646,6 @@ class TestRecordDtype(unittest.TestCase):
         got = cfunc(nbval1, nbval2)
         self.assertEqual(expected, got)
 
-
     def test_record_write_array(self):
         '''
         Testing writing to a 1D array within a structured type
@@ -537,7 +661,6 @@ class TestRecordDtype(unittest.TestCase):
         expected[0].h[1] = 4.0
         np.testing.assert_equal(expected, nbval)
 
-
     @tag('important')
     def test_record_write_2d_array(self):
         '''
@@ -554,7 +677,6 @@ class TestRecordDtype(unittest.TestCase):
                                       np.float32).reshape(3, 2)
         np.testing.assert_equal(expected, nbval)
 
-
     def test_record_read_array(self):
         '''
         Test reading from a 1D array within a structured type
@@ -571,7 +693,6 @@ class TestRecordDtype(unittest.TestCase):
         res = cfunc(nbval[0])
         np.testing.assert_equal(res, nbval[0].h[1])
 
-
     @tag('important')
     def test_record_read_2d_array(self):
         '''
@@ -593,7 +714,6 @@ class TestRecordDtype(unittest.TestCase):
         res = cfunc(nbval[0])
         np.testing.assert_equal(res, nbval[0].j[1, 0])
 
-
     @tag('important')
     def test_record_return(self):
         """
@@ -645,9 +765,9 @@ class TestRecordDtype(unittest.TestCase):
     def test_record_two_arrays(self):
         """
         Tests that comparison of NestedArrays by key is working correctly. If
-        the two NestedArrays in recordwith2arrays compare equal (same length and
-        ndim but different shape) incorrect code will be generated for one of the
-        functions.
+        the two NestedArrays in recordwith2arrays compare equal (same length
+        and ndim but different shape) incorrect code will be generated for one
+        of the functions.
         """
         nbrecord = numpy_support.from_dtype(recordwith2arrays)
         rec = np.recarray(1, dtype=recordwith2arrays)[0]
@@ -693,6 +813,15 @@ class TestRecordDtype(unittest.TestCase):
         got = cfunc(arr.copy())
         np.testing.assert_equal(expect, got)
 
+    def test_record_dtype_with_titles_roundtrip(self):
+        recdtype = np.dtype([(("title a", 'a'), np.float), ('b', np.float)])
+        nbtype = numpy_support.from_dtype(recdtype)
+        self.assertTrue(nbtype.is_title('title a'))
+        self.assertFalse(nbtype.is_title('a'))
+        self.assertFalse(nbtype.is_title('b'))
+        got = numpy_support.as_dtype(nbtype)
+        self.assertTrue(got, recdtype)
+
 
 def _get_cfunc_nopython(pyfunc, argspec):
     return jit(argspec, nopython=True)(pyfunc)
@@ -732,7 +861,8 @@ class TestRecordDtypeWithStructArrays(TestRecordDtype):
         self.nbsample1d2 = np.zeros(3, dtype=recordtype2)
         self.nbsample1d3 = np.zeros(3, dtype=recordtype)
 
-class TestRecordDtypeWithStructArraysAndDispatcher(TestRecordDtypeWithStructArrays):
+
+class TestRecordDtypeWithStructArraysAndDispatcher(TestRecordDtypeWithStructArrays):    # noqa: E501
     '''
     Same as TestRecordDtypeWithStructArrays, stressing the Dispatcher's type
     dispatch mechanism (issue #384) and caching of ndarray typecodes for void
@@ -755,7 +885,7 @@ class TestRecordDtypeWithCharSeq(unittest.TestCase):
 
         arr[0]['n'] = 'abcde'  # no null-byte
         arr[1]['n'] = 'xyz'  # null-byte
-        arr[2]['n'] = 'u\x00v\x00\x00'  # null-byte at the middle and then at end
+        arr[2]['n'] = 'u\x00v\x00\x00'  # null-byte at the middle and at the end
 
     def setUp(self):
         self._createSampleaArray()
@@ -807,9 +937,9 @@ class TestRecordDtypeWithCharSeq(unittest.TestCase):
         cfunc = cres.entry_point
 
         for i in range(self.refsample1d.size):
-            chars = "{0}".format(hex(i+10))
-            expected = pyfunc(self.refsample1d, i, chars)
-            got = cfunc(self.nbsample1d, i, chars)
+            chars = "{0}".format(hex(i + 10))
+            pyfunc(self.refsample1d, i, chars)
+            cfunc(self.nbsample1d, i, chars)
             np.testing.assert_equal(self.refsample1d, self.nbsample1d)
 
     def test_py_argument_char_seq_near_overflow(self):
@@ -829,7 +959,8 @@ class TestRecordDtypeWithCharSeq(unittest.TestCase):
                          recordwithcharseq['n'].itemsize)
 
         cfunc(self.nbsample1d, 0, cs_near_overflow)
-        self.assertEqual(self.nbsample1d[0]['n'].decode('ascii'), cs_near_overflow)
+        self.assertEqual(self.nbsample1d[0]['n'].decode('ascii'),
+                         cs_near_overflow)
         # Check that we didn't overwrite
         np.testing.assert_equal(self.refsample1d[1:], self.nbsample1d[1:])
 
