@@ -13,7 +13,7 @@ class TestCase(unittest.TestCase):
     upstream unittest, and run the numba test suite only in a subprocess."""
 
     def get_testsuite_listing(self, args):
-        cmd = ['python', '-m', 'numba.runtests', '-l'] + list(args)
+        cmd = [sys.executable, '-m', 'numba.runtests', '-l'] + list(args)
         lines = subprocess.check_output(cmd).decode('UTF-8').splitlines()
         lines = [line for line in lines if line.strip()]
         return lines
@@ -30,7 +30,7 @@ class TestCase(unittest.TestCase):
         """
         lines = self.get_testsuite_listing(args)
         last_line = lines[-1]
-        self.assertTrue(last_line.endswith('tests found'))
+        self.assertTrue('tests found' in last_line)
         number = int(last_line.split(' ')[0])
         # There may be some "skipped" messages at the beginning,
         # so do an approximate check.
@@ -67,11 +67,11 @@ class TestCase(unittest.TestCase):
         self.check_listing_prefix('numba.cuda.tests.cudasim')
 
     def test_module(self):
-        self.check_testsuite_size(['numba.tests.test_utils'], 3)
-        self.check_testsuite_size(['numba.tests.test_nested_calls'], 5)
+        self.check_testsuite_size(['numba.tests.test_storeslice'], 2)
+        self.check_testsuite_size(['numba.tests.test_nested_calls'], 10)
         # Several modules
         self.check_testsuite_size(['numba.tests.test_nested_calls',
-                                   'numba.tests.test_utils'], 13)
+                                   'numba.tests.test_storeslice'], 12)
 
     def test_subpackage(self):
         self.check_testsuite_size(['numba.tests.npyufunc'], 50)
@@ -104,6 +104,22 @@ class TestCase(unittest.TestCase):
             excluded = get_count(['--exclude-tags=%s' % tag, 'numba.tests'])
             self.assertEqual(total, included + excluded)
 
+    def test_check_slice(self):
+        tmp = self.get_testsuite_listing(['-j','0,5,1'])
+        l = [x for x in tmp if x.startswith('numba.')]
+        self.assertEqual(len(l), 5)
+
+    def test_check_slicing_equivalent(self):
+        def filter_test(xs):
+            return [x for x in xs if x.startswith('numba.')]
+        full = filter_test(self.get_testsuite_listing([]))
+        sliced = []
+        for i in range(3):
+            subset = self.get_testsuite_listing(['-j', '{},None,3'.format(i)])
+            sliced.extend(filter_test(subset))
+        # The tests must be equivalent
+        self.assertEqual(sorted(full), sorted(sliced))
+
 
 if __name__ == '__main__':
     unittest.main()
