@@ -522,6 +522,28 @@ class TestStencil(TestStencilBase):
         cpfunc = self.compile_parallel(test_impl, (numba.float64[:],), stencil=False)
         self.assertNotIn('@do_scheduling', cpfunc.library.get_llvm_str())
 
+    @skip_unsupported
+    def test_stencil_nested1(self):
+        """Tests whether nested stencil decorator works.
+        """
+        @njit(parallel=True)
+        def test_impl(n):
+            @stencil
+            def fun(a):
+                c = 2
+                return a[-c+1]
+            B = fun(n)
+            return B
+
+        def test_impl_seq(n):
+            B = np.zeros(len(n), dtype=int)
+            for i in range(1, len(n)):
+                B[i] = n[i-1]
+            return B
+
+        n = np.arange(10)
+        np.testing.assert_equal(test_impl(n), test_impl_seq(n))
+
     @skip_unsupported
     def test_out_kwarg_w_cval(self):
         """ Issue #3518, out kwarg did not work with cval."""
@@ -813,59 +835,216 @@ class pyStencilGenerator:
             indexing.
             """
 
-            node = self.generic_visit(node)
-            if (node.value.id in self._argnames) and (
-                    node.value.id not in self._standard_indexing):
-                # 2D index
-                if isinstance(node.slice.value, ast.Tuple):
-                    idx = []
-                    for x, val in enumerate(node.slice.value.elts):
-                        useval = self._const_assigns.get(val, val)
-                        idx.append(
-                            ast.BinOp(
-                                left=ast.Name(
-                                    id=self._id_pat %
-                                    self.ids[x],
-                                    ctx=ast.Load()),
+            def handle2dindex(node):
+                idx = []
+                for x, val in enumerate(node.slice.value.elts):
+                    useval = self._const_assigns.get(val, val)
+                    idx.append(
+                        ast.BinOp(
+                            left=ast.Name(
+                                id=self._id_pat % self.ids[x],
+                                ctx=ast.Load()),
+                            op=ast.Add(),
+                            right=useval,
+                            ctx=ast.Load()))
+                if self._idx_len == -1:
+                    self._idx_len = len(idx)
+                else:
+                    if(self._idx_len != len(idx)):
+                        raise ValueError(
+                            "Relative indexing mismatch detected")
+                if isinstance(node.ctx, ast.Store):
+                    msg = ("Assignments to array passed to "
+                            "stencil kernels is not allowed")
+                    raise ValueError(msg)
+                context = ast.Load()
+                newnode = ast.Subscript(
+                    value=node.value,
+                    slice=ast.Index(
+                        value=ast.Tuple(
+                            elts=idx,
+                            ctx=ast.Load()),
+                        ctx=ast.Load()),
+                    ctx=context)
+                ast.copy_location(newnode, node)
+                ast.fix_missing_locations(newnode)
+
+                # now work out max/min for index ranges i.e. stencil size
+                if self._mins is None and self._maxes is None:
+                    # first pass
+                    self._mins = [self._imax] * self._idx_len
+                    self._maxes = [self._imin] * self._idx_len
+
+                if not self._neighborhood:
+                    for x, lnode in enumerate(node.slice.value.elts):
+                        if isinstance(lnode, ast.Num) or\
+                                isinstance(lnode, ast.UnaryOp):
+                            relvalue = self.get_val_from_num(lnode)
+                        elif (hasattr(lnode, 'id') and
+                                lnode.id in self._const_assigns):
+                            relvalue = self._const_assigns[lnode.id]
+                        else:
+                            raise ValueError(
+                                "Cannot interpret indexing value")
+                        if relvalue < self._mins[x]:
+                            self._mins[x] = relvalue
+                        if relvalue > self._maxes[x]:
+                            self._maxes[x] = relvalue
+                else:
+                    for x, lnode in enumerate(self._neighborhood):
+                        self._mins[x] = self._neighborhood[x][0]
+                        self._maxes[x] = self._neighborhood[x][1]
+
+                return newnode
+
+            def handle1dindex(node):
+                useval = self._const_assigns.get(
+                    node.slice.value, node.slice.value)
+                idx = ast.BinOp(left=ast.Name(
+                                id=self._id_pat % self.ids[0],
+                                ctx=ast.Load()),
                                 op=ast.Add(),
                                 right=useval,
-                                ctx=ast.Load()))
-                    if self._idx_len == -1:
-                        self._idx_len = len(idx)
+                                ctx=ast.Load())
+                if self._idx_len == -1:
+                    self._idx_len = 1
+                else:
+                    if(self._idx_len != 1):
+                        raise ValueError(
+                            "Relative indexing mismatch detected")
+                if isinstance(node.ctx, ast.Store):
+                    msg = ("Assignments to array passed to "
+                            "stencil kernels is not allowed")
+                    raise ValueError(msg)
+                context = ast.Load()
+                newnode = ast.Subscript(
+                    value=node.value,
+                    slice=ast.Index(
+                        value=idx,
+                        ctx=ast.Load()),
+                    ctx=context)
+                ast.copy_location(newnode, node)
+                ast.fix_missing_locations(newnode)
+
+                # now work out max/min for index ranges i.e. stencil size
+                if self._mins is None and self._maxes is None:
+                    # first pass
+                    self._mins = [self._imax, ]
+                    self._maxes = [self._imin, ]
+
+                if not self._neighborhood:
+                    if isinstance(node.slice.value, ast.Num) or\
+                            isinstance(node.slice.value, ast.UnaryOp):
+                        relvalue = self.get_val_from_num(node.slice.value)
+                    elif (hasattr(node.slice.value, 'id') and
+                            node.slice.value.id in self._const_assigns):
+                        relvalue = self._const_assigns[node.slice.value.id]
                     else:
-                        if(self._idx_len != len(idx)):
-                            raise ValueError(
-                                "Relative indexing mismatch detected")
-                    if isinstance(node.ctx, ast.Store):
-                        msg = ("Assignments to array passed to "
-                               "stencil kernels is not allowed")
-                        raise ValueError(msg)
-                    context = ast.Load()
-                    newnode = ast.Subscript(
-                        value=node.value,
-                        slice=ast.Index(
-                            value=ast.Tuple(
-                                elts=idx,
+                        raise ValueError("Cannot interpret indexing value")
+                    if relvalue < self._mins[0]:
+                        self._mins[0] = relvalue
+                    if relvalue > self._maxes[0]:
+                        self._maxes[0] = relvalue
+                else:
+                    self._mins[0] = self._neighborhood[0][0]
+                    self._maxes[0] = self._neighborhood[0][1]
+                ast.copy_location(newnode, node)
+                ast.fix_missing_locations(newnode)
+                return newnode
+
+            def computeSlice(i, node):
+                def gen_idx(val, x):
+                    useval = self._const_assigns.get(val, val)
+                    value = self.get_val_from_num(val)
+                    tmp = ast.BinOp(
+                            left=ast.Name(
+                                id=self._id_pat % self.ids[x],
                                 ctx=ast.Load()),
-                            ctx=ast.Load()),
-                        ctx=context)
-                    ast.copy_location(newnode, node)
-                    ast.fix_missing_locations(newnode)
+                            op=ast.Add(),
+                            right=useval,
+                            ctx=ast.Load())
+                    ast.copy_location(tmp, node)
+                    ast.fix_missing_locations(tmp)
+                    return tmp
+                newnode = ast.Slice(gen_idx(node.lower, i),
+                                 gen_idx(node.upper, i),
+                                 node.step)
+                ast.copy_location(newnode, node)
+                ast.fix_missing_locations(newnode)
+                return newnode
 
-                    # now work out max/min for index ranges i.e. stencil size
-                    if self._mins is None and self._maxes is None:
-                        # first pass
-                        self._mins = [self._imax] * self._idx_len
-                        self._maxes = [self._imin] * self._idx_len
+            def computeIndex(i, node):
+                useval = self._const_assigns.get(node.value, node.value)
+                idx = ast.BinOp(left=ast.Name(
+                                id=self._id_pat % self.ids[i],
+                                ctx=ast.Load()),
+                                op=ast.Add(),
+                                right=useval,
+                                ctx=ast.Load())
+                newnode = ast.Index(value=idx, ctx=ast.Load())
+                ast.copy_location(newnode, node)
+                ast.fix_missing_locations(newnode)
+                return newnode
 
-                    if not self._neighborhood:
-                        for x, lnode in enumerate(node.slice.value.elts):
-                            if isinstance(lnode, ast.Num) or\
-                                    isinstance(lnode, ast.UnaryOp):
-                                relvalue = self.get_val_from_num(lnode)
-                            elif (hasattr(lnode, 'id') and
-                                  lnode.id in self._const_assigns):
-                                relvalue = self._const_assigns[lnode.id]
+            def handleExtSlice(node):
+                idx = []
+                for i, val in enumerate(node.slice.dims):
+                    if isinstance(val, ast.Slice):
+                        idx.append(computeSlice(i, val))
+                    if isinstance(val, ast.Index):
+                        idx.append(computeIndex(i, val))
+                    # TODO: handle more node types
+                if self._idx_len == -1:
+                    self._idx_len = len(node.slice.dims)
+                else:
+                    if(self._idx_len != len(node.slice.dims)):
+                        raise ValueError(
+                            "Relative indexing mismatch detected")
+
+                if isinstance(node.ctx, ast.Store):
+                    msg = ("Assignments to array passed to "
+                            "stencil kernels is not allowed")
+                    raise ValueError(msg)
+                context = ast.Load()
+                newnode = ast.Subscript(
+                    value=node.value,
+                    slice=ast.ExtSlice(
+                        dims=idx,
+                        ctx=ast.Load()),
+                        ctx=context
+                        )
+
+                # now work out max/min for index ranges i.e. stencil size
+                if self._mins is None and self._maxes is None:
+                    # first pass
+                    self._mins = [self._imax] * self._idx_len
+                    self._maxes = [self._imin] * self._idx_len
+
+                if not self._neighborhood:
+                    for x, anode in enumerate(node.slice.dims):
+                        if isinstance(anode, ast.Slice):
+                            for lnode in [anode.lower, anode.upper]:
+                                if isinstance(lnode, ast.Num) or\
+                                        isinstance(lnode, ast.UnaryOp):
+                                    relvalue = self.get_val_from_num(lnode)
+                                elif (hasattr(lnode, 'id') and
+                                        lnode.id in self._const_assigns):
+                                    relvalue = self._const_assigns[lnode.id]
+                                else:
+                                    raise ValueError(
+                                        "Cannot interpret indexing value")
+                                if relvalue < self._mins[x]:
+                                    self._mins[x] = relvalue
+                                if relvalue > self._maxes[x]:
+                                    self._maxes[x] = relvalue
+                        else:
+                            val = anode.value
+                            if isinstance(val, ast.Num) or\
+                                    isinstance(val, ast.UnaryOp):
+                                relvalue = self.get_val_from_num(val)
+                            elif (hasattr(val, 'id') and
+                                    val.id in self._const_assigns):
+                                relvalue = self._const_assigns[val.id]
                             else:
                                 raise ValueError(
                                     "Cannot interpret indexing value")
@@ -873,67 +1052,79 @@ class pyStencilGenerator:
                                 self._mins[x] = relvalue
                             if relvalue > self._maxes[x]:
                                 self._maxes[x] = relvalue
-                    else:
-                        for x, lnode in enumerate(self._neighborhood):
-                            self._mins[x] = self._neighborhood[x][0]
-                            self._maxes[x] = self._neighborhood[x][1]
+                else:
+                    for x, lnode in enumerate(self._neighborhood):
+                        self._mins[x] = self._neighborhood[x][0]
+                        self._maxes[x] = self._neighborhood[x][1]
+                ast.copy_location(newnode, node)
+                ast.fix_missing_locations(newnode)
+                return newnode
 
-                    return newnode
+
+            def handleSlice(node):
+                idx = computeSlice(0, node.slice)
+                idx.ctx=ast.Load()
+                if isinstance(node.ctx, ast.Store):
+                    msg = ("Assignments to array passed to "
+                            "stencil kernels is not allowed")
+                    raise ValueError(msg)
+                context = ast.Load()
+                newnode = ast.Subscript(
+                    value=node.value,
+                    slice=idx,
+                    ctx=context)
+                ast.copy_location(newnode, node)
+                ast.fix_missing_locations(newnode)
+
+                if self._idx_len == -1:
+                    self._idx_len = 1
+                else:
+                    if(self._idx_len != 1):
+                        raise ValueError(
+                            "Relative indexing mismatch detected")
+
+                # now work out max/min for index ranges i.e. stencil size
+                if self._mins is None and self._maxes is None:
+                    # first pass
+                    self._mins = [self._imax]
+                    self._maxes = [self._imin]
+
+                if not self._neighborhood:
+                    if isinstance(node.slice.value, ast.Num) or\
+                            isinstance(node.slice.value, ast.UnaryOp):
+                        relvalue = self.get_val_from_num(node.slice.value)
+                    elif (hasattr(node.slice.value, 'id') and
+                            node.slice.value.id in self._const_assigns):
+                        relvalue = self._const_assigns[node.slice.value.id]
+                    else:
+                        raise ValueError("Cannot interpret indexing value")
+                    if relvalue < self._mins[0]:
+                        self._mins[0] = relvalue
+                    if relvalue > self._maxes[0]:
+                        self._maxes[0] = relvalue
+                else:
+                    self._mins[0] = self._neighborhood[0][0]
+                    self._maxes[0] = self._neighborhood[0][1]
+
+                return newnode
+
+
+            node = self.generic_visit(node)
+            if (node.value.id in self._argnames) and (
+                    node.value.id not in self._standard_indexing):
+
+                # fancy slice
+                if isinstance(node.slice, ast.ExtSlice):
+                    return handleExtSlice(node)
+                # plain slice
+                if isinstance(node.slice, ast.Slice):
+                    return handleSlice(node)
+                # 2D index
+                if isinstance(node.slice.value, ast.Tuple):
+                    return handle2dindex(node)
                 # 1D index
                 elif isinstance(node.slice, ast.Index):
-                    useval = self._const_assigns.get(
-                        node.slice.value, node.slice.value)
-                    idx = ast.BinOp(left=ast.Name(
-                                    id=self._id_pat %
-                                    self.ids[0],
-                                    ctx=ast.Load()),
-                                    op=ast.Add(),
-                                    right=useval,
-                                    ctx=ast.Load())
-                    if self._idx_len == -1:
-                        self._idx_len = 1
-                    else:
-                        if(self._idx_len != 1):
-                            raise ValueError(
-                                "Relative indexing mismatch detected")
-                    if isinstance(node.ctx, ast.Store):
-                        msg = ("Assignments to array passed to "
-                               "stencil kernels is not allowed")
-                        raise ValueError(msg)
-                    context = ast.Load()
-                    newnode = ast.Subscript(
-                        value=node.value,
-                        slice=ast.Index(
-                            value=idx,
-                            ctx=ast.Load()),
-                        ctx=context)
-                    ast.copy_location(newnode, node)
-                    ast.fix_missing_locations(newnode)
-
-                    # now work out max/min for index ranges i.e. stencil size
-                    if self._mins is None and self._maxes is None:
-                        # first pass
-                        self._mins = [self._imax, ]
-                        self._maxes = [self._imin, ]
-
-                    if not self._neighborhood:
-                        if isinstance(node.slice.value, ast.Num) or\
-                                isinstance(node.slice.value, ast.UnaryOp):
-                            relvalue = self.get_val_from_num(node.slice.value)
-                        elif (hasattr(node.slice.value, 'id') and
-                              node.slice.value.id in self._const_assigns):
-                            relvalue = self._const_assigns[node.slice.value.id]
-                        else:
-                            raise ValueError("Cannot interpret indexing value")
-                        if relvalue < self._mins[0]:
-                            self._mins[0] = relvalue
-                        if relvalue > self._maxes[0]:
-                            self._maxes[0] = relvalue
-                    else:
-                        self._mins[0] = self._neighborhood[0][0]
-                        self._maxes[0] = self._neighborhood[0][1]
-
-                    return newnode
+                    return handle1dindex(node)
                 else:  # unknown
                     raise ValueError("Unhandled subscript")
             else:
@@ -1099,7 +1290,8 @@ class pyStencilGenerator:
                 vararg=None,
                 kwarg=None,
                 kwonlyargs=[],
-                kw_defaults=[])
+                kw_defaults=[],
+                posonlyargs=[])
             new = ast.FunctionDef(
                 name='__%s' %
                 node.name,
@@ -1390,6 +1582,7 @@ class TestManyStencils(TestStencilBase):
         stencil_args.update(options)
 
         expected_present = True
+
         try:
             # ast impl
             ast_impl = pyStencil(func_or_mode=pyfunc, **options)
@@ -2568,5 +2761,35 @@ class TestManyStencils(TestStencilBase):
         A = np.array(np.arange(20) % 2).reshape(4, 5).astype(np.bool_)
         self.check(kernel, A, options={'cval': True})
 
+    def test_basic94(self):
+        """ Issue #3528. Support for slices. """
+        def kernel(a):
+            return np.median(a[-1:2, -1:2])
+        a = np.arange(20, dtype=np.uint32).reshape(4, 5)
+        self.check(kernel, a, options={'neighborhood': ((-1, 1), (-1, 1),)})
+
+    @unittest.skip("not yet supported")
+    def test_basic95(self):
+        """ Slice, calculate neighborhood. """
+        def kernel(a):
+            return np.median(a[-1:2, -3:4])
+        a = np.arange(20, dtype=np.uint32).reshape(4, 5)
+        self.check(kernel, a)
+
+    def test_basic96(self):
+        """ 1D slice. """
+        def kernel(a):
+            return np.median(a[-1:2])
+        a = np.arange(20, dtype=np.uint32)
+        self.check(kernel, a, options={'neighborhood': ((-1, 1),)})
+
+    @unittest.skip("not yet supported")
+    def test_basic97(self):
+        """ 2D slice and index. """
+        def kernel(a):
+            return np.median(a[-1:2, 3])
+        a = np.arange(20, dtype=np.uint32).reshape(4, 5)
+        self.check(kernel, a)
+
 if __name__ == "__main__":
     unittest.main()
