@@ -7,7 +7,7 @@ import numpy as np
 
 from numba import unittest_support as unittest
 from numba.compiler import compile_isolated
-from numba import jit, types, errors, utils
+from numba import njit, jit, types, errors, utils
 from .support import TestCase, MemoryLeakMixin, tag
 
 
@@ -95,6 +95,51 @@ def identity(tup):
 def index_method_usecase(tup, value):
     return tup.index(value)
 
+def tuple_unpack_static_getitem_err():
+    # see issue3895, `c` is imprecise
+    a, b, c, d = [], [], [], 0.0
+    a.append(1)
+    b.append(1)
+    return
+
+
+class TestTupleLengthError(unittest.TestCase):
+
+    def test_tuple_length_error(self):
+        # issue 2195
+        # raise an error on tuples greater than 1000 in length
+        @njit
+        def eattuple(tup):
+            return len(tup)
+
+        with self.assertRaises(errors.UnsupportedError) as raises:
+            tup = tuple(range(1001))
+            eattuple(tup)
+
+        expected = "Tuple 'tup' length must be smaller than 1000"
+        self.assertIn(expected, str(raises.exception))
+
+class TestTupleTypeNotIterable(unittest.TestCase):
+    '''
+    issue 4369
+    raise an error if 'type' is not iterable
+    '''
+    def test_namedtuple_types_exception(self):
+        with self.assertRaises(errors.TypingError) as raises:
+            types.NamedTuple(types.uint32, 'p')
+        self.assertIn(
+            "Argument 'types' is not iterable",
+            str(raises.exception)
+        )
+
+    def test_tuple_types_exception(self):
+        with self.assertRaises(errors.TypingError) as raises:
+            types.Tuple((types.uint32))
+        self.assertIn(
+            "Argument 'types' is not iterable",
+            str(raises.exception)
+        )
+
 
 class TestTupleReturn(TestCase):
 
@@ -205,6 +250,12 @@ class TestOperations(TestCase):
             cr.entry_point((), 0)
         self.assertEqual("tuple index out of range", str(raises.exception))
 
+        # test uintp indexing (because, e.g., parfor generates unsigned prange)
+        cr = compile_isolated(pyfunc,
+                              [types.UniTuple(types.int64, 3), types.uintp])
+        for i in range(len(tup)):
+            self.assertPreciseEqual(cr.entry_point(tup, types.uintp(i)), tup[i])
+
         # With a compile-time static index (the code generation path is different)
         pyfunc = tuple_index_static
         for typ in (types.UniTuple(types.int64, 4),
@@ -217,6 +268,14 @@ class TestOperations(TestCase):
         with self.assertTypingError():
             cr = compile_isolated(pyfunc, (typ,))
 
+        # test unpack, staticgetitem with imprecise type (issue #3895)
+        pyfunc = tuple_unpack_static_getitem_err
+        with self.assertTypingError() as raises:
+            cr = compile_isolated(pyfunc, ())
+        msg = ("Cannot infer the type of variable 'c', have imprecise type: "
+               "list(undefined).")
+        self.assertIn(msg, str(raises.exception))
+
 
     def test_in(self):
         pyfunc = in_usecase
@@ -355,6 +414,10 @@ class TestNamedTuple(TestCase, MemoryLeakMixin):
         for i in range(len(p)):
             self.assertPreciseEqual(cfunc(p, i), pyfunc(p, i))
 
+        # test uintp indexing (because, e.g., parfor generates unsigned prange)
+        for i in range(len(p)):
+            self.assertPreciseEqual(cfunc(p, types.uintp(i)), pyfunc(p, i))
+
     def test_bool(self):
         def check(p):
             pyfunc = bool_usecase
@@ -465,6 +528,17 @@ class TestNamedTuple(TestCase, MemoryLeakMixin):
         r = call(123, 0)
         self.assertEqual(r, Rect(width=123, height=-321))
 
+    @unittest.skipIf(utils.PYVERSION < (3, 0), "needs Python 3")
+    def test_string_literal_in_ctor(self):
+        # Test for issue #3813
+
+        @jit(nopython=True)
+        def foo():
+            return Rect(10, 'somestring')
+
+        r = foo()
+        self.assertEqual(r, Rect(width=10, height='somestring'))
+
 
 class TestTupleNRT(TestCase, MemoryLeakMixin):
     def test_tuple_add(self):
@@ -518,7 +592,7 @@ class TestConversions(TestCase):
 
         with self.assertRaises(errors.TypingError) as raises:
             check(fromty, types.Tuple((types.float32,)), (4, 5))
-        msg = "No conversion from tuple(int32 x 2) to tuple(float32 x 1)"
+        msg = "No conversion from UniTuple(int32 x 2) to UniTuple(float32 x 1)"
         self.assertIn(msg, str(raises.exception))
 
 
