@@ -6,6 +6,7 @@ import re
 import sys
 import warnings
 import threading
+import operator
 
 import numpy as np
 
@@ -17,7 +18,7 @@ from numba import jit, vectorize
 from numba.config import PYVERSION
 from numba.errors import LoweringError, TypingError
 from .support import TestCase, CompilationCache, MemoryLeakMixin, tag
-
+from numba.six import exec_, string_types
 from numba.typing.npydecl import supported_ufuncs, all_ufuncs
 
 is32bits = tuple.__itemsize__ == 4
@@ -54,7 +55,7 @@ def _make_ufunc_usecase(ufunc):
     ldict = {}
     arg_str = ','.join(['a{0}'.format(i) for i in range(ufunc.nargs)])
     func_str = 'def fn({0}):\n    np.{1}({0})'.format(arg_str, ufunc.__name__)
-    exec(func_str, globals(), ldict)
+    exec_(func_str, globals(), ldict)
     fn = ldict['fn']
     fn.__name__ = '{0}_usecase'.format(ufunc.__name__)
     return fn
@@ -63,7 +64,7 @@ def _make_ufunc_usecase(ufunc):
 def _make_unary_ufunc_usecase(ufunc):
     ufunc_name = ufunc.__name__
     ldict = {}
-    exec("def fn(x,out):\n    np.{0}(x,out)".format(ufunc_name), globals(), ldict)
+    exec_("def fn(x,out):\n    np.{0}(x,out)".format(ufunc_name), globals(), ldict)
     fn = ldict["fn"]
     fn.__name__ = "{0}_usecase".format(ufunc_name)
     return fn
@@ -71,7 +72,7 @@ def _make_unary_ufunc_usecase(ufunc):
 
 def _make_unary_ufunc_op_usecase(ufunc_op):
     ldict = {}
-    exec("def fn(x):\n    return {0}(x)".format(ufunc_op), globals(), ldict)
+    exec_("def fn(x):\n    return {0}(x)".format(ufunc_op), globals(), ldict)
     fn = ldict["fn"]
     fn.__name__ = "usecase_{0}".format(hash(ufunc_op))
     return fn
@@ -80,7 +81,7 @@ def _make_unary_ufunc_op_usecase(ufunc_op):
 def _make_binary_ufunc_usecase(ufunc):
     ufunc_name = ufunc.__name__
     ldict = {}
-    exec("def fn(x,y,out):\n    np.{0}(x,y,out)".format(ufunc_name), globals(), ldict);
+    exec_("def fn(x,y,out):\n    np.{0}(x,y,out)".format(ufunc_name), globals(), ldict);
     fn = ldict['fn']
     fn.__name__ = "{0}_usecase".format(ufunc_name)
     return fn
@@ -88,17 +89,26 @@ def _make_binary_ufunc_usecase(ufunc):
 
 def _make_binary_ufunc_op_usecase(ufunc_op):
     ldict = {}
-    exec("def fn(x,y):\n    return x{0}y".format(ufunc_op), globals(), ldict)
+    exec_("def fn(x,y):\n    return x{0}y".format(ufunc_op), globals(), ldict)
     fn = ldict["fn"]
     fn.__name__ = "usecase_{0}".format(hash(ufunc_op))
     return fn
 
 
 def _make_inplace_ufunc_op_usecase(ufunc_op):
-    ldict = {}
-    exec("def fn(x,y):\n    x{0}y".format(ufunc_op), globals(), ldict)
-    fn = ldict["fn"]
-    fn.__name__ = "usecase_{0}".format(hash(ufunc_op))
+    """Generates a function to be compiled that performs an inplace operation
+
+    ufunc_op can be a string like '+=' or a function like operator.iadd
+    """
+    if isinstance(ufunc_op, string_types):
+        ldict = {}
+        exec_("def fn(x,y):\n    x{0}y".format(ufunc_op), globals(), ldict)
+        fn = ldict["fn"]
+        fn.__name__ = "usecase_{0}".format(hash(ufunc_op))
+    else:
+        def inplace_op(x, y):
+            ufunc_op(x, y)
+        fn = inplace_op
     return fn
 
 
@@ -139,6 +149,11 @@ class BaseUFuncTest(MemoryLeakMixin):
             (np.array([-1,0,1], dtype='i8'), types.Array(types.int64, 1, 'C')),
             (np.array([-0.5, 0.0, 0.5], dtype='f4'), types.Array(types.float32, 1, 'C')),
             (np.array([-0.5, 0.0, 0.5], dtype='f8'), types.Array(types.float64, 1, 'C')),
+
+            (np.array([0,1], dtype=np.int8), types.Array(types.int8, 1, 'C')),
+            (np.array([0,1], dtype=np.int16), types.Array(types.int16, 1, 'C')),
+            (np.array([0,1], dtype=np.uint8), types.Array(types.uint8, 1, 'C')),
+            (np.array([0,1], dtype=np.uint16), types.Array(types.uint16, 1, 'C')),
             ]
         self.cache = CompilationCache()
 
@@ -260,15 +275,19 @@ class TestUFuncs(BaseUFuncTest, TestCase):
             input_operand = input_tuple[0]
             input_type = input_tuple[1]
 
+            is_tuple = isinstance(input_operand, tuple)
+            lhs = input_operand[0] if is_tuple else input_operand
+            rhs = input_operand[1] if is_tuple else input_operand
+
             if input_type in skip_inputs:
                 continue
-            if positive_only and np.any(input_operand < 0):
+            if positive_only and np.any(lhs < 0):
                 continue
 
             # Some ufuncs don't allow all kinds of arguments, and implicit
             # conversion has become stricter in 1.10.
             if (numpy_support.strict_ufunc_typing and
-                input_operand.dtype.kind not in kinds):
+                lhs.dtype.kind not in kinds):
                 continue
 
             output_type = self._determine_output_type(
@@ -278,16 +297,16 @@ class TestUFuncs(BaseUFuncTest, TestCase):
                                     flags=flags)
             cfunc = cr.entry_point
 
-            if isinstance(input_operand, np.ndarray):
-                result = np.zeros(input_operand.size,
+            if isinstance(lhs, np.ndarray):
+                result = np.zeros(lhs.size,
                                   dtype=output_type.dtype.name)
-                expected = np.zeros(input_operand.size,
+                expected = np.zeros(lhs.size,
                                     dtype=output_type.dtype.name)
             else:
                 result = np.zeros(1, dtype=output_type.dtype.name)
                 expected = np.zeros(1, dtype=output_type.dtype.name)
-            cfunc(input_operand, input_operand, result)
-            pyfunc(input_operand, input_operand, expected)
+            cfunc(lhs, rhs, result)
+            pyfunc(lhs, rhs, expected)
             np.testing.assert_array_almost_equal(expected, result)
 
     def unary_int_ufunc_test(self, name=None, flags=no_pyobj_flags):
@@ -355,13 +374,24 @@ class TestUFuncs(BaseUFuncTest, TestCase):
         self.binary_ufunc_test(np.power, flags=flags,
                                positive_only=after_numpy_112)
 
+    def test_gcd_ufunc(self, flags=no_pyobj_flags):
+        if numpy_support.version >= (1, 15):
+            self.binary_ufunc_test(np.gcd, flags=flags, kinds="iu")
+
+    def test_lcm_ufunc(self, flags=no_pyobj_flags):
+        if numpy_support.version >= (1, 15):
+            self.binary_ufunc_test(np.lcm, flags=flags, kinds="iu")
+
     @tag('important')
     def test_remainder_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.remainder, flags=flags)
 
     @tag('important')
     def test_mod_ufunc(self, flags=no_pyobj_flags):
-        self.binary_ufunc_test(np.mod, flags=flags)
+        self.binary_ufunc_test(np.mod, flags=flags, kinds='ifcu',
+            additional_inputs = [
+                ((np.uint64(np.iinfo(np.uint64).max), np.uint64(16)), types.uint64)
+            ])
 
     @tag('important')
     def test_fmod_ufunc(self, flags=no_pyobj_flags):
@@ -609,17 +639,33 @@ class TestUFuncs(BaseUFuncTest, TestCase):
 
     ############################################################################
     # Floating functions
+
+    def bool_additional_inputs(self):
+        return [
+            (np.array([True, False], dtype=np.bool_),
+             types.Array(types.bool_, 1, 'C')),
+            ]
+
     @tag('important')
     def test_isfinite_ufunc(self, flags=no_pyobj_flags):
-        self.unary_ufunc_test(np.isfinite, flags=flags)
+        self.unary_ufunc_test(
+            np.isfinite, flags=flags, kinds='ifcb',
+            additional_inputs=self.bool_additional_inputs(),
+        )
 
     @tag('important')
     def test_isinf_ufunc(self, flags=no_pyobj_flags):
-        self.unary_ufunc_test(np.isinf, flags=flags)
+        self.unary_ufunc_test(
+            np.isinf, flags=flags, kinds='ifcb',
+            additional_inputs=self.bool_additional_inputs(),
+        )
 
     @tag('important')
     def test_isnan_ufunc(self, flags=no_pyobj_flags):
-        self.unary_ufunc_test(np.isnan, flags=flags)
+        self.unary_ufunc_test(
+            np.isnan, flags=flags, kinds='ifcb',
+            additional_inputs=self.bool_additional_inputs(),
+        )
 
     @tag('important')
     def test_signbit_ufunc(self, flags=no_pyobj_flags):
@@ -952,6 +998,9 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
         inputs = list(self.inputs)
         inputs.extend(additional_inputs)
         pyfunc = operator_func
+        # when generating arbitrary sequences, we use a fixed seed
+        # for deterministic testing
+        random_state = np.random.RandomState(1)
         for input_tuple in inputs:
             input_operand1, input_type = input_tuple
             input_dtype = numpy_support.as_dtype(
@@ -971,7 +1020,7 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
                 if positive_rhs and np.any(input_operand1 < zero):
                     continue
             else:
-                input_operand0 = (np.random.random(10) * 100).astype(
+                input_operand0 = (random_state.uniform(0, 100, 10)).astype(
                     input_dtype)
                 input_type0 = typeof(input_operand0)
                 if positive_rhs and input_operand1 < zero:
@@ -1074,43 +1123,55 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
     @tag('important')
     def test_inplace_add(self):
         self.inplace_float_op_test('+=', [-1, 1.5, 3], [-5, 0, 2.5])
+        self.inplace_float_op_test(operator.iadd, [-1, 1.5, 3], [-5, 0, 2.5])
 
     @tag('important')
     def test_inplace_sub(self):
         self.inplace_float_op_test('-=', [-1, 1.5, 3], [-5, 0, 2.5])
+        self.inplace_float_op_test(operator.isub, [-1, 1.5, 3], [-5, 0, 2.5])
 
     @tag('important')
     def test_inplace_mul(self):
         self.inplace_float_op_test('*=', [-1, 1.5, 3], [-5, 0, 2.5])
+        self.inplace_float_op_test(operator.imul, [-1, 1.5, 3], [-5, 0, 2.5])
 
     def test_inplace_floordiv(self):
         self.inplace_float_op_test('//=', [-1, 1.5, 3], [-5, 1.25, 2.5])
+        self.inplace_float_op_test(operator.ifloordiv, [-1, 1.5, 3], [-5, 1.25, 2.5])
 
     def test_inplace_div(self):
         self.inplace_float_op_test('/=', [-1, 1.5, 3], [-5, 0, 2.5])
+        self.inplace_float_op_test(operator.itruediv, [-1, 1.5, 3], [-5, 1.25, 2.5])
 
     def test_inplace_remainder(self):
         self.inplace_float_op_test('%=', [-1, 1.5, 3], [-5, 2, 2.5])
+        self.inplace_float_op_test(operator.imod, [-1, 1.5, 3], [-5, 2, 2.5])
 
     @tag('important')
     def test_inplace_pow(self):
         self.inplace_float_op_test('**=', [-1, 1.5, 3], [-5, 2, 2.5])
+        self.inplace_float_op_test(operator.ipow, [-1, 1.5, 3], [-5, 2, 2.5])
 
     def test_inplace_and(self):
         self.inplace_bitwise_op_test('&=', [0, 1, 2, 3, 51], [0, 13, 16, 42, 255])
+        self.inplace_bitwise_op_test(operator.iand, [0, 1, 2, 3, 51], [0, 13, 16, 42, 255])
 
     def test_inplace_or(self):
         self.inplace_bitwise_op_test('|=', [0, 1, 2, 3, 51], [0, 13, 16, 42, 255])
+        self.inplace_bitwise_op_test(operator.ior, [0, 1, 2, 3, 51], [0, 13, 16, 42, 255])
 
     def test_inplace_xor(self):
         self.inplace_bitwise_op_test('^=', [0, 1, 2, 3, 51], [0, 13, 16, 42, 255])
+        self.inplace_bitwise_op_test(operator.ixor, [0, 1, 2, 3, 51], [0, 13, 16, 42, 255])
 
     @tag('important')
     def test_inplace_lshift(self):
         self.inplace_int_op_test('<<=', [0, 5, -10, -51], [0, 1, 4, 14])
+        self.inplace_int_op_test(operator.ilshift, [0, 5, -10, -51], [0, 1, 4, 14])
 
     def test_inplace_rshift(self):
         self.inplace_int_op_test('>>=', [0, 5, -10, -51], [0, 1, 4, 14])
+        self.inplace_int_op_test(operator.irshift, [0, 5, -10, -51], [0, 1, 4, 14])
 
     def test_unary_positive_array_op(self):
         '''
@@ -1159,7 +1220,7 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
     @tag('important')
     def test_floor_divide_array_op(self):
         # Avoid floating-point zeros as x // 0.0 can have varying results
-        # depending on the algorithm (which changed accross Numpy versions)
+        # depending on the algorithm (which changed across Numpy versions)
         self.inputs = [
             (np.uint32(1), types.uint32),
             (np.int32(-2), types.int32),
@@ -1711,13 +1772,13 @@ class TestLoopTypesDatetimeNoPython(_LoopTypesTester):
     def test_add(self):
         ufunc = np.add
         fn = _make_ufunc_usecase(ufunc)
-        # heterogenous inputs
+        # heterogeneous inputs
         self._check_ufunc_with_dtypes(fn, ufunc, ['m8[s]', 'm8[m]', 'm8[s]'])
         self._check_ufunc_with_dtypes(fn, ufunc, ['m8[m]', 'm8[s]', 'm8[s]'])
         if not numpy_support.strict_ufunc_typing:
             self._check_ufunc_with_dtypes(fn, ufunc, ['m8[m]', 'm8', 'm8[m]'])
             self._check_ufunc_with_dtypes(fn, ufunc, ['m8', 'm8[m]', 'm8[m]'])
-        # heterogenous inputs, scaled output
+        # heterogeneous inputs, scaled output
         self._check_ufunc_with_dtypes(fn, ufunc, ['m8[s]', 'm8[m]', 'm8[ms]'])
         self._check_ufunc_with_dtypes(fn, ufunc, ['m8[m]', 'm8[s]', 'm8[ms]'])
         # Cannot upscale result (Numpy would accept this)
@@ -1727,10 +1788,10 @@ class TestLoopTypesDatetimeNoPython(_LoopTypesTester):
     def test_subtract(self):
         ufunc = np.subtract
         fn = _make_ufunc_usecase(ufunc)
-        # heterogenous inputs
+        # heterogeneous inputs
         self._check_ufunc_with_dtypes(fn, ufunc, ['M8[s]', 'M8[m]', 'm8[s]'])
         self._check_ufunc_with_dtypes(fn, ufunc, ['M8[m]', 'M8[s]', 'm8[s]'])
-        # heterogenous inputs, scaled output
+        # heterogeneous inputs, scaled output
         self._check_ufunc_with_dtypes(fn, ufunc, ['M8[s]', 'M8[m]', 'm8[ms]'])
         self._check_ufunc_with_dtypes(fn, ufunc, ['M8[m]', 'M8[s]', 'm8[ms]'])
         # Cannot upscale result (Numpy would accept this)
@@ -1750,7 +1811,7 @@ class TestLoopTypesDatetimeNoPython(_LoopTypesTester):
     def test_true_divide(self):
         ufunc = np.true_divide
         fn = _make_ufunc_usecase(ufunc)
-        # heterogenous inputs
+        # heterogeneous inputs
         self._check_ufunc_with_dtypes(fn, ufunc, ['m8[m]', 'm8[s]', 'd'])
         self._check_ufunc_with_dtypes(fn, ufunc, ['m8[s]', 'm8[m]', 'd'])
         # scaled output
@@ -1839,7 +1900,7 @@ class TestUFuncCompilationThreadSafety(TestCase):
                 a = np.ones((10,), dtype = np.float64)
                 expected = np.ones((10,), dtype = np.float64) + 1.
                 np.testing.assert_array_equal(foo(a), expected)
-            except BaseException as e:
+            except Exception as e:
                 errors.append(e)
 
         threads = [threading.Thread(target=wrapper) for i in range(16)]
