@@ -15,16 +15,8 @@ from numba.targets.registry import cpu_target
 from numba.targets import cpu
 from numba.compiler import compile_ir, DEFAULT_FLAGS
 from numba import njit, typeof, objmode
-from .support import MemoryLeak, TestCase, captured_stdout
-
-
-try:
-    import scipy
-except ImportError:
-    scipy = None
-
-_msg = "SciPy needed for test"
-skip_unless_scipy = unittest.skipIf(scipy is None, _msg)
+from numba.extending import overload
+from .support import MemoryLeak, TestCase, captured_stdout, skip_unless_scipy
 
 
 def get_func_ir(func):
@@ -605,7 +597,7 @@ class TestLiftObj(MemoryLeak, TestCase):
         self.assert_equal_return_and_stdout(foo, x)
 
     def test_case11_define_function_in_context(self):
-        # should this work? no, `make_function` opcode not supported
+        # should this work? no, global name 'bar' is not defined
         def foo(x):
             with objmode_context():
                 def bar(y):
@@ -614,10 +606,10 @@ class TestLiftObj(MemoryLeak, TestCase):
 
         x = np.array([1, 2, 3])
         cfoo = njit(foo)
-        with self.assertRaises(errors.TypingError) as raises:
+        with self.assertRaises(NameError) as raises:
             cfoo(x)
         self.assertIn(
-            'op code: make_function',
+            "global name 'bar' is not defined",
             str(raises.exception),
         )
 
@@ -698,7 +690,10 @@ class TestLiftObj(MemoryLeak, TestCase):
             with objmode_context(k='int64'):
                 print(x)
                 spx = sp.csr_matrix(x)
-                k = spx[0, 0]
+                # the np.int64 call is pointless, works around:
+                # https://github.com/scipy/scipy/issues/10206
+                # which hit the SciPy 1.3 release.
+                k = np.int64(spx[0, 0])
             return k
         x = np.array([1, 2, 3])
         self.assert_equal_return_and_stdout(foo, x)
@@ -778,6 +773,25 @@ class TestLiftObj(MemoryLeak, TestCase):
         self.assertPreciseEqual(foo(), foo.py_func())
         self.assertIs(objmode, objmode_context)
 
+    def test_objmode_in_overload(self):
+        def foo(s):
+            pass
+
+        @overload(foo)
+        def foo_overload(s):
+            def impl(s):
+                with objmode(out='intp'):
+                    out = s + 3
+                return out
+            return impl
+
+        @numba.njit
+        def f():
+            return foo(1)
+
+        self.assertEqual(f(), 1 + 3)
+
+
 class TestBogusContext(BaseTestWithLifting):
     def test_undefined_global(self):
         the_ir = get_func_ir(lift_undefiend)
