@@ -5,12 +5,16 @@ Implement transformation on Numba IR
 from __future__ import absolute_import, print_function
 
 from collections import namedtuple, defaultdict
+import logging
 
 from numba.analysis import compute_cfg_from_blocks, find_top_level_loops
 from numba import ir, errors, ir_utils
 from numba.analysis import compute_use_defs
 
 
+_logger = logging.getLogger(__name__)
+
+
 def _extract_loop_lifting_candidates(cfg, blocks):
     """
     Returns a list of loops that are candidate for loop lifting
@@ -25,13 +29,18 @@ def _extract_loop_lifting_candidates(cfg, blocks):
                 # If the exit point has no successor, it contains an return
                 # statement, which is not handled by the looplifting code.
                 # Thus, this loop is not a candidate.
+                _logger.debug("return-statement in loop.")
                 return False
             outedges |= succs
-        return len(outedges) == 1
+        ok = len(outedges) == 1
+        _logger.debug("same_exit_point=%s (%s)", ok, outedges)
+        return ok
 
     def one_entry(loop):
         "there is one entry"
-        return len(loop.entries) == 1
+        ok = len(loop.entries) == 1
+        _logger.debug("one_entry=%s", ok)
+        return ok
 
     def cannot_yield(loop):
         "cannot have yield inside the loop"
@@ -40,11 +49,23 @@ def _extract_loop_lifting_candidates(cfg, blocks):
             for inst in blk.body:
                 if isinstance(inst, ir.Assign):
                     if isinstance(inst.value, ir.Yield):
+                        _logger.debug("has yield")
                         return False
+        _logger.debug("no yield")
         return True
 
-    return [loop for loop in find_top_level_loops(cfg)
-            if same_exit_point(loop) and one_entry(loop) and cannot_yield(loop)]
+    _logger.info('finding looplift candidates')
+    # the check for cfg.entry_point in the loop.entries is to prevent a bad
+    # rewrite where a prelude for a lifted loop would get written into block -1
+    # if a loop entry were in block 0
+    candidates = []
+    for loop in find_top_level_loops(cfg):
+        _logger.debug("top-level loop: %s", loop)
+        if (same_exit_point(loop) and one_entry(loop) and cannot_yield(loop) and
+            cfg.entry_point() not in loop.entries):
+            candidates.append(loop)
+            _logger.debug("add candidate: %s", loop)
+    return candidates
 
 
 def find_region_inout_vars(blocks, livemap, callfrom, returnto, body_block_ids):
@@ -86,11 +107,17 @@ def _loop_lift_get_candidate_infos(cfg, blocks, livemap):
     loops = _extract_loop_lifting_candidates(cfg, blocks)
     loopinfos = []
     for loop in loops:
+
         [callfrom] = loop.entries   # requirement checked earlier
         an_exit = next(iter(loop.exits))  # anyone of the exit block
-        [(returnto, _)] = cfg.successors(an_exit)  # requirement checked earlier
+        if len(loop.exits) > 1:
+            # Pre-Py3.8 may have multiple exits
+            [(returnto, _)] = cfg.successors(an_exit)  # requirement checked earlier
+        else:
+            # Post-Py3.8 DO NOT have multiple exits
+            returnto = an_exit
 
-        local_block_ids = set(loop.body) | set(loop.entries) | set(loop.exits)
+        local_block_ids = set(loop.body) | set(loop.entries)
         inputs, outputs = find_region_inout_vars(
             blocks=blocks,
             livemap=livemap,
@@ -155,7 +182,11 @@ def _loop_lift_modify_blocks(func_ir, loopinfo, blocks,
 
     # Copy loop blocks
     loop = loopinfo.loop
-    loopblockkeys = set(loop.body) | set(loop.entries) | set(loop.exits)
+
+    loopblockkeys = set(loop.body) | set(loop.entries)
+    if len(loop.exits) > 1:
+        # Pre-Py3.8 may have multiple exits
+        loopblockkeys |= loop.exits
     loopblocks = dict((k, blocks[k].copy()) for k in loopblockkeys)
     # Modify the loop blocks
     _loop_lift_prepare_loop_func(loopinfo, loopblocks)
@@ -192,6 +223,9 @@ def loop_lifting(func_ir, typingctx, targetctx, flags, locals):
     loopinfos = _loop_lift_get_candidate_infos(cfg, blocks,
                                                func_ir.variable_lifetime.livemap)
     loops = []
+    if loopinfos:
+        _logger.debug('loop lifting this IR with %d candidates:\n%s',
+                      len(loopinfos), func_ir.dump_to_string())
     for loopinfo in loopinfos:
         lifted = _loop_lift_modify_blocks(func_ir, loopinfo, blocks,
                                           typingctx, targetctx, flags, locals)
@@ -300,6 +334,7 @@ def with_lifting(func_ir, typingctx, targetctx, flags, locals):
             # Lifted with-block uses object mode
             myflags.enable_pyobject = True
             myflags.force_pyobject = True
+            myflags.no_cpython_wrapper = False
             cls = ObjModeLiftedWith
         else:
             cls = LiftedWith
