@@ -14,24 +14,31 @@ Constraints push types forward following the dataflow.
 
 from __future__ import print_function, division, absolute_import
 
+import logging
 import operator
 import contextlib
 import itertools
 from pprint import pprint
-import traceback
-from collections import OrderedDict
+from collections import OrderedDict, defaultdict
+from functools import reduce
 
 from numba import ir, types, utils, config, typing
 from .errors import (TypingError, UntypedAttributeError, new_error_context,
-                     termcolor, UnsupportedError)
+                     termcolor, UnsupportedError, ForceLiteralArg)
 from .funcdesc import qualifying_prefix
 
 
-class NOTSET: pass
+_logger = logging.getLogger(__name__)
+
+
+class NOTSET:
+    pass
+
 
 # terminal color markup
 _termcolor = termcolor()
 
+
 class TypeVar(object):
     def __init__(self, context, var):
         self.context = context
@@ -139,19 +146,25 @@ class ConstraintNetwork(object):
                                                    lineno=loc.line):
                 try:
                     constraint(typeinfer)
+                except ForceLiteralArg as e:
+                    errors.append(e)
                 except TypingError as e:
-                    e = TypingError(str(e),
-                                    loc=constraint.loc,
-                                    highlighting=False)
-                    errors.append(e)
-                except Exception:
-                    msg = "Internal error at {con}:\n{sep}\n{err}{sep}\n"
-                    e = TypingError(msg.format(con=constraint,
-                                               err=traceback.format_exc(),
-                                               sep='--%<' + '-' * 76),
-                                    loc=constraint.loc,
-                                    highlighting=False)
-                    errors.append(e)
+                    _logger.debug("captured error", exc_info=e)
+                    new_exc = TypingError(
+                        str(e), loc=constraint.loc,
+                        highlighting=False,
+                    )
+                    errors.append(utils.chain_exception(new_exc, e))
+                except Exception as e:
+                    _logger.debug("captured error", exc_info=e)
+                    msg = ("Internal error at {con}.\n"
+                           "{err}\nEnable logging at debug level for details.")
+                    new_exc = TypingError(
+                        msg.format(con=constraint, err=str(e)),
+                        loc=constraint.loc,
+                        highlighting=False,
+                    )
+                    errors.append(utils.chain_exception(new_exc, e))
         return errors
 
 
@@ -195,7 +208,8 @@ class ArgConstraint(object):
             ty = src.getone()
             if isinstance(ty, types.Omitted):
                 ty = typeinfer.context.resolve_value_type(ty.value)
-            assert ty.is_precise()
+            if not ty.is_precise():
+                raise TypingError('non-precise type {}'.format(ty))
             typeinfer.add_type(self.dst, ty, loc=self.loc)
 
 
@@ -227,7 +241,8 @@ class _BuildContainerConstraint(object):
         self.loc = loc
 
     def __call__(self, typeinfer):
-        with new_error_context("typing of list at {0}", self.loc):
+        with new_error_context("typing of {0} at {1}",
+                               self.container_type, self.loc):
             typevars = typeinfer.typevars
             tsets = [typevars[i.name].get() for i in self.items]
             if not tsets:
@@ -251,6 +266,30 @@ class BuildSetConstraint(_BuildContainerConstraint):
     container_type = types.Set
 
 
+class BuildMapConstraint(object):
+
+    def __init__(self, target, items, loc):
+        self.target = target
+        self.items = items
+        self.loc = loc
+
+    def __call__(self, typeinfer):
+        with new_error_context("typing of dict at {0}", self.loc):
+            typevars = typeinfer.typevars
+            tsets = [(typevars[k.name].getone(), typevars[v.name].getone())
+                     for k, v in self.items]
+            if not tsets:
+                typeinfer.add_type(self.target,
+                                   types.DictType(types.undefined,
+                                                  types.undefined),
+                                   loc=self.loc)
+            else:
+                key_type, value_type = tsets[0]
+                typeinfer.add_type(self.target,
+                                   types.DictType(key_type, value_type),
+                                   loc=self.loc)
+
+
 class ExhaustIterConstraint(object):
     def __init__(self, target, count, iterator, loc):
         self.target = target
@@ -280,9 +319,9 @@ class ExhaustIterConstraint(object):
                     assert tup.is_precise()
                     typeinfer.add_type(self.target, tup, loc=self.loc)
                     break
-            else:
-                raise TypingError("failed to unpack {}".format(tp),
-                                  loc=self.loc)
+                else:
+                    raise TypingError("failed to unpack {}".format(tp),
+                                      loc=self.loc)
 
 
 class PairFirstConstraint(object):
@@ -336,10 +375,14 @@ class StaticGetItemConstraint(object):
         with new_error_context("typing of static-get-item at {0}", self.loc):
             typevars = typeinfer.typevars
             for ty in typevars[self.value.name].get():
-                itemty = typeinfer.context.resolve_static_getitem(
-                            value=ty, index=self.index)
-                if itemty is not None:
-                    assert itemty.is_precise()
+                sig = typeinfer.context.resolve_static_getitem(
+                    value=ty, index=self.index,
+                )
+
+                if sig is not None:
+                    itemty = sig.return_type
+                    # if the itemty is not precise, let it through, unification
+                    # will catch it and produce a better error message
                     typeinfer.add_type(self.target, itemty, loc=self.loc)
                 elif self.fallback is not None:
                     self.fallback(typeinfer)
@@ -349,6 +392,27 @@ class StaticGetItemConstraint(object):
         return self.fallback and self.fallback.get_call_signature()
 
 
+class TypedGetItemConstraint(object):
+    def __init__(self, target, value, dtype, index, loc):
+        self.target = target
+        self.value = value
+        self.dtype = dtype
+        self.index = index
+        self.loc = loc
+
+    def __call__(self, typeinfer):
+        with new_error_context("typing of typed-get-item at {0}", self.loc):
+            typevars = typeinfer.typevars
+            idx_ty = typevars[self.index.name].get()
+            ty = typevars[self.value.name].get()
+            from numba.typing.templates import Signature
+            self.signature = Signature(self.dtype, ty + idx_ty, None)
+            typeinfer.add_type(self.target, self.dtype, loc=self.loc)
+
+    def get_call_signature(self):
+        return self.signature
+
+
 def fold_arg_vars(typevars, args, vararg, kws):
     """
     Fold and resolve the argument variables of a function call.
@@ -433,18 +497,31 @@ class CallConstraint(object):
 
         # Check argument to be precise
         for a in itertools.chain(pos_args, kw_args.values()):
-            if not a.is_precise():
-                # Getitem on non-precise array is allowed to
-                # support array-comprehension
-                if fnty == operator.getitem and isinstance(pos_args[0], types.Array):
-                    pass
-                # Otherwise, don't compute type yet
-                else:
-                    return
+            # Forbids imprecise type except array of undefined dtype
+            if not a.is_precise() and not isinstance(a, types.Array):
+                return
 
         # Resolve call type
-        sig = typeinfer.resolve_call(fnty, pos_args, kw_args)
-
+        try:
+            sig = typeinfer.resolve_call(fnty, pos_args, kw_args)
+        except ForceLiteralArg as e:
+            # Adjust for bound methods
+            folding_args = ((fnty.this,) + tuple(self.args)
+                            if isinstance(fnty, types.BoundFunction)
+                            else self.args)
+            folded = e.fold_arguments(folding_args, self.kws)
+            requested = set()
+            unsatisified = set()
+            for idx in e.requested_args:
+                maybe_arg = typeinfer.func_ir.get_definition(folded[idx])
+                if isinstance(maybe_arg, ir.Arg):
+                    requested.add(maybe_arg.index)
+                else:
+                    unsatisified.add(idx)
+            if unsatisified:
+                raise TypingError("Cannot request literal type.", loc=self.loc)
+            elif requested:
+                raise ForceLiteralArg(requested, loc=self.loc)
         if sig is None:
             # Note: duplicated error checking.
             #       See types.BaseFunction.get_call_type
@@ -465,6 +542,13 @@ class CallConstraint(object):
                 and sig.recvr is not None
                 and sig.recvr != fnty.this):
             refined_this = context.unify_pairs(sig.recvr, fnty.this)
+            if (refined_this is None and
+                    fnty.this.is_precise() and
+                    sig.recvr.is_precise()):
+                msg = "Cannot refine type {} to {}".format(
+                    sig.recvr, fnty.this,
+                )
+                raise TypingError(msg, loc=self.loc)
             if refined_this is not None and refined_this.is_precise():
                 refined_fnty = fnty.copy(this=refined_this)
                 typeinfer.propagate_refined_type(self.func, refined_fnty)
@@ -483,11 +567,20 @@ class CallConstraint(object):
                     sig = sig.replace(return_type=targetty)
 
         self.signature = sig
+        self._add_refine_map(typeinfer, typevars, sig)
 
+    def _add_refine_map(self, typeinfer, typevars, sig):
+        """Add this expression to the refine_map base on the type of target_type
+        """
         target_type = typevars[self.target].getone()
+        # Array
         if (isinstance(target_type, types.Array)
                 and isinstance(sig.return_type.dtype, types.Undefined)):
             typeinfer.refine_map[self.target] = self
+        # DictType
+        if (isinstance(target_type, types.DictType) and
+                not target_type.is_precise()):
+            typeinfer.refine_map[self.target] = self
 
     def refine(self, typeinfer, updated_type):
         # Is getitem?
@@ -499,6 +592,9 @@ class CallConstraint(object):
                 assert updated_type.is_precise()
                 newtype = aryty.copy(dtype=updated_type.dtype)
                 typeinfer.add_type(self.args[0].name, newtype, loc=self.loc)
+        else:
+            m = 'no type refinement implemented for function {} updating to {}'
+            raise TypingError(m.format(self.func, updated_type))
 
     def get_call_signature(self):
         return self.signature
@@ -549,7 +645,27 @@ class GetAttrConstraint(object):
             value=self.value, attr=self.attr)
 
 
-class SetItemConstraint(object):
+class SetItemRefinement(object):
+    """A mixin class to provide the common refinement logic in setitem
+    and static setitem.
+    """
+
+    def _refine_target_type(self, typeinfer, targetty, idxty, valty, sig):
+        """Refine the target-type given the known index type and value type.
+        """
+        # For array setitem, refine imprecise array dtype
+        if _is_array_not_precise(targetty):
+            typeinfer.add_type(self.target.name, sig.args[0], loc=self.loc)
+        # For Dict setitem
+        if isinstance(targetty, types.DictType) and not targetty.is_precise():
+            refined = targetty.refine(idxty, valty)
+            typeinfer.add_type(
+                self.target.name, refined,
+                loc=self.loc,
+            )
+
+
+class SetItemConstraint(SetItemRefinement):
     def __init__(self, target, index, value, loc):
         self.target = target
         self.index = index
@@ -571,18 +687,14 @@ class SetItemConstraint(object):
                 raise TypingError("Cannot resolve setitem: %s[%s] = %s" %
                                   (targetty, idxty, valty), loc=self.loc)
 
-            # For array setitem, refine imprecise array dtype
-            if _is_array_not_precise(targetty):
-                assert sig.args[0].is_precise()
-                typeinfer.add_type(self.target.name, sig.args[0], loc=self.loc)
-
             self.signature = sig
+            self._refine_target_type(typeinfer, targetty, idxty, valty, sig)
 
     def get_call_signature(self):
         return self.signature
 
 
-class StaticSetItemConstraint(object):
+class StaticSetItemConstraint(SetItemRefinement):
     def __init__(self, target, index, index_var, value, loc):
         self.target = target
         self.index = index
@@ -608,6 +720,7 @@ class StaticSetItemConstraint(object):
                 raise TypingError("Cannot resolve setitem: %s[%r] = %s" %
                                   (targetty, self.index, valty), loc=self.loc)
             self.signature = sig
+            self._refine_target_type(typeinfer, targetty, idxty, valty, sig)
 
     def get_call_signature(self):
         return self.signature
@@ -709,6 +822,9 @@ class TypeVarMap(dict):
 
 # A temporary mapping of {function name: dispatcher object}
 _temporary_dispatcher_map = {}
+# A temporary mapping of {function name: dispatcher object reference count}
+# Reference: https://github.com/numba/numba/issues/3658
+_temporary_dispatcher_map_ref_count = defaultdict(int)
 
 
 @contextlib.contextmanager
@@ -723,10 +839,13 @@ def register_dispatcher(disp):
     assert callable(disp.py_func)
     name = disp.py_func.__name__
     _temporary_dispatcher_map[name] = disp
+    _temporary_dispatcher_map_ref_count[name] += 1
     try:
         yield
     finally:
-        del _temporary_dispatcher_map[name]
+        _temporary_dispatcher_map_ref_count[name] -= 1
+        if not _temporary_dispatcher_map_ref_count[name]:
+            del _temporary_dispatcher_map[name]
 
 
 typeinfer_extensions = {}
@@ -794,6 +913,9 @@ class TypeInferer(object):
                 rets.append(inst.value)
         return rets
 
+    def get_argument_types(self):
+        return [self.typevars[k].getone() for k in self.arg_names.values()]
+
     def seed_argument(self, name, index, typ):
         name = self._mangle_arg_name(name)
         self.seed_type(name, typ)
@@ -857,7 +979,12 @@ class TypeInferer(object):
             self.debug.propagate_finished()
         if errors:
             if raise_errors:
-                raise errors[0]
+                force_lit_args = [e for e in errors
+                                  if isinstance(e, ForceLiteralArg)]
+                if not force_lit_args:
+                    raise errors[0]
+                else:
+                    raise reduce(operator.or_, force_lit_args)
             else:
                 return errors
 
@@ -887,7 +1014,7 @@ class TypeInferer(object):
         if source_constraint is not None:
             source_constraint.refine(self, updated_type)
 
-    def unify(self):
+    def unify(self, raise_errors=True):
         """
         Run the final unification pass over all inferred types, and
         catch imprecise types.
@@ -904,7 +1031,7 @@ class TypeInferer(object):
                 if offender is not None:
                     if not exhaustive:
                         break
-                    try: # simple assignment
+                    try:  # simple assignment
                         hasattr(offender.value, 'name')
                         offender_value = offender.value.name
                     except (AttributeError, KeyError):
@@ -938,7 +1065,7 @@ http://numba.pydata.org/numba-doc/latest/user/troubleshoot.html#my-code-has-an-u
                             return list_msg
                         # or might be `foo = list()`
                         elif offender.value.op == 'call':
-                            try: # assignment involving a call
+                            try:  # assignment involving a call
                                 call_name = offender.value.func.name
                                 # find the offender based on the call name
                                 offender = find_offender(call_name)
@@ -947,28 +1074,36 @@ http://numba.pydata.org/numba-doc/latest/user/troubleshoot.html#my-code-has-an-u
                                         return list_msg
                             except (AttributeError, KeyError):
                                 pass
-            return "" # no help possible
+            return ""  # no help possible
 
         def check_var(name):
             tv = self.typevars[name]
             if not tv.defined:
-                offender = find_offender(name)
-                val = getattr(offender, 'value', 'unknown operation')
-                loc = getattr(offender, 'loc', ir.unknown_loc)
-                msg = ("Type of variable '%s' cannot be determined, operation:"
-                      " %s, location: %s")
-                raise TypingError(msg % (var, val, loc), loc)
+                if raise_errors:
+                    offender = find_offender(name)
+                    val = getattr(offender, 'value', 'unknown operation')
+                    loc = getattr(offender, 'loc', ir.unknown_loc)
+                    msg = ("Type of variable '%s' cannot be determined, "
+                           "operation: %s, location: %s")
+                    raise TypingError(msg % (var, val, loc), loc)
+                else:
+                    typdict[var] = types.unknown
+                    return
             tp = tv.getone()
             if not tp.is_precise():
                 offender = find_offender(name, exhaustive=True)
                 msg = ("Cannot infer the type of variable '%s'%s, "
-                      "have imprecise type: %s. %s")
+                       "have imprecise type: %s. %s")
                 istmp = " (temporary variable)" if var.startswith('$') else ""
                 loc = getattr(offender, 'loc', ir.unknown_loc)
                 # is this an untyped list? try and provide help
                 extra_msg = diagnose_imprecision(offender)
-                raise TypingError(msg % (var, istmp, tp, extra_msg), loc)
-            else: # type is precise, hold it
+                if raise_errors:
+                    raise TypingError(msg % (var, istmp, tp, extra_msg), loc)
+                else:
+                    typdict[var] = types.unknown
+                    return
+            else:  # type is precise, hold it
                 typdict[var] = tp
 
         # For better error display, check first user-visible vars, then
@@ -980,30 +1115,92 @@ http://numba.pydata.org/numba-doc/latest/user/troubleshoot.html#my-code-has-an-u
         for var in sorted(temps):
             check_var(var)
 
-        retty = self.get_return_type(typdict)
-        fntys = self.get_function_types(typdict)
+        try:
+            retty = self.get_return_type(typdict)
+        except Exception as e:
+            # partial type inference may raise e.g. attribute error if a
+            # constraint has no computable signature, ignore this as needed
+            if raise_errors:
+                raise e
+            else:
+                retty = None
+
+        try:
+            fntys = self.get_function_types(typdict)
+        except Exception as e:
+            # partial type inference may raise e.g. attribute error if a
+            # constraint has no computable signature, ignore this as needed
+            if raise_errors:
+                raise e
+            else:
+                fntys = None
+
         if self.generator_info:
-            retty = self.get_generator_type(typdict, retty)
+            retty = self.get_generator_type(typdict, retty,
+                                            raise_errors=raise_errors)
 
         self.debug.unify_finished(typdict, retty, fntys)
 
         return typdict, retty, fntys
 
-    def get_generator_type(self, typdict, retty):
+    def get_generator_type(self, typdict, retty, raise_errors=True):
         gi = self.generator_info
         arg_types = [None] * len(self.arg_names)
         for index, name in self.arg_names.items():
             arg_types[index] = typdict[name]
-        state_types = [typdict[var_name] for var_name in gi.state_vars]
-        yield_types = [typdict[y.inst.value.name]
-                       for y in gi.get_yield_points()]
+
+        state_types = None
+        try:
+            state_types = [typdict[var_name] for var_name in gi.state_vars]
+        except KeyError:
+            msg = "Cannot type generator: state variable types cannot be found"
+            if raise_errors:
+                raise TypingError(msg)
+            state_types = [types.unknown for _ in gi.state_vars]
+
+        yield_types = None
+        try:
+            yield_types = [typdict[y.inst.value.name]
+                           for y in gi.get_yield_points()]
+        except KeyError:
+            msg = "Cannot type generator: yield type cannot be found"
+            if raise_errors:
+                raise TypingError(msg)
         if not yield_types:
             msg = "Cannot type generator: it does not yield any value"
-            raise TypingError(msg)
+            if raise_errors:
+                raise TypingError(msg)
+            yield_types = [types.unknown for _ in gi.get_yield_points()]
+
+        if not yield_types or all(yield_types) == types.unknown:
+            # unknown yield, probably partial type inference, escape
+            return types.Generator(self.func_id.func, types.unknown, arg_types,
+                                   state_types, has_finalizer=True)
+
         yield_type = self.context.unify_types(*yield_types)
-        if yield_type is None:
+        if yield_type is None or isinstance(yield_type, types.Optional):
             msg = "Cannot type generator: cannot unify yielded types %s"
-            raise TypingError(msg % (yield_types,))
+            yp_highlights = []
+            for y in gi.get_yield_points():
+                msg = (_termcolor.errmsg("Yield of: IR '%s', type '%s', "
+                                         "location: %s"))
+                yp_highlights.append(msg % (str(y.inst),
+                                            typdict[y.inst.value.name],
+                                            y.inst.loc.strformat()))
+
+            explain_ty = set()
+            for ty in yield_types:
+                if isinstance(ty, types.Optional):
+                    explain_ty.add(ty.type)
+                    explain_ty.add(types.NoneType('none'))
+                else:
+                    explain_ty.add(ty)
+            if raise_errors:
+                raise TypingError("Can't unify yield type from the "
+                                  "following types: %s"
+                                  % ", ".join(sorted(map(str, explain_ty))) +
+                                  "\n\n" + "\n".join(yp_highlights))
+
         return types.Generator(self.func_id.func, yield_type, arg_types,
                                state_types, has_finalizer=True)
 
@@ -1039,6 +1236,7 @@ http://numba.pydata.org/numba-doc/latest/user/troubleshoot.html#my-code-has-an-u
                                     returns[x] = instr
                                     break
 
+                    interped = ""
                     for name, offender in returns.items():
                         loc = getattr(offender, 'loc', ir.unknown_loc)
                         msg = ("Return of: IR name '%s', type '%s', "
@@ -1084,9 +1282,11 @@ http://numba.pydata.org/numba-doc/latest/user/troubleshoot.html#my-code-has-an-u
             self.typeof_setattr(inst)
         elif isinstance(inst, ir.Print):
             self.typeof_print(inst)
+        elif isinstance(inst, ir.StoreMap):
+            self.typeof_storemap(inst)
         elif isinstance(inst, (ir.Jump, ir.Branch, ir.Return, ir.Del)):
             pass
-        elif isinstance(inst, ir.StaticRaise):
+        elif isinstance(inst, (ir.StaticRaise, ir.StaticTryRaise)):
             pass
         elif type(inst) in typeinfer_extensions:
             # let external calls handle stmt if type matches
@@ -1102,6 +1302,12 @@ http://numba.pydata.org/numba-doc/latest/user/troubleshoot.html#my-code-has-an-u
         self.constraints.append(constraint)
         self.calls.append((inst, constraint))
 
+    def typeof_storemap(self, inst):
+        constraint = SetItemConstraint(target=inst.dct, index=inst.key,
+                                       value=inst.value, loc=inst.loc)
+        self.constraints.append(constraint)
+        self.calls.append((inst, constraint))
+
     def typeof_static_setitem(self, inst):
         constraint = StaticSetItemConstraint(target=inst.target,
                                              index=inst.index,
@@ -1226,6 +1432,8 @@ http://numba.pydata.org/numba-doc/latest/user/troubleshoot.html#my-code-has-an-u
 
             sig = typing.signature(return_type, *args)
             sig.pysig = pysig
+            # Keep track of unique return_type
+            frame.add_return_type(return_type)
             return sig
         else:
             # Normal non-recursive call
@@ -1241,8 +1449,19 @@ http://numba.pydata.org/numba-doc/latest/user/troubleshoot.html#my-code-has-an-u
                 # as a global variable
                 typ = types.Dispatcher(_temporary_dispatcher_map[gvar.name])
             else:
-                msg = _termcolor.errmsg("Untyped global name '%s':") + " %s"
-                e.patch_message(msg % (gvar.name, e))
+                nm = gvar.name
+                msg = _termcolor.errmsg("Untyped global name '%s':" % nm)
+                msg += " %s"  # interps the actual error
+
+                # if the untyped global is a numba internal function then add
+                # to the error message asking if it's been imported.
+                from numba import special
+                if nm in special.__all__:
+                    tmp = ("\n'%s' looks like a Numba internal function, has "
+                           "it been imported (i.e. 'from numba import %s')?\n" %
+                           (nm, nm))
+                    msg += _termcolor.errmsg(tmp)
+                e.patch_message(msg % e)
                 raise
 
         if isinstance(typ, types.Dispatcher) and typ.dispatcher.is_compiling:
@@ -1306,9 +1525,16 @@ http://numba.pydata.org/numba-doc/latest/user/troubleshoot.html#my-code-has-an-u
             self.constraints.append(constraint)
             self.calls.append((inst.value, constraint))
         elif expr.op == 'getitem':
-            self.typeof_intrinsic_call(
-                inst, target, operator.getitem, expr.value, expr.index,
-                )
+            self.typeof_intrinsic_call(inst, target, operator.getitem,
+                                       expr.value, expr.index,)
+        elif expr.op == 'typed_getitem':
+            constraint = TypedGetItemConstraint(target.name, value=expr.value,
+                                                dtype=expr.dtype,
+                                                index=expr.index,
+                                                loc=expr.loc)
+            self.constraints.append(constraint)
+            self.calls.append((inst.value, constraint))
+
         elif expr.op == 'getattr':
             constraint = GetAttrConstraint(target.name, attr=expr.attr,
                                            value=expr.value, loc=inst.loc,
@@ -1326,6 +1552,10 @@ http://numba.pydata.org/numba-doc/latest/user/troubleshoot.html#my-code-has-an-u
             constraint = BuildSetConstraint(target.name, items=expr.items,
                                             loc=inst.loc)
             self.constraints.append(constraint)
+        elif expr.op == 'build_map':
+            constraint = BuildMapConstraint(target.name, items=expr.items,
+                                            loc=inst.loc)
+            self.constraints.append(constraint)
         elif expr.op == 'cast':
             self.constraints.append(Propagate(dst=target.name,
                                               src=expr.value.name,
