@@ -3,7 +3,7 @@ Helper classes / mixins for defining types.
 """
 from __future__ import print_function, division, absolute_import
 
-from .abstract import *
+from .abstract import ArrayCompatible, Dummy, IterableType, IteratorType
 
 
 class Opaque(Dummy):
@@ -50,7 +50,7 @@ class Buffer(IterableType, ArrayCompatible):
         from .misc import unliteral
 
         if isinstance(dtype, Buffer):
-            raise TypeError("Buffer dtype cannot be buffer")
+            raise TypeError("Buffer dtype cannot be buffer, have dtype: {}".format(dtype))
         if layout not in self.LAYOUTS:
             raise ValueError("Invalid layout '%s'" % layout)
         self.dtype = unliteral(dtype)
