@@ -1,13 +1,10 @@
 from __future__ import print_function, division, absolute_import
 
 import traceback
-import inspect
-import sys
 
-from .abstract import *
-from .common import *
+from .abstract import Callable, DTypeSpec, Dummy, Literal, Type, weakref
+from .common import Opaque
 from .misc import unliteral
-from numba.ir import Loc
 from numba import errors
 
 # terminal color markup
@@ -75,6 +72,12 @@ class _ResolutionFailures(object):
             frame = traceback.extract_tb(error.__traceback__)[-1]
             return "{}:{}".format(frame[0], frame[1])
 
+    def raise_error(self):
+        for _tempcls, e in self._failures:
+            if isinstance(e, errors.ForceLiteralArg):
+                raise e
+        raise errors.TypingError(self.format())
+
 
 class BaseFunction(Callable):
     """
@@ -82,12 +85,13 @@ class BaseFunction(Callable):
     """
 
     def __init__(self, template):
+
         if isinstance(template, (list, tuple)):
             self.templates = tuple(template)
             keys = set(temp.key for temp in self.templates)
             if len(keys) != 1:
                 raise ValueError("incompatible templates: keys = %s"
-                                 % (this,))
+                                 % (keys,))
             self.typing_key, = keys
         else:
             self.templates = (template,)
@@ -116,15 +120,12 @@ class BaseFunction(Callable):
         return self._impl_keys[sig.args]
 
     def get_call_type(self, context, args, kws):
-        return self.get_call_type_with_literals(context, args, kws)
-
-    def get_call_type_with_literals(self, context, args, kws, literals=None):
         failures = _ResolutionFailures(context, self, args, kws)
         for temp_cls in self.templates:
             temp = temp_cls(context)
-            for support_literals in [True, False]:
+            for uselit in [True, False]:
                 try:
-                    if support_literals:
+                    if uselit:
                         sig = temp.apply(args, kws)
                     else:
                         nolitargs = tuple([unliteral(a) for a in args])
@@ -138,16 +139,15 @@ class BaseFunction(Callable):
                         self._impl_keys[sig.args] = temp.get_impl_key(sig)
                         return sig
                     else:
-                        haslit= '' if support_literals else 'out'
+                        haslit= '' if uselit else 'out'
                         msg = "All templates rejected with%s literals." % haslit
                         failures.add_error(temp_cls, msg)
 
         if len(failures) == 0:
             raise AssertionError("Internal Error. "
                                  "Function resolution ended with no failures "
-                                 "or successfull signature")
-
-        raise errors.TypingError(failures.format())
+                                 "or successful signature")
+        failures.raise_error()
 
     def get_call_signatures(self):
         sigs = []
@@ -208,7 +208,7 @@ class BoundFunction(Callable, Opaque):
         # Try with Literal
         try:
             out = template.apply(args, kws)
-        except Exception as e:
+        except Exception:
             out = None
         # If that doesn't work, remove literals
         if out is None:
@@ -219,12 +219,6 @@ class BoundFunction(Callable, Opaque):
             raise e
         return out
 
-    def get_call_type_with_literals(self, context, args, kws, literals):
-        if literals is not None and self.template.support_literals:
-            return self.template(context).apply(*literals)
-        else:
-            return self.get_call_type(context, args, kws)
-
     def get_call_signatures(self):
         sigs = getattr(self.template, 'cases', [])
         is_param = hasattr(self.template, 'generic')
@@ -371,26 +365,6 @@ class ExternalFunction(Function):
         return self.symbol, self.sig
 
 
-class NumbaFunction(Function):
-    """
-    A named native function with the Numba calling convention
-    (resolvable by LLVM).
-    For internal use only.
-    """
-
-    def __init__(self, fndesc, sig):
-        from .. import typing
-        self.fndesc = fndesc
-        self.sig = sig
-        template = typing.make_concrete_template(fndesc.qualname,
-                                                 fndesc.qualname, [sig])
-        super(NumbaFunction, self).__init__(template)
-
-    @property
-    def key(self):
-        return self.fndesc.unique_name, self.sig
-
-
 class NamedTupleClass(Callable, Opaque):
     """
     Type class for namedtuple classes.
@@ -402,7 +376,7 @@ class NamedTupleClass(Callable, Opaque):
         super(NamedTupleClass, self).__init__(name)
 
     def get_call_type(self, context, args, kws):
-        # Overriden by the __call__ constructor resolution in typing.collections
+        # Overridden by the __call__ constructor resolution in typing.collections
         return None
 
     def get_call_signatures(self):
@@ -424,7 +398,7 @@ class NumberClass(Callable, DTypeSpec, Opaque):
         super(NumberClass, self).__init__(name)
 
     def get_call_type(self, context, args, kws):
-        # Overriden by the __call__ constructor resolution in typing.builtins
+        # Overridden by the __call__ constructor resolution in typing.builtins
         return None
 
     def get_call_signatures(self):
