@@ -1,7 +1,7 @@
 from __future__ import print_function, division, absolute_import
 
-from .abstract import *
-from .common import *
+from .abstract import Callable, Literal, Type
+from .common import Dummy, IterableType, Opaque, SimpleIteratorType
 from ..typeconv import Conversion
 from ..errors import TypingError, LiteralTypingError
 
@@ -58,12 +58,14 @@ def unliteral(lit_type):
 def literal(value):
     """Returns a Literal instance or raise LiteralTypingError
     """
-    assert not isinstance(value, Literal)
     ty = type(value)
+    if isinstance(value, Literal):
+        msg = "the function does not accept a Literal type; got {} ({})"
+        raise ValueError(msg.format(value, ty))
     try:
         ctor = Literal.ctor_map[ty]
     except KeyError:
-        raise LiteralTypingError(ty)
+        raise LiteralTypingError("{} cannot be used as a literal".format(ty))
     else:
         return ctor(value)
 
@@ -207,10 +209,14 @@ class Optional(Type):
 
     def __init__(self, typ):
         assert not isinstance(typ, (Optional, NoneType))
-        self.type = unliteral(typ)
-        name = "?%s" % typ
+        typ = unliteral(typ)
+        self.type = typ
+        name = "OptionalType(%s)" % self.type
         super(Optional, self).__init__(name)
 
+    def __str__(self):
+        return "%s i.e. the type '%s or None'" % (self.name, self.type)
+
     @property
     def key(self):
         return self.type
@@ -356,7 +362,7 @@ class ClassInstanceType(Type):
 
     @property
     def classname(self):
-        return self.class_type.class_def.__name__
+        return self.class_type.class_name
 
     @property
     def jitprops(self):
@@ -386,17 +392,16 @@ class ClassType(Callable, Opaque):
 
     def __init__(self, class_def, ctor_template_cls, struct, jitmethods,
                  jitprops):
-        self.class_def = class_def
-        self.ctor_template = self._specialize_template(ctor_template_cls)
+        self.class_name = class_def.__name__
+        self.class_doc = class_def.__doc__
+        self._ctor_template_class = ctor_template_cls
         self.jitmethods = jitmethods
         self.jitprops = jitprops
         self.struct = struct
-        self.methods = dict((k, v.py_func) for k, v in self.jitmethods.items())
         fielddesc = ','.join("{0}:{1}".format(k, v) for k, v in struct.items())
-        name = "{0}.{1}#{2:x}<{3}>".format(self.name_prefix, class_def.__name__,
-                                           id(class_def), fielddesc)
+        name = "{0}.{1}#{2:x}<{3}>".format(self.name_prefix, self.class_name,
+                                           id(self), fielddesc)
         super(ClassType, self).__init__(name)
-        self.instance_type = self.instance_type_class(self)
 
     def get_call_type(self, context, args, kws):
         return self.ctor_template(context).apply(args, kws)
@@ -404,6 +409,19 @@ class ClassType(Callable, Opaque):
     def get_call_signatures(self):
         return (), True
 
+    @property
+    def methods(self):
+        methods = dict((k, v.py_func) for k, v in self.jitmethods.items())
+        return methods
+
+    @property
+    def instance_type(self):
+        return ClassInstanceType(self)
+
+    @property
+    def ctor_template(self):
+        return self._specialize_template(self._ctor_template_class)
+
     def _specialize_template(self, basecls):
         return type(basecls.__name__, (basecls,), dict(key=self))
 
@@ -474,7 +492,19 @@ class ContextManager(Callable, Phantom):
         return typing.signature(self, *posargs)
 
 
-class UnicodeType(Type):
+class UnicodeType(IterableType):
+
     def __init__(self, name):
         super(UnicodeType, self).__init__(name)
 
+    @property
+    def iterator_type(self):
+        return UnicodeIteratorType(self)
+
+
+class UnicodeIteratorType(SimpleIteratorType):
+
+    def __init__(self, dtype):
+        name = "iter_unicode"
+        self.data = dtype
+        super(UnicodeIteratorType, self).__init__(name, dtype)
