@@ -168,10 +168,8 @@ class GetItemBuffer(AbstractTemplate):
         if out is not None:
             return signature(out.result, ary, out.index)
 
-@infer
+@infer_global(operator.setitem)
 class SetItemBuffer(AbstractTemplate):
-    key = "setitem"
-
     def generic(self, args, kws):
         assert not kws
         ary, idx, val = args
@@ -314,19 +312,22 @@ class ArrayAttribute(AttributeTemplate):
                 assert ary.ndim == 1
                 return signature(ary, *args)
 
+            if isinstance(shape, types.NoneType):
+                return signature(self.resolve_T(ary))
+
             shape = normalize_shape(shape)
             if shape is None:
                 return
 
             assert ary.ndim == shape.count
-            return signature(self.resolve_T(ary), shape)
+            return signature(self.resolve_T(ary).copy(layout="A"), shape)
 
         else:
             if any(not sentry_shape_scalar(a) for a in args):
                 raise TypeError("transpose({0}) is not supported".format(
                     ', '.join(args)))
             assert ary.ndim == len(args)
-            return signature(self.resolve_T(ary), *args)
+            return signature(self.resolve_T(ary).copy(layout="A"), *args)
 
     @bound_function("array.copy")
     def resolve_copy(self, ary, args, kws):
@@ -451,7 +452,9 @@ class ArrayAttribute(AttributeTemplate):
                             "cannot convert from %s to %s"
                             % (dtype, ary, ary.dtype, dtype))
         layout = ary.layout if ary.layout in 'CF' else 'C'
-        retty = ary.copy(dtype=dtype, layout=layout)
+        # reset the write bit irrespective of whether the cast type is the same
+        # as the current dtype, this replicates numpy
+        retty = ary.copy(dtype=dtype, layout=layout, readonly=False)
         return signature(retty, *args)
 
     @bound_function("array.ravel")
@@ -518,7 +521,8 @@ class StaticGetItemArray(AbstractTemplate):
         if (isinstance(ary, types.Array) and isinstance(idx, str) and
             isinstance(ary.dtype, types.Record)):
             if idx in ary.dtype.fields:
-                return ary.copy(dtype=ary.dtype.typeof(idx), layout='A')
+                ret = ary.copy(dtype=ary.dtype.typeof(idx), layout='A')
+                return signature(ret, *args)
 
 
 @infer_getattr
@@ -540,7 +544,7 @@ class StaticGetItemRecord(AbstractTemplate):
         if isinstance(record, types.Record) and isinstance(idx, str):
             ret = record.typeof(idx)
             assert ret
-            return ret
+            return signature(ret, *args)
 
 @infer
 class StaticSetItemRecord(AbstractTemplate):
@@ -596,43 +600,88 @@ def _expand_integer(ty):
     else:
         return ty
 
+
 def generic_homog(self, args, kws):
     assert not args
     assert not kws
     return signature(self.this.dtype, recvr=self.this)
 
+
 def generic_expand(self, args, kws):
     assert not args
     assert not kws
     return signature(_expand_integer(self.this.dtype), recvr=self.this)
 
+
 def sum_expand(self, args, kws):
     """
-    sum can be called with or without an axis parameter.
+    sum can be called with or without an axis parameter, and with or without
+    a dtype parameter
     """
     pysig = None
-    if kws:
+    if 'axis' in kws and 'dtype' not in kws:
         def sum_stub(axis):
             pass
         pysig = utils.pysignature(sum_stub)
         # rewrite args
         args = list(args) + [kws['axis']]
-        kws = None
+    elif 'dtype' in kws and 'axis' not in kws:
+        def sum_stub(dtype):
+            pass
+        pysig = utils.pysignature(sum_stub)
+        # rewrite args
+        args = list(args) + [kws['dtype']]
+    elif 'dtype' in kws and 'axis' in kws:
+        def sum_stub(axis, dtype):
+            pass
+        pysig = utils.pysignature(sum_stub)
+        # rewrite args
+        args = list(args) + [kws['axis'], kws['dtype']]
+
     args_len = len(args)
-    assert args_len <= 1
+    assert args_len <= 2
     if args_len == 0:
-        # No axis parameter so the return type of the summation is a scalar
+        # No axis or dtype parameter so the return type of the summation is a scalar
         # of the type of the array.
         out = signature(_expand_integer(self.this.dtype), *args,
                         recvr=self.this)
-    else:
-        # There is an axis paramter so the return type of this summation is
-        # an array of dimension one less than the input array.
-        return_type = types.Array(dtype=_expand_integer(self.this.dtype),
-                                  ndim=self.this.ndim-1, layout='C')
+    elif args_len == 1 and 'dtype' not in kws:
+        # There is an axis parameter, either arg or kwarg
+        if self.this.ndim == 1:
+            # 1d reduces to a scalar
+            return_type = self.this.dtype
+        else:
+            # the return type of this summation is  an array of dimension one
+            # less than the input array.
+            return_type = types.Array(dtype=_expand_integer(self.this.dtype),
+                                    ndim=self.this.ndim-1, layout='C')
         out = signature(return_type, *args, recvr=self.this)
+
+    elif args_len == 1 and 'dtype' in kws:
+        # No axis parameter so the return type of the summation is a scalar
+        # of the dtype parameter.
+        from .npydecl import _parse_dtype
+        dtype, = args
+        dtype = _parse_dtype(dtype)
+        out = signature(dtype, *args, recvr=self.this)
+
+    elif args_len == 2:
+        # There is an axis and dtype parameter, either arg or kwarg
+        from .npydecl import _parse_dtype
+        dtype = _parse_dtype(args[1])
+        return_type = dtype
+        if self.this.ndim != 1:
+            # 1d reduces to a scalar, 2d and above reduce dim by 1
+            # the return type of this summation is  an array of dimension one
+            # less than the input array.
+            return_type = types.Array(dtype=return_type,
+                                    ndim=self.this.ndim-1, layout='C')
+        out = signature(return_type, *args, recvr=self.this)
+    else:
+        pass
     return out.replace(pysig=pysig)
 
+
 def generic_expand_cumulative(self, args, kws):
     assert not args
     assert not kws
@@ -641,6 +690,7 @@ def generic_expand_cumulative(self, args, kws):
                               ndim=1, layout='C')
     return signature(return_type, recvr=self.this)
 
+
 def generic_hetero_real(self, args, kws):
     assert not args
     assert not kws
@@ -662,11 +712,9 @@ def generic_index(self, args, kws):
     assert not kws
     return signature(types.intp, recvr=self.this)
 
-def install_array_method(name, generic, support_literals=False):
+def install_array_method(name, generic):
     my_attr = {"key": "array." + name, "generic": generic}
     temp_class = type("Array_" + name, (AbstractTemplate,), my_attr)
-    if support_literals:
-        temp_class.support_literals = support_literals
     def array_attribute_attachment(self, ary):
         return types.BoundFunction(temp_class, ary)
 
@@ -678,7 +726,7 @@ for fname in ["min", "max"]:
 
 # Functions that return a machine-width type, to avoid overflows
 install_array_method("prod", generic_expand)
-install_array_method("sum", sum_expand, support_literals=True)
+install_array_method("sum", sum_expand)
 
 # Functions that return a machine-width type, to avoid overflows
 for fname in ["cumsum", "cumprod"]:
