@@ -5,7 +5,7 @@ import itertools
 import numpy as np
 import operator
 
-from numba import types, prange
+from numba import types, prange, errors
 from numba.parfor import internal_prange
 
 from numba.utils import PYVERSION, RANGE_ITER_OBJECTS, IS_PY3
@@ -37,16 +37,18 @@ class PrintItem(AbstractTemplate):
 @infer_global(abs)
 class Abs(ConcreteTemplate):
     int_cases = [signature(ty, ty) for ty in sorted(types.signed_domain)]
+    uint_cases = [signature(ty, ty) for ty in sorted(types.unsigned_domain)]
     real_cases = [signature(ty, ty) for ty in sorted(types.real_domain)]
     complex_cases = [signature(ty.underlying_float, ty)
                      for ty in sorted(types.complex_domain)]
-    cases = int_cases + real_cases + complex_cases
+    cases = int_cases + uint_cases +  real_cases + complex_cases
 
 
 @infer_global(slice)
 class Slice(ConcreteTemplate):
     cases = [
-        signature(types.slice2_type),
+        signature(types.slice2_type, types.intp),
+        signature(types.slice2_type, types.none),
         signature(types.slice2_type, types.none, types.none),
         signature(types.slice2_type, types.none, types.intp),
         signature(types.slice2_type, types.intp, types.none),
@@ -54,7 +56,11 @@ class Slice(ConcreteTemplate):
         signature(types.slice3_type, types.intp, types.intp, types.intp),
         signature(types.slice3_type, types.none, types.intp, types.intp),
         signature(types.slice3_type, types.intp, types.none, types.intp),
+        signature(types.slice3_type, types.intp, types.intp, types.none),
+        signature(types.slice3_type, types.intp, types.none, types.none),
+        signature(types.slice3_type, types.none, types.intp, types.none),
         signature(types.slice3_type, types.none, types.none, types.intp),
+        signature(types.slice3_type, types.none, types.none, types.none),
     ]
 
 
@@ -88,7 +94,16 @@ class GetIter(AbstractTemplate):
         assert not kws
         [obj] = args
         if isinstance(obj, types.IterableType):
-            return signature(obj.iterator_type, obj)
+            # Raise this here to provide a very specific message about this
+            # common issue, delaying the error until later leads to something
+            # less specific being noted as the problem (e.g. no support for
+            # getiter on array(<>, 2, 'C')).
+            if isinstance(obj, types.Array) and obj.ndim > 1:
+                msg = ("Direct iteration is not supported for arrays with "
+                       "dimension > 1. Try using indexing instead.")
+                raise errors.TypingError(msg)
+            else:
+                return signature(obj.iterator_type, obj)
 
 
 @infer
@@ -549,10 +564,8 @@ class GetItemCPointer(AbstractTemplate):
             return signature(ptr.dtype, ptr, normalize_1d_index(idx))
 
 
-@infer
+@infer_global(operator.setitem)
 class SetItemCPointer(AbstractTemplate):
-    key = "setitem"
-
     def generic(self, args, kws):
         assert not kws
         ptr, idx, val = args
@@ -610,9 +623,10 @@ class StaticGetItemTuple(AbstractTemplate):
         if not isinstance(tup, types.BaseTuple):
             return
         if isinstance(idx, int):
-            return tup.types[idx]
+            ret = tup.types[idx]
         elif isinstance(idx, slice):
-            return types.BaseTuple.from_types(tup.types[idx])
+            ret = types.BaseTuple.from_types(tup.types[idx])
+        return signature(ret, *args)
 
 
 # Generic implementation for "not in"
@@ -718,6 +732,20 @@ class SliceAttribute(AttributeTemplate):
     def resolve_step(self, ty):
         return types.intp
 
+    @bound_function("slice.indices")
+    def resolve_indices(self, ty, args, kws):
+        assert not kws
+        if len(args) != 1:
+            raise TypeError(
+                "indices() takes exactly one argument (%d given)" % len(args)
+            )
+        typ, = args
+        if not isinstance(typ, types.Integer):
+            raise TypeError(
+                "'%s' object cannot be interpreted as an integer" % typ
+            )
+        return signature(types.UniTuple(types.intp, 3), types.intp)
+
 
 #-------------------------------------------------------------------------------
 
@@ -745,15 +773,33 @@ class NumberClassAttribute(AttributeTemplate):
         return types.Function(make_callable_template(key=ty, typer=typer))
 
 
-def register_number_classes(register_global):
-    nb_types = set(types.number_domain)
-    nb_types.add(types.bool_)
+@infer_getattr
+class TypeRefAttribute(AttributeTemplate):
+    key = types.TypeRef
 
-    for ty in nb_types:
-        register_global(ty, types.NumberClass(ty))
+    def resolve___call__(self, classty):
+        """
+        Resolve a number class's constructor (e.g. calling int(...))
 
+        Note:
 
-register_number_classes(infer_global)
+        This is needed because of the limitation of the current type-system
+        implementation.  Specifically, the lack of a higher-order type
+        (i.e. passing the ``DictType`` vs ``DictType(key_type, value_type)``)
+        """
+        ty = classty.instance_type
+
+        if isinstance(ty, type) and issubclass(ty, types.Type):
+            # Redirect the typing to a:
+            #   @type_callable(ty)
+            #   def typeddict_call(context):
+            #        ...
+            # For example, see numba/typed/typeddict.py
+            #   @type_callable(DictType)
+            #   def typeddict_call(context):
+            def redirect(*args, **kwargs):
+                return self.context.resolve_function_type(ty, args, kwargs)
+            return types.Function(make_callable_template(key=ty, typer=redirect))
 
 
 #------------------------------------------------------------------------------
@@ -820,16 +866,6 @@ class Round(ConcreteTemplate):
     ]
 
 
-@infer_global(hash)
-class Hash(AbstractTemplate):
-
-    def generic(self, args, kws):
-        assert not kws
-        arg, = args
-        if isinstance(arg, types.Hashable):
-            return signature(types.intp, *args)
-
-
 #------------------------------------------------------------------------------
 
 
