@@ -5,6 +5,7 @@ import types as pytypes
 import weakref
 import threading
 import contextlib
+import operator
 
 import numba
 from numba import types, errors
@@ -13,12 +14,7 @@ from . import templates
 from .typeof import typeof, Purpose
 
 from numba import utils
-from numba.config import PYVERSION
-
-if PYVERSION >= (3, 3):
-    from collections.abc import Sequence
-else:
-    from collections import Sequence
+from numba.six import Sequence
 
 
 class Rating(object):
@@ -114,10 +110,23 @@ class CallFrame(object):
         self.typeinfer = typeinfer
         self.func_id = func_id
         self.args = args
+        self._inferred_retty = set()
 
     def __repr__(self):
         return "CallFrame({}, {})".format(self.func_id, self.args)
 
+    def add_return_type(self, return_type):
+        """Add *return_type* to the list of inferred return-types.
+        If there are too many, raise `TypingError`.
+        """
+        # The maximum limit is picked arbitrarily.
+        # Don't think that this needs to be user configurable.
+        RETTY_LIMIT = 16
+        self._inferred_retty.add(return_type)
+        if len(self._inferred_retty) >= RETTY_LIMIT:
+            m = "Return type of recursive function does not converge"
+            raise errors.TypingError(m)
+
 
 class BaseContext(object):
     """A typing context for storing function typing constrain template.
@@ -138,7 +147,7 @@ class BaseContext(object):
 
     def init(self):
         """
-        Initialize the typing context.  Can be overriden by subclasses.
+        Initialize the typing context.  Can be overridden by subclasses.
         """
 
     def refresh(self):
@@ -260,10 +269,9 @@ class BaseContext(object):
         The attribute's type is returned, or None if resolution failed.
         """
         def core(typ):
-            for attrinfo in self._get_attribute_templates(typ):
-                ret = attrinfo.resolve(typ, attr)
-                if ret is not None:
-                    return ret
+            out = self.find_matching_getattr_template(typ, attr)
+            if out:
+                return out['return_type']
 
         out = core(typ)
         if out is not None:
@@ -279,6 +287,15 @@ class BaseContext(object):
             if attrty is not None:
                 return attrty
 
+    def find_matching_getattr_template(self, typ, attr):
+        for template in self._get_attribute_templates(typ):
+            return_type = template.resolve(typ, attr)
+            if return_type is not None:
+                return {
+                    'template': template,
+                    'return_type': return_type,
+                }
+
     def resolve_setattr(self, target, attr, value):
         """
         Resolve setting the attribute *attr* (a string) on the *target* type
@@ -306,14 +323,16 @@ class BaseContext(object):
 
     def resolve_setitem(self, target, index, value):
         assert isinstance(index, types.Type), index
-        args = target, index, value
-        kws = {}
-        return self.resolve_function_type("setitem", args, kws)
+        fnty = self.resolve_value_type(operator.setitem)
+        sig = fnty.get_call_type(self, (target, index, value), {})
+        return sig
 
     def resolve_delitem(self, target, index):
         args = target, index
         kws = {}
-        return self.resolve_function_type("delitem", args, kws)
+        fnty = self.resolve_value_type(operator.delitem)
+        sig = fnty.get_call_type(self, args, kws)
+        return sig
 
     def resolve_module_constants(self, typ, attr):
         """
@@ -358,7 +377,7 @@ class BaseContext(object):
         else:
             return ty
 
-        if isinstance(val, (types.ExternalFunction, types.NumbaFunction)):
+        if isinstance(val, types.ExternalFunction):
             return val
 
         # Try to look up target specific typing information
@@ -377,13 +396,15 @@ class BaseContext(object):
 
     def _load_builtins(self):
         # Initialize declarations
-        from . import builtins, arraydecl, npdatetime
-        from . import ctypes_utils, bufproto
+        from . import builtins, arraydecl, npdatetime  # noqa: F401
+        from . import ctypes_utils, bufproto           # noqa: F401
+        from numba.unsafe import eh                    # noqa: F401
+
         self.install_registry(templates.builtin_registry)
 
     def load_additional_registries(self):
         """
-        Load target-specific registries.  Can be overriden by subclasses.
+        Load target-specific registries.  Can be overridden by subclasses.
         """
 
     def install_registry(self, registry):
@@ -500,7 +521,8 @@ class BaseContext(object):
             else:
                 return min(forward, backward)
 
-    def _rate_arguments(self, actualargs, formalargs, unsafe_casting=True):
+    def _rate_arguments(self, actualargs, formalargs, unsafe_casting=True,
+                        exact_match_required=False):
         """
         Rate the actual arguments for compatibility against the formal
         arguments.  A Rating instance is returned, or None if incompatible.
@@ -514,6 +536,8 @@ class BaseContext(object):
                 return None
             elif not unsafe_casting and conv >= Conversion.unsafe:
                 return None
+            elif exact_match_required and conv != Conversion.exact:
+                return None
 
             if conv == Conversion.promote:
                 rate.promote += 1
@@ -548,7 +572,8 @@ class BaseContext(object):
         return True
 
     def resolve_overload(self, key, cases, args, kws,
-                         allow_ambiguous=True, unsafe_casting=True):
+                         allow_ambiguous=True, unsafe_casting=True,
+                         exact_match_required=False):
         """
         Given actual *args* and *kws*, find the best matching
         signature in *cases*, or None if none matches.
@@ -560,6 +585,7 @@ class BaseContext(object):
         assert not kws, "Keyword arguments are not supported, yet"
         options = {
             'unsafe_casting': unsafe_casting,
+            'exact_match_required': exact_match_required,
         }
         # Rate each case
         candidates = []
@@ -641,7 +667,8 @@ class BaseContext(object):
             # Can convert from second to first
             return first
 
-        if isinstance(first, types.Literal) or isinstance(second, types.Literal):
+        if isinstance(first, types.Literal) or \
+           isinstance(second, types.Literal):
             first = types.unliteral(first)
             second = types.unliteral(second)
             return self.unify_pairs(first, second)
@@ -653,8 +680,17 @@ class BaseContext(object):
 class Context(BaseContext):
 
     def load_additional_registries(self):
-        from . import (cffi_utils, cmathdecl, enumdecl, listdecl, mathdecl,
-                       npydecl, randomdecl, setdecl)
+        from . import (
+            cffi_utils,
+            cmathdecl,
+            enumdecl,
+            listdecl,
+            mathdecl,
+            npydecl,
+            randomdecl,
+            setdecl,
+            dictdecl,
+        )
         self.install_registry(cffi_utils.registry)
         self.install_registry(cmathdecl.registry)
         self.install_registry(enumdecl.registry)
@@ -663,3 +699,4 @@ class Context(BaseContext):
         self.install_registry(npydecl.registry)
         self.install_registry(randomdecl.registry)
         self.install_registry(setdecl.registry)
+        self.install_registry(dictdecl.registry)
