@@ -29,21 +29,6 @@ class ListBuiltin(AbstractTemplate):
             return signature(types.List(types.undefined))
 
 
-@infer_global(sorted)
-class SortedBuiltin(CallableTemplate):
-
-    def generic(self):
-        def typer(iterable, reverse=None):
-            if not isinstance(iterable, types.IterableType):
-                return
-            if (reverse is not None and
-                not isinstance(reverse, types.Boolean)):
-                return
-            return types.List(iterable.iterator_type.yield_type)
-
-        return typer
-
-
 @infer_getattr
 class ListAttribute(AttributeTemplate):
     key = types.List
@@ -138,18 +123,6 @@ class ListAttribute(AttributeTemplate):
         assert not kws
         return signature(types.none)
 
-    def resolve_sort(self, list):
-        def typer(reverse=None):
-            if (reverse is not None and
-                not isinstance(reverse, types.Boolean)):
-                return
-            return types.none
-
-        return types.BoundFunction(make_callable_template(key="list.sort",
-                                                          typer=typer,
-                                                          recvr=list),
-                                   list)
-
 
 @infer_global(operator.add)
 class AddList(AbstractTemplate):
