@@ -5,7 +5,7 @@ import warnings
 import numpy as np
 import operator
 
-from .. import types, utils
+from .. import types, utils, config
 from .templates import (AttributeTemplate, AbstractTemplate, CallableTemplate,
                         Registry, signature)
 
@@ -14,7 +14,7 @@ from ..numpy_support import (ufunc_find_matching_loop,
                              from_dtype, as_dtype, resolve_output_type,
                              carray, farray)
 from ..numpy_support import version as numpy_version
-from ..errors import TypingError, PerformanceWarning
+from ..errors import TypingError, NumbaPerformanceWarning
 from numba import pndindex
 
 registry = Registry()
@@ -285,6 +285,9 @@ _math_operations = [ "add", "subtract", "multiply",
                      "sqrt", "square", "reciprocal",
                      "divide", "mod", "abs", "fabs" ]
 
+if numpy_version >= (1, 15):
+    _math_operations += ["gcd", "lcm"]
+
 _trigonometric_functions = [ "sin", "cos", "tan", "arcsin",
                              "arccos", "arctan", "arctan2",
                              "hypot", "sinh", "cosh", "tanh",
@@ -378,9 +381,18 @@ class Numpy_method_redirection(AbstractTemplate):
         pysig = None
         if kws:
             if self.method_name == 'sum':
-                def sum_stub(arr, axis):
-                    pass
-                pysig = utils.pysignature(sum_stub)
+                if 'axis' in kws and 'dtype' not in kws:
+                    def sum_stub(arr, axis):
+                        pass
+                    pysig = utils.pysignature(sum_stub)
+                elif 'dtype' in kws and 'axis' not in kws:
+                    def sum_stub(arr, dtype):
+                        pass
+                    pysig = utils.pysignature(sum_stub)
+                elif 'dtype' in kws and 'axis' in kws:
+                    def sum_stub(arr, axis, dtype):
+                        pass
+                    pysig = utils.pysignature(sum_stub)
             elif self.method_name == 'argsort':
                 def argsort_stub(arr, kind='quicksort'):
                     pass
@@ -404,9 +416,6 @@ def _numpy_redirect(fname):
     cls = type("Numpy_redirect_{0}".format(fname), (Numpy_method_redirection,),
                dict(key=numpy_function, method_name=fname))
     infer_global(numpy_function, types.Function(cls))
-    # special case literal support for 'sum'
-    if fname in ['sum', 'argsort']:
-        cls.support_literals = True
 
 for func in ['min', 'max', 'sum', 'prod', 'mean', 'var', 'std',
              'cumsum', 'cumprod', 'argmin', 'argmax', 'argsort',
@@ -627,28 +636,6 @@ def _infer_dtype_from_inputs(inputs):
     return dtype
 
 
-@infer_global(np.arange)
-class NdArange(AbstractTemplate):
-
-    def generic(self, args, kws):
-        assert not kws
-        if len(args) >= 4:
-            dtype = _parse_dtype(args[3])
-            bounds = args[:3]
-        else:
-            bounds = args
-            if any(isinstance(arg, types.Complex) for arg in bounds):
-                dtype = types.complex128
-            elif any(isinstance(arg, types.Float) for arg in bounds):
-                dtype = types.float64
-            else:
-                dtype = max(bounds)
-        if not all(isinstance(arg, types.Number) for arg in bounds):
-            return
-        return_type = types.Array(ndim=1, dtype=dtype, layout='C')
-        return signature(return_type, *args)
-
-
 @infer_global(np.linspace)
 class NdLinspace(AbstractTemplate):
 
@@ -964,9 +951,11 @@ class MatMulTyperMixin(object):
         else:
             all_args = (a, b)
 
-        if not all(x.layout in 'CF' for x in (a, b)):
-            warnings.warn("%s is faster on contiguous arrays, called on %s"
-                          % (self.func_name, (a, b)), PerformanceWarning)
+        if not (config.DISABLE_PERFORMANCE_WARNINGS or
+                all(x.layout in 'CF' for x in (a, b))):
+            msg = ("%s is faster on contiguous arrays, called on %s" %
+                   (self.func_name, (a, b)))
+            warnings.warn(NumbaPerformanceWarning(msg))
         if not all(x.dtype == a.dtype for x in all_args):
             raise TypingError("%s arguments must all have "
                               "the same dtype" % (self.func_name,))
@@ -1006,7 +995,7 @@ class VDot(CallableTemplate):
                 raise TypingError("np.vdot() only supported on 1-D arrays")
             if not all(x.layout in 'CF' for x in (a, b)):
                 warnings.warn("np.vdot() is faster on contiguous arrays, called on %s"
-                              % ((a, b),), PerformanceWarning)
+                              % ((a, b),), NumbaPerformanceWarning)
             if not all(x.dtype == a.dtype for x in (a, b)):
                 raise TypingError("np.vdot() arguments must all have "
                                   "the same dtype")
@@ -1150,11 +1139,16 @@ class Where(AbstractTemplate):
                         as_dtype(getattr(args[2], 'dtype', args[2]))))
             if isinstance(cond, types.Array):
                 # array where()
-                if (cond.ndim == x.ndim == y.ndim):
-                    if x.layout == y.layout == cond.layout:
-                        retty = types.Array(retdty, x.ndim, x.layout)
-                    else:
-                        retty = types.Array(retdty, x.ndim, 'C')
+                if isinstance(x, types.Array) and isinstance(y, types.Array):
+                    if (cond.ndim == x.ndim == y.ndim):
+                        if x.layout == y.layout == cond.layout:
+                            retty = types.Array(retdty, x.ndim, x.layout)
+                        else:
+                            retty = types.Array(retdty, x.ndim, 'C')
+                        return signature(retty, *args)
+                else:
+                    # x and y both scalar
+                    retty = types.Array(retdty, cond.ndim, cond.layout)
                     return signature(retty, *args)
             else:
                 # scalar where()
