@@ -6,7 +6,7 @@ import enum
 
 import numpy as np
 
-from numba import numpy_support, types, utils, smartarray
+from numba import numpy_support, types, utils
 from numba import errors
 
 # terminal color markup
@@ -93,6 +93,14 @@ def typeof_type(val, c):
     if issubclass(val, np.generic):
         return types.NumberClass(numpy_support.from_dtype(val))
 
+    from numba.typed import Dict
+    if issubclass(val, Dict):
+        return types.TypeRef(types.DictType)
+
+    from numba.typed import List
+    if issubclass(val, List):
+        return types.TypeRef(types.ListType)
+
 
 @typeof_impl.register(bool)
 def _typeof_bool(val, c):
@@ -214,8 +222,27 @@ def _typeof_ndarray(val, c):
     readonly = not val.flags.writeable
     return types.Array(dtype, val.ndim, layout, readonly=readonly)
 
-@typeof_impl.register(smartarray.SmartArray)
-def typeof_array(val, c):
-    arrty = typeof_impl(val.get('host'), c)
-    return types.SmartArrayType(arrty.dtype, arrty.ndim, arrty.layout, type(val))
 
+@typeof_impl.register(types.NumberClass)
+def typeof_number_class(val, c):
+    return val
+
+
+@typeof_impl.register(types.Literal)
+def typeof_literal(val, c):
+    return val
+
+
+@typeof_impl.register(types.TypeRef)
+def typeof_typeref(val, c):
+    return val
+
+
+@typeof_impl.register(types.Type)
+def typeof_typeref(val, c):
+    if isinstance(val, types.BaseFunction):
+        return val
+    elif isinstance(val, (types.Number, types.Boolean)):
+        return types.NumberClass(val)
+    else:
+        return types.TypeRef(val)
